@@ -1,39 +1,37 @@
 #include <filezilla.h>
 #include "ControlSocket.h"
 #include "directorycache.h"
-#include "local_filesys.h"
+#include "engineprivate.h"
 #include "local_path.h"
 #include "logging_private.h"
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sizeformatting_base.h"
 
+#include <libfilezilla/event_loop.hpp>
+#include <libfilezilla/iputils.hpp>
+#include <libfilezilla/local_filesys.hpp>
+
 #include <wx/file.h>
 #include <wx/filename.h>
 
 #ifndef __WXMSW__
-#include <idna.h>
-extern "C" {
-#include <idn-free.h>
-}
+	#define mutex mutex_override // Sadly on some platforms system headers include conflicting names
+	#include <netdb.h>
+	#undef mutex
+	#ifndef AI_IDN
+		#include <idna.h>
+		extern "C" {
+			#include <idn-free.h>
+		}
+	#endif
 #endif
 
-#include <errno.h>
-
-DECLARE_EVENT_TYPE(fzOBTAINLOCK, -1)
-DEFINE_EVENT_TYPE(fzOBTAINLOCK)
-
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
+struct obtain_lock_event_type;
+typedef fz::simple_event<obtain_lock_event_type> CObtainLockEvent;
 
 std::list<CControlSocket::t_lockInfo> CControlSocket::m_lockInfoList;
 
-BEGIN_EVENT_TABLE(CControlSocket, wxEvtHandler)
-	EVT_TIMER(wxID_ANY, CControlSocket::OnTimer)
-	EVT_COMMAND(wxID_ANY, fzOBTAINLOCK, CControlSocket::OnObtainLock)
-END_EVENT_TABLE()
-
 COpData::COpData(Command op_Id)
 	: opId(op_Id)
 {
@@ -51,21 +49,18 @@ COpData::~COpData()
 	delete pNextOpData;
 }
 
-CControlSocket::CControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CLogging(pEngine)
+CControlSocket::CControlSocket(CFileZillaEnginePrivate & engine)
+	: CLogging(engine)
+	, event_handler(engine.event_loop_)
+	, engine_(engine)
 {
-	m_pEngine = pEngine;
 	m_pCurOpData = 0;
 	m_nOpState = 0;
 	m_pCurrentServer = 0;
-	m_pTransferStatus = 0;
-	m_transferStatusSendState = 0;
 
 	m_pCSConv = 0;
 	m_useUTF8 = false;
 
-	m_timer.SetOwner(this);
-
 	m_closed = false;
 
 	m_invalidateCurrentPath = false;
@@ -73,6 +68,8 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate *pEngine)
 
 CControlSocket::~CControlSocket()
 {
+	remove_handler();
+
 	DoClose();
 
 	delete m_pCSConv;
@@ -92,27 +89,28 @@ Command CControlSocket::GetCurrentCommandId() const
 	if (m_pCurOpData)
 		return m_pCurOpData->opId;
 
-	return m_pEngine->GetCurrentCommandId();
+	return engine_.GetCurrentCommandId();
 }
 
 void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData)
 {
-	if (m_pTransferStatus && (nErrorCode == FZ_REPLY_OK || m_pTransferStatus->madeProgress))
-	{
-		int elapsed = wxTimeSpan(wxDateTime::Now() - m_pTransferStatus->started).GetSeconds().GetLo();
+	bool tmp;
+
+	CTransferStatus const status = engine_.transfer_status_.Get(tmp);
+	if (!status.empty() && (nErrorCode == FZ_REPLY_OK || status.madeProgress)) {
+		int elapsed = static_cast<int>((fz::datetime::now() - status.started).get_seconds());
 		if (elapsed <= 0)
 			elapsed = 1;
 		wxString time = wxString::Format(
 			wxPLURAL("%d second", "%d seconds", elapsed),
 			elapsed);
 
-		wxLongLong transferred = m_pTransferStatus->currentOffset - m_pTransferStatus->startOffset;
-		wxString size = CSizeFormatBase::Format(m_pEngine->GetOptions(), transferred, true);
+		int64_t transferred = status.currentOffset - status.startOffset;
+		wxString size = CSizeFormatBase::Format(&engine_.GetOptions(), transferred, true);
 
 		MessageType msgType = MessageType::Error;
 		wxString msg;
-		if (nErrorCode == FZ_REPLY_OK)
-		{
+		if (nErrorCode == FZ_REPLY_OK) {
 			msgType = MessageType::Status;
 			msg = _("File transfer successful, transferred %s in %s");
 		}
@@ -124,12 +122,10 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 			msg = _("File transfer failed after transferring %s in %s");
 		LogMessage(msgType, msg, size, time);
 	}
-	else
-	{
+	else {
 		if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
 			LogMessage(MessageType::Error, _("File transfer aborted by user"));
-		else if (nErrorCode == FZ_REPLY_OK)
-		{
+		else if (nErrorCode == FZ_REPLY_OK) {
 			if (pData->transferInitiated)
 				LogMessage(MessageType::Status, _("File transfer successful"));
 			else
@@ -146,16 +142,14 @@ int CControlSocket::ResetOperation(int nErrorCode)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CControlSocket::ResetOperation(%d)"), nErrorCode);
 
-	if (nErrorCode & FZ_REPLY_WOULDBLOCK)
-	{
+	if (nErrorCode & FZ_REPLY_WOULDBLOCK) {
 		LogMessage(MessageType::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
 	}
 
 	if (m_pCurOpData && m_pCurOpData->holdsLock)
 		UnlockCache();
 
-	if (m_pCurOpData && m_pCurOpData->pNextOpData)
-	{
+	if (m_pCurOpData && m_pCurOpData->pNextOpData) {
 		COpData *pNext = m_pCurOpData->pNextOpData;
 		m_pCurOpData->pNextOpData = 0;
 		delete m_pCurOpData;
@@ -177,8 +171,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		prefix = _("Critical error:") + _T(" ");
 	}
 
-	if (m_pCurOpData)
-	{
+	if (m_pCurOpData) {
 		const Command commandId = m_pCurOpData->opId;
 		switch (commandId)
 		{
@@ -198,23 +191,25 @@ int CControlSocket::ResetOperation(int nErrorCode)
 				LogMessage(MessageType::Error, prefix + _("Directory listing aborted by user"));
 			else if (nErrorCode != FZ_REPLY_OK)
 				LogMessage(MessageType::Error, prefix + _("Failed to retrieve directory listing"));
-			else
-				LogMessage(MessageType::Status, _("Directory listing successful"));
+			else {
+				if (m_CurrentPath.empty()) {
+					LogMessage(MessageType::Status, _("Directory listing successful"));
+				}
+				else {
+					LogMessage(MessageType::Status, _("Directory listing of \"%s\" successful"), m_CurrentPath.GetPath());
+				}
+			}
 			break;
 		case Command::transfer:
 			{
 				CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
-				if (!pData->download && pData->transferInitiated)
-				{
+				if (!pData->download && pData->transferInitiated) {
 					if (!m_pCurrentServer)
 						LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurrentServer is 0"));
-					else
-					{
-						CDirectoryCache cache;
-						bool updated = cache.UpdateFile(*m_pCurrentServer, pData->remotePath, pData->remoteFile, true, CDirectoryCache::file, (nErrorCode == FZ_REPLY_OK) ? pData->localFileSize : -1);
-
+					else {
+						bool updated = engine_.GetDirectoryCache().UpdateFile(*m_pCurrentServer, pData->remotePath, pData->remoteFile, true, CDirectoryCache::file, (nErrorCode == FZ_REPLY_OK) ? pData->localFileSize : -1);
 						if (updated)
-							m_pEngine->SendDirectoryListingNotification(pData->remotePath, false, true, false);
+							engine_.SendDirectoryListingNotification(pData->remotePath, false, true, false);
 					}
 				}
 				LogTransferResultMessage(nErrorCode, pData);
@@ -230,7 +225,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		m_pCurOpData = 0;
 	}
 
-	ResetTransferStatus();
+	engine_.transfer_status_.Reset();
 
 	SetWait(false);
 
@@ -239,14 +234,13 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		m_invalidateCurrentPath = false;
 	}
 
-	return m_pEngine->ResetOperation(nErrorCode);
+	return engine_.ResetOperation(nErrorCode);
 }
 
 int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
 	LogMessage(MessageType::Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
-	if (m_closed)
-	{
+	if (m_closed) {
 		wxASSERT(!m_pCurOpData);
 		return nErrorCode;
 	}
@@ -264,15 +258,16 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 wxString CControlSocket::ConvertDomainName(wxString const& domain)
 {
 #ifdef __WXMSW__
-	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, 0, 0);
-	if( !len ) {
+	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, 0, 0);
+	if (!len) {
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 
 	wchar_t* output = new wchar_t[len];
-	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, output, len);
-	if( !res ) {
+	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, output, len);
+	if (!res) {
+		delete [] output;
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
@@ -280,8 +275,9 @@ wxString CControlSocket::ConvertDomainName(wxString const& domain)
 	wxString ret(output);
 	delete [] output;
 	return ret;
+#elif defined(AI_IDN)
+	return domain;
 #else
-
 	wxScopedCharBuffer const utf8 = domain.utf8_str();
 
 	char *output = 0;
@@ -307,84 +303,6 @@ void CControlSocket::Cancel()
 	}
 }
 
-void CControlSocket::ResetTransferStatus()
-{
-	delete m_pTransferStatus;
-	m_pTransferStatus = 0;
-
-	m_pEngine->AddNotification(new CTransferStatusNotification(0));
-
-	m_transferStatusSendState = 0;
-}
-
-void CControlSocket::InitTransferStatus(wxFileOffset totalSize, wxFileOffset startOffset, bool list)
-{
-	if (startOffset < 0)
-		startOffset = 0;
-
-	delete m_pTransferStatus;
-	m_pTransferStatus = new CTransferStatus();
-
-	m_pTransferStatus->list = list;
-	m_pTransferStatus->totalSize = totalSize;
-	m_pTransferStatus->startOffset = startOffset;
-	m_pTransferStatus->currentOffset = startOffset;
-	m_pTransferStatus->madeProgress = false;
-}
-
-void CControlSocket::SetTransferStatusStartTime()
-{
-	if (!m_pTransferStatus)
-		return;
-
-	m_pTransferStatus->started = wxDateTime::Now();
-}
-
-void CControlSocket::SetTransferStatusMadeProgress()
-{
-	if (!m_pTransferStatus)
-		return;
-
-	m_pTransferStatus->madeProgress = true;
-}
-
-void CControlSocket::UpdateTransferStatus(wxFileOffset transferredBytes)
-{
-	if (!m_pTransferStatus)
-		return;
-
-	m_pTransferStatus->currentOffset += transferredBytes;
-
-	if (!m_transferStatusSendState)
-		m_pEngine->AddNotification(new CTransferStatusNotification(new CTransferStatus(*m_pTransferStatus)));
-	m_transferStatusSendState = 2;
-}
-
-bool CControlSocket::GetTransferStatus(CTransferStatus &status, bool &changed)
-{
-	if (!m_pTransferStatus)
-	{
-		changed = false;
-		m_transferStatusSendState = 0;
-		return false;
-	}
-
-	status = *m_pTransferStatus;
-	if (m_transferStatusSendState == 2)
-	{
-		changed = true;
-		m_transferStatusSendState = 1;
-		return true;
-	}
-	else
-	{
-		changed = false;
-		m_transferStatusSendState = 0;
-		return true;
-	}
-}
-
-
 const CServer* CControlSocket::GetCurrentServer() const
 {
 	return m_pCurrentServer;
@@ -459,21 +377,19 @@ int CControlSocket::CheckOverwriteFile()
 	CDirentry entry;
 	bool dirDidExist;
 	bool matchedCase;
-	CDirectoryCache cache;
 	CServerPath remotePath;
 	if (pData->tryAbsolutePath || m_CurrentPath.empty())
 		remotePath = pData->remotePath;
 	else
 		remotePath = m_CurrentPath;
-	bool found = cache.LookupFile(entry, *m_pCurrentServer, remotePath, pData->remoteFile, dirDidExist, matchedCase);
+	bool found = engine_.GetDirectoryCache().LookupFile(entry, *m_pCurrentServer, remotePath, pData->remoteFile, dirDidExist, matchedCase);
 
 	// Ignore entries with wrong case
 	if (found && !matchedCase)
 		found = false;
 
-	if (!pData->download)
-	{
-		if (!found && pData->remoteFileSize == -1 && !pData->fileTime.IsValid())
+	if (!pData->download) {
+		if (!found && pData->remoteFileSize < 0 && !pData->fileTime.empty())
 			return FZ_REPLY_OK;
 	}
 
@@ -487,20 +403,20 @@ int CControlSocket::CheckOverwriteFile()
 	pNotification->remoteSize = pData->remoteFileSize;
 	pNotification->ascii = !pData->transferSettings.binary;
 
-	if (pData->download && pNotification->localSize != -1)
+	if (pData->download && pNotification->localSize >= 0)
 		pNotification->canResume = true;
-	else if (!pData->download && pNotification->remoteSize != -1)
+	else if (!pData->download && pNotification->remoteSize >= 0)
 		pNotification->canResume = true;
 	else
 		pNotification->canResume = false;
 
-	pNotification->localTime = CLocalFileSystem::GetModificationTime(pData->localFile);
+	pNotification->localTime = fz::local_filesys::get_modification_time(fz::to_native(pData->localFile));
 
-	if (pData->fileTime.IsValid())
+	if (pData->fileTime.empty())
 		pNotification->remoteTime = pData->fileTime;
 
 	if (found) {
-		if (!pData->fileTime.IsValid()) {
+		if (!pData->fileTime.empty()) {
 			if (entry.has_date()) {
 				pNotification->remoteTime = entry.time;
 				pData->fileTime = entry.time;
@@ -516,9 +432,7 @@ int CControlSocket::CheckOverwriteFile()
 CFileTransferOpData::CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path) :
 	COpData(Command::transfer),
 	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
-	download(is_download),
-	localFileSize(-1), remoteFileSize(-1),
-	tryAbsolutePath(false), resume(false), transferInitiated(false)
+	download(is_download)
 {
 }
 
@@ -526,32 +440,28 @@ CFileTransferOpData::~CFileTransferOpData()
 {
 }
 
-wxString CControlSocket::ConvToLocal(const char* buffer)
+wxString CControlSocket::ConvToLocal(const char* buffer, size_t len)
 {
-	if (m_useUTF8)
-	{
-		wxChar* out = ConvToLocalBuffer(buffer, wxConvUTF8);
-		if (out)
-		{
-			wxString str = out;
+	size_t outLen{};
+	if (m_useUTF8) {
+		wxChar* out = ConvToLocalBuffer(buffer, wxConvUTF8, len, outLen);
+		if (out) {
+			wxString str(out, outLen - 1);
 			delete [] out;
 			return str;
 		}
 
 		// Fall back to local charset on error
-		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8)
-		{
+		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8) {
 			LogMessage(MessageType::Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
 			m_useUTF8 = false;
 		}
 	}
 
-	if (m_pCSConv)
-	{
-		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv);
-		if (out)
-		{
-			wxString str = out;
+	if (m_pCSConv) {
+		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv, len, outLen);
+		if (out) {
+			wxString str(out, outLen - 1);
 			delete [] out;
 			return str;
 		}
@@ -565,42 +475,53 @@ wxString CControlSocket::ConvToLocal(const char* buffer)
 	return str;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv)
+wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
 {
-	size_t len = conv.MB2WC(0, buffer, 0);
-	if (!len || len == wxCONV_FAILED)
+	wxASSERT(buffer && len > 0 && !buffer[len - 1]);
+	outlen = conv.ToWChar(0, 0, buffer, len);
+	if (!outlen || outlen == wxCONV_FAILED)
 		return 0;
 
-	wchar_t* unicode = new wchar_t[len + 1];
-	conv.MB2WC(unicode, buffer, len + 1);
+	wchar_t* unicode = new wchar_t[outlen];
+	conv.ToWChar(unicode, outlen, buffer, len);
 	return unicode;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
+wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
 {
-	if (m_useUTF8)
-	{
-		wxChar* res = ConvToLocalBuffer(buffer, wxConvUTF8);
+	if (m_useUTF8) {
+#ifdef __WXMSW__
+		// wxConvUTF8 is generic and slow.
+		// Use the highly optimized MultiByteToWideChar on Windows
+		// This helps when processing large directory listings.
+		int outlen2 = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, 0, 0);
+		if (outlen2 > 0) {
+			wxChar* out = new wxChar[outlen2];
+			MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, out, outlen2);
+			outlen = static_cast<size_t>(outlen2);
+			return out;
+		}
+#else
+		wxChar* res = ConvToLocalBuffer(buffer, wxConvUTF8, len, outlen);
 		if (res && *res)
 			return res;
+#endif
 
 		// Fall back to local charset on error
-		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8)
-		{
+		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8) {
 			LogMessage(MessageType::Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
 			m_useUTF8 = false;
 		}
 	}
 
-	if (m_pCSConv)
-	{
-		wxChar* res = ConvToLocalBuffer(buffer, *m_pCSConv);
+	if (m_pCSConv) {
+		wxChar* res = ConvToLocalBuffer(buffer, *m_pCSConv, len, outlen);
 		if (res && *res)
 			return res;
 	}
 
 	// Fallback: Conversion using current locale
-	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent);
+	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent, len, outlen);
 
 	return res;
 }
@@ -626,43 +547,52 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /
 	return buffer;
 }
 
-void CControlSocket::OnTimer(wxTimerEvent&)
+void CControlSocket::OnTimer(fz::timer_id)
 {
-	int timeout = m_pEngine->GetOptions()->GetOptionVal(OPTION_TIMEOUT);
-	if (!timeout)
-		return;
+	m_timer = 0; // It's a one-shot timer, no need to stop it
 
-	if (m_pCurOpData && m_pCurOpData->waitForAsyncRequest)
-		return;
+	int const timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
+	if (timeout > 0) {
+		fz::duration elapsed = fz::monotonic_clock::now() - m_lastActivity;
 
-	if (IsWaitingForLock())
-		return;
+		if ((!m_pCurOpData || !m_pCurOpData->waitForAsyncRequest) && !IsWaitingForLock()) {
+			if (elapsed > fz::duration::from_seconds(timeout)) {
+				LogMessage(MessageType::Error, wxPLURAL("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
+				DoClose(FZ_REPLY_TIMEOUT);
+				return;
+			}
+		}
+		else {
+			elapsed = fz::duration();
+		}
 
-	if (m_stopWatch.Time() > (timeout * 1000))
-	{
-		LogMessage(MessageType::Error, _("Connection timed out"));
-		DoClose(FZ_REPLY_TIMEOUT);
-		wxASSERT(!m_timer.IsRunning());
+		m_timer = add_timer(fz::duration::from_milliseconds(timeout * 1000) - elapsed, true);
 	}
 }
 
 void CControlSocket::SetAlive()
 {
-	m_stopWatch.Start();
+	m_lastActivity = fz::monotonic_clock::now();
 }
 
 void CControlSocket::SetWait(bool wait)
 {
-	if (wait)
-	{
-		if (m_timer.IsRunning())
+	if (wait) {
+		if (m_timer)
 			return;
 
-		m_stopWatch.Start();
-		m_timer.Start(1000);
+		m_lastActivity = fz::monotonic_clock::now();
+
+		int timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
+		if (!timeout)
+			return;
+
+		m_timer = add_timer(fz::duration::from_milliseconds(timeout * 1000 + 100), true); // Add a bit of slack
+	}
+	else {
+		stop_timer(m_timer);
+		m_timer = 0;
 	}
-	else if (m_timer.IsRunning())
-		m_timer.Stop();
 }
 
 int CControlSocket::SendNextCommand()
@@ -792,31 +722,27 @@ void CControlSocket::UnlockCache()
 	m_lockInfoList.erase(iter);
 
 	// Find other instance waiting for the lock
-	if (!m_pCurrentServer)
-	{
+	if (!m_pCurrentServer) {
 		LogMessage(MessageType::Debug_Warning, _T("UnlockCache called with !m_pCurrentServer"));
 		return;
 	}
-	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); ++iter)
-	{
-		if (!iter->pControlSocket->m_pCurrentServer)
-		{
+	for (auto & lockInfo : m_lockInfoList) {
+		if (!lockInfo.pControlSocket->m_pCurrentServer){
 			LogMessage(MessageType::Debug_Warning, _T("UnlockCache found other instance with !m_pCurrentServer"));
 			continue;
 		}
 
-		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
+		if (*m_pCurrentServer != *lockInfo.pControlSocket->m_pCurrentServer)
 			continue;
 
-		if (iter->directory != directory)
+		if (lockInfo.directory != directory)
 			continue;
 
-		if (iter->reason != reason)
+		if (lockInfo.reason != reason)
 			continue;
 
 		// Send notification
-		wxCommandEvent evt(fzOBTAINLOCK);
-		iter->pControlSocket->AddPendingEvent(evt);
+		lockInfo.pControlSocket->send_event<CObtainLockEvent>();
 		break;
 	}
 }
@@ -854,7 +780,7 @@ enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 	return own->reason;
 }
 
-void CControlSocket::OnObtainLock(wxCommandEvent&)
+void CControlSocket::OnObtainLock()
 {
 	if (ObtainLockFromEvent() == lock_unknown)
 		return;
@@ -888,39 +814,39 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 	}
 }
 
-wxTimeSpan CControlSocket::GetTimezoneOffset()
+fz::duration CControlSocket::GetTimezoneOffset() const
 {
-	if (!m_pCurrentServer)
-		return wxTimeSpan();
-
-	int seconds = 0;
-	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) != yes)
-		return wxTimeSpan();
-
-	return wxTimeSpan(0, 0, seconds);
+	fz::duration ret;
+	if (m_pCurrentServer) {
+		int seconds = 0;
+		if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) == yes) {
+			ret = fz::duration::from_seconds(seconds);
+		}
+	}
+	return ret;
 }
 
 void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
 {
 	wxASSERT(pNotification);
 
-	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
+	pNotification->requestNumber = engine_.GetNextAsyncRequestNumber();
 
 	if (m_pCurOpData)
 		m_pCurOpData->waitForAsyncRequest = true;
-	m_pEngine->AddNotification(pNotification);
+	engine_.AddNotification(pNotification);
 }
 
 // ------------------
 // CRealControlSocket
 // ------------------
 
-CRealControlSocket::CRealControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CControlSocket(pEngine)
+CRealControlSocket::CRealControlSocket(CFileZillaEnginePrivate & engine)
+	: CControlSocket(engine)
 {
 	m_pSocket = new CSocket(this);
 
-	m_pBackend = new CSocketBackend(this, m_pSocket);
+	m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 	m_pProxyBackend = 0;
 
 	m_pSendBuffer = 0;
@@ -941,8 +867,7 @@ CRealControlSocket::~CRealControlSocket()
 bool CRealControlSocket::Send(const char *buffer, int len)
 {
 	SetWait(true);
-	if (m_pSendBuffer)
-	{
+	if (m_pSendBuffer) {
 		char *tmp = m_pSendBuffer;
 		m_pSendBuffer = new char[m_nSendBufferLen + len];
 		memcpy(m_pSendBuffer, tmp, m_nSendBufferLen);
@@ -950,14 +875,11 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 		m_nSendBufferLen += len;
 		delete [] tmp;
 	}
-	else
-	{
+	else {
 		int error;
 		int written = m_pBackend->Write(buffer, len, error);
-		if (written < 0)
-		{
-			if (error != EAGAIN)
-			{
+		if (written < 0) {
+			if (error != EAGAIN) {
 				LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
 				LogMessage(MessageType::Error, _("Disconnected from server"));
 				DoClose();
@@ -967,70 +889,76 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 		}
 
 		if (written)
-			m_pEngine->SetActive(CFileZillaEngine::send);
+			SetActive(CFileZillaEngine::send);
 
-		if (written < len)
-		{
-			char *tmp = m_pSendBuffer;
-			m_pSendBuffer = new char[m_nSendBufferLen + len - written];
-			memcpy(m_pSendBuffer, tmp, m_nSendBufferLen);
-			memcpy(m_pSendBuffer + m_nSendBufferLen, buffer, len - written);
-			m_nSendBufferLen += len - written;
-			delete [] tmp;
+		if (written < len) {
+			m_nSendBufferLen = len - written;
+			m_pSendBuffer = new char[m_nSendBufferLen];
+			memcpy(m_pSendBuffer, buffer, len - written);
 		}
 	}
 
 	return true;
 }
 
-void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
+void CRealControlSocket::operator()(fz::event_base const& ev)
+{
+	if (!fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+		&CRealControlSocket::OnSocketEvent,
+		&CRealControlSocket::OnHostAddress))
+	{
+		CControlSocket::operator()(ev);
+	}
+}
+
+void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
 	if (!m_pBackend)
 		return;
 
-	switch (event.GetType())
+	switch (t)
 	{
-	case CSocketEvent::hostaddress:
-		{
-			const wxString& address = event.GetData();
-			LogMessage(MessageType::Status, _("Connecting to %s..."), address);
-		}
+	case SocketEventType::connection_next:
+		if (error)
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
+		SetAlive();
 		break;
-	case CSocketEvent::connection_next:
-		if (event.GetError())
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()));
-		break;
-	case CSocketEvent::connection:
-		if (event.GetError())
-		{
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()));
-			OnClose(event.GetError());
+	case SocketEventType::connection:
+		if (error) {
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(error));
+			OnClose(error);
 		}
-		else
-		{
-			if (m_pProxyBackend && !m_pProxyBackend->Detached())
-			{
+		else {
+			if (m_pProxyBackend && !m_pProxyBackend->Detached()) {
 				m_pProxyBackend->Detach();
-				m_pBackend = new CSocketBackend(this, m_pSocket);
+				m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 			}
 			OnConnect();
 		}
 		break;
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
-	case CSocketEvent::close:
-		OnClose(event.GetError());
+	case SocketEventType::close:
+		OnClose(error);
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
+		LogMessage(MessageType::Debug_Warning, _T("Unhandled socket event %d"), t);
 		break;
 	}
 }
 
+void CRealControlSocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+{
+	if (!m_pBackend)
+		return;
+
+	LogMessage(MessageType::Status, _("Connecting to %s..."), address);
+}
+
 void CRealControlSocket::OnConnect()
 {
 }
@@ -1064,20 +992,16 @@ void CRealControlSocket::OnSend()
 			return;
 		}
 
-		if (written)
-		{
-			SetAlive();
-			m_pEngine->SetActive(CFileZillaEngine::send);
+		if (written) {
+			SetActive(CFileZillaEngine::send);
 		}
 
-		if (written == m_nSendBufferLen)
-		{
+		if (written == m_nSendBufferLen) {
 			m_nSendBufferLen = 0;
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 		}
-		else
-		{
+		else {
 			memmove(m_pSendBuffer, m_pSendBuffer + written, m_nSendBufferLen - written);
 			m_nSendBufferLen -= written;
 		}
@@ -1122,20 +1046,20 @@ int CRealControlSocket::ContinueConnect()
 	wxString host;
 	unsigned int port = 0;
 
-	const int proxy_type = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE);
+	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
-		LogMessage(MessageType::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost());
+		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->FormatHost(), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
-		host = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_HOST);
-		port = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_PORT);
+		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
+		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
 
 		delete m_pBackend;
 		m_pProxyBackend = new CProxySocket(this, m_pSocket, this);
 		m_pBackend = m_pProxyBackend;
 		int res = m_pProxyBackend->Handshake((enum CProxySocket::ProxyType)proxy_type,
-											 m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
-											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_USER),
-											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_PASS));
+											m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
+											engine_.GetOptions().GetOption(OPTION_PROXY_USER),
+											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
 
 		if (res != EINPROGRESS) {
 			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res));
@@ -1154,8 +1078,9 @@ int CRealControlSocket::ContinueConnect()
 			port = m_pCurrentServer->GetPort();
 		}
 	}
-	if (!IsIpAddress(host))
+	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
+	}
 
 	int res = m_pSocket->Connect(host, port);
 
@@ -1215,11 +1140,11 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::overwriteNewer:
-		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
 		else
 		{
@@ -1238,7 +1163,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 	case CFileExistsNotification::overwriteSize:
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
-		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0))
 			SendNextCommand();
 		else {
 			if (pData->download) {
@@ -1252,15 +1177,15 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::overwriteSizeOrNewer:
-		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
-		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0))
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
 		else
 		{
@@ -1277,52 +1202,46 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::resume:
-		if (pData->download && pData->localFileSize != -1)
+		if (pData->download && pData->localFileSize >= 0)
 			pData->resume = true;
-		else if (!pData->download && pData->remoteFileSize != -1)
+		else if (!pData->download && pData->remoteFileSize >= 0)
 			pData->resume = true;
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::rename:
-		if (pData->download)
-		{
+		if (pData->download) {
 			wxFileName fn = pData->localFile;
 			fn.SetFullName(pFileExistsNotification->newName);
 			pData->localFile = fn.GetFullPath();
 
-			wxLongLong size;
+			int64_t size;
 			bool isLink;
-			if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
-				pData->localFileSize = size.GetValue();
+			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file)
+				pData->localFileSize = size;
 			else
 				pData->localFileSize = -1;
 
 			if (CheckOverwriteFile() == FZ_REPLY_OK)
 				SendNextCommand();
 		}
-		else
-		{
+		else {
 			pData->remoteFile = pFileExistsNotification->newName;
 
-			CDirectoryCache cache;
-
 			CDirentry entry;
 			bool dir_did_exist;
 			bool matched_case;
-			if (cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
+			if (engine_.GetDirectoryCache().LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
 				matched_case)
 			{
-				wxLongLong size = entry.size;
-				pData->remoteFileSize = size.GetValue();
+				pData->remoteFileSize = entry.size;
 				if (entry.has_date())
 					pData->fileTime = entry.time;
 
 				if (CheckOverwriteFile() != FZ_REPLY_OK)
 					break;
 			}
-			else
-			{
-				pData->fileTime = CDateTime();
+			else {
+				pData->fileTime = fz::datetime();
 				pData->remoteFileSize = -1;
 			}
 
@@ -1359,21 +1278,19 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 
 	// Only go back as far as needed. On comparison, wxWidgets'
 	// wxFileName::Mkdir always starts at the root.
-	std::list<wxString> segments;
-	while (!local_path.Exists() && local_path.HasParent())
-	{
+	std::vector<wxString> segments;
+	while (!local_path.Exists() && local_path.HasParent()) {
 		wxString segment;
 		local_path.MakeParent(&segment);
-		segments.push_front(segment);
+		segments.push_back(segment);
 	}
 
 	CLocalPath last_successful;
-	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
-	{
+	for (auto iter = segments.rbegin(); iter != segments.rend(); ++iter) {
 		local_path.AddSegment(*iter);
 
 #ifdef __WXMSW__
-		BOOL res = CreateDirectory(local_path.GetPath(), 0);
+		BOOL res = CreateDirectory(local_path.GetPath().wc_str(), 0);
 		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
 			break;
 #else
@@ -1392,7 +1309,7 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	// Send out notification
 	CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
 	n->dir = last_successful;
-	m_pEngine->AddNotification(n);
+	engine_.AddNotification(n);
 }
 
 int CControlSocket::List(CServerPath, wxString, int)
@@ -1412,7 +1329,7 @@ int CControlSocket::RawCommand(const wxString&)
 	return FZ_REPLY_NOTSUPPORTED;
 }
 
-int CControlSocket::Delete(const CServerPath&, const std::list<wxString>&)
+int CControlSocket::Delete(const CServerPath&, std::deque<wxString>&&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
@@ -1436,3 +1353,16 @@ int CControlSocket::Chmod(const CChmodCommand&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
+
+void CControlSocket::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<fz::timer_event, CObtainLockEvent>(ev, this,
+		&CControlSocket::OnTimer,
+		&CControlSocket::OnObtainLock);
+}
+
+void CControlSocket::SetActive(CFileZillaEngine::_direction direction)
+{
+	SetAlive();
+	engine_.SetActive(direction);
+}
