@@ -4,27 +4,30 @@
 #include "ratelimiter.h"
 #include "socket.h"
 
-class CBackend : public CRateLimiterObject
+class CBackend : public CRateLimiterObject, public CSocketEventSource
 {
 public:
-	CBackend(CSocketEventHandler* pEvtHandler);
-	virtual ~CBackend() {}
+	CBackend(fz::event_handler* pEvtHandler);
+	virtual ~CBackend();
+
+	CBackend(CBackend const&) = delete;
+	CBackend& operator=(CBackend const&) = delete;
 
 	virtual int Read(void *buffer, unsigned int size, int& error) = 0;
 	virtual int Peek(void *buffer, unsigned int size, int& error) = 0;
 	virtual int Write(const void *buffer, unsigned int size, int& error) = 0;
 
-	virtual void OnRateAvailable(enum CRateLimiter::rate_direction direction) = 0;
+	virtual void OnRateAvailable(CRateLimiter::rate_direction direction) = 0;
 
 protected:
-	CSocketEventHandler* const m_pEvtHandler;
+	fz::event_handler* const m_pEvtHandler;
 };
 
 class CSocket;
-class CSocketBackend final : public CBackend, public CSocketEventSource
+class CSocketBackend final : public CBackend
 {
 public:
-	CSocketBackend(CSocketEventHandler* pEvtHandler, CSocket* pSocket);
+	CSocketBackend(fz::event_handler* pEvtHandler, CSocket & socket, CRateLimiter& rateLimiter);
 	virtual ~CSocketBackend();
 	// Backend definitions
 	virtual int Read(void *buffer, unsigned int size, int& error);
@@ -32,9 +35,10 @@ public:
 	virtual int Write(const void *buffer, unsigned int size, int& error);
 
 protected:
-	virtual void OnRateAvailable(enum CRateLimiter::rate_direction direction);
+	virtual void OnRateAvailable(CRateLimiter::rate_direction direction);
 
-	CSocket* m_pSocket;
+	CSocket &socket_;
+	CRateLimiter& m_rateLimiter;
 };
 
 #endif //__BACKEND_H__
