@@ -14,7 +14,9 @@ but for some operations the engine/interface prefers to retrieve a clean
 version.
 */
 
-#include <mutex.h>
+#include <libfilezilla/mutex.hpp>
+
+#include <set>
 
 const int CACHE_TIMEOUT = 900; // In seconds
 
@@ -35,12 +37,12 @@ public:
 	CDirectoryCache& operator=(CDirectoryCache const&) = delete;
 
 	void Store(const CDirectoryListing &listing, const CServer &server);
-	bool GetChangeTime(CMonotonicClock& time, const CServer &server, const CServerPath &path);
+	bool GetChangeTime(fz::monotonic_clock& time, const CServer &server, const CServerPath &path);
 	bool Lookup(CDirectoryListing &listing, const CServer &server, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated);
 	bool DoesExist(const CServer &server, const CServerPath &path, int &hasUnsureEntries, bool &is_outdated);
 	bool LookupFile(CDirentry &entry, const CServer &server, const CServerPath &path, const wxString& file, bool &dirDidExist, bool &matchedCase);
 	bool InvalidateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool *wasDir = 0);
-	bool UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type = file, wxLongLong size = -1);
+	bool UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type = file, int64_t size = -1);
 	bool RemoveFile(const CServer &server, const CServerPath &path, const wxString& filename);
 	void InvalidateServer(const CServer& server);
 	void RemoveDir(const CServer& server, const CServerPath& path, const wxString& filename, const CServerPath& target);
@@ -51,20 +53,26 @@ protected:
 	class CCacheEntry final
 	{
 	public:
-		CCacheEntry() : lruIt() { };
-		CCacheEntry(const CCacheEntry &entry);
+		CCacheEntry() = default;
+		CCacheEntry(CCacheEntry const& entry) = default;
+		CCacheEntry(CCacheEntry && entry) noexcept = default;
+
 		explicit CCacheEntry(CDirectoryListing const& l)
 			: listing(l)
-			, modificationTime(CMonotonicClock::now())
+			, modificationTime(fz::monotonic_clock::now())
 		{}
-		~CCacheEntry() { };
 
 		CDirectoryListing listing;
-		CMonotonicClock modificationTime;
+		fz::monotonic_clock modificationTime;
 
-		CCacheEntry& operator=(const CCacheEntry &a);
+		CCacheEntry& operator=(CCacheEntry const& a) = default;
+		CCacheEntry& operator=(CCacheEntry && a) noexcept = default;
 
 		void* lruIt{}; // void* to break cyclic declaration dependency
+
+		bool operator<(CCacheEntry const& op) const noexcept {
+			return listing.path < op.listing.path;
+		}
 	};
 
 	class CServerEntry final
@@ -76,7 +84,7 @@ protected:
 		{}
 
 		CServer server;
-		std::list<CCacheEntry> cacheList;
+		std::set<CCacheEntry> cacheList;
 	};
 
 	typedef std::list<CServerEntry>::iterator tServerIter;
@@ -84,12 +92,12 @@ protected:
 	tServerIter CreateServerEntry(const CServer& server);
 	tServerIter GetServerEntry(const CServer& server);
 
-	typedef std::list<CCacheEntry>::iterator tCacheIter;
-	typedef std::list<CCacheEntry>::const_iterator tCacheConstIter;
+	typedef std::set<CCacheEntry>::iterator tCacheIter;
+	typedef std::set<CCacheEntry>::const_iterator tCacheConstIter;
 
 	bool Lookup(tCacheIter &cacheIter, tServerIter &sit, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated);
 
-	mutex mutex_;
+	fz::mutex mutex_;
 
 	std::list<CServerEntry> m_serverList;
 
