@@ -1,52 +1,16 @@
 #include <filezilla.h>
 
-CDirectoryListing::CDirectoryListing()
-	: m_flags()
-	, m_entryCount()
-{
-}
-
-CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
-	: path(listing.path)
-	, m_firstListTime(listing.m_firstListTime)
-	, m_flags(listing.m_flags)
-	, m_entries(listing.m_entries), m_searchmap_case(listing.m_searchmap_case), m_searchmap_nocase(listing.m_searchmap_nocase)
-	, m_entryCount(listing.m_entryCount)
-{
-}
-
-CDirectoryListing& CDirectoryListing::operator=(const CDirectoryListing &a)
-{
-	if (&a == this)
-		return *this;
-
-	m_entries = a.m_entries;
-
-	path = a.path;
-
-	m_flags = a.m_flags;
-
-	m_entryCount = a.m_entryCount;
-
-	m_firstListTime = a.m_firstListTime;
-
-	m_searchmap_case = a.m_searchmap_case;
-	m_searchmap_nocase = a.m_searchmap_nocase;
-
-	return *this;
-}
-
 wxString CDirentry::dump() const
 {
-	wxString str = wxString::Format(_T("name=%s\nsize=%s\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n"),
-				name, size.ToString(), *permissions, *ownerGroup, flags & flag_dir, flags & flag_link,
-				target ? *target : wxString(), flags & flag_unsure);
+	wxString str = wxString::Format(_T("name=%s\nsize=%lld\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n"),
+				name, static_cast<long long>(size), *permissions, *ownerGroup, flags & flag_dir, flags & flag_link,
+				target ? *target : std::wstring(), flags & flag_unsure);
 
 	if( has_date() ) {
-		str += _T("date=") + time.Format(_T("%Y-%m-%d"), CDateTime::local) + _T("\n");
+		str += _T("date=") + time.format(_T("%Y-%m-%d"), fz::datetime::local) + _T("\n");
 	}
 	if( has_time() ) {
-		str += _T("time=") + time.Format(_T("%H-%M-%S"), CDateTime::local) + _T("\n");
+		str += _T("time=") + time.format(_T("%H-%M-%S"), fz::datetime::local) + _T("\n");
 	}
 	return str;
 }
@@ -159,7 +123,7 @@ bool CDirectoryListing::RemoveEntry(unsigned int index)
 	return true;
 }
 
-void CDirectoryListing::GetFilenames(std::vector<wxString> &names) const
+void CDirectoryListing::GetFilenames(std::vector<std::wstring> &names) const
 {
 	names.reserve(GetCount());
 	for (unsigned int i = 0; i < GetCount(); ++i)
@@ -175,7 +139,7 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 		m_searchmap_case.Get();
 
 	// Search map
-	std::multimap<wxString, unsigned int>::const_iterator iter = m_searchmap_case->find(name);
+	auto iter = m_searchmap_case->find(to_wstring(name));
 	if (iter != m_searchmap_case->end())
 		return iter->second;
 
@@ -183,14 +147,13 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 	if (i == m_entryCount)
 		return -1;
 
-	std::multimap<wxString, unsigned int>& searchmap_case = m_searchmap_case.Get();
+	auto & searchmap_case = m_searchmap_case.Get();
 
 	// Build map if not yet complete
 	std::vector<CRefcountObject<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
-	for (; entry_iter != m_entries->end(); ++entry_iter, ++i)
-	{
-		const wxString& entry_name = (*entry_iter)->name;
-		searchmap_case.insert(std::pair<const wxString, unsigned int>(entry_name, i));
+	for (; entry_iter != m_entries->end(); ++entry_iter, ++i) {
+		std::wstring const& entry_name = (*entry_iter)->name;
+		searchmap_case.insert(std::pair<std::wstring const, unsigned int>(entry_name, i));
 
 		if (entry_name == name)
 			return i;
@@ -211,7 +174,7 @@ int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 	name.MakeLower();
 
 	// Search map
-	std::multimap<wxString, unsigned int>::const_iterator iter = m_searchmap_nocase->find(name);
+	auto iter = m_searchmap_nocase->find(to_wstring(name));
 	if (iter != m_searchmap_nocase->end())
 		return iter->second;
 
@@ -219,15 +182,14 @@ int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 	if (i == m_entryCount)
 		return -1;
 
-	std::multimap<wxString, unsigned int>& searchmap_nocase = m_searchmap_nocase.Get();
+	auto& searchmap_nocase = m_searchmap_nocase.Get();
 
 	// Build map if not yet complete
 	std::vector<CRefcountObject<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
-	for (; entry_iter != m_entries->end(); ++entry_iter, ++i)
-	{
+	for (; entry_iter != m_entries->end(); ++entry_iter, ++i) {
 		wxString entry_name = (*entry_iter)->name;
 		entry_name.MakeLower();
-		searchmap_nocase.insert(std::pair<const wxString, unsigned int>(entry_name, i));
+		searchmap_nocase.insert(std::pair<std::wstring const, unsigned int>(to_wstring(entry_name), i));
 
 		if (entry_name == name)
 			return i;
