@@ -47,20 +47,23 @@ namespace listingEncoding
 }
 
 
-class CDirectoryListingParser
+class CDirectoryListingParser final
 {
 public:
-	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding = listingEncoding::unknown);
+	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding = listingEncoding::unknown, bool sftp_mode = false);
 	~CDirectoryListingParser();
 
+	CDirectoryListingParser(CDirectoryListingParser const&) = delete;
+	CDirectoryListingParser& operator=(CDirectoryListingParser const&) = delete;
+
 	CDirectoryListing Parse(const CServerPath &path);
 
 	bool AddData(char *pData, int len);
-	bool AddLine(const wxChar* pLine);
+	bool AddLine(const wchar_t* pLine);
 
 	void Reset();
 
-	void SetTimezoneOffset(const wxTimeSpan& span) { m_timezoneOffset = span; }
+	void SetTimezoneOffset(fz::duration const& span) { m_timezoneOffset = span; }
 
 	void SetServer(const CServer& server) { m_server = server; };
 
@@ -69,47 +72,47 @@ protected:
 
 	bool ParseData(bool partial);
 
-	bool ParseLine(CLine *pLine, const enum ServerType serverType, bool concatenated);
+	bool ParseLine(CLine &line, const enum ServerType serverType, bool concatenated);
 
-	bool ParseAsUnix(CLine *pLine, CDirentry &entry, bool expect_date);
-	bool ParseAsDos(CLine *pLine, CDirentry &entry);
-	bool ParseAsEplf(CLine *pLine, CDirentry &entry);
-	bool ParseAsVms(CLine *pLine, CDirentry &entry);
-	bool ParseAsIbm(CLine *pLine, CDirentry &entry);
-	bool ParseOther(CLine *pLine, CDirentry &entry);
-	bool ParseAsWfFtp(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_Migrated(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_Tape(CLine *pLine, CDirentry &entry);
-	int ParseAsMlsd(CLine *pLine, CDirentry &entry);
-	bool ParseAsOS9(CLine *pLine, CDirentry &entry);
+	bool ParseAsUnix(CLine &line, CDirentry &entry, bool expect_date);
+	bool ParseAsDos(CLine &line, CDirentry &entry);
+	bool ParseAsEplf(CLine &line, CDirentry &entry);
+	bool ParseAsVms(CLine &line, CDirentry &entry);
+	bool ParseAsIbm(CLine &line, CDirentry &entry);
+	bool ParseOther(CLine &line, CDirentry &entry);
+	bool ParseAsWfFtp(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_PDS(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry);
+	int ParseAsMlsd(CLine &line, CDirentry &entry);
+	bool ParseAsOS9(CLine &line, CDirentry &entry);
 
 	// Only call this if servertype set to ZVM since it conflicts
 	// with other formats.
-	bool ParseAsZVM(CLine *pLine, CDirentry &entry);
+	bool ParseAsZVM(CLine &line, CDirentry &entry);
 
 	// Only call this if servertype set to HPNONSTOP since it conflicts
 	// with other formats.
-	bool ParseAsHPNonstop(CLine *pLine, CDirentry &entry);
+	bool ParseAsHPNonstop(CLine &line, CDirentry &entry);
 
 	// Date / time parsers
-	bool ParseUnixDateTime(CLine *pLine, int &index, CDirentry &entry);
+	bool ParseUnixDateTime(CLine &line, int &index, CDirentry &entry);
 	bool ParseShortDate(CToken &token, CDirentry &entry, bool saneFieldOrder = false);
 	bool ParseTime(CToken &token, CDirentry &entry);
 
 	// Parse file sizes given like this: 123.4M
-	bool ParseComplexFileSize(CToken& token, wxLongLong& size, int blocksize = -1);
+	bool ParseComplexFileSize(CToken& token, int64_t& size, int blocksize = -1);
 
-	bool GetMonthFromName(const wxString& name, int &month);
+	bool GetMonthFromName(std::wstring const& name, int &month);
 
 	void DeduceEncoding();
 	void ConvertEncoding(char *pData, int len);
 
 	CControlSocket* m_pControlSocket;
 
-	static std::map<wxString, int> m_MonthNamesMap;
+	static std::map<std::wstring, int> m_MonthNamesMap;
 
 	struct t_list
 	{
@@ -124,22 +127,24 @@ protected:
 
 	int m_currentOffset;
 
-	std::list<t_list> m_DataList;
-	std::list<CDirentry> m_entryList;
-	wxLongLong m_totalData;
+	std::deque<t_list> m_DataList;
+	std::deque<CRefcountObject<CDirentry>> m_entryList;
+	int64_t m_totalData;
 
 	CLine *m_prevLine;
 
 	CServer m_server;
 
 	bool m_fileListOnly;
-	std::list<wxString> m_fileList;
+	std::vector<wxString> m_fileList;
 
 	bool m_maybeMultilineVms;
 
-	wxTimeSpan m_timezoneOffset;
+	fz::duration m_timezoneOffset;
 
 	listingEncoding::type m_listingEncoding;
+
+	bool sftp_mode_{};
 };
 
 #endif
