@@ -1,34 +1,29 @@
 #include <filezilla.h>
 #include "externalipresolver.h"
-#include "wx/regex.h"
 #include "socket.h"
-#include <errno.h>
 #include "misc.h"
 
-const wxEventType fzEVT_EXTERNALIPRESOLVE = wxNewEventType();
+#include <libfilezilla/iputils.hpp>
 
-fzExternalIPResolveEvent::fzExternalIPResolveEvent(int id)
-	: wxEvent(id, fzEVT_EXTERNALIPRESOLVE)
-{
+#include <wx/regex.h>
+
+namespace {
+fz::mutex s_sync;
+wxString ip;
+bool checked = false;
 }
 
-wxEvent* fzExternalIPResolveEvent::Clone() const
-{
-	return new fzExternalIPResolveEvent(GetId());
-}
-
-wxString CExternalIPResolver::m_ip;
-bool CExternalIPResolver::m_checked = false;
-
-CExternalIPResolver::CExternalIPResolver(wxEvtHandler* handler, int id /*=wxID_ANY*/)
-	: m_handler(handler)
-	, m_id(id)
+CExternalIPResolver::CExternalIPResolver(event_handler & handler)
+	: event_handler(handler.event_loop_)
+	, m_handler(&handler)
 {
 	ResetHttpData(true);
 }
 
 CExternalIPResolver::~CExternalIPResolver()
 {
+	remove_handler();
+
 	delete [] m_pSendBuffer;
 	m_pSendBuffer = 0;
 	delete [] m_pRecvBuffer;
@@ -38,16 +33,17 @@ CExternalIPResolver::~CExternalIPResolver()
 	m_pSocket = 0;
 }
 
-void CExternalIPResolver::GetExternalIP(const wxString& address, enum CSocket::address_family protocol, bool force /*=false*/)
+void CExternalIPResolver::GetExternalIP(const wxString& address, CSocket::address_family protocol, bool force /*=false*/)
 {
-	if (m_checked)
 	{
-		if (force)
-			m_checked = false;
-		else
-		{
-			m_done = true;
-			return;
+		fz::scoped_lock l(s_sync);
+		if (checked) {
+			if (force)
+				checked = false;
+			else {
+				m_done = true;
+				return;
+			}
 		}
 	}
 
@@ -66,8 +62,7 @@ void CExternalIPResolver::GetExternalIP(const wxString& address, enum CSocket::a
 
 	wxString hostWithPort = host;
 
-	if ((pos = host.Find(':', true)) != -1)
-	{
+	if ((pos = host.Find(':', true)) != -1) {
 		wxString port = host.Mid(pos + 1);
 		if (!port.ToULong(&m_port) || m_port < 1 || m_port > 65535)
 			m_port = 80;
@@ -94,23 +89,28 @@ void CExternalIPResolver::GetExternalIP(const wxString& address, enum CSocket::a
 	strcpy(m_pSendBuffer, buffer.mb_str());
 }
 
-void CExternalIPResolver::OnSocketEvent(CSocketEvent& event)
+void CExternalIPResolver::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<CSocketEvent>(ev, this, &CExternalIPResolver::OnSocketEvent);
+}
+
+void CExternalIPResolver::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
 	if (!m_pSocket)
 		return;
 
-	switch (event.GetType())
+	switch (t)
 	{
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::connection:
-		OnConnect(event.GetError());
+	case SocketEventType::connection:
+		OnConnect(error);
 		break;
-	case CSocketEvent::close:
+	case SocketEventType::close:
 		OnClose();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
 	default:
@@ -147,21 +147,18 @@ void CExternalIPResolver::OnReceive()
 		unsigned int len = m_recvBufferLen - m_recvBufferPos;
 		int error;
 		int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, len, error);
-		if (read == -1)
-		{
+		if (read == -1) {
 			if (error != EAGAIN)
 				Close(false);
 			return;
 		}
 
-		if (!read)
-		{
+		if (!read) {
 			Close(false);
 			return;
 		}
 
-		if (m_finished)
-		{
+		if (m_finished) {
 			// Just ignore all further data
 			m_recvBufferPos = 0;
 			return;
@@ -171,8 +168,7 @@ void CExternalIPResolver::OnReceive()
 
 		if (!m_gotHeader)
 			OnHeader();
-		else
-		{
+		else {
 			if (m_transferEncoding == chunked)
 				OnChunkedData();
 			else
@@ -183,26 +179,22 @@ void CExternalIPResolver::OnReceive()
 
 void CExternalIPResolver::OnSend()
 {
-	while (m_pSendBuffer)
-	{
+	while (m_pSendBuffer) {
 		unsigned int len = strlen(m_pSendBuffer + m_sendBufferPos);
 		int error;
 		int written = m_pSocket->Write(m_pSendBuffer + m_sendBufferPos, len, error);
-		if (written == -1)
-		{
+		if (written == -1) {
 			if (error != EAGAIN)
 				Close(false);
 			return;
 		}
 
-		if (!written)
-		{
+		if (!written) {
 			Close(false);
 			return;
 		}
 
-		if (written == (int)len)
-		{
+		if (written == (int)len) {
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 
@@ -229,13 +221,16 @@ void CExternalIPResolver::Close(bool successful)
 
 	m_done = true;
 
-	if (!successful)
-		m_ip.clear();
-	m_checked = true;
+	{
+		fz::scoped_lock l(s_sync);
+		if (!successful) {
+			ip.clear();
+		}
+		checked = true;
+	}
 
 	if (m_handler) {
-		fzExternalIPResolveEvent event;
-		wxPostEvent(m_handler, event);
+		m_handler->send_event<CExternalIPResolveEvent>();
 		m_handler = 0;
 	}
 }
@@ -246,26 +241,20 @@ void CExternalIPResolver::OnHeader()
 	// We do just the neccessary parsing and silently ignore most header fields
 	// Redirects are supported though if the server sends the Location field.
 
-	for (;;)
-	{
+	for (;;) {
 		// Find line ending
 		unsigned int i = 0;
-		for (i = 0; (i + 1) < m_recvBufferPos; i++)
-		{
-			if (m_pRecvBuffer[i] == '\r')
-			{
-				if (m_pRecvBuffer[i + 1] != '\n')
-				{
+		for (i = 0; (i + 1) < m_recvBufferPos; ++i) {
+			if (m_pRecvBuffer[i] == '\r') {
+				if (m_pRecvBuffer[i + 1] != '\n') {
 					Close(false);
 					return;
 				}
 				break;
 			}
 		}
-		if ((i + 1) >= m_recvBufferPos)
-		{
-			if (m_recvBufferPos == m_recvBufferLen)
-			{
+		if ((i + 1) >= m_recvBufferPos) {
+			if (m_recvBufferPos == m_recvBufferLen) {
 				// We don't support header lines larger than 4096
 				Close(false);
 				return;
@@ -275,11 +264,9 @@ void CExternalIPResolver::OnHeader()
 
 		m_pRecvBuffer[i] = 0;
 
-		if (!m_responseCode)
-		{
+		if (!m_responseCode) {
 			m_responseString = wxString(m_pRecvBuffer, wxConvLocal);
-			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7))
-			{
+			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7)) {
 				// Invalid HTTP Status-Line
 				Close(false);
 				return;
@@ -296,29 +283,24 @@ void CExternalIPResolver::OnHeader()
 
 			m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
-			if (m_responseCode >= 400)
-			{
+			if (m_responseCode >= 400) {
 				// Failed request
 				Close(false);
 				return;
 			}
 
-			if (m_responseCode == 305)
-			{
+			if (m_responseCode == 305) {
 				// Unsupported redirect
 				Close(false);
 				return;
 			}
 		}
-		else
-		{
-			if (!i)
-			{
+		else {
+			if (!i) {
 				// End of header, data from now on
 
 				// Redirect if neccessary
-				if (m_responseCode >= 300)
-				{
+				if (m_responseCode >= 300) {
 					delete m_pSocket;
 					m_pSocket = 0;
 
@@ -337,8 +319,7 @@ void CExternalIPResolver::OnHeader()
 
 				memmove(m_pRecvBuffer, m_pRecvBuffer + 2, m_recvBufferPos - 2);
 				m_recvBufferPos -= 2;
-				if (m_recvBufferPos)
-				{
+				if (m_recvBufferPos) {
 					if (m_transferEncoding == chunked)
 						OnChunkedData();
 					else
@@ -346,12 +327,10 @@ void CExternalIPResolver::OnHeader()
 				}
 				return;
 			}
-			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10))
-			{
+			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10)) {
 				m_location = wxString(m_pRecvBuffer + 10, wxConvLocal);
 			}
-			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19))
-			{
+			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19)) {
 				if (!strcmp(m_pRecvBuffer + 19, "chunked"))
 					m_transferEncoding = chunked;
 				else if (!strcmp(m_pRecvBuffer + 19, "identity"))
@@ -371,15 +350,12 @@ void CExternalIPResolver::OnHeader()
 
 void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 {
-	if (buffer)
-	{
+	if (buffer) {
 		unsigned int i;
-		for (i = 0; i < len; i++)
-		{
+		for (i = 0; i < len; ++i) {
 			if (buffer[i] == '\r' || buffer[i] == '\n')
 				break;
-			if (buffer[i] & 0x80)
-			{
+			if (buffer[i] & 0x80) {
 				Close(false);
 				return;
 			}
@@ -392,12 +368,9 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 			return;
 	}
 
-	if (m_protocol == CSocket::ipv6)
-	{
-		if (!m_data.empty() && m_data[0] == '[')
-		{
-			if (m_data.Last() != ']')
-			{
+	if (m_protocol == CSocket::ipv6) {
+		if (!m_data.empty() && m_data[0] == '[') {
+			if (m_data.Last() != ']') {
 				Close(false);
 				return;
 			}
@@ -405,16 +378,15 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 			m_data = m_data.Mid(1);
 		}
 
-		if (GetIPV6LongForm(m_data).empty())
-		{
+		if (fz::get_ipv6_long_form(m_data.ToStdWstring()).empty()) {
 			Close(false);
 			return;
 		}
 
-		m_ip = m_data;
+		fz::scoped_lock l(s_sync);
+		ip = m_data;
 	}
-	else
-	{
+	else {
 
 		// Validate ip address
 		wxString digit = _T("0*[0-9]{1,3}");
@@ -423,13 +395,13 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 		wxRegEx regex;
 		regex.Compile(exp);
 
-		if (!regex.Matches(m_data))
-		{
+		if (!regex.Matches(m_data)) {
 			Close(false);
 			return;
 		}
 
-		m_ip = regex.GetMatch(m_data, 2);
+		fz::scoped_lock l(s_sync);
+		ip = regex.GetMatch(m_data, 2);
 	}
 
 	Close(true);
@@ -446,9 +418,7 @@ void CExternalIPResolver::ResetHttpData(bool resetRedirectCount)
 
 	m_transferEncoding = unknown;
 
-	m_chunkData.getTrailer = false;
-	m_chunkData.size = 0;
-	m_chunkData.terminateChunk = false;
+	m_chunkData = t_chunkData();
 
 	m_finished = false;
 }
@@ -458,13 +428,11 @@ void CExternalIPResolver::OnChunkedData()
 	char* p = m_pRecvBuffer;
 	unsigned int len = m_recvBufferPos;
 
-	for (;;)
-	{
-		if (m_chunkData.size != 0)
-		{
+	for (;;) {
+		if (m_chunkData.size != 0) {
 			unsigned int dataLen = len;
 			if (m_chunkData.size < len)
-				dataLen = m_chunkData.size.GetLo();
+				dataLen = static_cast<unsigned int>(m_chunkData.size);
 			OnData(p, dataLen);
 			if (!m_pRecvBuffer)
 				return;
@@ -482,22 +450,17 @@ void CExternalIPResolver::OnChunkedData()
 
 		// Find line ending
 		unsigned int i = 0;
-		for (i = 0; (i + 1) < len; i++)
-		{
-			if (p[i] == '\r')
-			{
-				if (p[i + 1] != '\n')
-				{
+		for (i = 0; (i + 1) < len; ++i) {
+			if (p[i] == '\r') {
+				if (p[i + 1] != '\n') {
 					Close(false);
 					return;
 				}
 				break;
 			}
 		}
-		if ((i + 1) >= len)
-		{
-			if (len == m_recvBufferLen)
-			{
+		if ((i + 1) >= len) {
+			if (len == m_recvBufferLen) {
 				// We don't support lines larger than 4096
 				Close(false);
 				return;
@@ -507,20 +470,16 @@ void CExternalIPResolver::OnChunkedData()
 
 		p[i] = 0;
 
-		if (m_chunkData.terminateChunk)
-		{
-			if (i)
-			{
+		if (m_chunkData.terminateChunk) {
+			if (i) {
 				// Chunk has to end with CRLF
 				Close(false);
 				return;
 			}
 			m_chunkData.terminateChunk = false;
 		}
-		else if (m_chunkData.getTrailer)
-		{
-			if (!i)
-			{
+		else if (m_chunkData.getTrailer) {
+			if (!i) {
 				m_finished = true;
 				m_recvBufferPos = 0;
 				return;
@@ -528,31 +487,25 @@ void CExternalIPResolver::OnChunkedData()
 
 			// Ignore the trailer
 		}
-		else
-		{
+		else {
 			// Read chunk size
 			char* q = p;
-			while (*q)
-			{
-				if (*q >= '0' && *q <= '9')
-				{
+			while (*q) {
+				if (*q >= '0' && *q <= '9') {
 					m_chunkData.size *= 16;
 					m_chunkData.size += *q - '0';
 				}
-				else if (*q >= 'A' && *q <= 'F')
-				{
+				else if (*q >= 'A' && *q <= 'F') {
 					m_chunkData.size *= 10;
 					m_chunkData.size += *q - 'A' + 10;
 				}
-				else if (*q >= 'a' && *q <= 'f')
-				{
+				else if (*q >= 'a' && *q <= 'f') {
 					m_chunkData.size *= 10;
 					m_chunkData.size += *q - 'a' + 10;
 				}
 				else if (*q == ';' || *q == ' ')
 					break;
-				else
-				{
+				else {
 					// Invalid size
 					Close(false);
 					return;
@@ -570,9 +523,20 @@ void CExternalIPResolver::OnChunkedData()
 			break;
 	}
 
-	if (p != m_pRecvBuffer)
-	{
+	if (p != m_pRecvBuffer) {
 		memmove(m_pRecvBuffer, p, len);
 		m_recvBufferPos = len;
 	}
 }
+
+bool CExternalIPResolver::Successful() const
+{
+	fz::scoped_lock l(s_sync);
+	return !ip.empty();
+}
+
+wxString CExternalIPResolver::GetIP() const
+{
+	fz::scoped_lock l(s_sync);
+	return ip;
+}
