@@ -1,13 +1,14 @@
 #include <filezilla.h>
 
 #include "ControlSocket.h"
+#include "engineprivate.h"
 #include "httpcontrolsocket.h"
-#include "local_filesys.h"
 #include "tlssocket.h"
 
-#include <wx/file.h>
+#include <libfilezilla/iputils.hpp>
+#include <libfilezilla/local_filesys.hpp>
 
-#include <errno.h>
+#include <wx/file.h>
 
 #define FZ_REPLY_REDIRECTED FZ_REPLY_ALREADYCONNECTED
 
@@ -34,29 +35,18 @@ public:
 	CHttpOpData(COpData* pOpData)
 		: m_pOpData(pOpData)
 	{
-		m_gotHeader = false;
-		m_responseCode = -1;
-		m_redirectionCount = 0;
-		m_transferEncoding = unknown;
-
-		m_chunkData.getTrailer = false;
-		m_chunkData.size = 0;
-		m_chunkData.terminateChunk = false;
-
-		m_totalSize = -1;
-		m_receivedData = 0;
 	}
 
-	virtual ~CHttpOpData() {}
+	virtual ~CHttpOpData() = default;
 
-	bool m_gotHeader;
-	int m_responseCode;
+	bool m_gotHeader{};
+	int m_responseCode{-1};
 	wxString m_responseString;
 	wxURI m_newLocation;
-	int m_redirectionCount;
+	int m_redirectionCount{};
 
-	wxLongLong m_totalSize;
-	wxLongLong m_receivedData;
+	int64_t m_totalSize{-1};
+	int64_t m_receivedData{};
 
 	COpData* m_pOpData;
 
@@ -66,13 +56,13 @@ public:
 		chunked,
 		unknown
 	};
-	enum transferEncodings m_transferEncoding;
+	transferEncodings m_transferEncoding{unknown};
 
 	struct t_chunkData
 	{
-		bool getTrailer;
-		bool terminateChunk;
-		wxLongLong size;
+		bool getTrailer{};
+		bool terminateChunk{};
+		int64_t size{};
 	} m_chunkData;
 };
 
@@ -93,8 +83,8 @@ public:
 	wxFile* pFile;
 };
 
-CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CRealControlSocket(pEngine)
+CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
+	: CRealControlSocket(engine)
 {
 	m_pRecvBuffer = 0;
 	m_recvBufferPos = 0;
@@ -104,6 +94,7 @@ CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate *pEngine)
 
 CHttpControlSocket::~CHttpControlSocket()
 {
+	remove_handler();
 	DoClose();
 	delete [] m_pRecvBuffer;
 }
@@ -140,7 +131,7 @@ int CHttpControlSocket::SendNextCommand()
 
 int CHttpControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() m_pEngine=%p"), m_pEngine);
+	LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() &engine_=%p"), &engine_);
 	if (GetCurrentCommandId() != Command::connect ||
 		!m_pCurrentServer)
 	{
@@ -174,19 +165,19 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				return false;
 			}
 
-			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
+			CFileExistsNotification *pFileExistsNotification = static_cast<CFileExistsNotification *>(pNotification);
 			return SetFileExistsAction(pFileExistsNotification);
 		}
 		break;
 	case reqId_certificate:
 		{
-			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::verifycert)
+			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::TlsState::verifycert)
 			{
 				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %d"), pNotification->GetRequestID());
 				return false;
 			}
 
-			CCertificateNotification* pCertificateNotification = reinterpret_cast<CCertificateNotification *>(pNotification);
+			CCertificateNotification* pCertificateNotification = static_cast<CCertificateNotification *>(pNotification);
 			m_pTlsSocket->TrustCurrentCert(pCertificateNotification->m_trusted);
 		}
 		break;
@@ -230,10 +221,9 @@ int CHttpControlSocket::DoReceive()
 			return 0;
 		}
 
-		m_pEngine->SetActive(CFileZillaEngine::recv);
+		SetActive(CFileZillaEngine::recv);
 
-		if (!m_pCurOpData || m_pCurOpData->opId == Command::connect)
-		{
+		if (!m_pCurOpData || m_pCurOpData->opId == Command::connect) {
 			// Just ignore all further data
 			m_recvBufferPos = 0;
 			return 0;
@@ -241,8 +231,7 @@ int CHttpControlSocket::DoReceive()
 
 		m_recvBufferPos += read;
 
-		if (!m_pHttpOpData->m_gotHeader)
-		{
+		if (!m_pHttpOpData->m_gotHeader) {
 			if (!read)
 			{
 				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
@@ -296,7 +285,7 @@ void CHttpControlSocket::OnConnect()
 			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
 
 			delete m_pBackend;
-			m_pTlsSocket = new CTlsSocket(this, m_pSocket, this);
+			m_pTlsSocket = new CTlsSocket(this, *m_pSocket, this);
 			m_pBackend = m_pTlsSocket;
 
 			if (!m_pTlsSocket->Init()) {
@@ -305,19 +294,12 @@ void CHttpControlSocket::OnConnect()
 				return;
 			}
 
-			const wxString trusted_rootcert = m_pEngine->GetOptions()->GetOption(OPTION_INTERNAL_ROOTCERT);
-			if (!trusted_rootcert.empty() && !m_pTlsSocket->AddTrustedRootCertificate(trusted_rootcert)) {
-				LogMessage(MessageType::Error, _("Failed to parse trusted root cert."));
-				DoClose();
-				return;
-			}
-
 			int res = m_pTlsSocket->Handshake();
 			if (res == FZ_REPLY_ERROR)
 				DoClose();
 		}
 		else {
-			LogMessage(MessageType::Status, _("TLS/SSL connection established, sending HTTP request"));
+			LogMessage(MessageType::Status, _("TLS connection established, sending HTTP request"));
 			ResetOperation(FZ_REPLY_OK);
 		}
 
@@ -364,7 +346,7 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	m_current_uri = wxURI(m_pCurrentServer->FormatServer() + pData->remotePath.FormatFilename(pData->remoteFile));
 
 	if (!localFile.empty()) {
-		pData->localFileSize = CLocalFileSystem::GetSize(pData->localFile).GetValue();
+		pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
 
 		pData->opState = filetransfer_waitfileexists;
 		int res = CheckOverwriteFile();
@@ -473,8 +455,9 @@ int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool
 	pData->port = port;
 	pData->tls = tls;
 
-	if (!IsIpAddress(host))
+	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
+	}
 
 	pData->host = host;
 	return DoInternalConnect();
@@ -494,7 +477,7 @@ int CHttpControlSocket::DoInternalConnect()
 	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
 
 	delete m_pBackend;
-	m_pBackend = new CSocketBackend(this, m_pSocket);
+	m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 
 	int res = m_pSocket->Connect(pData->host, pData->port);
 	if (!res)
@@ -510,8 +493,7 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferParseResponse(%p, %d)"), p, len);
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -519,37 +501,32 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 
 	CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
 
-	if (!p)
-	{
+	if (!p) {
 		ResetOperation(FZ_REPLY_OK);
 		return FZ_REPLY_OK;
 	}
 
-	if (!m_pTransferStatus)
-	{
-		InitTransferStatus(pData->m_totalSize.GetValue(), 0, false);
-		SetTransferStatusStartTime();
+	if (engine_.transfer_status_.empty()) {
+		engine_.transfer_status_.Init(pData->m_totalSize, 0, false);
+		engine_.transfer_status_.SetStartTime();
 	}
 
-	if (pData->localFile.empty())
-	{
+	if (pData->localFile.empty()) {
 		char* q = new char[len];
 		memcpy(q, p, len);
-		m_pEngine->AddNotification(new CDataNotification(q, len));
+		engine_.AddNotification(new CDataNotification(q, len));
 	}
-	else
-	{
+	else {
 		wxASSERT(pData->pFile);
 
-		if (pData->pFile->Write(p, len) != len)
-		{
+		if (pData->pFile->Write(p, len) != len) {
 			LogMessage(MessageType::Error, _("Failed to write to file %s"), pData->localFile);
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
 		}
 	}
 
-	UpdateTransferStatus(len);
+	engine_.transfer_status_.Update(len);
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -560,8 +537,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 	// We do just the neccessary parsing and silently ignore most header fields
 	// Redirects are supported though if the server sends the Location field.
 
-	for (;;)
-	{
+	for (;;) {
 		// Find line ending
 		unsigned int i = 0;
 		for (i = 0; (i + 1) < m_recvBufferPos; i++)
@@ -590,7 +566,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 		}
 
 		m_pRecvBuffer[i] = 0;
-		const wxString& line = wxString(m_pRecvBuffer, wxConvLocal);
+		wxString const line = wxString(m_pRecvBuffer, wxConvLocal);
 		if (!line.empty())
 			LogMessageRaw(MessageType::Response, line);
 
@@ -618,7 +594,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			pData->m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
 			if( pData->m_responseCode == 416 ) {
-				CHttpFileTransferOpData* pTransfer = reinterpret_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+				CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
 				if( pTransfer->resume ) {
 					// Sad, the server does not like our attempt to resume.
 					// Get full file instead.
@@ -697,7 +673,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				}
 
 				if( pData->m_pOpData && pData->m_pOpData->opId == Command::transfer) {
-					CHttpFileTransferOpData* pTransfer = reinterpret_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+					CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
 					if( pTransfer->resume && pData->m_responseCode != 206 ) {
 						pTransfer->resume = false;
 						int res = OpenFile(pTransfer);
@@ -780,7 +756,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 		{
 			unsigned int dataLen = len;
 			if (pData->m_chunkData.size < len)
-				dataLen = pData->m_chunkData.size.GetLo();
+				dataLen = static_cast<unsigned int>(pData->m_chunkData.size);
 			pData->m_receivedData += dataLen;
 			int res = ProcessData(p, dataLen);
 			if (res != FZ_REPLY_WOULDBLOCK)
@@ -918,8 +894,7 @@ void CHttpControlSocket::OnClose(int error)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::OnClose(%d)"), error);
 
-	if (error)
-	{
+	if (error) {
 		LogMessage(MessageType::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
@@ -929,30 +904,24 @@ void CHttpControlSocket::OnClose(int error)
 	if (!m_pCurOpData)
 		return;
 
-	if (m_pCurOpData->pNextOpData)
-	{
+	if (m_pCurOpData->pNextOpData) {
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
 
-	if (!m_pHttpOpData->m_gotHeader)
-	{
+	if (!m_pHttpOpData->m_gotHeader) {
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
 
-	if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked)
-	{
-		if (!m_pHttpOpData->m_chunkData.getTrailer)
-		{
+	if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked) {
+		if (!m_pHttpOpData->m_chunkData.getTrailer) {
 			ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 			return;
 		}
 	}
-	else
-	{
-		if (m_pHttpOpData->m_totalSize != -1 && m_pHttpOpData->m_receivedData != m_pHttpOpData->m_totalSize)
-		{
+	else {
+		if (m_pHttpOpData->m_totalSize != -1 && m_pHttpOpData->m_receivedData != m_pHttpOpData->m_totalSize) {
 			ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 			return;
 		}
@@ -1004,8 +973,7 @@ int CHttpControlSocket::ProcessData(char* p, int len)
 int CHttpControlSocket::ParseSubcommandResult(int prevResult)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand(%d)"), prevResult);
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
@@ -1030,22 +998,21 @@ int CHttpControlSocket::Disconnect()
 	return FZ_REPLY_OK;
 }
 
-int CHttpControlSocket::OpenFile( CHttpFileTransferOpData* pData)
+int CHttpControlSocket::OpenFile(CHttpFileTransferOpData* pData)
 {
 	delete pData->pFile;
 	pData->pFile = new wxFile();
 	CreateLocalDir(pData->localFile);
 
-	if (!pData->pFile->Open(pData->localFile, pData->resume ? wxFile::write_append : wxFile::write))
-	{
+	if (!pData->pFile->Open(pData->localFile, pData->resume ? wxFile::write_append : wxFile::write)) {
 		LogMessage(MessageType::Error, _("Failed to open \"%s\" for writing"), pData->localFile);
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 	wxFileOffset end = pData->pFile->SeekEnd();
-	if( !end ) {
+	if (!end) {
 		pData->resume = false;
 	}
-	pData->localFileSize = CLocalFileSystem::GetSize(pData->localFile).GetValue();
+	pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
 	return FZ_REPLY_OK;
 }
