@@ -1,14 +1,19 @@
 #ifndef __LOGGING_PRIVATE_H__
 #define __LOGGING_PRIVATE_H__
 
+#include "engineprivate.h"
+#include <libfilezilla/mutex.hpp>
 #include <utility>
 
 class CLogging
 {
 public:
-	CLogging(CFileZillaEnginePrivate *pEngine);
+	explicit CLogging(CFileZillaEnginePrivate & engine);
 	virtual ~CLogging();
 
+	CLogging(CLogging const&) = delete;
+	CLogging& operator=(CLogging const&) = delete;
+
 	template<typename String, typename...Args>
 	void LogMessage(MessageType nMessageType, String&& msgFormat, Args&& ...args) const
 	{
@@ -20,7 +25,7 @@ public:
 		notification->msg.Printf(std::forward<String>(msgFormat), std::forward<Args>(args)...);
 
 		LogToFile(nMessageType, notification->msg);
-		m_pEngine->AddNotification(notification);
+		engine_.AddLogNotification(notification);
 	}
 
 	template<typename String>
@@ -30,11 +35,10 @@ public:
 			return;
 		}
 
-
 		CLogmsgNotification *notification = new CLogmsgNotification(nMessageType, std::forward<String>(msg));
 
 		LogToFile(nMessageType, notification->msg);
-		m_pEngine->AddNotification(notification);
+		engine_.AddLogNotification(notification);
 	}
 
 	template<typename String, typename String2, typename...Args>
@@ -60,15 +64,18 @@ public:
 		notification->msg.Printf(_T("%s(%d): %s   caller=%p"), source, nSourceLine, text, pInstance);
 
 		LogToFile(nMessageType, notification->msg);
-		m_pEngine->AddNotification(notification);
+		engine_.AddLogNotification(notification);
 	}
 
 	bool ShouldLog(MessageType nMessageType) const;
 
-private:
-	CFileZillaEnginePrivate *m_pEngine;
+	// Only affects calling thread
+	static void UpdateLogLevel(COptionsBase & options);
 
-	void InitLogFile() const;
+private:
+	CFileZillaEnginePrivate & engine_;
+
+	bool InitLogFile(fz::scoped_lock& l) const;
 	void LogToFile(MessageType nMessageType, const wxString& msg) const;
 
 	static bool m_logfile_initialized;
@@ -83,6 +90,15 @@ private:
 	static wxString m_file;
 
 	static int m_refcount;
+
+	static fz::mutex mutex_;
+
+#if HAVE_NO_THREAD_LOCAL
+	// Fixme: Get rid of this once OS X's clang supports it.
+	#define thread_local __thread
+#endif
+	static thread_local int debug_level_;
+	static thread_local int raw_listing_;
 };
 
 #endif
