@@ -1,8 +1,22 @@
 #include <filezilla.h>
 
-const wxEventType fzEVT_NOTIFICATION = wxNewEventType();
+wxDEFINE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
 
-wxFzEvent::wxFzEvent(int id /*=wxID_ANY*/) : wxEvent(id, fzEVT_NOTIFICATION)
+wxFzEvent::wxFzEvent()
+	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
+	, engine_()
+{
+}
+
+wxFzEvent::wxFzEvent(CFileZillaEngine const* engine)
+	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
+	, engine_(engine)
+{
+}
+
+wxFzEvent::wxFzEvent(int id)
+	: wxEvent(id, fzEVT_NOTIFICATION)
+	, engine_()
 {
 }
 
@@ -21,8 +35,10 @@ enum RequestId CFileExistsNotification::GetRequestID() const
 	return reqId_fileexists;
 }
 
-CInteractiveLoginNotification::CInteractiveLoginNotification(const wxString& challenge)
+CInteractiveLoginNotification::CInteractiveLoginNotification(type t, const wxString& challenge, bool repeated)
 	: m_challenge(challenge)
+	, m_type(t)
+	, m_repeated(repeated)
 {
 }
 
@@ -36,19 +52,14 @@ CActiveNotification::CActiveNotification(int direction)
 {
 }
 
-CTransferStatusNotification::CTransferStatusNotification(CTransferStatus *pStatus)
-	: m_pStatus(pStatus)
+CTransferStatusNotification::CTransferStatusNotification(CTransferStatus const& status)
+	: status_(status)
 {
 }
 
-CTransferStatusNotification::~CTransferStatusNotification()
+CTransferStatus const& CTransferStatusNotification::GetStatus() const
 {
-	delete m_pStatus;
-}
-
-const CTransferStatus* CTransferStatusNotification::GetStatus() const
-{
-	return m_pStatus;
+	return status_;
 }
 
 CHostKeyNotification::CHostKeyNotification(wxString host, int port, wxString fingerprint, bool changed /*=false*/)
@@ -95,49 +106,41 @@ char* CDataNotification::Detach(int& len)
 }
 
 CCertificate::CCertificate(
-		const unsigned char* rawData, unsigned int len,
-		wxDateTime activationTime, wxDateTime expirationTime,
-		const wxString& serial,
-		const wxString& pkalgoname, unsigned int bits,
-		const wxString& signalgoname,
-		const wxString& fingerprint_md5,
-		const wxString& fingerprint_sha1,
-		const wxString& subject,
-		const wxString& issuer)
+		unsigned char const* rawData, unsigned int len,
+		fz::datetime const& activationTime, fz::datetime const& expirationTime,
+		wxString const& serial,
+		wxString const& pkalgoname, unsigned int bits,
+		wxString const& signalgoname,
+		wxString const& fingerprint_sha256,
+		wxString const& fingerprint_sha1,
+		wxString const& issuer,
+		wxString const& subject,
+		std::vector<wxString> const& altSubjectNames)
+	: m_activationTime(activationTime)
+	, m_expirationTime(expirationTime)
+	, m_len(len)
+	, m_serial(serial)
+	, m_pkalgoname(pkalgoname)
+	, m_pkalgobits(bits)
+	, m_signalgoname(signalgoname)
+	, m_fingerprint_sha256(fingerprint_sha256)
+	, m_fingerprint_sha1(fingerprint_sha1)
+	, m_issuer(issuer)
+	, m_subject(subject)
+	, m_altSubjectNames(altSubjectNames)
 {
 	wxASSERT(len);
-	if (len)
-	{
+	if (len) {
 		m_rawData = new unsigned char[len];
 		memcpy(m_rawData, rawData, len);
 	}
-	else
-		m_rawData = 0;
-	m_len = len;
-
-	m_activationTime = activationTime;
-	m_expirationTime = expirationTime;
-
-	m_serial = serial;
-	m_pkalgoname = pkalgoname;
-	m_pkalgobits = bits;
-
-	m_signalgoname = signalgoname;
-
-	m_fingerprint_md5 = fingerprint_md5;
-	m_fingerprint_sha1 = fingerprint_sha1;
-
-	m_subject = subject;
-	m_issuer = issuer;
 }
 
 CCertificate::CCertificate(const CCertificate &op)
 {
-	if (op.m_rawData)
-	{
+	if (op.m_rawData) {
 		wxASSERT(op.m_len);
-		if (op.m_len)
-		{
+		if (op.m_len) {
 			m_rawData = new unsigned char[op.m_len];
 			memcpy(m_rawData, op.m_rawData, op.m_len);
 		}
@@ -157,11 +160,12 @@ CCertificate::CCertificate(const CCertificate &op)
 
 	m_signalgoname = op.m_signalgoname;
 
-	m_fingerprint_md5 = op.m_fingerprint_md5;
+	m_fingerprint_sha256 = op.m_fingerprint_sha256;
 	m_fingerprint_sha1 = op.m_fingerprint_sha1;
 
-	m_subject = op.m_subject;
 	m_issuer = op.m_issuer;
+	m_subject = op.m_subject;
+	m_altSubjectNames = op.m_altSubjectNames;
 }
 
 CCertificate::~CCertificate()
@@ -175,11 +179,9 @@ CCertificate& CCertificate::operator=(const CCertificate &op)
 		return *this;
 
 	delete [] m_rawData;
-	if (op.m_rawData)
-	{
+	if (op.m_rawData) {
 		wxASSERT(op.m_len);
-		if (op.m_len)
-		{
+		if (op.m_len) {
 			m_rawData = new unsigned char[op.m_len];
 			memcpy(m_rawData, op.m_rawData, op.m_len);
 		}
@@ -199,11 +201,12 @@ CCertificate& CCertificate::operator=(const CCertificate &op)
 
 	m_signalgoname = op.m_signalgoname;
 
-	m_fingerprint_md5 = op.m_fingerprint_md5;
+	m_fingerprint_sha256 = op.m_fingerprint_sha256;
 	m_fingerprint_sha1 = op.m_fingerprint_sha1;
 
-	m_subject = op.m_subject;
 	m_issuer = op.m_issuer;
+	m_subject = op.m_subject;
+	m_altSubjectNames = op.m_altSubjectNames;
 
 	return *this;
 }
@@ -213,9 +216,11 @@ CCertificateNotification::CCertificateNotification(const wxString& host, unsigne
 		const wxString& keyExchange,
 		const wxString& sessionCipher,
 		const wxString& sessionMac,
+		int algorithmWarnings,
 		const std::vector<CCertificate> &certificates)
-	: m_protocol( protocol )
-	, m_keyExchange( keyExchange )
+	: m_protocol(protocol)
+	, m_keyExchange(keyExchange)
+	, m_algorithmWarnings(algorithmWarnings)
 {
 	m_host = host;
 	m_port = port;
