@@ -1,8 +1,22 @@
+#include <wx/defs.h>
+#include <libfilezilla/libfilezilla.hpp>
+#ifdef FZ_WINDOWS
+  #include <libfilezilla/private/windows.hpp>
+  #include <winsock2.h>
+  #include <ws2tcpip.h>
+#else
+  #include <sys/socket.h>
+#endif
 #include <filezilla.h>
+#include "engineprivate.h"
 #include "proxy.h"
-#include <errno.h>
 #include "ControlSocket.h"
-#include <wx/sckaddr.h>
+
+#include <libfilezilla/iputils.hpp>
+
+#ifndef FZ_WINDOWS
+#include <netdb.h>
+#endif
 
 enum handshake_state
 {
@@ -17,8 +31,9 @@ enum handshake_state
 	socks4_handshake
 };
 
-CProxySocket::CProxySocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
-	: CBackend(pEvtHandler)
+CProxySocket::CProxySocket(fz::event_handler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+	: fz::event_handler(pOwner->event_loop_)
+	, CBackend(pEvtHandler)
 	, m_pSocket(pSocket)
 	, m_pOwner(pOwner)
 {
@@ -27,18 +42,21 @@ CProxySocket::CProxySocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, C
 
 CProxySocket::~CProxySocket()
 {
+	remove_handler();
+
 	if (m_pSocket)
 		m_pSocket->SetEventHandler(0);
 	delete [] m_pSendBuffer;
 	delete [] m_pRecvBuffer;
 }
 
-static wxString base64encode(const wxString& str)
+namespace {
+wxString base64encode(const wxString& str)
 {
 	// Code shamelessly taken from wxWidgets and adopted to encode UTF-8 strings.
 	// wxWidget's http class encodes string from arbitrary encoding into base64,
 	// could as well encode /dev/random
-	static const char *base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	static char const*const base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 	wxString buf;
 
@@ -56,7 +74,8 @@ static wxString base64encode(const wxString& str)
 		buf << wxString::Format(wxT("%c"), base64[(from[0] >> 2) & 0x3f]);
 		if (len == 1) {
 			buf << wxString::Format(wxT("%c="), base64[(from[0] << 4) & 0x30]);
-		} else {
+		}
+		else {
 			buf << wxString::Format(wxT("%c%c"), base64[((from[0] << 4) & 0x30) | ((from[1] >> 4) & 0xf)], base64[(from[1] << 2) & 0x3c]);
 		}
 		buf << wxString::Format(wxT("="));
@@ -64,8 +83,23 @@ static wxString base64encode(const wxString& str)
 
 	return buf;
 }
+}
 
-int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
+wxString CProxySocket::Name(ProxyType t)
+{
+	switch (t) {
+	case HTTP:
+		return _T("HTTP");
+	case SOCKS4:
+		return _T("SOCKS4");
+	case SOCKS5:
+		return _T("SOCKS5");
+	default:
+		return _("unknown");
+	}
+}
+
+int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
 {
 	if (type == CProxySocket::unknown || host.empty() || port < 1 || port > 65535)
 		return EINVAL;
@@ -89,21 +123,17 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 	if (type == HTTP) {
 		m_handshakeState = http_wait;
 
-		wxWX2MBbuf challenge;
-		int challenge_len;
+		wxWX2MBbuf challenge{};
+		int challenge_len{};
 		if (!user.empty()) {
 			challenge = base64encode(user + _T(":") + pass).mb_str(wxConvUTF8);
 			challenge_len = strlen(challenge);
 		}
-		else {
-			challenge = (size_t)0;
-			challenge_len = 0;
-		}
 
 		// Bit oversized, but be on the safe side
 		m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + challenge_len + 23];
 
-		if (!challenge) {
+		if (!challenge || !challenge_len) {
 			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nUser-Agent: FileZilla\r\n\r\n",
 				(const char*)host_raw, port,
 				(const char*)host_raw, port);
@@ -121,18 +151,29 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 	}
 	else if (type == SOCKS4) {
 		wxString ip = m_host;
-		if (!GetIPV6LongForm(m_host).empty()) {
+		if (!fz::get_ipv6_long_form(m_host.ToStdWstring()).empty()) {
 			m_pOwner->LogMessage(MessageType::Error, _("IPv6 addresses are not supported with SOCKS4 proxy"));
 			return EINVAL;
 		}
 
-		if (!IsIpAddress(m_host)) {
-			wxIPV4address address;
-			if (!address.Hostname(host)) {
+		if (fz::get_address_type(m_host.ToStdWstring()) == fz::address_type::unknown) {
+			addrinfo hints{};
+			hints.ai_family = AF_INET;
+			hints.ai_socktype = SOCK_STREAM;
+
+			addrinfo * result{};
+			int res = getaddrinfo(host.ToStdString().c_str(), 0, &hints, &result);
+			if (!res && result) {
+				if (result->ai_family == AF_INET) {
+					ip = CSocket::AddressToString(result->ai_addr, result->ai_addrlen, false);
+				}
+				freeaddrinfo(result);
+			}
+
+			if (ip.empty()) {
 				m_pOwner->LogMessage(MessageType::Error, _("Cannot resolve hostname to IPv4 address for use with SOCKS4 proxy."));
 				return EINVAL;
 			}
-			ip = address.IPAddress();
 		}
 
 		m_pOwner->LogMessage(MessageType::Status, _("SOCKS4 proxy will connect to: %s"), ip);
@@ -186,51 +227,47 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 	return EINPROGRESS;
 }
 
-void CProxySocket::OnSocketEvent(CSocketEvent& event)
+void CProxySocket::operator()(fz::event_base const& ev)
 {
-	switch (event.GetType())
-	{
-	case CSocketEvent::hostaddress:
-		{
-			const wxString& address = event.GetData();
-			m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
-		}
+	fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+		&CProxySocket::OnSocketEvent,
+		&CProxySocket::OnHostAddress);
+}
+
+void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+{
+	switch (t) {
+	case SocketEventType::connection_next:
+		if (error)
+			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
 		break;
-	case CSocketEvent::connection_next:
-		if (event.GetError())
-			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()));
-		break;
-	case CSocketEvent::connection:
-		if (event.GetError())
-		{
+	case SocketEventType::connection:
+		if (error) {
 			if (m_proxyState == handshake)
 				m_proxyState = noconn;
-			CSocketEvent *forwarded_event = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, event.GetError());
-			CSocketEventDispatcher::Get().SendEvent(forwarded_event);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, error);
 		}
-		else
-		{
+		else {
 			m_pOwner->LogMessage(MessageType::Status, _("Connection with proxy established, performing handshake..."));
 		}
 		break;
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
-	case CSocketEvent::close:
+	case SocketEventType::close:
 		OnReceive();
-		/*if (m_proxyState == handshake)
-			m_proxyState = noconn;
-		m_pEvtHandler->AddPendingEvent(event);*/
-		break;
-	default:
-		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
 
+void CProxySocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+{
+	m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
+}
+
 void CProxySocket::Detach()
 {
 	if (!m_pSocket)
@@ -250,59 +287,46 @@ void CProxySocket::OnReceive()
 	switch (m_handshakeState)
 	{
 	case http_wait:
-		for (;;)
-		{
+		for (;;) {
 			int error;
 			int do_read = m_recvBufferLen - m_recvBufferPos - 1;
 			char* end = 0;
-			for (int i = 0; i < 2; i++)
-			{
+			for (int i = 0; i < 2; ++i) {
 				int read;
 				if (!i)
 					read = m_pSocket->Peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
 				else
 					read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
-				if (read == -1)
-				{
-					if (error != EAGAIN)
-					{
+				if (read == -1) {
+					if (error != EAGAIN) {
 						m_proxyState = noconn;
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 					}
 					else
 						m_can_read = false;
 					return;
 				}
-				if (!read)
-				{
+				if (!read) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
-				if (m_pSendBuffer)
-				{
+				if (m_pSendBuffer) {
 					m_proxyState = noconn;
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming data before requst fully sent"));
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming data before request fully sent"));
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 
-				if (!i)
-				{
+				if (!i) {
 					// Response ends with strstr
 					m_pRecvBuffer[m_recvBufferPos + read] = 0;
 					end = strstr(m_pRecvBuffer, "\r\n\r\n");
-					if (!end)
-					{
-						if (m_recvBufferPos + read + 1 == m_recvBufferLen)
-						{
+					if (!end) {
+						if (m_recvBufferPos + read + 1 == m_recvBufferLen) {
 							m_proxyState = noconn;
 							m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming header too large"));
-							CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ENOMEM);
-							CSocketEventDispatcher::Get().SendEvent(evt);
+							m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ENOMEM);
 							return;
 						}
 						do_read = read;
@@ -310,14 +334,11 @@ void CProxySocket::OnReceive()
 					else
 						do_read = end - m_pRecvBuffer + 4 - m_recvBufferPos;
 				}
-				else
-				{
-					if (read != do_read)
-					{
+				else {
+					if (read != do_read) {
 						m_proxyState = noconn;
 						m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Could not read what got peeked"));
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 					m_recvBufferPos += read;
@@ -327,48 +348,38 @@ void CProxySocket::OnReceive()
 			if (!end)
 				continue;
 
-			end = strchr(m_pRecvBuffer, '\r');
-			wxASSERT(end);
+			end = strchr(m_pRecvBuffer, '\r'); // Never fails as old value of end exists and starts with CR, we just look for an earlier case.
 			*end = 0;
 			wxString reply(m_pRecvBuffer, wxConvUTF8);
 			m_pOwner->LogMessage(MessageType::Response, _("Proxy reply: %s"), reply);
 
-			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2"))
-			{
+			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2")) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNRESET);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
 				return;
 			}
 
 			m_proxyState = conn;
-			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
-			CSocketEventDispatcher::Get().SendEvent(evt);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 			return;
 		}
 	case socks4_handshake:
-		while (m_recvBufferLen && m_can_read && m_proxyState == handshake)
-		{
+		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int read_error;
 			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
-			if (read == -1)
-			{
-				if (read_error != EAGAIN)
-				{
+			if (read == -1) {
+				if (read_error != EAGAIN) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, read_error);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, read_error);
 				}
 				else
 					m_can_read = false;
 				return;
 			}
 
-			if (!read)
-			{
+			if (!read) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -397,13 +408,11 @@ void CProxySocket::OnReceive()
 				}
 				m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: %s"), error);
 				m_proxyState = noconn;
-				CSocketEventDispatcher::Get()
-					.SendEvent(new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED));
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_proxyState = conn;
-			CSocketEventDispatcher::Get()
-				.SendEvent(new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0));
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 		}
 		return;
 	case socks5_method:
@@ -413,27 +422,21 @@ void CProxySocket::OnReceive()
 	case socks5_request_address:
 		if (m_pSendBuffer)
 			return;
-		while (m_recvBufferLen && m_can_read && m_proxyState == handshake)
-		{
+		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int error;
 			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
-			if (read == -1)
-			{
-				if (error != EAGAIN)
-				{
+			if (read == -1) {
+				if (error != EAGAIN) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 				}
 				else
 					m_can_read = false;
 				return;
 			}
-			if (!read)
-			{
+			if (!read) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -445,25 +448,20 @@ void CProxySocket::OnReceive()
 			m_recvBufferPos = 0;
 
 			// All data got read, parse it
-			switch (m_handshakeState)
-			{
+			switch (m_handshakeState) {
 			default:
-				if (m_pRecvBuffer[0] != 5)
-				{
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
+				if (m_pRecvBuffer[0] != 5) {
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
 			case socks5_auth:
-				if (m_pRecvBuffer[0] != 1)
-				{
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
+				if (m_pRecvBuffer[0] != 1) {
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -472,8 +470,7 @@ void CProxySocket::OnReceive()
 				// Nothing to do
 				break;
 			}
-			switch (m_handshakeState)
-			{
+			switch (m_handshakeState) {
 			case socks5_method:
 				{
 					const char method = m_pRecvBuffer[1];
@@ -486,10 +483,9 @@ void CProxySocket::OnReceive()
 						m_handshakeState = socks5_auth;
 						break;
 					default:
-						m_pOwner->LogMessage(MessageType::Debug_Warning, _("No supported SOCKS5 auth method"));
+						m_pOwner->LogMessage(MessageType::Error, _("No supported SOCKS5 auth method"));
 						m_proxyState = noconn;
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 				}
@@ -497,59 +493,59 @@ void CProxySocket::OnReceive()
 			case socks5_auth:
 				if (m_pRecvBuffer[1] != 0)
 				{
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy authentication failed"));
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy authentication failed"));
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request;
 				break;
 			case socks5_request:
-				if (m_pRecvBuffer[1])
-				{
-					wxString error;
+				if (m_pRecvBuffer[1]) {
+					wxString errorMsg;
 					switch (m_pRecvBuffer[1])
 					{
 					case 1:
-						error = _("General SOCKS server failure");
+						errorMsg = _("General SOCKS server failure");
 						break;
 					case 2:
-						error = _("Connection not allowed by ruleset");
+						errorMsg = _("Connection not allowed by ruleset");
 						break;
 					case 3:
-						error = _("Network unreachable");
+						errorMsg = _("Network unreachable");
 						break;
 					case 4:
-						error = _("Host unreachable");
+						errorMsg = _("Host unreachable");
 						break;
 					case 5:
-						error = _("Connection refused");
+						errorMsg = _("Connection refused");
 						break;
 					case 6:
-						error = _("TTL expired");
+						errorMsg = _("TTL expired");
 						break;
 					case 7:
-						error = _("Command not supported");
+						errorMsg = _("Command not supported");
 						break;
 					case 8:
-						error = _("Address type not supported");
+						errorMsg = _("Address type not supported");
 						break;
 					default:
-						error.Printf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
+						errorMsg.Printf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
 						break;
 					}
 
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: %s"), error);
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed. Reply from proxy: %s"), errorMsg);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_addrtype;
 				m_recvBufferLen = 3;
 				break;
 			case socks5_request_addrtype:
+				// We need to parse the returned address type to determine the length of the address that follows.
+				// Unfortunately the information in the type and address is useless, many proxies just return
+				// syntactically valid bogus values
 				switch (m_pRecvBuffer[1])
 				{
 				case 1:
@@ -562,10 +558,9 @@ void CProxySocket::OnReceive()
 					m_recvBufferLen = 17;
 					break;
 				default:
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: Unknown address type in CONNECT reply"));
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: Unknown address type in CONNECT reply"));
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_address;
@@ -574,8 +569,7 @@ void CProxySocket::OnReceive()
 				{
 					// We're done
 					m_proxyState = conn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 					return;
 				}
 			default:
@@ -613,43 +607,36 @@ void CProxySocket::OnReceive()
 					m_pSendBuffer[1] = 1; // CONNECT
 					m_pSendBuffer[2] = 0; // Reserved
 
-					if (IsIpAddress(m_host))
-					{
-						// Quite ugly
-						wxString ipv6 = GetIPV6LongForm(m_host);
-						if (!ipv6.empty())
-						{
-							ipv6.Replace(_T(":"), _T(""));
-							addrlen = 16;
-							for (int i = 0; i < 16; i++)
-								m_pSendBuffer[4 + i] = (DigitHexToDecNum(ipv6[i * 2]) << 4) + DigitHexToDecNum(ipv6[i * 2 + 1]);
-
-							m_pSendBuffer[3] = 4; // IPv6
+					auto type = fz::get_address_type(m_host.ToStdWstring());
+					if (type == fz::address_type::ipv6) {
+						auto ipv6 = fz::get_ipv6_long_form(m_host.ToStdWstring());
+						addrlen = 16;
+						for (int i = 0; i < 16; ++i) {
+							m_pSendBuffer[4 + i] = (fz::hex_char_to_int(ipv6[i * 2 + i / 2]) << 4) + fz::hex_char_to_int(ipv6[i * 2 + 1 + i / 2]);
 						}
-						else
-						{
-							unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
-							int i = 0;
-							memset(buf, 0, 4);
-							for (const wxChar* p = m_host.c_str(); *p && i < 4; p++)
-							{
-								const wxChar& c = *p;
-								if (c == '.')
-								{
-									i++;
-									continue;
-								}
-								buf[i] *= 10;
-								buf[i] += c - '0';
-							}
 
-							addrlen = 4;
-
-							m_pSendBuffer[3] = 1; // IPv4
-						}
+						m_pSendBuffer[3] = 4; // IPv6
 					}
-					else
-					{
+					else if (type == fz::address_type::ipv4) {
+						unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
+						int i = 0;
+						memset(buf, 0, 4);
+						for (const wxChar* p = m_host.c_str(); *p && i < 4; ++p) {
+							const wxChar& c = *p;
+							if (c == '.')
+							{
+								i++;
+								continue;
+							}
+							buf[i] *= 10;
+							buf[i] += c - '0';
+						}
+
+						addrlen = 4;
+
+						m_pSendBuffer[3] = 1; // IPv4
+					}
+					else {
 						m_pSendBuffer[3] = 3; // Domain name
 						m_pSendBuffer[4] = hostlen;
 						memcpy(m_pSendBuffer + 5, (const char*)host, hostlen);
@@ -679,8 +666,7 @@ void CProxySocket::OnReceive()
 	default:
 		m_proxyState = noconn;
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unhandled handshake state %d"), m_handshakeState);
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 		return;
 	}
 }
@@ -691,17 +677,13 @@ void CProxySocket::OnSend()
 	if (m_proxyState != handshake || !m_pSendBuffer)
 		return;
 
-	for (;;)
-	{
+	for (;;) {
 		int error;
 		int written = m_pSocket->Write(m_pSendBuffer, m_sendBufferLen, error);
-		if (written == -1)
-		{
-			if (error != EAGAIN)
-			{
+		if (written == -1) {
+			if (error != EAGAIN) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 			}
 			else
 				m_can_write = false;
@@ -709,8 +691,7 @@ void CProxySocket::OnSend()
 			return;
 		}
 
-		if (written == m_sendBufferLen)
-		{
+		if (written == m_sendBufferLen) {
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 
