@@ -3,7 +3,7 @@
 
 #include "socket.h"
 
-class CLatencyMeasurement : public CCallback
+class CLatencyMeasurement final : public CCallback
 {
 public:
 	CLatencyMeasurement();
@@ -23,12 +23,12 @@ public:
 	virtual void cb();
 
 protected:
-	wxDateTime m_start;
+	fz::monotonic_clock m_start;
 
-	wxLongLong m_summed_latency;
-	int m_measurements;
+	int64_t m_summed_latency{};
+	int m_measurements{};
 
-	mutable wxCriticalSection m_sync;
+	mutable fz::mutex m_sync;
 };
 
 #endif
