@@ -21,13 +21,13 @@ enum capabilityNames
 	mlsd_command,
 	opst_mlst_command, // Arguments for OPTS MLST command
 	mfmt_command,
-	pret_command,
 	mdtm_command,
 	size_command,
 	mode_z_support,
 	tvfs_support, // Trivial virtual file store (RFC 3659)
 	list_hidden_support, // LIST -a command
 	rest_stream, // supports REST+STOR in addition to APPE
+	epsv_command,
 
 	// FTPS and HTTPS
 	tls_resume, // Does the server support resuming of TLS sessions?
@@ -36,12 +36,15 @@ enum capabilityNames
 	// can return different times than the UTC based times using the MLST or
 	// MDTM commands.
 	// Note that the user can invoke an additional timezone offset on top of
-	// this for server not supporting auto-detection or to compensate 
+	// this for server not supporting auto-detection or to compensate
 	// unsynchronized clocks.
-	timezone_offset
+	timezone_offset,
+
+	auth_tls_command,
+	auth_ssl_command
 };
 
-class CCapabilities
+class CCapabilities final
 {
 public:
 	enum capabilities GetCapability(enum capabilityNames name, wxString* pOption = 0) const;
@@ -59,7 +62,7 @@ protected:
 	std::map<enum capabilityNames, struct t_cap> m_capabilityMap;
 };
 
-class CServerCapabilities
+class CServerCapabilities final
 {
 public:
 	// If return value isn't 'yes', pOptions remains unchanged
