@@ -1,6 +1,5 @@
 #include <filezilla.h>
 #include "serverpath.h"
-#include "string_coalescer.h"
 
 #define FTP_MVS_DOUBLE_QUOTE (wxChar)0xDC
 
@@ -42,14 +41,12 @@ bool CServerPathData::operator==(const CServerPathData& cmp) const
 }
 
 CServerPath::CServerPath()
-	: m_bEmpty(true)
-	, m_type(DEFAULT)
+	: m_type(DEFAULT)
 {
 }
 
-CServerPath::CServerPath(const CServerPath &path, wxString subdir)
-	: m_bEmpty(path.m_bEmpty)
-	, m_type(path.m_type)
+CServerPath::CServerPath(CServerPath const& path, wxString subdir)
+	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
 	if (subdir.empty())
@@ -59,13 +56,6 @@ CServerPath::CServerPath(const CServerPath &path, wxString subdir)
 		clear();
 }
 
-CServerPath::CServerPath(const CServerPath &path)
-	: m_bEmpty(path.m_bEmpty)
-	, m_type(path.m_type)
-	, m_data(path.m_data)
-{
-}
-
 CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
 	: m_type(type)
 {
@@ -74,8 +64,6 @@ CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
 
 void CServerPath::clear()
 {
-	m_bEmpty = true;
-	m_type = DEFAULT;
 	m_data.clear();
 }
 
@@ -92,11 +80,9 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	if (path.empty())
 		return false;
 
-	if (m_type == DEFAULT)
-	{
+	if (m_type == DEFAULT) {
 		int pos1 = path.Find(_T(":["));
-		if (pos1 != -1)
-		{
+		if (pos1 != -1) {
 			int pos2 = path.Find(']', true);
 			if (pos2 != -1 && static_cast<size_t>(pos2) == (path.Length() - 1) && !isFile)
 				m_type = VMS;
@@ -123,7 +109,6 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	}
 
 	m_data.clear();
-	m_bEmpty = true;
 
 	if (!ChangePath(path, isFile))
 		return false;
@@ -135,7 +120,7 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 
 wxString CServerPath::GetPath() const
 {
-	if (m_bEmpty)
+	if (empty())
 		return wxString();
 
 	wxString path;
@@ -182,7 +167,7 @@ wxString CServerPath::GetPath() const
 
 bool CServerPath::HasParent() const
 {
-	if (m_bEmpty)
+	if (empty())
 		return false;
 
 	if (!traits[m_type].has_root)
@@ -193,7 +178,7 @@ bool CServerPath::HasParent() const
 
 CServerPath CServerPath::GetParent() const
 {
-	if (!HasParent())
+	if (empty() || !HasParent())
 		return CServerPath();
 
 	CServerPath parent(*this);
@@ -209,7 +194,7 @@ CServerPath CServerPath::GetParent() const
 
 wxString CServerPath::GetLastSegment() const
 {
-	if (!HasParent())
+	if (empty() || !HasParent())
 		return wxString();
 
 	if (!m_data->m_segments.empty())
@@ -242,7 +227,7 @@ wxChar* fast_sprint_number(wxChar* s, size_t n)
 
 wxString CServerPath::GetSafePath() const
 {
-	if (m_bEmpty)
+	if (empty())
 		return wxString();
 
 	#define INTLENGTH 20 // 2^64 - 1
@@ -284,10 +269,19 @@ wxString CServerPath::GetSafePath() const
 	return safepath;
 }
 
-bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
+bool CServerPath::SetSafePath(const wxString& path)
+{
+	bool const ret = DoSetSafePath(path);
+	if (!ret) {
+		clear();
+	}
+
+	return ret;
+}
+
+bool CServerPath::DoSetSafePath(const wxString& path)
 {
 	CServerPathData& data = m_data.Get();
-	m_bEmpty = true;
 	data.m_prefix.clear();
 	data.m_segments.clear();
 
@@ -332,14 +326,11 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 	}
 	while (*p && *p != ' ');
 
-	if (!*p)
-	{
+	if (!*p) {
 		if (prefix_len != 0)
 			return false;
-		else
-		{
+		else {
 			// Is root directory, like / on unix like systems.
-			m_bEmpty = false;
 			return true;
 		}
 	}
@@ -353,8 +344,6 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 		*(p + prefix_len) = 0;
 		if( *p ) {
 			data.m_prefix = CSparseOptional<wxString>(p);
-			if (coalesce)
-				::Coalesce(*data.m_prefix);
 		}
 
 		p += prefix_len + 1;
@@ -384,21 +373,17 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 			return false;
 		*(p + segment_len) = 0;
 		wxString s(p);
-		if (coalesce)
-			::Coalesce(s);
 		data.m_segments.push_back(s);
 
 		p += segment_len + 1;
 	}
 
-	m_bEmpty = false;
-
 	return true;
 }
 
-bool CServerPath::SetType(enum ServerType type)
+bool CServerPath::SetType(ServerType type)
 {
-	if (!m_bEmpty && m_type != DEFAULT)
+	if (!empty() && m_type != DEFAULT)
 		return false;
 
 	m_type = type;
@@ -406,14 +391,14 @@ bool CServerPath::SetType(enum ServerType type)
 	return true;
 }
 
-enum ServerType CServerPath::GetType() const
+ServerType CServerPath::GetType() const
 {
 	return m_type;
 }
 
 bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 {
-	if (m_bEmpty || path.m_bEmpty)
+	if (empty() || path.empty())
 		return false;
 
 	if (m_type != path.m_type)
@@ -464,9 +449,6 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 
 bool CServerPath::IsParentOf(const CServerPath &path, bool cmpNoCase) const
 {
-	if (!this)
-		return false;
-
 	return path.IsSubdirOf(*this, cmpNoCase);
 }
 
@@ -477,18 +459,28 @@ bool CServerPath::ChangePath(wxString subdir)
 }
 
 bool CServerPath::ChangePath(wxString &subdir, bool isFile)
+{
+	bool ret = DoChangePath(subdir, isFile);
+	if (!ret) {
+		clear();
+	}
+
+	return ret;
+}
+
+bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 {
 	wxString dir = subdir;
 	wxString file;
 
-	if (dir.empty())
-	{
+	if (dir.empty()) {
 		if (empty() || isFile)
 			return false;
 		else
 			return true;
 	}
 
+	bool const was_empty = empty();
 	CServerPathData& data = m_data.Get();
 
 	switch (m_type)
@@ -504,7 +496,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 				if (isFile)
 				{
-					if (empty())
+					if (was_empty)
 						return false;
 
 					file = dir;
@@ -534,7 +526,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 			if (!Segmentize(dir, data.m_segments))
 				return false;
-			if (data.m_segments.empty() && m_bEmpty)
+			if (data.m_segments.empty() && was_empty)
 				return false;
 		}
 		break;
@@ -565,7 +557,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 			if (!Segmentize(dir, data.m_segments))
 				return false;
-			if (data.m_segments.empty() && m_bEmpty)
+			if (data.m_segments.empty() && was_empty)
 				return false;
 		}
 		break;
@@ -600,10 +592,10 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		}
 		else if (dir.Last() == traits[m_type].right_enclosure)
 			return false;
-		else if (m_bEmpty)
+		else if (was_empty)
 			return false;
 
-		if (dir.Last() == ')') {
+		if (!dir.empty() && dir.Last() == ')') {
 			// Partitioned dataset member
 			if (!isFile)
 				return false;
@@ -615,13 +607,13 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 			file = dir.Mid(pos + 1);
 			dir = dir.Left(pos);
 
-			if (!m_bEmpty && !data.m_prefix && !dir.empty())
+			if (!was_empty && !data.m_prefix && !dir.empty())
 				return false;
 
 			data.m_prefix.clear();
 		}
 		else {
-			if (!m_bEmpty && !data.m_prefix)
+			if (!was_empty && !data.m_prefix)
 			{
 				if (dir.Find('.') != -1 || !isFile)
 					return false;
@@ -632,7 +624,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 					return false;
 				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			}
-			else if (dir.Last() == '.')
+			else if (!dir.empty() && dir.Last() == '.')
 				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			else
 				data.m_prefix.clear();
@@ -650,7 +642,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 		if (!Segmentize(dir, data.m_segments))
 			return false;
-		if (data.m_segments.empty() && m_bEmpty)
+		if (data.m_segments.empty() && was_empty)
 			return false;
 
 		break;
@@ -658,7 +650,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		{
 			if (dir[0] != ':')
 			{
-				if (empty())
+				if (was_empty)
 					return false;
 			}
 			else
@@ -686,7 +678,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				data.m_segments.clear();
 				data.m_prefix.clear();
 			}
-			else if (m_bEmpty)
+			else if (was_empty)
 				return false;
 			if (dir.Left(2) == _T("//"))
 			{
@@ -705,7 +697,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		{
 			if (wxString(traits[m_type].separators).Contains(dir[0]))
 				data.m_segments.clear();
-			else if (m_bEmpty)
+			else if (was_empty)
 				return false;
 
 			if (isFile && !ExtractFile(dir, file))
@@ -730,13 +722,12 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		subdir = file;
 	}
 
-	m_bEmpty = false;
 	return true;
 }
 
 bool CServerPath::operator==(const CServerPath &op) const
 {
-	if (m_bEmpty != op.m_bEmpty)
+	if (empty() != op.empty())
 		return false;
 	else if (m_type != op.m_type)
 		return false;
@@ -748,20 +739,16 @@ bool CServerPath::operator==(const CServerPath &op) const
 
 bool CServerPath::operator!=(const CServerPath &op) const
 {
-	if (!this)
-		return false;
-
 	return !(*this == op);
 }
 
 bool CServerPath::operator<(const CServerPath &op) const
 {
-	if (m_bEmpty)
-	{
-		if (!op.m_bEmpty)
+	if (empty()) {
+		if (!op.empty())
 			return false;
 	}
-	else if (op.m_bEmpty)
+	else if (op.empty())
 		return true;
 
 	if( m_data->m_prefix || op.m_data->m_prefix ) {
@@ -795,15 +782,12 @@ bool CServerPath::operator<(const CServerPath &op) const
 
 wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=false*/) const
 {
-	if (m_bEmpty)
+	if (empty())
 		return filename;
 
 	if (filename.empty())
 		return wxString();
 
-	if (m_bEmpty)
-		return wxString();
-
 	if (omitPath && (!traits[m_type].prefixmode || (m_data->m_prefix && *m_data->m_prefix == _T("."))))
 		return filename;
 
@@ -814,14 +798,14 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 	switch (m_type)
 	{
 		case VXWORKS:
-			if (!wxString(traits[m_type].separators).Contains(result.Last()) && !m_data->m_segments.empty())
+			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()) && !m_data->m_segments.empty())
 				result += traits[m_type].separators[0];
 			break;
 		case VMS:
 		case MVS:
 			break;
 		default:
-			if (!wxString(traits[m_type].separators).Contains(result.Last()))
+			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()))
 				result += traits[m_type].separators[0];
 			break;
 	}
@@ -839,7 +823,7 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 
 int CServerPath::CmpNoCase(const CServerPath &op) const
 {
-	if (m_bEmpty != op.m_bEmpty)
+	if (empty() != op.empty())
 		return 1;
 	else if (m_data->m_prefix != op.m_data->m_prefix)
 		return 1;
@@ -878,7 +862,7 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	if (*this == path)
 		return *this;
 
-	if (m_bEmpty || path.m_bEmpty)
+	if (empty() || path.empty())
 		return CServerPath();
 
 	if (m_type != path.m_type ||
@@ -903,7 +887,6 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	}
 
 	CServerPath parent;
-	parent.m_bEmpty = false;
 	parent.m_type = m_type;
 
 	CServerPathData& parentData = parent.m_data.Get();
@@ -950,60 +933,68 @@ wxString CServerPath::FormatSubdir(const wxString &subdir) const
 	return res;
 }
 
-bool CServerPath::Segmentize(wxString str, tSegmentList& segments)
+bool CServerPath::SegmentizeAddSegment(wxString & segment, tSegmentList& segments, bool& append)
 {
-	bool append = false;
-	while (!str.empty())
-	{
-		wxString segment;
-		int pos = str.find_first_of(traits[m_type].separators);
-		if (pos == -1)
-		{
-			segment = str,
-			str.clear();
-		}
-		else if (!pos)
-		{
-			str = str.Mid(1);
-			continue;
-		}
-		else
-		{
-			segment = str.Left(pos);
-			str = str.Mid(pos + 1);
-		}
-
-		if (traits[m_type].has_dots)
-		{
-			if (segment == _T("."))
-				continue;
-			else if (segment == _T(".."))
-			{
-				if (segments.empty())
-					return false;
-				else
-				{
-					segments.pop_back();
-					continue;
-				}
+	if (traits[m_type].has_dots) {
+		if (segment == _T("."))
+			return true;
+		else if (segment == _T("..")) {
+			if (segments.empty())
+				return false;
+			else {
+				segments.pop_back();
+				return true;
 			}
 		}
+	}
 
-		bool append_next = false;
-		if (traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape)
-		{
-			append_next = true;
-			segment.RemoveLast();
-			segment += traits[m_type].separators[0];
+	bool append_next = false;
+	if (!segment.empty() && traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape) {
+		append_next = true;
+		segment[segment.size() - 1] = traits[m_type].separators[0];
+	}
+
+	if (append)
+		segments.back() += segment;
+	else
+		segments.push_back(std::move(segment));
+
+	append = append_next;
+
+	return true;
+}
+
+bool CServerPath::Segmentize(wxString const& str, tSegmentList& segments)
+{
+	bool append = false;
+	size_t start = 0;
+
+	size_t pos;
+	while (true) {
+		pos = str.find_first_of(traits[m_type].separators, start);
+		if (pos == std::string::npos) {
+			break;
+		}
+		if (start == pos) {
+			++start;
+			continue;
 		}
 
-		if (append)
-			segments.back() += segment;
-		else
-			segments.push_back(segment);
+		wxString segment = str.substr(start, pos - start);
+		start = pos + 1;
 
-		append = append_next;
+		if (!SegmentizeAddSegment(segment, segments, append)) {
+			return false;
+		}
 	}
+
+	if (start < str.size()) {
+		wxString segment = str.substr(start);
+		if (!SegmentizeAddSegment(segment, segments, append)) {
+			return false;
+		}
+	}
+
 	if (append)
 		return false;
 
@@ -1016,8 +1007,7 @@ bool CServerPath::ExtractFile(wxString& dir, wxString& file)
 	if (pos == (int)dir.Length() - 1)
 		return false;
 
-	if (pos == -1)
-	{
+	if (pos == -1) {
 		file = dir;
 		dir.clear();
 		return true;
@@ -1037,13 +1027,7 @@ void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
 	}
 }
 
-void CServerPath::Coalesce()
+size_t CServerPath::SegmentCount() const
 {
-	CServerPathData& data = m_data.Get();
-	if( data.m_prefix ) {
-		::Coalesce(*data.m_prefix);
-	}
-	for (auto& segment : data.m_segments) {
-		::Coalesce(segment);
-	}
-}
+	return empty() ? 0 : m_data->m_segments.size();
+}
\ No newline at end of file
