@@ -2,7 +2,6 @@
 #define __SFTPCONTROLSOCKET_H__
 
 #include "ControlSocket.h"
-#include <wx/process.h>
 
 enum class sftpEvent {
 	Unknown = -1,
@@ -16,14 +15,14 @@ enum class sftpEvent {
 	Close,
 	Request,
 	Listentry,
-	Read,
-	Write,
+	Transfer,
 	RequestPreamble,
 	RequestInstruction,
 	UsedQuotaRecv,
 	UsedQuotaSend,
 	KexAlgorithm,
 	KexHash,
+	KexCurve,
 	CipherClientToServer,
 	CipherServerToClient,
 	MacClientToServer,
@@ -41,17 +40,22 @@ enum sftpRequestTypes
 	sftpReqUnknown
 };
 
+namespace fz {
+class process;
+}
+
 class CSftpInputThread;
-class CSftpControlSocket : public CControlSocket, public CRateLimiterObject
+
+class CSftpControlSocket final : public CControlSocket, public CRateLimiterObject
 {
 public:
-	CSftpControlSocket(CFileZillaEnginePrivate* pEngine);
+	CSftpControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CSftpControlSocket();
 
 	virtual int Connect(const CServer &server);
 
 	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), int flags = 0);
-	virtual int Delete(const CServerPath& path, const std::list<wxString>& files);
+	virtual int Delete(const CServerPath& path, std::deque<wxString>&& files);
 	virtual int RemoveDir(const CServerPath& path = CServerPath(), const wxString& subDir = _T(""));
 	virtual int Mkdir(const CServerPath& path);
 	virtual int Rename(const CRenameCommand& command);
@@ -62,8 +66,6 @@ public:
 
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification);
 
-	void SetActive(bool recv);
-
 protected:
 	// Replaces filename"with"quotes with
 	// "filename""with""quotes"
@@ -75,7 +77,7 @@ protected:
 	virtual int SendNextCommand();
 	virtual int ParseSubcommandResult(int prevResult);
 
-	int ProcessReply(bool successful, const wxString& reply = _T(""));
+	void ProcessReply(int result, const wxString& reply = _T(""));
 
 	int ConnectParseResponse(bool successful, const wxString& reply);
 	int ConnectSend();
@@ -85,7 +87,7 @@ protected:
 							 const CFileTransferCommand::t_transferSettings& transferSettings);
 	int FileTransferSubcommandResult(int prevResult);
 	int FileTransferSend();
-	int FileTransferParseResponse(bool successful, const wxString& reply);
+	int FileTransferParseResponse(int result, const wxString& reply);
 
 	int ListSubcommandResult(int prevResult);
 	int ListSend();
@@ -117,25 +119,18 @@ protected:
 	bool SendCommand(wxString const& cmd, const wxString& show = wxString());
 	bool AddToStream(const wxString& cmd, bool force_utf8 = false);
 
-	virtual void OnRateAvailable(enum CRateLimiter::rate_direction direction);
-	void OnQuotaRequest(enum CRateLimiter::rate_direction direction);
+	virtual void OnRateAvailable(CRateLimiter::rate_direction direction);
+	void OnQuotaRequest(CRateLimiter::rate_direction direction);
 
 	// see src/putty/wildcard.c
 	wxString WildcardEscape(const wxString& file);
 
-	wxProcess* m_pProcess;
-	CSftpInputThread* m_pInputThread;
-	int m_pid;
+	fz::process* m_pProcess{};
+	CSftpInputThread* m_pInputThread{};
 
-	DECLARE_EVENT_TABLE()
-	void OnTerminate(wxProcessEvent& event);
-	void OnSftpEvent(wxCommandEvent& event);
-
-	// Set to true in destructor of CSftpControlSocket
-	// If sftp process dies (or gets killed), OnTerminate will be called.
-	// To avoid a race condition, abort OnTerminate if m_inDestructor is set
-	bool m_inDestructor;
-	bool m_termindatedInDestructor;
+	virtual void operator()(fz::event_base const& ev);
+	void OnSftpEvent();
+	void OnTerminate();
 
 	wxString m_requestPreamble;
 	wxString m_requestInstruction;
