@@ -1,14 +1,19 @@
 #include <filezilla.h>
+#include "engineprivate.h"
 #include "tlssocket.h"
 #include "ControlSocket.h"
 
-#include <gnutls/gnutls.h>
+#include <libfilezilla/iputils.hpp>
+
 #include <gnutls/x509.h>
-#include <errno.h>
 
-char const ciphers[] = "SECURE256:+SECURE128:+ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP";
+#if FZ_USE_GNUTLS_SYSTEM_CIPHERS
+char const ciphers[] = "@SYSTEM";
+#else
+char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP:-VERS-SSL3.0";
+#endif
 
-//#define TLSDEBUG 1
+#define TLSDEBUG 0
 #if TLSDEBUG
 // This is quite ugly
 CControlSocket* pLoggingControlSocket;
@@ -22,46 +27,22 @@ void log_func(int level, const char* msg)
 }
 #endif
 
-CTlsSocket::CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
-	: CBackend(pEvtHandler), m_pOwner(pOwner)
+CTlsSocket::CTlsSocket(event_handler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
+	: event_handler(pOwner->event_loop_)
+	, CBackend(pEvtHandler)
+	, m_pOwner(pOwner)
+	, m_socket(socket)
 {
-	m_pSocket = pSocket;
-	m_pSocketBackend = new CSocketBackend(this, m_pSocket);
-
-	m_session = 0;
-	m_initialized = false;
-	m_certCredentials = 0;
-
-	m_canReadFromSocket = true;
-	m_canWriteToSocket = true;
-	m_canCheckCloseSocket = false;
-
-	m_canTriggerRead = false;
-	m_canTriggerWrite = true;
-
-	m_tlsState = noconn;
-
-	m_lastReadFailed = true;
-	m_lastWriteFailed = false;
-	m_writeSkip = 0;
-
-	m_peekData = 0;
-	m_peekDataLen = 0;
-
-	m_socketClosed = false;
+	m_pSocketBackend = new CSocketBackend(this, m_socket, m_pOwner->GetEngine().GetRateLimiter());
 
 	m_implicitTrustedCert.data = 0;
 	m_implicitTrustedCert.size = 0;
-
-	m_shutdown_requested = false;
-
-	m_socket_eof = false;
-
-	m_require_root_trust = false;
 }
 
 CTlsSocket::~CTlsSocket()
 {
+	remove_handler();
+
 	Uninit();
 	delete m_pSocketBackend;
 }
@@ -71,24 +52,21 @@ bool CTlsSocket::Init()
 	// This function initializes GnuTLS
 	m_initialized = true;
 	int res = gnutls_global_init();
-	if (res)
-	{
+	if (res) {
 		LogError(res, _T("gnutls_global_init"));
 		Uninit();
 		return false;
 	}
 
 #if TLSDEBUG
-	if (!pLoggingControlSocket)
-	{
+	if (!pLoggingControlSocket) {
 		pLoggingControlSocket = m_pOwner;
 		gnutls_global_set_log_function(log_func);
 		gnutls_global_set_log_level(99);
 	}
 #endif
 	res = gnutls_certificate_allocate_credentials(&m_certCredentials);
-	if (res < 0)
-	{
+	if (res < 0) {
 		LogError(res, _T("gnutls_certificate_allocate_credentials"));
 		Uninit();
 		return false;
@@ -104,11 +82,32 @@ bool CTlsSocket::Init()
 	return true;
 }
 
+bool CTlsSocket::SetClientCertificate(fz::native_string const& keyfile, fz::native_string const& certs, fz::native_string const& password)
+{
+	if (!m_certCredentials) {
+		return false;
+	}
+
+	int res = gnutls_certificate_set_x509_key_file2(m_certCredentials, fz::to_string(certs).c_str(),
+		fz::to_string(keyfile).c_str(), GNUTLS_X509_FMT_PEM, password.empty() ? 0 : fz::to_utf8(password).c_str(), 0);
+	if (res < 0) {
+		LogError(res, _T("gnutls_certificate_set_x509_key_file2"));
+		Uninit();
+		return false;
+	}
+
+	return true;
+}
+
 bool CTlsSocket::InitSession()
 {
+	if (!m_certCredentials) {
+		Uninit();
+		return false;
+	}
+
 	int res = gnutls_init(&m_session, GNUTLS_CLIENT);
-	if (res)
-	{
+	if (res) {
 		LogError(res, _T("gnutls_init"));
 		Uninit();
 		return false;
@@ -121,14 +120,13 @@ bool CTlsSocket::InitSession()
 	gnutls_db_set_cache_expiration(m_session, 100000000);
 
 	res = gnutls_priority_set_direct(m_session, ciphers, 0);
-	if (res)
-	{
+	if (res) {
 		LogError(res, _T("gnutls_priority_set_direct"));
 		Uninit();
 		return false;
 	}
 
-	gnutls_dh_set_prime_bits(m_session, 512);
+	gnutls_dh_set_prime_bits(m_session, 1024);
 
 	gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, m_certCredentials);
 
@@ -144,19 +142,17 @@ void CTlsSocket::Uninit()
 {
 	UninitSession();
 
-	if (m_certCredentials)
-	{
+	if (m_certCredentials) {
 		gnutls_certificate_free_credentials(m_certCredentials);
 		m_certCredentials = 0;
 	}
 
-	if (m_initialized)
-	{
+	if (m_initialized) {
 		m_initialized = false;
 		gnutls_global_deinit();
 	}
 
-	m_tlsState = noconn;
+	m_tlsState = TlsState::noconn;
 
 	delete [] m_peekData;
 	m_peekData = 0;
@@ -165,8 +161,6 @@ void CTlsSocket::Uninit()
 	delete [] m_implicitTrustedCert.data;
 	m_implicitTrustedCert.data = 0;
 
-	m_require_root_trust = false;
-
 #if TLSDEBUG
 	if (pLoggingControlSocket == m_pOwner)
 		pLoggingControlSocket = 0;
@@ -176,8 +170,7 @@ void CTlsSocket::Uninit()
 
 void CTlsSocket::UninitSession()
 {
-	if (m_session)
-	{
+	if (m_session) {
 		gnutls_deinit(m_session);
 		m_session = 0;
 	}
@@ -186,39 +179,53 @@ void CTlsSocket::UninitSession()
 
 void CTlsSocket::LogError(int code, const wxString& function, MessageType logLevel)
 {
-	if (code == GNUTLS_E_WARNING_ALERT_RECEIVED || code == GNUTLS_E_FATAL_ALERT_RECEIVED)
-		PrintAlert();
-
-	const char* error = gnutls_strerror(code);
-
-	if (error)
-	{
-		wxString str(error, wxConvLocal);
-		if (function.empty())
-			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d: %s"), code, str);
-		else
-			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d in %s: %s"), code, function, str);
+	if (code == GNUTLS_E_WARNING_ALERT_RECEIVED || code == GNUTLS_E_FATAL_ALERT_RECEIVED) {
+		PrintAlert(logLevel);
 	}
-	else
-	{
-		if (function.empty())
-			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d"), code);
-		else
-			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d in %s"), code, function);
+	else if (code == GNUTLS_E_PULL_ERROR) {
+		if (function.empty()) {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS could not read from socket: %s"), CSocket::GetErrorDescription(m_socket_error));
+		}
+		else {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS could not read from socket in %s: %s"), function, CSocket::GetErrorDescription(m_socket_error));
+		}
+	}
+	else if (code == GNUTLS_E_PUSH_ERROR) {
+		if (function.empty()) {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS could not write to socket: %s"), CSocket::GetErrorDescription(m_socket_error));
+		}
+		else {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS could not write to socket in %s: %s"), function, CSocket::GetErrorDescription(m_socket_error));
+		}
+	}
+	else {
+		const char* error = gnutls_strerror(code);
+		if (error) {
+			wxString str(error, wxConvLocal);
+			if (function.empty())
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d: %s"), code, str);
+			else
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d in %s: %s"), code, function, str);
+		}
+		else {
+			if (function.empty())
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d"), code);
+			else
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d in %s"), code, function);
+		}
 	}
 }
 
-void CTlsSocket::PrintAlert()
+void CTlsSocket::PrintAlert(MessageType logLevel)
 {
 	gnutls_alert_description_t last_alert = gnutls_alert_get(m_session);
 	const char* alert = gnutls_alert_get_name(last_alert);
-	if (alert)
-	{
+	if (alert) {
 		wxString str(alert, wxConvLocal);
-		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS alert %d: %s"), last_alert, str);
+		m_pOwner->LogMessage(logLevel, _("Received TLS alert from the server: %s (%d)"), str, last_alert);
 	}
 	else
-		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS alert %d"), last_alert);
+		m_pOwner->LogMessage(logLevel, _("Received unknown TLS alert %d from the server"), last_alert);
 }
 
 ssize_t CTlsSocket::PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len)
@@ -234,16 +241,14 @@ ssize_t CTlsSocket::PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t
 ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 {
 #if TLSDEBUG
-	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PushFunction(%x, %d)"), data, len);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PushFunction(%d)"), len);
 #endif
-	if (!m_canWriteToSocket)
-	{
+	if (!m_canWriteToSocket) {
 		gnutls_transport_set_errno(m_session, EAGAIN);
 		return -1;
 	}
 
-	if (!m_pSocketBackend)
-	{
+	if (!m_pSocketBackend) {
 		gnutls_transport_set_errno(m_session, 0);
 		return -1;
 	}
@@ -251,16 +256,15 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 	int error;
 	int written = m_pSocketBackend->Write(data, len, error);
 
-	if (written < 0)
-	{
-		if (error == EAGAIN)
-		{
-			m_canWriteToSocket = false;
-			gnutls_transport_set_errno(m_session, EAGAIN);
-			return -1;
+	if (written < 0) {
+		m_canWriteToSocket = false;
+		if (error == EAGAIN) {
+			m_socket_error = error;
 		}
-
-		gnutls_transport_set_errno(m_session, 0);
+		gnutls_transport_set_errno(m_session, error);
+#if TLSDEBUG
+		m_pOwner->LogMessage(MessageType::Debug_Debug, _T("  returning -1 due to %d"), error);
+#endif
 		return -1;
 	}
 
@@ -274,10 +278,9 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 {
 #if TLSDEBUG
-	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PullFunction(%x, %d)"), data, len);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PullFunction(%d)"),  (int)len);
 #endif
-	if (!m_pSocketBackend)
-	{
+	if (!m_pSocketBackend) {
 		gnutls_transport_set_errno(m_session, 0);
 		return -1;
 	}
@@ -285,38 +288,37 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	if (m_socketClosed)
 		return 0;
 
-	if (!m_canReadFromSocket)
-	{
+	if (!m_canReadFromSocket) {
 		gnutls_transport_set_errno(m_session, EAGAIN);
 		return -1;
 	}
 
 	int error;
 	int read = m_pSocketBackend->Read(data, len, error);
-	if (read < 0)
-	{
-		if (error == EAGAIN)
-		{
-			m_canReadFromSocket = false;
-			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound))
-			{
-				CSocketEvent *evt = new CSocketEvent(this, m_pSocketBackend, CSocketEvent::close);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+	if (read < 0) {
+		m_canReadFromSocket = false;
+		if (error == EAGAIN) {
+			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound)) {
+				send_event<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
 			}
 		}
-
+		else {
+			m_socket_error = error;
+		}
 		gnutls_transport_set_errno(m_session, error);
+#if TLSDEBUG
+		m_pOwner->LogMessage(MessageType::Debug_Debug, _T("  returning -1 due to %d"), error);
+#endif
 		return -1;
 	}
 
-	if (m_canCheckCloseSocket)
-	{
-		CSocketEvent *evt = new CSocketEvent(this, m_pSocketBackend, CSocketEvent::close);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+	if (m_canCheckCloseSocket) {
+		send_event<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
 	}
 
-	if (!read)
+	if (!read) {
 		m_socket_eof = true;
+	}
 
 #if TLSDEBUG
 	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("  returning %d"), read);
@@ -325,32 +327,33 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	return read;
 }
 
-void CTlsSocket::OnSocketEvent(CSocketEvent& event)
+void CTlsSocket::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<CSocketEvent>(ev, this, &CTlsSocket::OnSocketEvent);
+}
+
+void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
-	wxASSERT(m_pSocket);
 	if (!m_session)
 		return;
 
-	switch (event.GetType())
+	switch (t)
 	{
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnRead();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
-	case CSocketEvent::close:
+	case SocketEventType::close:
 		{
 			m_canCheckCloseSocket = true;
 			char tmp[100];
-			int error;
 			int peeked = m_pSocketBackend->Peek(&tmp, 100, error);
-			if (peeked >= 0)
-			{
+			if (peeked >= 0) {
 				if (peeked > 0)
 					m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
-				else
-				{
+				else {
 					m_socket_eof = true;
 					m_socketClosed = true;
 				}
@@ -363,8 +366,7 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 			m_pOwner->LogMessage(MessageType::Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
 
 			//Uninit();
-			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
-			CSocketEventDispatcher::Get().SendEvent(evt);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 		}
 		break;
 	default:
@@ -382,15 +384,16 @@ void CTlsSocket::OnRead()
 		return;
 
 	const int direction = gnutls_record_get_direction(m_session);
-	if (direction && !m_lastReadFailed)
+	if (direction && !m_lastReadFailed) {
+		m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::Postponing read"));
 		return;
+	}
 
-	if (m_tlsState == handshake)
+	if (m_tlsState == TlsState::handshake)
 		ContinueHandshake();
-	if (m_tlsState == closing)
+	if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -409,12 +412,11 @@ void CTlsSocket::OnSend()
 	if (!direction && !m_lastWriteFailed)
 		return;
 
-	if (m_tlsState == handshake)
+	if (m_tlsState == TlsState::handshake)
 		ContinueHandshake();
-	else if (m_tlsState == closing)
+	else if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -454,7 +456,7 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Handshake()"));
 	wxASSERT(m_session);
 
-	m_tlsState = handshake;
+	m_tlsState = TlsState::handshake;
 
 	wxString hostname;
 
@@ -462,31 +464,29 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
-		if (cert_list && cert_list_size)
-		{
+		if (cert_list && cert_list_size) {
 			delete [] m_implicitTrustedCert.data;
 			m_implicitTrustedCert.data = new unsigned char[cert_list[0].size];
 			memcpy(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size);
 			m_implicitTrustedCert.size = cert_list[0].size;
 		}
 
-		if (try_resume)
-		{
+		if (try_resume) {
 			if (!CopySessionData(pPrimarySocket))
 				return FZ_REPLY_ERROR;
 		}
 
-		hostname = pPrimarySocket->m_pSocket->GetPeerHost();
+		hostname = pPrimarySocket->m_socket.GetPeerHost();
 	}
 	else {
-		hostname = m_pSocket->GetPeerHost();
+		hostname = m_socket.GetPeerHost();
 	}
 
-	if( !hostname.empty() && !IsIpAddress(hostname) ) {
+	if (!hostname.empty() && fz::get_address_type(hostname.ToStdWstring()) == fz::address_type::unknown) {
 		const wxWX2MBbuf utf8 = hostname.mb_str(wxConvUTF8);
-		if( utf8 ) {
+		if (utf8) {
 			int res = gnutls_server_name_set( m_session, GNUTLS_NAME_DNS, utf8, strlen(utf8) );
-			if( res ) {
+			if (res) {
 				LogError(res, _T("gnutls_server_name_set"), MessageType::Debug_Warning );
 			}
 		}
@@ -499,18 +499,16 @@ int CTlsSocket::ContinueHandshake()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
 	wxASSERT(m_session);
-	wxASSERT(m_tlsState == handshake);
+	wxASSERT(m_tlsState == TlsState::handshake);
 
 	int res = gnutls_handshake(m_session);
-	while (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
-	{
-		if ( gnutls_record_get_direction(m_session) != 1 || !m_canWriteToSocket )
+	while (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
+		if (!(gnutls_record_get_direction(m_session) ? m_canWriteToSocket : m_canReadFromSocket)) {
 			break;
-
+		}
 		res = gnutls_handshake(m_session);
 	}
-	if (!res)
-	{
+	if (!res) {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("TLS Handshake successful"));
 
 		if (ResumedSession())
@@ -527,13 +525,10 @@ int CTlsSocket::ContinueHandshake()
 		if (res != FZ_REPLY_OK)
 			return res;
 
-		if (m_shutdown_requested)
-		{
+		if (m_shutdown_requested) {
 			int error = Shutdown();
-			if (!error || error != EAGAIN)
-			{
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+			if (!error || error != EAGAIN) {
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 			}
 		}
 
@@ -542,42 +537,36 @@ int CTlsSocket::ContinueHandshake()
 	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
 		return FZ_REPLY_WOULDBLOCK;
 
-	Failure(res, ECONNABORTED);
+	Failure(res, true);
 
 	return FZ_REPLY_ERROR;
 }
 
 int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 {
-	if (m_tlsState == handshake || m_tlsState == verifycert)
-	{
+	if (m_tlsState == TlsState::handshake || m_tlsState == TlsState::verifycert) {
 		error = EAGAIN;
 		return -1;
 	}
-	else if (m_tlsState != conn)
-	{
+	else if (m_tlsState != TlsState::conn) {
 		error = ENOTCONN;
 		return -1;
 	}
-	else if (m_lastReadFailed)
-	{
+	else if (m_lastReadFailed) {
 		error = EAGAIN;
 		return -1;
 	}
 
-	if (m_peekDataLen)
-	{
+	if (m_peekDataLen) {
 		unsigned int min = wxMin(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
-		if (min == m_peekDataLen)
-		{
+		if (min == m_peekDataLen) {
 			m_peekDataLen = 0;
 			delete [] m_peekData;
 			m_peekData = 0;
 		}
-		else
-		{
+		else {
 			memmove(m_peekData, m_peekData + min, m_peekDataLen - min);
 			m_peekDataLen -= min;
 		}
@@ -589,12 +578,10 @@ int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 	}
 
 	int res = DoCallGnutlsRecordRecv(buffer, len);
-	if (res >= 0)
-	{
+	if (res >= 0) {
 		if (res > 0)
 			TriggerEvents();
-		else
-		{
+		else {
 			// Peer did already initiate a shutdown, reply to it
 			gnutls_bye(m_session, GNUTLS_SHUT_WR);
 			// Note: Theoretically this could return a write error.
@@ -607,15 +594,13 @@ int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 		return res;
 	}
 
-	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
-	{
+	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
 		error = EAGAIN;
 		m_lastReadFailed = true;
 	}
-	else
-	{
-		Failure(res, 0, _T("gnutls_record_recv"));
-		error = ECONNABORTED;
+	else {
+		Failure(res, false, _T("gnutls_record_recv"));
+		error = m_socket_error;
 	}
 
 	return -1;
@@ -623,25 +608,21 @@ int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 
 int CTlsSocket::Write(const void *buffer, unsigned int len, int& error)
 {
-	if (m_tlsState == handshake || m_tlsState == verifycert)
-	{
+	if (m_tlsState == TlsState::handshake || m_tlsState == TlsState::verifycert) {
 		error = EAGAIN;
 		return -1;
 	}
-	else if (m_tlsState != conn)
-	{
+	else if (m_tlsState != TlsState::conn) {
 		error = ENOTCONN;
 		return -1;
 	}
 
-	if (m_lastWriteFailed)
-	{
+	if (m_lastWriteFailed) {
 		error = EAGAIN;
 		return -1;
 	}
 
-	if (m_writeSkip >= len)
-	{
+	if (m_writeSkip >= len) {
 		m_writeSkip -= len;
 		return len;
 	}
@@ -654,8 +635,7 @@ int CTlsSocket::Write(const void *buffer, unsigned int len, int& error)
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
 		res = gnutls_record_send(m_session, 0, 0);
 
-	if (res >= 0)
-	{
+	if (res >= 0) {
 		error = 0;
 		int written = res + m_writeSkip;
 		m_writeSkip = 0;
@@ -664,54 +644,45 @@ int CTlsSocket::Write(const void *buffer, unsigned int len, int& error)
 		return written;
 	}
 
-	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
-	{
-		if (m_writeSkip)
-		{
+	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
+		if (m_writeSkip) {
 			error = 0;
 			int written = m_writeSkip;
 			m_writeSkip = 0;
 			return written;
 		}
-		else
-		{
+		else {
 			error = EAGAIN;
 			m_lastWriteFailed = true;
 			return -1;
 		}
 	}
-	else
-	{
-		Failure(res, 0, _T("gnutls_record_send"));
-		error = ECONNABORTED;
+	else {
+		Failure(res, false, _T("gnutls_record_send"));
+		error = m_socket_error;
 		return -1;
 	}
 }
 
 void CTlsSocket::TriggerEvents()
 {
-	if (m_tlsState != conn)
+	if (m_tlsState != TlsState::conn)
 		return;
 
-	if (m_canTriggerRead)
-	{
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::read);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+	if (m_canTriggerRead) {
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::read, 0);
 		m_canTriggerRead = false;
 	}
 
-	if (m_canTriggerWrite)
-	{
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::write);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+	if (m_canTriggerWrite) {
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::write, 0);
 		m_canTriggerWrite = false;
 	}
 }
 
 void CTlsSocket::CheckResumeFailedReadWrite()
 {
-	if (m_lastWriteFailed)
-	{
+	if (m_lastWriteFailed) {
 		int res = GNUTLS_E_AGAIN;
 		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
 			res = gnutls_record_send(m_session, 0, 0);
@@ -719,9 +690,8 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
 			return;
 
-		if (res < 0)
-		{
-			Failure(res, ECONNABORTED);
+		if (res < 0) {
+			Failure(res, true);
 			return;
 		}
 
@@ -729,8 +699,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 		m_lastWriteFailed = false;
 		m_canTriggerWrite = true;
 	}
-	if (m_lastReadFailed)
-	{
+	if (m_lastReadFailed) {
 		wxASSERT(!m_peekData);
 
 		m_peekDataLen = 65536;
@@ -742,7 +711,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 			delete [] m_peekData;
 			m_peekData = 0;
 			if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN)
-				Failure(res, ECONNABORTED);
+				Failure(res, true);
 			return;
 		}
 
@@ -759,14 +728,12 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 	}
 }
 
-void CTlsSocket::Failure(int code, int socket_error, const wxString& function)
+void CTlsSocket::Failure(int code, bool send_close, const wxString& function)
 {
-	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::Failure(%d, %d)"), code, socket_error);
-	if (code)
-	{
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::Failure(%d)"), code);
+	if (code) {
 		LogError(code, function);
-		if (m_socket_eof)
-		{
+		if (m_socket_eof) {
 			if (code == GNUTLS_E_UNEXPECTED_PACKET_LENGTH
 #ifdef GNUTLS_E_PREMATURE_TERMINATION
 				|| code == GNUTLS_E_PREMATURE_TERMINATION
@@ -779,17 +746,14 @@ void CTlsSocket::Failure(int code, int socket_error, const wxString& function)
 	}
 	Uninit();
 
-	if (socket_error)
-	{
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, socket_error);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+	if (send_close) {
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, m_socket_error);
 	}
 }
 
 int CTlsSocket::Peek(void *buffer, unsigned int len, int& error)
 {
-	if (m_peekData)
-	{
+	if (m_peekData) {
 		int min = wxMin(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
@@ -812,39 +776,37 @@ int CTlsSocket::Shutdown()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Shutdown()"));
 
-	if (m_tlsState == closed)
+	if (m_tlsState == TlsState::closed)
 		return 0;
 
-	if (m_tlsState == closing)
+	if (m_tlsState == TlsState::closing)
 		return EAGAIN;
 
-	if (m_tlsState == handshake || m_tlsState == verifycert)
-	{
+	if (m_tlsState == TlsState::handshake || m_tlsState == TlsState::verifycert) {
 		// Shutdown during handshake is not a good idea.
 		m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("Shutdown during handshake, postponing"));
 		m_shutdown_requested = true;
 		return EAGAIN;
 	}
 
-	if (m_tlsState != conn)
+	if (m_tlsState != TlsState::conn)
 		return ECONNABORTED;
 
-	m_tlsState = closing;
+	m_tlsState = TlsState::closing;
 
 	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
 		res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
-	if (!res)
-	{
-		m_tlsState = closed;
+	if (!res) {
+		m_tlsState = TlsState::closed;
 		return 0;
 	}
 
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
 		return EAGAIN;
 
-	Failure(res, 0);
-	return ECONNABORTED;
+	Failure(res, false);
+	return m_socket_error;
 }
 
 void CTlsSocket::ContinueShutdown()
@@ -854,40 +816,34 @@ void CTlsSocket::ContinueShutdown()
 	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
 		res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
-	if (!res)
-	{
-		m_tlsState = closed;
+	if (!res) {
+		m_tlsState = TlsState::closed;
 
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 
 		return;
 	}
 
 	if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN)
-		Failure(res, ECONNABORTED);
+		Failure(res, true);
 }
 
 void CTlsSocket::TrustCurrentCert(bool trusted)
 {
-	if (m_tlsState != verifycert)
-	{
+	if (m_tlsState != TlsState::verifycert) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("TrustCurrentCert called at wrong time."));
 		return;
 	}
 
-	if (trusted)
-	{
-		m_tlsState = conn;
+	if (trusted) {
+		m_tlsState = TlsState::conn;
 
 		if (m_lastWriteFailed)
 			m_lastWriteFailed = false;
 		CheckResumeFailedReadWrite();
 
-		if (m_tlsState == conn)
-		{
-			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection);
-			CSocketEventDispatcher::Get().SendEvent(evt);
+		if (m_tlsState == TlsState::conn) {
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 		}
 
 		TriggerEvents();
@@ -896,7 +852,7 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 	}
 
 	m_pOwner->LogMessage(MessageType::Error, _("Remote certificate not trusted."));
-	Failure(0, ECONNABORTED);
+	Failure(0, true);
 }
 
 static wxString bin2hex(const unsigned char* in, size_t size)
@@ -913,26 +869,22 @@ static wxString bin2hex(const unsigned char* in, size_t size)
 }
 
 
-bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
+bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 {
-	const gnutls_datum_t* datum = reinterpret_cast<const gnutls_datum_t*>(in);
-
 	gnutls_x509_crt_t cert;
-	if (gnutls_x509_crt_init(&cert))
-	{
+	if (gnutls_x509_crt_init(&cert)) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
 		return false;
 	}
 
-	if (gnutls_x509_crt_import(cert, datum, GNUTLS_X509_FMT_DER))
-	{
+	if (gnutls_x509_crt_import(cert, datum, GNUTLS_X509_FMT_DER)) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
 		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
 
-	wxDateTime expirationTime = gnutls_x509_crt_get_expiration_time(cert);
-	wxDateTime activationTime = gnutls_x509_crt_get_activation_time(cert);
+	fz::datetime expirationTime(gnutls_x509_crt_get_expiration_time(cert), fz::datetime::seconds);
+	fz::datetime activationTime(gnutls_x509_crt_get_activation_time(cert), fz::datetime::seconds);
 
 	// Get the serial number of the certificate
 	unsigned char buffer[40];
@@ -947,8 +899,7 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 	unsigned int pkBits;
 	int pkAlgo = gnutls_x509_crt_get_pk_algorithm(cert, &pkBits);
 	wxString pkAlgoName;
-	if (pkAlgo >= 0)
-	{
+	if (pkAlgo >= 0) {
 		const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)pkAlgo);
 		if (pAlgo)
 			pkAlgoName = wxString(pAlgo, wxConvUTF8);
@@ -956,8 +907,7 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 
 	int signAlgo = gnutls_x509_crt_get_signature_algorithm(cert);
 	wxString signAlgoName;
-	if (signAlgo >= 0)
-	{
+	if (signAlgo >= 0) {
 		const char* pAlgo = gnutls_sign_algorithm_get_name((gnutls_sign_algorithm_t)signAlgo);
 		if (pAlgo)
 			signAlgoName = wxString(pAlgo, wxConvUTF8);
@@ -967,8 +917,7 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 
 	size = 0;
 	res = gnutls_x509_crt_get_dn(cert, 0, &size);
-	if (size)
-	{
+	if (size) {
 		char* dn = new char[size + 1];
 		dn[size] = 0;
 		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size)))
@@ -988,14 +937,14 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 		return false;
 	}
 
+	std::vector<wxString> alt_subject_names = GetCertSubjectAltNames(cert);
+
 	size = 0;
 	res = gnutls_x509_crt_get_issuer_dn(cert, 0, &size);
-	if (size)
-	{
+	if (size) {
 		char* dn = new char[++size + 1];
 		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size)))
-		{
+		if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size))) {
 			dn[size] = 0;
 			issuer = wxString(dn, wxConvUTF8);
 		}
@@ -1011,19 +960,17 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 		return false;
 	}
 
-	wxString fingerprint_md5;
+	wxString fingerprint_sha256;
 	wxString fingerprint_sha1;
 
 	unsigned char digest[100];
 	size = sizeof(digest) - 1;
-	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_MD5, digest, &size))
-	{
+	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA256, digest, &size)) {
 		digest[size] = 0;
-		fingerprint_md5 = bin2hex(digest, size);
+		fingerprint_sha256 = bin2hex(digest, size);
 	}
 	size = sizeof(digest) - 1;
-	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA1, digest, &size))
-	{
+	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA1, digest, &size)) {
 		digest[size] = 0;
 		fingerprint_sha1 = bin2hex(digest, size);
 	}
@@ -1036,101 +983,190 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 		serial,
 		pkAlgoName, pkBits,
 		signAlgoName,
-		fingerprint_md5,
+		fingerprint_sha256,
 		fingerprint_sha1,
+		issuer,
 		subject,
-		issuer);
+		alt_subject_names);
 
 	return true;
 }
 
+
+std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
+{
+	std::vector<wxString> ret;
+
+	char san[4096];
+	for (unsigned int i = 0; i < 10000; ++i) { // I assume this is a sane limit
+		size_t san_size = sizeof(san) - 1;
+		int const type_or_error = gnutls_x509_crt_get_subject_alt_name(cert, i, san, &san_size, 0);
+		if (type_or_error == GNUTLS_E_SHORT_MEMORY_BUFFER) {
+			continue;
+		}
+		else if (type_or_error < 0) {
+			break;
+		}
+
+		if (type_or_error == GNUTLS_SAN_DNSNAME || type_or_error == GNUTLS_SAN_RFC822NAME) {
+			wxString dns = wxString(san, wxConvUTF8);
+			if (!dns.empty()) {
+				ret.push_back(dns);
+			}
+		}
+		else if (type_or_error == GNUTLS_SAN_IPADDRESS) {
+			wxString ip = CSocket::AddressToString(san, san_size);
+			if (!ip.empty()) {
+				ret.push_back(ip);
+			}
+		}
+	}
+
+	return ret;
+}
+
+bool CTlsSocket::CertificateIsBlacklisted(std::vector<CCertificate> const&)
+{
+	return false;
+}
+
+
+int CTlsSocket::GetAlgorithmWarnings()
+{
+	int algorithmWarnings{};
+
+	switch (gnutls_protocol_get_version(m_session))
+	{
+		case GNUTLS_SSL3:
+		case GNUTLS_VERSION_UNKNOWN:
+			algorithmWarnings |= CCertificateNotification::tlsver;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_cipher_get(m_session)) {
+		case GNUTLS_CIPHER_UNKNOWN:
+		case GNUTLS_CIPHER_NULL:
+		case GNUTLS_CIPHER_ARCFOUR_128:
+		case GNUTLS_CIPHER_3DES_CBC:
+		case GNUTLS_CIPHER_ARCFOUR_40:
+		case GNUTLS_CIPHER_RC2_40_CBC:
+		case GNUTLS_CIPHER_DES_CBC:
+			algorithmWarnings |= CCertificateNotification::cipher;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_mac_get(m_session)) {
+		case GNUTLS_MAC_UNKNOWN:
+		case GNUTLS_MAC_NULL:
+		case GNUTLS_MAC_MD5:
+		case GNUTLS_MAC_MD2:
+		case GNUTLS_MAC_UMAC_96:
+			algorithmWarnings |= CCertificateNotification::mac;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_kx_get(m_session)) {
+		case GNUTLS_KX_UNKNOWN:
+		case GNUTLS_KX_ANON_DH:
+		case GNUTLS_KX_RSA_EXPORT:
+		case GNUTLS_KX_ANON_ECDH:
+			algorithmWarnings |= CCertificateNotification::kex;
+		default:
+			break;
+	}
+
+	return algorithmWarnings;
+}
+
+
 int CTlsSocket::VerifyCertificate()
 {
-	if (m_tlsState != handshake)
-	{
+	if (m_tlsState != TlsState::handshake) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("VerifyCertificate called at wrong time"));
 		return FZ_REPLY_ERROR;
 	}
 
-	m_tlsState = verifycert;
+	m_tlsState = TlsState::verifycert;
 
-	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509)
-	{
+	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509) {
 		m_pOwner->LogMessage(MessageType::Error, _("Unsupported certificate type"));
-		Failure(0, ECONNABORTED);
+		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
 	unsigned int status = 0;
-	if (gnutls_certificate_verify_peers2(m_session, &status) < 0)
-	{
+	int const verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+	if (verifyResult < 0) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_certificate_verify_peers2 returned %d with status %u"), verifyResult, status);
 		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
-		Failure(0, ECONNABORTED);
+		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_REVOKED)
-	{
+	if (status & GNUTLS_CERT_REVOKED) {
 		m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
-		Failure(0, ECONNABORTED);
+		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_SIGNER_NOT_CA)
-	{
+	if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
 		m_pOwner->LogMessage(MessageType::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
-		Failure(0, ECONNABORTED);
-		return FZ_REPLY_ERROR;
-	}
-
-	if (m_require_root_trust && status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-	{
-		m_pOwner->LogMessage(MessageType::Error, _("Root certificate is not trusted"));
-		Failure(0, ECONNABORTED);
+		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
 	unsigned int cert_list_size;
 	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
-	if (!cert_list || !cert_list_size)
-	{
+	if (!cert_list || !cert_list_size) {
 		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
-		Failure(0, ECONNABORTED);
+		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (m_implicitTrustedCert.data)
-	{
+	if (m_implicitTrustedCert.data) {
 		if (m_implicitTrustedCert.size != cert_list[0].size ||
 			memcmp(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size))
 		{
 			m_pOwner->LogMessage(MessageType::Error, _("Primary connection and data connection certificates don't match."));
-			Failure(0, ECONNABORTED);
+			Failure(0, true);
 			return FZ_REPLY_ERROR;
 		}
 
 		TrustCurrentCert(true);
 
-		if (m_tlsState != conn)
+		if (m_tlsState != TlsState::conn)
 			return FZ_REPLY_ERROR;
 		return FZ_REPLY_OK;
 	}
 
+	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
+
 	std::vector<CCertificate> certificates;
-	for (unsigned int i = 0; i < cert_list_size; i++)
-	{
+	for (unsigned int i = 0; i < cert_list_size; ++i) {
 		CCertificate cert;
 		if (ExtractCert(cert_list, cert))
 			certificates.push_back(cert);
-		else
-		{
-			Failure(0, ECONNABORTED);
+		else {
+			Failure(0, true);
 			return FZ_REPLY_ERROR;
 		}
 
 		++cert_list;
 	}
 
+	if (CertificateIsBlacklisted(certificates)) {
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
+	int const algorithmWarnings = GetAlgorithmWarnings();
+
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		m_pOwner->GetCurrentServer()->GetHost(),
 		m_pOwner->GetCurrentServer()->GetPort(),
@@ -1138,16 +1174,15 @@ int CTlsSocket::VerifyCertificate()
 		GetKeyExchange(),
 		GetCipherName(),
 		GetMacName(),
+		algorithmWarnings,
 		certificates);
 
 	m_pOwner->SendAsyncRequest(pNotification);
 
-	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
-
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-void CTlsSocket::OnRateAvailable(enum CRateLimiter::rate_direction)
+void CTlsSocket::OnRateAvailable(CRateLimiter::rate_direction)
 {
 }
 
@@ -1191,25 +1226,6 @@ wxString CTlsSocket::GetMacName()
 		return _T("unknown");
 }
 
-bool CTlsSocket::AddTrustedRootCertificate(const wxString& cert)
-{
-	if (!m_initialized)
-		return false;
-
-	if (cert.empty())
-		return false;
-
-	m_require_root_trust = true;
-
-	const wxWX2MBbuf str = cert.mb_str();
-
-	gnutls_datum_t datum;
-	datum.data = (unsigned char*)(const char*)str;
-	datum.size = strlen(str);
-
-	return gnutls_certificate_set_x509_trust_mem(m_certCredentials, &datum, GNUTLS_X509_FMT_PEM) > 0;
-}
-
 wxString CTlsSocket::ListTlsCiphers(wxString priority)
 {
 	if (priority.empty())
