@@ -9,8 +9,10 @@
 #include "proxy.h"
 #include "servercapabilities.h"
 
+#include <libfilezilla/util.hpp>
+
 CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode)
-: CEventHandler(controlSocket.event_loop_)
+: fz::event_handler(controlSocket.event_loop_)
 , engine_(engine)
 , controlSocket_(controlSocket)
 , m_transferMode(transferMode)
@@ -19,7 +21,7 @@ CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSo
 
 CTransferSocket::~CTransferSocket()
 {
-	RemoveHandler();
+	remove_handler();
 	if (m_transferEndReason == TransferEndReason::none)
 		m_transferEndReason = TransferEndReason::successful;
 	ResetSocket();
@@ -310,7 +312,7 @@ void CTransferSocket::OnReceive()
 			FinalizeWrite();
 		}
 		else {
-			SendEvent<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
+			send_event<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
 		}
 	}
 	else if (m_transferMode == TransferMode::resumetest) {
@@ -411,7 +413,7 @@ void CTransferSocket::OnSend()
 		}
 	}
 	else if (written > 0) {
-		SendEvent<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
+		send_event<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
 	}
 }
 
@@ -524,10 +526,12 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 	wxString bindAddress;
 	if (m_pProxyBackend) {
 		bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+		controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
 	}
 	else {
 		if (controlSocket_.m_pSocket->GetPeerIP(true) == host || controlSocket_.m_pSocket->GetPeerIP(false) == host) {
 			bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+			controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
 		}
 		else {
 			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Destination IP of data connection does not match peer IP of control connection. Not binding source address of data connection."));
@@ -571,7 +575,7 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 
 	ResetSocket();
 
-	engine_.SendEvent<CFileZillaEngineEvent>(engineTransferEnd);
+	engine_.send_event<CFileZillaEngineEvent>(engineTransferEnd);
 }
 
 CSocket* CTransferSocket::CreateSocketServer(int port)
@@ -615,7 +619,7 @@ CSocket* CTransferSocket::CreateSocketServer()
 		low = high;
 
 	if (start < low || start > high) {
-		start = GetRandomNumber(low, high);
+		start = fz::random_number(low, high);
 		wxASSERT(start >= low && start <= high);
 	}
 
@@ -701,6 +705,8 @@ void CTransferSocket::OnIOThreadEvent()
 void CTransferSocket::FinalizeWrite()
 {
 	bool res = ioThread_->Finalize(BUFFERSIZE - m_transferBufferLen);
+	m_transferBufferLen = BUFFERSIZE;
+
 	if (m_transferEndReason != TransferEndReason::none)
 		return;
 
@@ -722,7 +728,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	m_pSocket->SetFlags(m_pSocket->GetFlags() | CSocket::flag_nodelay);
 
 	wxASSERT(!m_pBackend);
-	m_pTlsSocket = new CTlsSocket(this, m_pSocket, &controlSocket_);
+	m_pTlsSocket = new CTlsSocket(this, *m_pSocket, &controlSocket_);
 
 	if (!m_pTlsSocket->Init()) {
 		delete m_pTlsSocket;
@@ -791,9 +797,9 @@ void CTransferSocket::SetSocketBufferSizes(CSocket* pSocket)
 	pSocket->SetBufferSizes(size_read, size_write);
 }
 
-void CTransferSocket::operator()(CEventBase const& ev)
+void CTransferSocket::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent, CIOThreadEvent>(ev, this,
+	fz::dispatch<CSocketEvent, CIOThreadEvent>(ev, this,
 		&CTransferSocket::OnSocketEvent,
 		&CTransferSocket::OnIOThreadEvent);
 }
