@@ -9,9 +9,9 @@
 // For example the event table entry could look like this:
 //     EVT_FZ_NOTIFICATION(wxID_ANY, CMainFrame::OnEngineEvent)
 // and the handler function has the following declaration:
-//     void OnEngineEvent(wxEvent& event);
-// You can get the engine which sent the event by calling
-//     event.GetEventObject()
+//     void OnEngineEvent(wxFzEvent& event);
+// You can get the engine which sent the event by reading
+//     event.engine_
 // Whenever you get a notification event,
 // CFileZillaEngine::GetNextNotification has to be called until it returns 0,
 // or you will lose important notifications or your memory will fill with
@@ -22,23 +22,27 @@
 // CFileZillaEngine::SetAsyncRequestReply to continue the current operation.
 
 #include "local_path.h"
-#include "timeex.h"
-
-extern const wxEventType fzEVT_NOTIFICATION;
-#define EVT_FZ_NOTIFICATION(id, fn) \
-	DECLARE_EVENT_TABLE_ENTRY(      \
-		fzEVT_NOTIFICATION, id, -1, \
-		(wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( wxEventFunction, &fn ), \
-		(wxObject *) NULL           \
-	),
+#include <libfilezilla/time.hpp>
 
+class CFileZillaEngine;
 class wxFzEvent final : public wxEvent
 {
 public:
-	wxFzEvent(int id = wxID_ANY);
+	wxFzEvent();
+
+	explicit wxFzEvent(int id);
+	explicit wxFzEvent(CFileZillaEngine const* engine);
+
 	virtual wxEvent *Clone() const;
+
+	CFileZillaEngine const* engine_;
 };
 
+wxDECLARE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
+
+#define EVT_FZ_NOTIFICATION(id, func) \
+	wx__DECLARE_EVT1(fzEVT_NOTIFICATION, id, &func)
+
 enum NotificationId
 {
 	nId_logmsg,				// notification about new messages for the message log
@@ -60,7 +64,7 @@ enum RequestId
 	reqId_interactiveLogin, // gives a challenge prompt for a password
 	reqId_hostkey,			// used only by SSH/SFTP to indicate new host key
 	reqId_hostkeyChanged,	// used only by SSH/SFTP to indicate changed host key
-	reqId_certificate		// sent after a successful TLS/SSL handshake to allow certificate
+	reqId_certificate		// sent after a successful TLS handshake to allow certificate
 							// validation
 };
 
@@ -153,20 +157,20 @@ public:
 	bool download{};
 
 	wxString localFile;
-	wxLongLong localSize{-1};
-	CDateTime localTime;
+	int64_t localSize{-1};
+	fz::datetime localTime;
 
 	wxString remoteFile;
 	CServerPath remotePath;
-	wxLongLong remoteSize{-1};
-	CDateTime remoteTime;
+	int64_t remoteSize{-1};
+	fz::datetime remoteTime;
 
 	bool ascii{};
 
 	bool canResume{};
 
 	// overwriteAction will be set by the request handler
-	enum OverwriteAction
+	enum OverwriteAction : signed char
 	{
 		unknown = -1,
 		ask,
@@ -192,7 +196,12 @@ public:
 class CInteractiveLoginNotification final : public CAsyncRequestNotification
 {
 public:
-	CInteractiveLoginNotification(const wxString& challenge);
+	enum type {
+		interactive,
+		keyfile
+	};
+
+	CInteractiveLoginNotification(type t, const wxString& challenge, bool repeated);
 	virtual enum RequestId GetRequestID() const;
 
 	// Set to true if you have set a password
@@ -203,9 +212,17 @@ public:
 
 	const wxString& GetChallenge() const { return m_challenge; }
 
+	type GetType() const { return m_type; }
+
+	bool IsRepeated() const { return m_repeated; }
+
 protected:
 	// Password prompt string as given by the server
-	const wxString m_challenge;
+	wxString const m_challenge;
+
+	type const m_type;
+
+	bool const m_repeated;
 };
 
 // Indicate network action.
@@ -222,11 +239,24 @@ protected:
 class CTransferStatus final
 {
 public:
-	wxDateTime started;
+	CTransferStatus() {}
+	CTransferStatus(wxFileOffset total, wxFileOffset start, bool l)
+		: totalSize(total)
+		, startOffset(start)
+		, currentOffset(start)
+		, list(l)
+	{}
+
+	fz::datetime started;
 	wxFileOffset totalSize{-1};		// Total size of the file to transfer, -1 if unknown
 	wxFileOffset startOffset{-1};
 	wxFileOffset currentOffset{-1};
 
+	void clear() { startOffset = -1; }
+	bool empty() const { return startOffset < 0; }
+
+	explicit operator bool() const { return !empty(); }
+
 	// True on download notifications iff currentOffset != startOffset.
 	// True on FTP upload notifications iff currentOffset != startOffset
 	// AND after the first accepted data after the first wxSOCKET_WOULDBLOCK.
@@ -239,13 +269,13 @@ public:
 class CTransferStatusNotification final : public CNotificationHelper<nId_transferstatus>
 {
 public:
-	CTransferStatusNotification(CTransferStatus *pStatus);
-	virtual ~CTransferStatusNotification();
+	CTransferStatusNotification() {}
+	CTransferStatusNotification(CTransferStatus const& status);
 
-	const CTransferStatus *GetStatus() const;
+	CTransferStatus const& GetStatus() const;
 
 protected:
-	CTransferStatus *m_pStatus;
+	CTransferStatus const status_;
 };
 
 // Notification about new or changed hostkeys, only used by SSH/SFTP transfers.
@@ -293,21 +323,23 @@ class CCertificate final
 public:
 	CCertificate() = default;
 	CCertificate(
-		const unsigned char* rawData, unsigned int len,
-		wxDateTime activationTime, wxDateTime expirationTime,
-		const wxString& serial,
-		const wxString& pkalgoname, unsigned int bits,
-		const wxString& signalgoname,
-		const wxString& fingerprint_md5,
-		const wxString& fingerprint_sha1,
-		const wxString& subject,
-		const wxString& issuer);
-	CCertificate(const CCertificate& op);
+		unsigned char const* rawData, unsigned int len,
+		fz::datetime const& activationTime, fz::datetime const& expirationTime,
+		wxString const& serial,
+		wxString const& pkalgoname, unsigned int bits,
+		wxString const& signalgoname,
+		wxString const& fingerprint_sha256,
+		wxString const& fingerprint_sha1,
+		wxString const& issuer,
+		wxString const& subject,
+		std::vector<wxString> const& altSubjectNames);
+
+	CCertificate(CCertificate const& op);
 	~CCertificate();
 
 	const unsigned char* GetRawData(unsigned int& len) const { len = m_len; return m_rawData; }
-	wxDateTime GetActivationTime() const { return m_activationTime; }
-	wxDateTime GetExpirationTime() const { return m_expirationTime; }
+	fz::datetime GetActivationTime() const { return m_activationTime; }
+	fz::datetime GetExpirationTime() const { return m_expirationTime; }
 
 	const wxString& GetSerial() const { return m_serial; }
 	const wxString& GetPkAlgoName() const { return m_pkalgoname; }
@@ -315,17 +347,19 @@ public:
 
 	const wxString& GetSignatureAlgorithm() const { return m_signalgoname; }
 
-	const wxString& GetFingerPrintMD5() const { return m_fingerprint_md5; }
+	const wxString& GetFingerPrintSHA256() const { return m_fingerprint_sha256; }
 	const wxString& GetFingerPrintSHA1() const { return m_fingerprint_sha1; }
 
 	const wxString& GetSubject() const { return m_subject; }
 	const wxString& GetIssuer() const { return m_issuer; }
 
-	CCertificate& operator=(const CCertificate &op);
+	CCertificate& operator=(CCertificate const& op);
+
+	std::vector<wxString> const& GetAltSubjectNames() const { return m_altSubjectNames; }
 
 private:
-	wxDateTime m_activationTime;
-	wxDateTime m_expirationTime;
+	fz::datetime m_activationTime;
+	fz::datetime m_expirationTime;
 
 	unsigned char* m_rawData{};
 	unsigned int m_len{};
@@ -336,11 +370,13 @@ private:
 
 	wxString m_signalgoname;
 
-	wxString m_fingerprint_md5;
+	wxString m_fingerprint_sha256;
 	wxString m_fingerprint_sha1;
 
-	wxString m_subject;
 	wxString m_issuer;
+	wxString m_subject;
+
+	std::vector<wxString> m_altSubjectNames;
 };
 
 class CCertificateNotification final : public CAsyncRequestNotification
@@ -351,6 +387,7 @@ public:
 		const wxString& keyExchange,
 		const wxString& sessionCipher,
 		const wxString& sessionMac,
+		int algorithmWarnings,
 		const std::vector<CCertificate> &certificates);
 	virtual enum RequestId GetRequestID() const { return reqId_certificate; }
 
@@ -367,7 +404,17 @@ public:
 	const wxString& GetProtocol() const { return m_protocol; }
 	const wxString& GetKeyExchange() const { return m_keyExchange; }
 
-protected:
+	enum algorithm_warnings_t
+	{
+		tlsver = 1,
+		cipher = 2,
+		mac = 4,
+		kex = 8
+	};
+
+	int GetAlgorithmWarnings() const { return m_algorithmWarnings; }
+
+private:
 	wxString m_host;
 	unsigned int m_port{};
 
@@ -375,6 +422,7 @@ protected:
 	wxString m_keyExchange;
 	wxString m_sessionCipher;
 	wxString m_sessionMac;
+	int m_algorithmWarnings{};
 
 	std::vector<CCertificate> m_certificates;
 };
@@ -385,6 +433,7 @@ public:
 	wxString hostKey;
 	wxString kexAlgorithm;
 	wxString kexHash;
+	wxString kexCurve;
 	wxString cipherClientToServer;
 	wxString cipherServerToClient;
 	wxString macClientToServer;
