@@ -14,6 +14,7 @@ enum engineOptions
 	OPTION_LIMITPORTS,
 	OPTION_LIMITPORTS_LOW,
 	OPTION_LIMITPORTS_HIGH,
+	OPTION_LIMITPORTS_OFFSET,
 	OPTION_EXTERNALIPMODE,		/* External IP Address mode for use in active mode
 								   Values: 0: ask operating system
 										   1: use provided IP
@@ -41,6 +42,8 @@ enum engineOptions
 	OPTION_SPEEDLIMIT_OUTBOUND,
 	OPTION_SPEEDLIMIT_BURSTTOLERANCE,
 
+	OPTION_PREALLOCATE_SPACE,
+
 	OPTION_VIEW_HIDDEN_FILES,
 
 	OPTION_PRESERVE_TIMESTAMPS,
@@ -57,6 +60,7 @@ enum engineOptions
 	OPTION_FTP_PROXY_CUSTOMLOGINSEQUENCE,
 
 	OPTION_SFTP_KEYFILES,
+	OPTION_SFTP_COMPRESSION,
 
 	OPTION_PROXY_TYPE,
 	OPTION_PROXY_HOST,
@@ -66,8 +70,7 @@ enum engineOptions
 
 	OPTION_LOGGING_FILE,
 	OPTION_LOGGING_FILE_SIZELIMIT,
-
-	OPTION_INTERNAL_ROOTCERT,
+	OPTION_LOGGING_SHOW_DETAILED_LOGS,
 
 	OPTION_SIZE_FORMAT,
 	OPTION_SIZE_USETHOUSANDSEP,
