@@ -1,101 +1,53 @@
 #ifndef __SOCKET_H__
 #define __SOCKET_H__
 
+#include <libfilezilla/event_handler.hpp>
+
+#include <errno.h>
+
 // IPv6 capable, non-blocking socket class for use with wxWidgets.
 // Error codes are the same as used by the POSIX socket functions,
 // see 'man 2 socket', 'man 2 connect', ...
 
-class CSocketEventHandler;
-class CSocketEventSource;
-class CSocketEvent final
+enum class SocketEventType
 {
-public:
-	enum EventType
-	{
-		hostaddress,
+	// This is a nonfatal condition. It
+	// means there are additional addresses to try.
+	connection_next,
 
-		// This is a nonfatal condition. It
-		// means there are additional addresses to try.
-		connection_next,
-		connection,
-		read,
-		write,
-		close
-	};
-
-	CSocketEvent(CSocketEventHandler* pSocketEventHandler, CSocketEventSource* pSource, enum EventType type, const wxChar* data);
-	CSocketEvent(CSocketEventHandler* pSocketEventHandler, CSocketEventSource* pSource, enum EventType type, int error = 0);
-	~CSocketEvent();
-
-	CSocketEventSource* GetSocketEventSource() const { return m_pSource; }
-	enum EventType GetType() const { return m_type; }
-	CSocketEventHandler* GetSocketEventHandler() const { return m_pSocketEventHandler; }
-
-	wxString GetData() const;
-	int GetError() const { return m_error; }
-
-protected:
-	CSocketEventSource* m_pSource;
-	const enum EventType m_type;
-	wxChar *m_data;
-	int m_error;
-	CSocketEventHandler* m_pSocketEventHandler;
-
-	friend class CSocketEventDispatcher;
-};
-
-class CSocketEventDispatcher final : protected wxEvtHandler
-{
-public:
-	void SendEvent(CSocketEvent* evt);
-	void RemovePending(const CSocketEventHandler* pHandler);
-	void RemovePending(const CSocketEventSource* pSource);
-	void UpdatePending(const CSocketEventHandler* pOldHandler, const CSocketEventSource* pOldSource, CSocketEventHandler* pNewHandler, CSocketEventSource* pNewSource);
-
-	static CSocketEventDispatcher& Get();
-
-private:
-	CSocketEventDispatcher();
-	~CSocketEventDispatcher();
-
-	virtual bool ProcessEvent(wxEvent& event);
-
-	std::list<CSocketEvent*> m_pending_events;
-
-	wxCriticalSection m_sync;
-
-	static CSocketEventDispatcher m_dispatcher;
-
-	bool m_inside_loop{};
-};
-
-class CSocketEventHandler
-{
-public:
-	CSocketEventHandler() {};
-	virtual ~CSocketEventHandler();
-
-	virtual void OnSocketEvent(CSocketEvent& event) = 0;
+	connection,
+	read,
+	write,
+	close
 };
 
 class CSocketEventSource
 {
 public:
-	virtual ~CSocketEventSource();
+	virtual ~CSocketEventSource() {}
 };
 
+struct socket_event_type;
+typedef fz::simple_event<socket_event_type, CSocketEventSource*, SocketEventType, int> CSocketEvent;
+
+struct hostaddress_event_type;
+typedef fz::simple_event<hostaddress_event_type, CSocketEventSource*, wxString> CHostAddressEvent;
+
 class CCallback
 {
 public:
 	virtual void cb() {}
 };
 
+void RemoveSocketEvents(fz::event_handler * handler, CSocketEventSource const* const source);
+void ChangeSocketEventHandler(fz::event_handler * oldHandler, fz::event_handler * newHandler, CSocketEventSource const* const source);
+
 class CSocketThread;
-class CSocket : public CSocketEventSource
+class CSocket final : public CSocketEventSource
 {
 	friend class CSocketThread;
 public:
-	CSocket(CSocketEventHandler* pEvtHandler);
+	CSocket(fz::event_handler* pEvtHandler);
 	virtual ~CSocket();
 
 	CSocket(CSocket const&) = delete;
@@ -118,7 +70,7 @@ public:
 		closing,
 		closed
 	};
-	enum SocketState GetState();
+	SocketState GetState();
 
 	enum address_family
 	{
@@ -134,7 +86,7 @@ public:
 	// If host is a name that can be resolved, a hostaddress socket event gets sent.
 	// Once connections got established, a connection event gets sent. If
 	// connection could not be established, a close event gets sent.
-	int Connect(wxString host, unsigned int port, enum address_family family = unspec);
+	int Connect(wxString const& host, unsigned int port, address_family family = unspec, wxString const& bind = wxString());
 
 	// After receiving a send or receive event, you can call these functions
 	// as long as their return value is positive.
@@ -156,20 +108,21 @@ public:
 	int GetRemotePort(int& error);
 
 	// If connected, either ipv4 or ipv6, unspec otherwise
-	enum address_family GetAddressFamily() const;
+	address_family GetAddressFamily() const;
 
 	static wxString GetErrorString(int error);
 	static wxString GetErrorDescription(int error);
 
 	// Can only be called if the state is none
-	void SetEventHandler(CSocketEventHandler* pEvtHandler);
-	CSocketEventHandler* GetEventHandler() { return m_pEvtHandler; }
+	void SetEventHandler(fz::event_handler* pEvtHandler);
+	fz::event_handler* GetEventHandler() { return m_pEvtHandler; }
 
-	static bool Cleanup(bool force);
+	static void Cleanup(bool force);
 
 	static wxString AddressToString(const struct sockaddr* addr, int addr_len, bool with_port = true, bool strip_zone_index = false);
+	static wxString AddressToString(char const* buf, int buf_len);
 
-	int Listen(enum address_family family, int port = 0);
+	int Listen(address_family family, int port = 0);
 	CSocket* Accept(int& error);
 
 	enum Flags
@@ -183,7 +136,7 @@ public:
 
 	// If called on listen socket, sizes will be inherited by
 	// accepted sockets
-	void SetBufferSizes(int size_read, int size_write);
+	int SetBufferSizes(int size_read, int size_write);
 
 	void SetSynchronousReadCallback(CCallback* cb);
 
@@ -194,11 +147,11 @@ protected:
 
 	void DetachThread();
 
-	CSocketEventHandler* m_pEvtHandler;
+	fz::event_handler* m_pEvtHandler;
 
 	int m_fd;
 
-	enum SocketState m_state;
+	SocketState m_state;
 
 	CSocketThread* m_pSocketThread;
 
@@ -213,7 +166,7 @@ protected:
 	CCallback* m_synchronous_read_cb{};
 };
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 
 #ifndef EISCONN
 #define EISCONN WSAEISCONN
@@ -272,9 +225,12 @@ protected:
 #ifndef ECONNRESET
 #define ECONNRESET WSAECONNRESET
 #endif
+#ifndef EHOSTDOWN
+#define EHOSTDOWN WSAEHOSTDOWN
+#endif
 
 // For the future:
 // Handle ERROR_NETNAME_DELETED=64
-#endif //__WXMSW__
+#endif //FZ_WINDOWS
 
 #endif //__SOCKET_H__
