@@ -7,14 +7,12 @@
 #include "Options.h"
 #include "wrapengine.h"
 #include "buildinfo.h"
-#ifdef __WXMSW__
-#include <wx/msw/registry.h> // Needed by CheckForWin2003FirewallBug
-#endif
 #include <wx/tokenzr.h>
 #include "cmdline.h"
 #include "welcome_dialog.h"
 #include <msgbox.h>
-#include "local_filesys.h"
+
+#include <libfilezilla/local_filesys.hpp>
 
 #include <wx/xrc/xh_animatctrl.h>
 #include <wx/xrc/xh_bmpbt.h>
@@ -41,9 +39,7 @@
 #include <wx/xrc/xh_tree.h>
 #include <wx/xrc/xh_hyperlink.h>
 #include "xh_toolb_ex.h"
-#include "xh_menu_ex.h"
 #ifdef __WXMSW__
-#include <wx/socket.h>
 #include <wx/dynlib.h>
 #endif
 #ifdef WITH_LIBDBUS
@@ -61,10 +57,6 @@
 	#include "prefix.h"
 #endif
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 #ifndef __WXGTK__
 IMPLEMENT_APP(CFileZillaApp)
 #else
@@ -73,86 +65,20 @@ IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 
 CFileZillaApp::CFileZillaApp()
 {
-	m_pWrapEngine = 0;
-	m_pLocale = 0;
-	m_pCommandLine = 0;
 	m_profilingActive = true;
 	AddStartupProfileRecord(_T("CFileZillaApp::CFileZillaApp()"));
 }
 
 CFileZillaApp::~CFileZillaApp()
 {
-	delete m_pLocale;
-	delete m_pWrapEngine;
-	delete m_pCommandLine;
 	COptions::Destroy();
 }
 
 #ifdef __WXMSW__
-bool IsServiceRunning(const wxString& serviceName)
-{
-	SC_HANDLE hScm = OpenSCManager(0, 0, GENERIC_READ);
-	if (!hScm)
-	{
-		//wxMessageBoxEx(_T("OpenSCManager failed"));
-		return false;
-	}
-
-	SC_HANDLE hService = OpenService(hScm, serviceName, GENERIC_READ);
-	if (!hService)
-	{
-		CloseServiceHandle(hScm);
-		//wxMessageBoxEx(_T("OpenService failed"));
-		return false;
-	}
-
-	SERVICE_STATUS status;
-	if (!ControlService(hService, SERVICE_CONTROL_INTERROGATE, &status))
-	{
-		CloseServiceHandle(hService);
-		CloseServiceHandle(hScm);
-		//wxMessageBoxEx(_T("ControlService failed"));
-		return false;
-	}
-
-	CloseServiceHandle(hService);
-	CloseServiceHandle(hScm);
-
-	if (status.dwCurrentState == 0x07 || status.dwCurrentState == 0x01)
-		return false;
-
-	return true;
-}
-
-bool CheckForWin2003FirewallBug()
-{
-	const wxString os = ::wxGetOsDescription();
-	if (os.Find(_T("Windows Server 2003")) == -1)
-		return false;
-
-	if (!IsServiceRunning(_T("SharedAccess")))
-		return false;
-
-	if (!IsServiceRunning(_T("ALG")))
-		return false;
-
-	wxRegKey key(_T("HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\SharedAccess\\Parameters\\FirewallPolicy\\StandardProfile"));
-	if (!key.Exists() || !key.Open(wxRegKey::Read))
-		return false;
-
-	long value = 0;
-	if (!key.HasValue(_T("EnableFirewall")) || !key.QueryValue(_T("EnableFirewall"), &value))
-		return false;
-
-	if (!value)
-		return false;
-
-	return true;
-}
-
+namespace {
 extern "C"
 {
-	typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
+	typedef HRESULT(WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
 }
 
 static void SetAppId()
@@ -173,23 +99,40 @@ static void SetAppId()
 	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
 }
 
+static bool InitWinsock()
+{
+	WSADATA d{};
+	int res = WSAStartup((2 << 8) | 8, &d);
+	if (res != 0) {
+		int err = WSAGetLastError();
+		wxString msg = wxString::Format(_("Could not initialize Winsock (%d): %s"), err, wxSysErrorMsg(err));
+		wxMessageBoxEx(msg, _("Failed to initialize networking"), wxICON_EXCLAMATION);
+		return false;
+	}
+
+	return true;
+}
+
+static void UninitWinsock()
+{
+	WSACleanup();
+}
+}
+
 #endif //__WXMSW__
 
 void CFileZillaApp::InitLocale()
 {
 	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
 	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
-	if (!language.empty())
-	{
+	if (!language.empty()) {
 #ifdef __WXGTK__
-		if (CInitializer::error)
-		{
+		if (CInitializer::error) {
 			wxString error;
 
 			wxLocale *loc = wxGetLocale();
 			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
-			if (!loc || !currentInfo)
-			{
+			if (!loc || !currentInfo) {
 				if (!pInfo)
 					error.Printf(_("Failed to set language to %s, using default system language."),
 						language);
@@ -197,8 +140,7 @@ void CFileZillaApp::InitLocale()
 					error.Printf(_("Failed to set language to %s (%s), using default system language."),
 						pInfo->Description, language);
 			}
-			else
-			{
+			else {
 				wxString currentName = currentInfo->CanonicalName;
 
 				if (!pInfo)
@@ -222,9 +164,11 @@ void CFileZillaApp::InitLocale()
 			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
 				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
 				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
+					COptions::Get()->SetOption(OPTION_LANGUAGE, language);
 					return;
 				}
 			}
+			COptions::Get()->SetOption(OPTION_LANGUAGE, wxString());
 			if (pInfo && !pInfo->Description.empty())
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
 			else
@@ -240,13 +184,10 @@ bool CFileZillaApp::OnInit()
 
 	srand( (unsigned)time( NULL ) );
 
-#if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
-	//wxHandleFatalExceptions();
-#endif
-
 #ifdef __WXMSW__
-	// Need to call WSAStartup. Let wx do that for us
-	wxSocketBase::Initialize();
+	if (!InitWinsock()) {
+		return false;
+	}
 
 	SetAppId();
 #endif
@@ -273,9 +214,10 @@ bool CFileZillaApp::OnInit()
 
 	LoadLocales();
 
-	if (cmdline_result < 0)
-	{
-		m_pCommandLine->DisplayUsage();
+	if (cmdline_result < 0) {
+		if (m_pCommandLine) {
+			m_pCommandLine->DisplayUsage();
+		}
 		return false;
 	}
 
@@ -305,30 +247,13 @@ USE AT OWN RISK"), _T("Important Information"));
 	}
 #endif
 
-	if (!LoadResourceFiles())
-	{
+	if (!LoadResourceFiles()) {
 		COptions::Destroy();
 		return false;
 	}
 
 	CheckExistsFzsftp();
 
-#ifdef __WXMSW__
-	if (CheckForWin2003FirewallBug())
-	{
-		const wxString& error = _("Warning!\n\nA bug in Windows causes problems with FileZilla\n\n\
-The bug occurs if you have\n\
-- Windows Server 2003 or XP 64\n\
-- Windows Firewall enabled\n\
-- Application Layer Gateway service enabled\n\
-See http://support.microsoft.com/kb/931130 for background information.\n\n\
-Unless you either disable Windows Firewall or the Application Layer Gateway service,\n\
-FileZilla will timeout on big transfers.\
-");
-		wxMessageBoxEx(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
-	}
-#endif
-
 	// Turn off idle events, we don't need them
 	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
 
@@ -339,7 +264,7 @@ FileZilla will timeout on big transfers.\
 #endif
 
 	// Load the text wrapping engine
-	m_pWrapEngine = new CWrapEngine();
+	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
 	CMainFrame *frame = new CMainFrame();
@@ -365,7 +290,7 @@ int CFileZillaApp::OnExit()
 	CSessionManager::Uninit();
 #endif
 #ifdef __WXMSW__
-	wxSocketBase::Shutdown();
+	UninitWinsock();
 #endif
 	return wxApp::OnExit();
 }
@@ -387,8 +312,7 @@ bool CFileZillaApp::FileExists(const wxString& file) const
 
 	wxString subDir;
 	bool found = dir.GetFirst(&subDir, _T(""), wxDIR_DIRS);
-	while (found)
-	{
+	while (found) {
 		if (FileExists(file.Left(pos) + subDir + file.Mid(pos + 1)))
 			return true;
 
@@ -431,8 +355,7 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 #ifdef ENABLE_BINRELOC
 	const char* path = SELFPATH;
-	if (path && *path)
-	{
+	if (path && *path) {
 		wxString datadir(SELFPATH , *wxConvCurrent);
 		wxFileName fn(datadir);
 		datadir = fn.GetPath();
@@ -441,8 +364,7 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 	}
 	path = DATADIR;
-	if (path && *path)
-	{
+	if (path && *path) {
 		wxString datadir(DATADIR, *wxConvCurrent);
 		if (!datadir.empty())
 			pathList.Add(datadir);
@@ -450,8 +372,7 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 #elif defined __WXMSW__
 	wxChar path[1024];
 	int res = GetModuleFileName(0, path, 1000);
-	if (res > 0 && res < 1000)
-	{
+	if (res > 0 && res < 1000) {
 		wxFileName fn(path);
 		pathList.Add(fn.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
 	}
@@ -518,7 +439,7 @@ bool CFileZillaApp::LoadResourceFiles()
 #endif
 
 	pResource->AddHandler(new wxMenuXmlHandler);
-	pResource->AddHandler(new wxMenuBarXmlHandlerEx);
+	pResource->AddHandler(new wxMenuBarXmlHandler);
 	pResource->AddHandler(new wxDialogXmlHandler);
 	pResource->AddHandler(new wxPanelXmlHandler);
 	pResource->AddHandler(new wxSizerXmlHandler);
@@ -545,17 +466,17 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->AddHandler(new wxAnimationCtrlXmlHandler);
 	pResource->AddHandler(new wxStdDialogButtonSizerXmlHandler);
 
-	if (CLocalFileSystem::GetFileType(m_resourceDir.GetPath() + _T("xrc/resources.xrc")) == CLocalFileSystem::file) {
+	if (fz::local_filesys::get_file_type(fz::to_native(m_resourceDir.GetPath() + _T("xrc/resources.xrc"))) == fz::local_filesys::file) {
 		pResource->LoadFile(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
 	}
 	else {
-		CLocalFileSystem fs;
+		fz::local_filesys fs;
 		wxString dir = m_resourceDir.GetPath() + _T("xrc/");
-		bool found = fs.BeginFindFiles(dir, false);
+		bool found = fs.begin_find_files(fz::to_native(dir), false);
 		while (found) {
-			wxString name;
-			found = fs.GetNextFile(name);
-			if (name.Right(4) != _T(".xrc")) {
+			fz::native_string name;
+			found = fs.get_next_file(name);
+			if (name.size() <= 4 || name.substr(name.size() - 4) != fzT(".xrc")) {
 				continue;
 			}
 			pResource->LoadFile(dir + name);
@@ -619,30 +540,25 @@ bool CFileZillaApp::LoadLocales()
 bool CFileZillaApp::SetLocale(int language)
 {
 	// First check if we can load the new locale
-	wxLocale* pLocale = new wxLocale();
+	auto pLocale = std::make_unique<wxLocale>();
 	wxLogNull log;
 	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla")))
-	{
-		delete pLocale;
+	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
 		return false;
 	}
 
 	// Now unload old locale
 	// We unload new locale as well, else the internal locale chain in wxWidgets get's broken.
-	delete pLocale;
-	delete m_pLocale;
-	m_pLocale = 0;
+	pLocale.reset();
+	m_pLocale.reset();
 
 	// Finally load new one
-	pLocale = new wxLocale();
+	pLocale = std::make_unique<wxLocale>();
 	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla")))
-	{
-		delete pLocale;
+	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
 		return false;
 	}
-	m_pLocale = pLocale;
+	m_pLocale = std::move(pLocale);
 
 	return true;
 }
@@ -682,7 +598,7 @@ void CFileZillaApp::DisplayEncodingWarning()
 
 CWrapEngine* CFileZillaApp::GetWrapEngine()
 {
-	return m_pWrapEngine;
+	return m_pWrapEngine.get();
 }
 
 void CFileZillaApp::CheckExistsFzsftp()
@@ -714,14 +630,12 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 	// First check the FZ_FZSFTP environment variable
 	wxString executable;
-	if (wxGetEnv(_T("FZ_FZSFTP"), &executable))
-	{
+	if (wxGetEnv(_T("FZ_FZSFTP"), &executable)) {
 		if (wxFileName::FileExists(executable))
 			found = true;
 	}
 
-	if (!found)
-	{
+	if (!found) {
 		wxPathList pathList;
 
 		// Add current working directory
@@ -732,13 +646,11 @@ void CFileZillaApp::CheckExistsFzsftp()
 		// Add executable path
 		wxChar modulePath[1000];
 		DWORD len = GetModuleFileName(0, modulePath, 999);
-		if (len)
-		{
+		if (len) {
 			modulePath[len] = 0;
 			wxString path(modulePath);
 			int pos = path.Find('\\', true);
-			if (pos != -1)
-			{
+			if (pos != -1) {
 				path = path.Left(pos);
 				pathList.Add(path);
 			}
@@ -755,29 +667,25 @@ void CFileZillaApp::CheckExistsFzsftp()
 			found = true;
 	}
 
-	if (!found)
-	{
 #ifdef __UNIX__
+	if (!found) {
 		const wxString prefix = ((const wxStandardPaths&)wxStandardPaths::Get()).GetInstallPrefix();
-		if (prefix != _T("/usr/local"))
-		{
+		if (prefix != _T("/usr/local")) {
 			// /usr/local is the fallback value. /usr/local/bin is most likely in the PATH
 			// environment variable already so we don't have to check it. Furthermore, other
 			// directories might be listed before it (For example a developer's own
 			// application prefix)
 			wxFileName fn(prefix + _T("/bin/"), program);
 			fn.Normalize();
-			if (fn.FileExists())
-			{
+			if (fn.FileExists()) {
 				executable = fn.GetFullPath();
 				found = true;
 			}
 		}
-#endif
 	}
+#endif
 
-	if (!found)
-	{
+	if (!found) {
 		// Check PATH
 		wxPathList pathList;
 		pathList.AddEnvList(_T("PATH"));
@@ -786,22 +694,17 @@ void CFileZillaApp::CheckExistsFzsftp()
 			found = true;
 	}
 
-	// Quote path if it contains spaces
-	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
-		executable = _T("\"") + executable + _T("\"");
+	if (!found) {
+		// Quote path if it contains spaces
+		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+			executable = _T("\"") + executable + _T("\"");
 
-	if (!found)
-	{
 		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program, program, program),
 			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
 #endif
 
-	// Quote path if it contains spaces
-	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
-		executable = _T("\"") + executable + _T("\"");
-
 	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
 }
 
@@ -809,8 +712,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 {
 	HWND child = FindWindowEx(hwnd, 0, 0, _T("FileZilla process identificator 3919DB0A-082D-4560-8E2F-381A35969FB4"));
-	if (child)
-	{
+	if (child) {
 		::PostMessage(hwnd, WM_ENDSESSION, (WPARAM)TRUE, (LPARAM)ENDSESSION_LOGOFF);
 	}
 
@@ -821,21 +723,18 @@ extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 int CFileZillaApp::ProcessCommandLine()
 {
 	AddStartupProfileRecord(_T("CFileZillaApp::ProcessCommandLine"));
-	m_pCommandLine = new CCommandLine(argc, argv);
+	m_pCommandLine = std::make_unique<CCommandLine>(argc, argv);
 	int res = m_pCommandLine->Parse() ? 1 : -1;
 
-	if (res > 0)
-	{
-		if (m_pCommandLine->HasSwitch(CCommandLine::close))
-		{
+	if (res > 0) {
+		if (m_pCommandLine->HasSwitch(CCommandLine::close)) {
 #ifdef __WXMSW__
 			EnumWindows((WNDENUMPROC)EnumWindowCallback, 0);
 #endif
 			return 0;
 		}
 
-		if (m_pCommandLine->HasSwitch(CCommandLine::version))
-		{
+		if (m_pCommandLine->HasSwitch(CCommandLine::version)) {
 			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
 			if (!CBuildInfo::GetBuildType().empty())
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
@@ -849,49 +748,6 @@ int CFileZillaApp::ProcessCommandLine()
 	return res;
 }
 
-#if USE_CHUNKED_PROCESS_PENDING_EVENTS
-void CFileZillaApp::ProcessPendingEvents()
-{
-	// Code copied from wxAppConsole::ProcessPendingEvents, adapted to
-	// process only 100 events in one go.
-#if wxUSE_THREADS
-	if ( !wxPendingEventsLocker )
-		return;
-#endif
-
-	// ensure that we're the only thread to modify the pending events list
-	wxENTER_CRIT_SECT( *wxPendingEventsLocker );
-
-	if ( !wxPendingEvents )
-	{
-		wxLEAVE_CRIT_SECT( *wxPendingEventsLocker );
-		return;
-	}
-
-	// iterate until the list becomes empty or 100 events have been
-	// processed, whichever comes first
-	wxList::compatibility_iterator node = wxPendingEvents->GetFirst();
-	int idleCounter = 100;
-	while (node)
-	{
-		wxEvtHandler *handler = (wxEvtHandler *)node->GetData();
-		wxPendingEvents->Erase(node);
-
-		// In ProcessPendingEvents(), new handlers might be add
-		// and we can safely leave the critical section here.
-		wxLEAVE_CRIT_SECT( *wxPendingEventsLocker );
-
-		handler->ProcessPendingEvents();
-
-		wxENTER_CRIT_SECT( *wxPendingEventsLocker );
-
-		node = --idleCounter ? wxPendingEvents->GetFirst() : 0;
-	}
-
-	wxLEAVE_CRIT_SECT( *wxPendingEventsLocker );
-}
-#endif
-
 void CFileZillaApp::AddStartupProfileRecord(const wxString& msg)
 {
 	if (!m_profilingActive)
@@ -911,11 +767,10 @@ void CFileZillaApp::ShowStartupProfile()
 		return;
 
 	wxString msg = _T("Profile:\n");
-	for (std::list<std::pair<wxDateTime, wxString> >::const_iterator it = profile.begin(); it != profile.end(); ++it)
-	{
-		msg += it->first.Format(_T("%Y-%m-%d %H:%M:%S %l"));
+	for (auto const& p : profile) {
+		msg += p.first.Format(_T("%Y-%m-%d %H:%M:%S %l"));
 		msg += _T(" ");
-		msg += it->second;
+		msg += p.second;
 		msg += _T("\n");
 	}
 
