@@ -19,11 +19,13 @@
 #include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include "local_filesys.h"
 #include "filelist_statusbar.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
 
+#include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/recursive_remove.hpp>
+
 class CLocalListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
@@ -304,6 +306,7 @@ CLocalListView::~CLocalListView()
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
+	
 	CancelLabelEdit();
 
 	wxString focused;
@@ -371,9 +374,9 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 regular_dir:
 #endif
 		CFilterManager filter;
-		CLocalFileSystem local_filesystem;
+		fz::local_filesys local_filesys;
 
-		if (!local_filesystem.BeginFindFiles(m_dir.GetPath(), false)) {
+		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
 			SetItemCount(1);
 			return false;
 		}
@@ -387,12 +390,14 @@ regular_dir:
 		int num = m_fileData.size();
 		CLocalFileData data;
 		bool wasLink;
-		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
-			if (data.name.empty()) {
+		fz::native_string name;
+		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+			if (name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
+			data.name = name;
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
@@ -927,7 +932,7 @@ wxString CLocalListView::MenuMkdir()
 
 void CLocalListView::OnMenuDelete(wxCommandEvent&)
 {
-	std::list<wxString> pathsToDelete;
+	std::list<fz::native_string> pathsToDelete;
 	long item = -1;
 	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		if (!item && m_hasParent)
@@ -940,10 +945,10 @@ void CLocalListView::OnMenuDelete(wxCommandEvent&)
 		if (data->comparison_flags == fill)
 			continue;
 
-		pathsToDelete.push_back(m_dir.GetPath() + data->name);
+		pathsToDelete.push_back(fz::to_native(m_dir.GetPath() + data->name));
 	}
-	if (!CLocalFileSystem::RecursiveDelete(pathsToDelete, this))
-		wxGetApp().DisplayEncodingWarning();
+	gui_recursive_remove rmd(this);
+	rmd.remove(pathsToDelete);
 
 	m_pState->SetLocalDir(m_dir);
 }
@@ -1315,12 +1320,12 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir.GetPath() + file, wasLink, &data.size, &data.time, &data.attributes);
-	if (type == CLocalFileSystem::unknown)
+	enum fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
+	if (type == fz::local_filesys::unknown)
 		return;
 
 	data.name = file;
-	data.dir = type == CLocalFileSystem::dir;
+	data.dir = type == fz::local_filesys::dir;
 
 	CFilterManager filter;
 	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time))
@@ -1496,7 +1501,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, CDateTime& date)
+bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
 		return false;
