@@ -7,13 +7,11 @@
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "inputdialog.h"
-#include "local_filesys.h"
 #include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include "Options.h"
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
+#include <libfilezilla/local_filesys.hpp>
 
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
@@ -22,6 +20,8 @@
 #include "volume_enumerator.h"
 #endif
 
+#include <algorithm>
+
 class CTreeItemData : public wxTreeItemData
 {
 public:
@@ -29,11 +29,12 @@ public:
 	wxString m_known_subdir;
 };
 
-class CLocalTreeViewDropTarget : public wxDropTarget
+class CLocalTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CLocalTreeViewDropTarget(CLocalTreeView* pLocalTreeView)
-		: m_pLocalTreeView(pLocalTreeView), m_pFileDataObject(new wxFileDataObject()),
+		: CScrollableDropTarget(pLocalTreeView)
+		, m_pLocalTreeView(pLocalTreeView), m_pFileDataObject(new wxFileDataObject()),
 		m_pRemoteDataObject(new CRemoteDataObject())
 	{
 		m_pDataObject = new wxDataObjectComposite;
@@ -125,6 +126,10 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
+		if (!CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y)) {
+			return false;
+		}
+
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
@@ -138,21 +143,19 @@ public:
 		return true;
 	}
 
-	wxString DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint point)
 	{
 		wxTreeItemId hit = GetHit(point);
-		if (!hit)
-		{
+		if (!hit) {
 			ClearDropHighlight();
-			return wxString();
+			return hit;
 		}
 
 		wxString dir = GetDirFromItem(hit);
 
-		if (dir.empty())
-		{
+		if (dir.empty()) {
 			ClearDropHighlight();
-			return wxString();
+			return wxTreeItemId();
 		}
 
 		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
@@ -163,11 +166,13 @@ public:
 		m_pLocalTreeView->m_dropHighlight = hit;
 
 
-		return dir;
+		return hit;
 	}
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -176,8 +181,8 @@ public:
 			return def;
 		}
 
-		const wxString& dir = DisplayDropHighlight(wxPoint(x, y));
-		if (dir.empty())
+		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
+		if (!hit.IsOk())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -188,11 +193,13 @@ public:
 
 	virtual void OnLeave()
 	{
+		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -246,6 +253,8 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	SetImageList(GetSystemImageList());
 
 	UpdateSortMode();
+	RegisterOption(OPTION_FILELIST_NAMESORT);
+
 #ifdef __WXMSW__
 	m_pVolumeEnumeratorThread = 0;
 
@@ -328,15 +337,13 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId root = GetFirstChild(m_drives, value);
-	while (root)
-	{
+	while (root) {
 		if (!GetItemText(root).Left(drive.Length()).CmpNoCase(drive))
 			break;
 
 		root = GetNextSibling(root);
 	}
-	if (!root)
-	{
+	if (!root) {
 		if (drive[1] == ':')
 			return AddDrive(drive[0]);
 		return wxTreeItemId();
@@ -347,24 +354,23 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 		wxTreeItemId root = GetRootItem();
 #endif
 
-	while (!localDir.empty())
-	{
+	while (!localDir.empty()) {
 		wxString subDir;
-		int pos = localDir.Find(separator);
-		if (pos == -1)
+		int pos2 = localDir.Find(separator);
+		if (pos2 == -1)
 			subDir = localDir;
 		else
-			subDir = localDir.Left(pos);
+			subDir = localDir.Left(pos2);
 
 		wxTreeItemId child = GetSubdir(root, subDir);
 		if (!child)
 			return root;
 
-		if (!pos)
+		if (!pos2)
 			return child;
 
 		root = child;
-		localDir = localDir.Mid(pos + 1);
+		localDir = localDir.Mid(pos2 + 1);
 	}
 
 	return root;
@@ -422,14 +428,12 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir /*=_T("")*/)
 {
-	CLocalFileSystem local_filesystem;
+	fz::local_filesys local_filesys;
 
 	{
 		wxLogNull log;
-		if (!local_filesystem.BeginFindFiles(dirname, true))
-		{
-			if (!knownSubdir.empty())
-			{
+		if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
+			if (!knownSubdir.empty()) {
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
 				if (item != wxTreeItemId())
 					return;
@@ -463,35 +467,34 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 
 	bool matchedKnown = false;
 
-	wxString file;
+	fz::native_string file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
-	const wxLongLong size(-1);
-	CDateTime date;
-	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
-	{
+	static int64_t const size(-1);
+	fz::datetime date;
+	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty())
-		{
+		if (file.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
+		wxString file_wx = file;
 
-		wxString fullName = dirname + file;
+		wxString fullName = dirname + file_wx;
 #ifdef __WXMSW__
-		if (file.CmpNoCase(knownSubdir))
+		if (file_wx.CmpNoCase(knownSubdir))
 #else
-		if (file != knownSubdir)
+		if (file_wx != knownSubdir)
 #endif
 		{
-			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
+			if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
 				continue;
 		}
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(iconType::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, file_wx, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
@@ -502,8 +505,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		CheckSubdirStatus(item, fullName);
 	}
 
-	if (!matchedKnown && !knownSubdir.empty())
-	{
+	if (!matchedKnown && !knownSubdir.empty()) {
 		const wxString fullName = dirname + knownSubdir;
 		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
@@ -525,29 +527,28 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 
 	CFilterManager filter;
 
-	CLocalFileSystem local_filesystem;
-	if (!local_filesystem.BeginFindFiles(dirname, true))
+	fz::local_filesys local_filesys;
+	if (!local_filesys.begin_find_files(fz::to_native(dirname), true))
 		return wxString();
 
-	wxString file;
+	fz::native_string file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
-	const wxLongLong size(-1);
-	CDateTime date;
-	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
-	{
+	static int64_t const size(-1);
+	fz::datetime date;
+	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty())
-		{
+		if (file.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
+		wxString file_wx = file;
+		if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
 			continue;
 
-		return file;
+		return file_wx;
 	}
 
 	return wxString();
@@ -557,17 +558,14 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
-	while (!subDir.empty())
-	{
+	while (!subDir.empty()) {
 		int pos = subDir.Find(separator);
 		wxString segment;
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			segment = subDir;
 			subDir = _T("");
 		}
-		else
-		{
+		else {
 			segment = subDir.Left(pos);
 			subDir = subDir.Mid(pos + 1);
 		}
@@ -604,39 +602,33 @@ wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 	while (item)
 	{
 #ifdef __WXMSW__
-		if (item == m_desktop)
-		{
+		if (item == m_desktop) {
 			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-			{
-				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-				{
+			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
+				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
 					wxMessageBoxEx(_("Failed to get desktop path"));
 					return _T("/");
 				}
 			}
 			dir = path;
-			if (dir.Last() != separator)
+			if (dir.empty() || dir.Last() != separator)
 				dir += separator;
 			return dir;
 		}
-		else if (item == m_documents)
-		{
+		else if (item == m_documents) {
 			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-			{
+			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
 				wxMessageBoxEx(_("Failed to get 'My Documents' path"));
 				return _T("/");
 			}
 			dir = path;
-			if (dir.Last() != separator)
+			if (dir.empty() || dir.Last() != separator)
 				dir += separator;
 			return dir;
 		}
 		else if (item == m_drives)
 			return _T("/");
-		else if (GetItemParent(item) == m_drives)
-		{
+		else if (GetItemParent(item) == m_drives) {
 			wxString text = GetItemText(item);
 			int pos = text.Find(_T(" "));
 			if (pos == -1)
@@ -688,17 +680,12 @@ void CLocalTreeView::RefreshListing()
 
 	std::list<t_dir> dirsToCheck;
 
-	UpdateSortMode();
-
 #ifdef __WXMSW__
 	int prevErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 
 	wxTreeItemIdValue tmp;
-	wxTreeItemId child = GetFirstChild(m_drives, tmp);
-	while (child)
-	{
-		if (IsExpanded(child))
-		{
+	for (auto child = GetFirstChild(m_drives, tmp); child; child = GetNextSibling(child)) {
+		if (IsExpanded(child)) {
 			wxString drive = GetItemText(child);
 			int pos = drive.Find(_T(" "));
 			if (pos != -1)
@@ -709,8 +696,6 @@ void CLocalTreeView::RefreshListing()
 			dir.item = child;
 			dirsToCheck.push_back(dir);
 		}
-
-		child = GetNextSibling(child);
 	}
 #else
 	t_dir dir;
@@ -721,62 +706,55 @@ void CLocalTreeView::RefreshListing()
 
 	CFilterManager filter;
 
-	while (!dirsToCheck.empty())
-	{
+	while (!dirsToCheck.empty()) {
 		t_dir dir = dirsToCheck.front();
 		dirsToCheck.pop_front();
 
 		// Step 1: Check if directory exists
-		CLocalFileSystem local_filesystem;
-		if (!local_filesystem.BeginFindFiles(dir.dir, true))
-		{
+		fz::local_filesys local_filesys;
+		if (!local_filesys.begin_find_files(fz::to_native(dir.dir), true)) {
 			// Dir does exist (listed in parent) but may not be accessible.
 			// Recurse into children anyhow, they might be accessible again.
 			wxTreeItemIdValue value;
-			wxTreeItemId child = GetFirstChild(dir.item, value);
-			while (child)
-			{
+			for (auto child = GetFirstChild(dir.item, value); child; child = GetNextSibling(child)) {
 				t_dir subdir;
 				subdir.dir = dir.dir + GetItemText(child) + separator;
 				subdir.item = child;
 				dirsToCheck.push_back(subdir);
-
-				child = GetNextSibling(child);
 			}
 			continue;
 		}
 
 		// Step 2: Enumerate subdirectories on disk and sort them
-		wxArrayString dirs;
+		std::vector<wxString> dirs;
 
-		wxString file;
-		const wxLongLong size(-1);
+		fz::native_string file;
+		static int64_t const size(-1);
 		bool was_link;
 		bool is_dir;
 		int attributes;
-		CDateTime date;
-		while (local_filesystem.GetNextFile(file, was_link, is_dir, 0, &date, &attributes))
-		{
-			if (file.empty())
-			{
+		fz::datetime date;
+		while (local_filesys.get_next_file(file, was_link, is_dir, 0, &date, &attributes)) {
+			if (file.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date))
+			wxString file_wx = file;
+			if (filter.FilenameFiltered(file_wx, dir.dir, true, size, true, attributes, date))
 				continue;
 
-			dirs.push_back(file);
+			dirs.push_back(file_wx);
 		}
-		dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
+		auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
+		std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
 
 		bool inserted = false;
 
 		// Step 3: Merge list of subdirectories with subtree.
 		wxTreeItemId child = GetLastChild(dir.item);
-		wxArrayString::reverse_iterator iter = dirs.rbegin();
-		while (child || iter != dirs.rend())
-		{
+		auto iter = dirs.rbegin();
+		while (child || iter != dirs.rend()) {
 			int cmp;
 			if (child && iter != dirs.rend())
 #ifdef __WXMSW__
@@ -789,22 +767,18 @@ void CLocalTreeView::RefreshListing()
 			else
 				cmp = -1;
 
-			if (!cmp)
-			{
+			if (!cmp) {
 				// Found item with same name. Mark it for further processing
-				if (!IsExpanded(child))
-				{
+				if (!IsExpanded(child)) {
 					wxString path = dir.dir + *iter + separator;
-					if (!CheckSubdirStatus(child, path))
-					{
+					if (!CheckSubdirStatus(child, path)) {
 						t_dir subdir;
 						subdir.dir = path;
 						subdir.item = child;
 						dirsToCheck.push_front(subdir);
 					}
 				}
-				else
-				{
+				else {
 					t_dir subdir;
 					subdir.dir = dir.dir + *iter + separator;
 					subdir.item = child;
@@ -813,8 +787,7 @@ void CLocalTreeView::RefreshListing()
 				child = GetPrevSibling(child);
 				++iter;
 			}
-			else if (cmp > 0)
-			{
+			else if (cmp > 0) {
 				// Subdirectory currently in tree no longer exists.
 				// Delete child from tree, unless current selection
 				// is in the subtree.
@@ -826,8 +799,7 @@ void CLocalTreeView::RefreshListing()
 					Delete(child);
 				child = prev;
 			}
-			else if (cmp < 0)
-			{
+			else if (cmp < 0) {
 				// New subdirectory, add treeitem
 				wxString fullname = dir.dir + *iter + separator;
 				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(iconType::dir, fullname),
@@ -854,8 +826,7 @@ void CLocalTreeView::RefreshListing()
 
 void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 {
-	if (m_setSelection)
-	{
+	if (m_setSelection) {
 		event.Skip();
 		return;
 	}
@@ -867,8 +838,7 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	wxString dir = GetDirFromItem(item);
 
 	wxString error;
-	if (!m_pState->SetLocalDir(dir, &error))
-	{
+	if (!m_pState->SetLocalDir(dir, &error)) {
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
@@ -879,31 +849,11 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 }
 
-int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2)
-{
-	wxString label1 = GetItemText(item1);
-	wxString label2 = GetItemText(item2);
-
-	switch (m_nameSortMode)
-	{
-	case CFileListCtrlSortBase::namesort_casesensitive:
-		return CFileListCtrlSortBase::CmpCase(label1, label2);
-
-	default:
-	case CFileListCtrlSortBase::namesort_caseinsensitive:
-		return CFileListCtrlSortBase::CmpNoCase(label1, label2);
-
-	case CFileListCtrlSortBase::namesort_natural:
-		return CFileListCtrlSortBase::CmpNatural(label1, label2);
-	}
-}
-
-void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CLocalTreeView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		SetDir(m_pState->GetLocalDir().GetPath());
-	else
-	{
+	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		RefreshListing();
 	}
@@ -925,14 +875,14 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 
 #ifdef __WXMSW__
-	if (dir.Last() == '\\')
+	if (!dir.empty() && dir.Last() == '\\')
 		dir.RemoveLast();
 #endif
-	if (dir.Last() == '/')
+	if (!dir.empty() && dir.Last() == '/')
 		dir.RemoveLast();
 
 #ifdef __WXMSW__
-	if (dir.Last() == ':')
+	if (!dir.empty() && dir.Last() == ':')
 		return;
 #endif
 
@@ -1121,7 +1071,7 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 }
 
 // Create a new Directory
-void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
+void CLocalTreeView::OnMenuMkdir(wxCommandEvent&)
 {
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
@@ -1131,7 +1081,7 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 }
 
 // Create a new Directory and enter the new Directory
-void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent& event)
+void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
 	wxString newdir = MenuMkdir();
 	if (newdir.empty()) {
@@ -1198,22 +1148,20 @@ wxString CLocalTreeView::MenuMkdir()
 	return fn.GetPath();
 }
 
-void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
+void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
 
 #ifdef __WXMSW__
-	if (m_contextMenuItem == m_desktop || m_contextMenuItem == m_documents)
-	{
+	if (m_contextMenuItem == m_desktop || m_contextMenuItem == m_documents) {
 		wxBell();
 		return;
 	}
 #endif
 
 	CLocalPath path(GetDirFromItem(m_contextMenuItem));
-	if (!path.HasParent() || !path.IsWriteable())
-	{
+	if (!path.HasParent() || !path.IsWriteable()) {
 		wxBell();
 		return;
 	}
@@ -1221,7 +1169,7 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
 	EditLabel(m_contextMenuItem);
 }
 
-void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
+void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
@@ -1232,15 +1180,14 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 	if (!local_path.HasParent() || !local_path.IsWriteable())
 		return;
 
-	if (!CLocalFileSystem::RecursiveDelete(path, this))
-		wxGetApp().DisplayEncodingWarning();
+	gui_recursive_remove rmd(this);
+	rmd.remove(fz::to_native(path));
 
 	wxTreeItemId item = GetSelection();
 	while (item && item != m_contextMenuItem)
 		item = GetItemParent(item);
 
-	if (!item)
-	{
+	if (!item) {
 		if (GetItemParent(m_contextMenuItem) == GetSelection())
 			m_pState->RefreshLocal();
 		else
@@ -1248,7 +1195,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 		return;
 	}
 
-	if (path.Last() == wxFileName::GetPathSeparator())
+	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
 		path.RemoveLast();
 	int pos = path.Find(wxFileName::GetPathSeparator(), true);
 	if (pos < 1)
@@ -1285,8 +1232,7 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 
 void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 {
-	if (event.IsEditCancelled())
-	{
+	if (event.IsEditCancelled()) {
 		event.Veto();
 		return;
 	}
@@ -1294,8 +1240,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxTreeItemId item = event.GetItem();
 
 #ifdef __WXMSW__
-	if (item == m_desktop || item == m_documents)
-	{
+	if (item == m_desktop || item == m_documents) {
 		wxBell();
 		event.Veto();
 		return;
@@ -1305,14 +1250,13 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxString path = GetDirFromItem(item);
 
 	CLocalPath local_path(path);
-	if (!local_path.HasParent() || !local_path.IsWriteable())
-	{
+	if (!local_path.HasParent() || !local_path.IsWriteable()) {
 		wxBell();
 		event.Veto();
 		return;
 	}
 
-	if (path.Last() == wxFileName::GetPathSeparator())
+	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
 		path.RemoveLast();
 
 	int pos = path.Find(wxFileName::GetPathSeparator(), true);
@@ -1322,8 +1266,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(item);
 	const wxString& newName = event.GetLabel();
-	if (newName.empty())
-	{
+	if (newName.empty()) {
 		wxBell();
 		event.Veto();
 		return;
@@ -1351,8 +1294,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	if (item == currentSel)
-	{
+	if (item == currentSel) {
 		m_pState->SetLocalDir(parent + newName);
 		return;
 	}
@@ -1360,14 +1302,12 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxString sub;
 
 	wxTreeItemId tmp = currentSel;
-	while (tmp != GetRootItem() && tmp != item)
-	{
+	while (tmp != GetRootItem() && tmp != item) {
 		sub = wxFileName::GetPathSeparator() + GetItemText(tmp) + sub;
 		tmp = GetItemParent(tmp);
 	}
 
-	if (tmp == GetRootItem())
-	{
+	if (tmp == GetRootItem()) {
 		// Rename unrelated to current selection
 		return;
 	}
@@ -1394,7 +1334,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
 
-	static const wxLongLong size(-1);
+	static int64_t const size(-1);
 
 #ifdef __WXMAC__
 	// By default, OS X has a list of servers mounted into /net,
@@ -1404,7 +1344,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		CFilterManager filter;
 
 		const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
-		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, CDateTime()))
+		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, fz::datetime()))
 		{
 			if (!child)
 				AppendItem(item, _T(""));
@@ -1413,24 +1353,21 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	}
 #endif
 
-	if (child)
-	{
+	if (child) {
 		if (!GetItemText(child).empty())
 			return false;
 
 		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
-		if (pData)
-		{
+		if (pData) {
 			bool wasLink;
 			int attributes;
-			enum CLocalFileSystem::local_fileType type;
-			CDateTime date;
-			if (path.Last() == CLocalFileSystem::path_separator)
-				type = CLocalFileSystem::GetFileInfo(path + pData->m_known_subdir, wasLink, 0, &date, &attributes);
+			enum fz::local_filesys::type type;
+			fz::datetime date;
+			if (!path.empty() && path.Last() == fz::local_filesys::path_separator)
+				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
 			else
-				type = CLocalFileSystem::GetFileInfo(path + CLocalFileSystem::path_separator + pData->m_known_subdir, wasLink, 0, &date, &attributes);
-			if (type == CLocalFileSystem::dir)
-			{
+				type = fz::local_filesys::get_file_info(fz::to_native(path + fz::local_filesys::path_separator + pData->m_known_subdir), wasLink, 0, &date, &attributes);
+			if (type == fz::local_filesys::dir) {
 				CFilterManager filter;
 				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
 					return true;
@@ -1439,8 +1376,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	}
 
 	wxString sub = HasSubdir(path);
-	if (!sub.empty())
-	{
+	if (!sub.empty()) {
 		wxTreeItemId subItem = AppendItem(item, _T(""));
 		SetItemData(subItem, new CTreeItemData(sub));
 	}
@@ -1456,8 +1392,7 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 	if (!m_drives)
 		return;
 
-	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE)
-	{
+	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE) {
 		DEV_BROADCAST_HDR* pHdr = (DEV_BROADCAST_HDR*)lParam;
 		if (pHdr->dbch_devicetype != DBT_DEVTYP_VOLUME)
 			return;
@@ -1467,18 +1402,14 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 
 		wxChar drive = 'A';
 		int mask = 1;
-		while (drive <= 'Z')
-		{
-			if (pVolume->dbcv_unitmask & mask)
-			{
+		while (drive <= 'Z') {
+			if (pVolume->dbcv_unitmask & mask) {
 				if (wParam == DBT_DEVICEARRIVAL)
 					AddDrive(drive);
-				else
-				{
+				else {
 					RemoveDrive(drive);
 
-					if (pVolume->dbcv_flags & DBTF_MEDIA)
-					{
+					if (pVolume->dbcv_flags & DBTF_MEDIA) {
 						// E.g. disk removed from CD-ROM drive, need to keep the drive letter
 						AddDrive(drive);
 					}
@@ -1499,14 +1430,14 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 	drive += _T(":");
 
 	long drivesToHide = CVolumeDescriptionEnumeratorThread::GetDrivesToHide();
-	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive, drivesToHide) ) {
+	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive.wc_str(), drivesToHide) ) {
 		return wxTreeItemId();
 	}
 
 	// Get the label of the drive
 	wxChar volumeName[501];
 	int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-	BOOL res = GetVolumeInformation(drive + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
+	BOOL res = GetVolumeInformation((drive + _T("\\")).wc_str(), volumeName, 500, 0, 0, 0, 0, 0);
 	SetErrorMode(oldErrorMode);
 
 	wxString itemLabel = drive;
@@ -1592,3 +1523,11 @@ void CLocalTreeView::OnMenuOpen(wxCommandEvent&)
 
 	OpenInFileManager(path);
 }
+
+void CLocalTreeView::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILELIST_NAMESORT)) {
+		UpdateSortMode();
+		RefreshListing();
+	}
+}
