@@ -1,11 +1,18 @@
 #ifndef __MAINFRM_H__
 #define __MAINFRM_H__
 
+#include "statusbar.h"
+#include "engine_context.h"
+
+#include <wx/timer.h>
+
 #ifndef __WXMAC__
 #include <wx/taskbar.h>
 #endif
 
-#include "statusbar.h"
+#if FZ_MANUALUPDATECHECK
+#include "updater.h"
+#endif
 
 class CAsyncRequestQueue;
 class CContextControl;
@@ -23,13 +30,7 @@ class CThemeProvider;
 class CToolBar;
 class CWindowStateManager;
 
-
-#if FZ_MANUALUPDATECHECK
-#include "updater.h"
-#endif
-class CWindowStateManager;
-
-class CMainFrame : public wxNavigationEnabled<wxFrame>
+class CMainFrame final : public wxNavigationEnabled<wxFrame>, private COptionChangeEventHandler
 #if FZ_MANUALUPDATECHECK
 	, protected CUpdateHandler
 #endif
@@ -41,13 +42,10 @@ public:
 
 	void UpdateActivityLed(int direction);
 
-	void AddToRequestQueue(CFileZillaEngine* pEngine, CAsyncRequestNotification* pNotification);
 	CStatusView* GetStatusView() { return m_pStatusView; }
 	CQueueView* GetQueue() { return m_pQueueView; }
 	CQuickconnectBar* GetQuickconnectBar() { return m_pQuickconnectBar; }
 
-	void UpdateLayout(int layout = -1, int swap = -1, int messagelog_position = -1);
-
 	// Window size and position as well as pane sizes
 	void RememberSplitterPositions();
 	bool RestoreSplitterPositions();
@@ -67,9 +65,11 @@ public:
 
 	CContextControl* GetContextControl() { return m_pContextControl; }
 
-	bool ConnectToSite(CSiteManagerItemData_Site* const pData, bool newTab = false);
+	bool ConnectToSite(CSiteManagerItemData_Site & data, bool newTab = false);
 
-protected:
+	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
+private:
+	void UpdateLayout();
 	void FixTabOrder();
 
 	bool CloseDialogsAndQuit(wxCloseEvent &event);
@@ -91,6 +91,7 @@ protected:
 	CSplitterWindowEx* m_pBottomSplitter{}; // Top contains view splitter, bottom queue (or queuelog splitter if in position 1)
 	CSplitterWindowEx* m_pQueueLogSplitter{};
 
+	CFileZillaEngineContext m_engineContext;
 	CContextControl* m_pContextControl{};
 
 	CStatusView* m_pStatusView{};
@@ -104,8 +105,7 @@ protected:
 	wxTimer update_dialog_timer_;
 #endif
 
-	void ShowLocalTree(bool show);
-	void ShowRemoteTree(bool show);
+	void ShowDirectoryTree(bool local, bool show);
 
 	void ShowDropdownMenu(wxMenu* pMenu, wxToolBar* pToolBar, wxCommandEvent& event);
 
@@ -115,11 +115,15 @@ protected:
 
 	void HandleResize();
 
+	void SetupKeyboardAccelerators();
+
+	void OnOptionsChanged(changed_options_t const& options);
+
 	// Event handlers
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
-	void OnEngineEvent(wxEvent& event);
+	void OnEngineEvent(wxFzEvent& event);
 	void OnUpdateLedTooltip(wxCommandEvent&);
 	void OnDisconnect(wxCommandEvent&);
 	void OnCancel(wxCommandEvent&);
@@ -132,8 +136,7 @@ protected:
 	void OnMenuEditSettings(wxCommandEvent&);
 	void OnToggleToolBar(wxCommandEvent& event);
 	void OnToggleLogView(wxCommandEvent&);
-	void OnToggleLocalTreeView(wxCommandEvent& event);
-	void OnToggleRemoteTreeView(wxCommandEvent& event);
+	void OnToggleDirectoryTreeView(wxCommandEvent& event);
 	void OnToggleQueueView(wxCommandEvent& event);
 	void OnMenuHelpAbout(wxCommandEvent&);
 	void OnFilter(wxCommandEvent& event);
