@@ -1,12 +1,12 @@
 #include <filezilla.h>
 #include "Options.h"
-#include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include <wx/tokenzr.h>
 #include "ipcmutex.h"
 #include <option_change_event_handler.h>
 #include "sizeformatting.h"
 
+#include <algorithm>
 #include <string>
 
 #ifdef __WXMSW__
@@ -18,10 +18,6 @@
 	#endif
 #endif
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 COptions* COptions::m_theOptions = 0;
 
 enum Type
@@ -41,7 +37,7 @@ enum Flags
 struct t_Option
 {
 	const char name[30];
-	const enum Type type;
+	const Type type;
 	const wxString defaultValue; // Default values are stored as string even for numerical options
 	const Flags flags; // internal items won't get written to settings file nor loaded from there
 };
@@ -54,6 +50,10 @@ struct t_Option
 #define DEFAULT_FILENAME_SORT   _T("1")
 #endif
 
+// In C++14 we should be able to use this instead:
+//   static_assert(OPTIONS_NUM <= changed_options_t().size());
+static_assert(static_cast<int>(OPTIONS_NUM) <= static_cast<int>(changed_options_size), "OPTIONS_NUM too big for changed_options_t");
+
 static const t_Option options[OPTIONS_NUM] =
 {
 	// Note: A few options are versioned due to a changed
@@ -64,6 +64,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Limit local ports", number, _T("0"), normal },
 	{ "Limit ports low", number, _T("6000"), normal },
 	{ "Limit ports high", number, _T("7000"), normal },
+	{ "Limit ports offset", number, _T("0"), normal },
 	{ "External IP mode", number, _T("0"), normal },
 	{ "External IP", string, _T(""), normal },
 	{ "External address resolver", string, _T("http://ip.filezilla-project.org/ip.php"), normal },
@@ -81,6 +82,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Speedlimit inbound", number, _T("100"), normal },
 	{ "Speedlimit outbound", number, _T("20"), normal },
 	{ "Speedlimit burst tolerance", number, _T("0"), normal },
+	{ "Preallocate space", number, _T("0"), normal },
 	{ "View hidden files", number, _T("0"), normal },
 	{ "Preserve timestamps", number, _T("0"), normal },
 	{ "Socket recv buffer size (v2)", number, _T("4194304"), normal }, // Make it large enough by default
@@ -93,6 +95,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "FTP Proxy password", string, _T(""), normal },
 	{ "FTP Proxy login sequence", string, _T(""), normal },
 	{ "SFTP keyfiles", string, _T(""), normal },
+	{ "SFTP compression", number, _T(""), normal },
 	{ "Proxy type", number, _T("0"), normal },
 	{ "Proxy host", string, _T(""), normal },
 	{ "Proxy port", number, _T("0"), normal },
@@ -100,7 +103,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Proxy password", string, _T(""), normal },
 	{ "Logging file", string, _T(""), normal },
 	{ "Logging filesize limit", number, _T("10"), normal },
-	{ "Trusted root certificate", string, _T(""), internal },
+	{ "Logging show detailed logs", number, _T("0"), internal },
 	{ "Size format", number, _T("0"), normal },
 	{ "Size thousands separator", number, _T("1"), normal },
 	{ "Size decimal places", number, _T("1"), normal },
@@ -108,7 +111,7 @@ static const t_Option options[OPTIONS_NUM] =
 	// Interface settings
 	{ "Number of Transfers", number, _T("2"), normal },
 	{ "Ascii Binary mode", number, _T("0"), normal },
-	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
+	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|sha1|sha256|sha512|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
 	{ "Auto Ascii no extension", number, _T("1"), normal },
 	{ "Auto Ascii dotfiles", number, _T("1"), normal },
 	{ "Theme", string, _T("opencrystal/"), normal },
@@ -121,7 +124,6 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Last automatic update check", string, _T(""), normal },
 	{ "Update Check New Version", string, _T(""), normal },
 	{ "Update Check Check Beta", number, _T("0"), normal },
-	{ "Update Check Download Dir", string, _T(""), normal },
 	{ "Show debug menu", number, _T("0"), normal },
 	{ "File exists action download", number, _T("0"), normal },
 	{ "File exists action upload", number, _T("0"), normal },
@@ -187,6 +189,8 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Strip VMS revisions", number, _T("0"), normal },
 	{ "Show Site Manager on startup", number, _T("0"), normal },
 	{ "Prompt password change", number, _T("0"), normal },
+	{ "Persistent Choices", number, _T("0"), normal },
+	{ "Show notification bubbles", number, _T("1"), normal },
 
 	// Default/internal options
 	{ "Config Location", string, _T(""), default_only },
@@ -198,6 +202,26 @@ BEGIN_EVENT_TABLE(COptions, wxEvtHandler)
 EVT_TIMER(wxID_ANY, COptions::OnTimer)
 END_EVENT_TABLE()
 
+t_OptionsCache& t_OptionsCache::operator=(wxString const& v)
+{
+	strValue = v;
+	long l;
+	if (v.ToLong(&l)) {
+		numValue = l;
+	}
+	else {
+		numValue = 0;
+	}
+	return *this;
+}
+
+t_OptionsCache& t_OptionsCache::operator=(int v)
+{
+	numValue = v;
+	strValue.Printf("%d", v);
+	return *this;
+}
+
 COptions::COptions()
 {
 	m_theOptions = this;
@@ -227,10 +251,10 @@ COptions::COptions()
 	LoadOptions(nameOptionMap);
 }
 
-std::map<std::string, int> COptions::GetNameOptionMap() const
+std::map<std::string, unsigned int> COptions::GetNameOptionMap() const
 {
-	std::map<std::string, int> ret;
-	for (int i = 0; i < OPTIONS_NUM; ++i) {
+	std::map<std::string, unsigned int> ret;
+	for (unsigned int i = 0; i < OPTIONS_NUM; ++i) {
 		if (options[i].flags != internal)
 			ret.insert(std::make_pair(std::string(options[i].name), i));
 	}
@@ -239,7 +263,7 @@ std::map<std::string, int> COptions::GetNameOptionMap() const
 
 COptions::~COptions()
 {
-	COptionChangeEventHandler::UnregisterAll();
+	COptionChangeEventHandler::UnregisterAllHandlers();
 
 	delete m_pLastServer;
 	delete m_pXmlFile;
@@ -250,9 +274,7 @@ int COptions::GetOptionVal(unsigned int nID)
 	if (nID >= OPTIONS_NUM)
 		return 0;
 
-	if (options[nID].type != number)
-		return 0;
-
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].numValue;
 }
 
@@ -261,9 +283,7 @@ wxString COptions::GetOption(unsigned int nID)
 	if (nID >= OPTIONS_NUM)
 		return wxString();
 
-	if (options[nID].type != string)
-		return wxString::Format(_T("%d"), GetOptionVal(nID));
-
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].strValue;
 }
 
@@ -301,11 +321,18 @@ void COptions::ContinueSetOption(unsigned int nID, T const& value)
 {
 	T validated = Validate(nID, value);
 
-	if (m_optionsCache[nID] == validated) {
-		// Nothing to do
-		return;
+	{
+		fz::scoped_lock l(m_sync_);
+		if (m_optionsCache[nID] == validated) {
+			// Nothing to do
+			return;
+		}
+		m_optionsCache[nID] = validated;
 	}
-	m_optionsCache[nID] = validated;
+
+	// Fixme: Setting options from other threads
+	if (!wxIsMainThread())
+		return;
 
 	if (options[nID].flags == normal || options[nID].flags == default_priority) {
 		SetXmlValue(nID, validated);
@@ -314,7 +341,18 @@ void COptions::ContinueSetOption(unsigned int nID, T const& value)
 			m_save_timer.Start(15000, true);
 	}
 
-	COptionChangeEventHandler::DoNotify(nID);
+	if (changedOptions_.none()) {
+		CallAfter(&COptions::NotifyChangedOptions);
+	}
+	changedOptions_.set(nID);
+}
+
+void COptions::NotifyChangedOptions()
+{
+	// Reset prior to notifying to correctly handle the case of an option being set while notifying
+	auto changedOptions = changedOptions_;
+	changedOptions_.reset();
+	COptionChangeEventHandler::DoNotify(changedOptions);
 }
 
 bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
@@ -322,33 +360,32 @@ bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 	if (nID >= OPTIONS_NUM)
 		return false;
 
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].from_default;
 }
 
-TiXmlElement* COptions::CreateSettingsXmlElement()
+pugi::xml_node COptions::CreateSettingsXmlElement()
 {
 	if (!m_pXmlFile)
-		return 0;
+		return pugi::xml_node();
 
-	TiXmlElement* element = m_pXmlFile->GetElement();
+	auto element = m_pXmlFile->GetElement();
 	if (!element) {
-		return 0;
+		return element;
 	}
 
-	TiXmlElement* settings = element->FirstChildElement("Settings");
+	auto settings = element.child("Settings");
 	if (settings) {
 		return settings;
 	}
 
-	settings = new TiXmlElement("Settings");
-	element->LinkEndChild(settings);
-
+	settings = element.append_child("Settings");
 	for (int i = 0; i < OPTIONS_NUM; ++i) {
 		if (options[i].type == string) {
-			SetXmlValue(i, m_optionsCache[i].strValue);
+			SetXmlValue(i, GetOption(i));
 		}
 		else {
-			SetXmlValue(i, m_optionsCache[i].numValue);
+			SetXmlValue(i, GetOptionVal(i));
 		}
 	}
 
@@ -371,25 +408,21 @@ void COptions::SetXmlValue(unsigned int nID, wxString const& value)
 	if (!utf8)
 		return;
 
-	TiXmlElement *settings = CreateSettingsXmlElement();
+	auto settings = CreateSettingsXmlElement();
 	if (settings) {
-		TiXmlElement *setting = 0;
-		for (setting = settings->FirstChildElement("Setting"); setting; setting = setting->NextSiblingElement("Setting")) {
-			const char *attribute = setting->Attribute("name");
+		pugi::xml_node setting;
+		for (setting = settings.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
+			const char *attribute = setting.attribute("name").value();
 			if (!attribute)
 				continue;
 			if (!strcmp(attribute, options[nID].name))
 				break;
 		}
-		if (setting) {
-			setting->Clear();
+		if (!setting) {
+			setting = settings.append_child("Setting");
+			SetTextAttribute(setting, "name", options[nID].name);
 		}
-		else {
-			setting = new TiXmlElement("Setting");
-			setting->SetAttribute("name", options[nID].name);
-			settings->LinkEndChild(setting);
-		}
-		setting->LinkEndChild(new TiXmlText(utf8));
+		setting.text() = utf8;
 	}
 }
 
@@ -464,6 +497,17 @@ int COptions::Validate(unsigned int nID, int value)
 		if (value < 0 || value >= CSizeFormat::formats_count)
 			value = 0;
 		break;
+	case OPTION_TIMEOUT:
+		if (value <= 0) {
+			value = 0;
+		}
+		else if (value < 10) {
+			value = 10;
+		}
+		else if (value > 9999) {
+			value = 9999;
+		}
+		break;
 	}
 	return value;
 }
@@ -485,7 +529,7 @@ void COptions::SetServer(wxString path, const CServer& server)
 	if (path.empty())
 		return;
 
-	TiXmlElement *element = m_pXmlFile->GetElement();
+	auto element = m_pXmlFile->GetElement();
 
 	while (!path.empty()) {
 		wxString sub;
@@ -501,14 +545,11 @@ void COptions::SetServer(wxString path, const CServer& server)
 		wxScopedCharBuffer utf8 = sub.utf8_str();
 		if (!utf8)
 			return;
-		TiXmlElement *newElement = element->FirstChildElement(utf8);
+		auto newElement = element.child(utf8);
 		if (newElement)
 			element = newElement;
 		else {
-			TiXmlNode *node = element->LinkEndChild(new TiXmlElement(utf8));
-			if (!node || !node->ToElement())
-				return;
-			element = node->ToElement();
+			element = element.append_child(utf8);
 		}
 	}
 
@@ -528,7 +569,7 @@ bool COptions::GetServer(wxString path, CServer& server)
 
 	if (!m_pXmlFile)
 		return false;
-	TiXmlElement *element = m_pXmlFile->GetElement();
+	auto element = m_pXmlFile->GetElement();
 
 	while (!path.empty()) {
 		wxString sub;
@@ -544,7 +585,7 @@ bool COptions::GetServer(wxString path, CServer& server)
 		wxScopedCharBuffer utf8 = sub.utf8_str();
 		if (!utf8)
 			return false;
-		element = element->FirstChildElement(utf8);
+		element = element.child(utf8);
 		if (!element)
 			return false;
 	}
@@ -600,14 +641,14 @@ COptions* COptions::Get()
 	return m_theOptions;
 }
 
-void COptions::Import(TiXmlElement* pElement)
+void COptions::Import(pugi::xml_node element)
 {
-	LoadOptions(GetNameOptionMap(), pElement);
+	LoadOptions(GetNameOptionMap(), element);
 	if (!m_save_timer.IsRunning())
 		m_save_timer.Start(15000, true);
 }
 
-void COptions::LoadOptions(std::map<std::string, int> const& nameOptionMap, TiXmlElement* settings)
+void COptions::LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, pugi::xml_node settings)
 {
 	if (!settings) {
 		settings = CreateSettingsXmlElement();
@@ -616,18 +657,14 @@ void COptions::LoadOptions(std::map<std::string, int> const& nameOptionMap, TiXm
 		}
 	}
 
-	TiXmlNode *node = 0;
-	while ((node = settings->IterateChildren("Setting", node))) {
-		TiXmlElement *setting = node->ToElement();
-		if (!setting)
-			continue;
+	for (auto setting = settings.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
 		LoadOptionFromElement(setting, nameOptionMap, false);
 	}
 }
 
-void COptions::LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string, int> const& nameOptionMap, bool allowDefault)
+void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault)
 {
-	const char* name = pOption->Attribute("name");
+	const char* name = option.attribute("name").value();
 	if (!name)
 		return;
 
@@ -635,20 +672,14 @@ void COptions::LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string
 	if (iter != nameOptionMap.end()) {
 		if (!allowDefault && options[iter->second].flags == default_only)
 			return;
-		wxString value;
-
-		TiXmlNode *text = pOption->FirstChild();
-		if (text) {
-			if (!text->ToText())
-				return;
-
-			value = ConvLocal(text->Value());
-		}
-
+		wxString value = GetTextElement(option);
 		if (options[iter->second].flags == default_priority) {
-			if (allowDefault)
+			if (allowDefault) {
+				fz::scoped_lock l(m_sync_);
 				m_optionsCache[iter->second].from_default = true;
+			}
 			else {
+				fz::scoped_lock l(m_sync_);
 				if (m_optionsCache[iter->second].from_default)
 					return;
 			}
@@ -658,16 +689,18 @@ void COptions::LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string
 			long numValue = 0;
 			value.ToLong(&numValue);
 			numValue = Validate(iter->second, numValue);
-			m_optionsCache[iter->second].numValue = numValue;
+			fz::scoped_lock l(m_sync_);
+			m_optionsCache[iter->second] = numValue;
 		}
 		else {
 			value = Validate(iter->second, value);
-			m_optionsCache[iter->second].strValue = value;
+			fz::scoped_lock l(m_sync_);
+			m_optionsCache[iter->second] = value;
 		}
 	}
 }
 
-void COptions::LoadGlobalDefaultOptions(std::map<std::string, int> const& nameOptionMap)
+void COptions::LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
 	if (defaultsDir.empty())
@@ -677,20 +710,20 @@ void COptions::LoadGlobalDefaultOptions(std::map<std::string, int> const& nameOp
 	if (!file.Load())
 		return;
 
-	TiXmlElement* pElement = file.GetElement();
-	if (!pElement)
+	auto element = file.GetElement();
+	if (!element)
 		return;
 
-	pElement = pElement->FirstChildElement("Settings");
-	if (!pElement)
+	element = element.child("Settings");
+	if (!element)
 		return;
 
-	for (TiXmlElement* pSetting = pElement->FirstChildElement("Setting"); pSetting; pSetting = pSetting->NextSiblingElement("Setting")) {
-		LoadOptionFromElement(pSetting, nameOptionMap, true);
+	for (auto setting = element.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
+		LoadOptionFromElement(setting, nameOptionMap, true);
 	}
 }
 
-void COptions::OnTimer(wxTimerEvent& event)
+void COptions::OnTimer(wxTimerEvent&)
 {
 	Save();
 }
@@ -827,14 +860,9 @@ CLocalPath COptions::InitSettingsDir()
 
 void COptions::SetDefaultValues()
 {
+	fz::scoped_lock l(m_sync_);
 	for (int i = 0; i < OPTIONS_NUM; ++i) {
-		if (options[i].type == string)
-			m_optionsCache[i].strValue = options[i].defaultValue;
-		else {
-			long numValue = 0;
-			options[i].defaultValue.ToLong(&numValue);
-			m_optionsCache[i].numValue = numValue;
-		}
+		m_optionsCache[i] = options[i].defaultValue;
 		m_optionsCache[i].from_default = false;
 	}
 }
