@@ -5,10 +5,12 @@
 
 #include <option_change_event_handler.h>
 
-#include <mutex.h>
+#include <libfilezilla/mutex.hpp>
 
 #include <wx/timer.h>
 
+#include "xmlfunctions.h"
+
 enum interfaceOptions
 {
 	OPTION_NUMTRANSFERS = OPTIONS_ENGINE_NUM,
@@ -92,6 +94,7 @@ enum interfaceOptions
 	OPTION_INTERFACE_SITEMANAGER_ON_STARTUP,
 	OPTION_PROMPTPASSWORDSAVE,
 	OPTION_PERSISTENT_CHOICES,
+	OPTION_SHOW_NOTIFICATION_BUBBLE,
 
 	// Default/internal options
 	OPTION_DEFAULT_SETTINGSDIR, // guaranteed to be (back)slash-terminated
@@ -115,7 +118,6 @@ struct t_OptionsCache
 };
 
 class CXmlFile;
-class TiXmlElement;
 class COptions : public wxEvtHandler, public COptionsBase
 {
 public:
@@ -134,7 +136,7 @@ public:
 	static void Init();
 	static void Destroy();
 
-	void Import(TiXmlElement* pElement);
+	void Import(pugi::xml_node element);
 
 	void SaveIfNeeded();
 
@@ -155,12 +157,12 @@ protected:
 	void SetServer(wxString path, const CServer& server);
 	bool GetServer(wxString path, CServer& server);
 
-	TiXmlElement* CreateSettingsXmlElement();
+	pugi::xml_node CreateSettingsXmlElement();
 
 	std::map<std::string, unsigned int> GetNameOptionMap() const;
-	void LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, TiXmlElement* settings = 0);
+	void LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, pugi::xml_node settings = pugi::xml_node());
 	void LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap);
-	void LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault);
+	void LoadOptionFromElement(pugi::xml_node option, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault);
 	CLocalPath InitSettingsDir();
 	void SetDefaultValues();
 
@@ -181,7 +183,7 @@ protected:
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& event);
 
-	mutex m_sync_;
+	fz::mutex m_sync_;
 
 	changed_options_t changedOptions_;
 };
