@@ -4,7 +4,7 @@
 #include "dndobjects.h"
 #include "queue.h"
 
-#include <libfilezilla.h>
+#include <libfilezilla_engine.h>
 #include <option_change_event_handler.h>
 
 #include <set>
@@ -31,15 +31,13 @@ class t_newEntry : public CFolderProcessingEntry
 public:
 	t_newEntry()
 		: CFolderProcessingEntry(CFolderProcessingEntry::file)
-		, attributes()
-		, dir()
 	{}
 
 	wxString name;
-	wxLongLong size;
-	CDateTime time;
-	int attributes;
-	bool dir;
+	int64_t size{};
+	fz::datetime time;
+	int attributes{};
+	bool dir{};
 };
 
 enum ActionAfterState
@@ -112,6 +110,8 @@ class CAsyncRequestQueue;
 class CQueue;
 #if WITH_LIBDBUS
 class CDesktopNotification;
+#elif defined(__WXGTK__) || defined(__WXMSW__)
+class wxNotificationMessage;
 #endif
 
 class CQueueView : public CQueueViewBase, public COptionChangeEventHandler
@@ -126,7 +126,7 @@ public:
 	bool QueueFile(const bool queueOnly, const bool download,
 		const wxString& localFile, const wxString& remoteFile,
 		const CLocalPath& localPath, const CServerPath& remotePath,
-		const CServer& server, const wxLongLong size, enum CEditHandler::fileType edit = CEditHandler::none,
+		const CServer& server, int64_t size, enum CEditHandler::fileType edit = CEditHandler::none,
 		QueuePriority priority = QueuePriority::normal);
 
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
@@ -143,21 +143,19 @@ public:
 	// This includes queued folders which are yet to be processed
 	void SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction);
 
-	void UpdateItemSize(CFileItem* pItem, wxLongLong size);
+	void UpdateItemSize(CFileItem* pItem, int64_t size);
 
 	void RemoveAll();
 
 	void LoadQueue();
 	void LoadQueueFromXML();
-	void ImportQueue(TiXmlElement* pElement, bool updateSelections);
+	void ImportQueue(pugi::xml_node element, bool updateSelections);
 
 	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem);
 
 	virtual void CommitChanges();
 
-	void WriteToFile(TiXmlElement* pElement) const;
-
-	void ProcessNotification(CFileZillaEngine* pEngine, CNotification* pNotification);
+	void ProcessNotification(CFileZillaEngine* pEngine, std::unique_ptr<CNotification>&& pNotification);
 
 	void RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString& newName, bool local);
 
@@ -177,7 +175,7 @@ protected:
 	WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
 #endif
 
-	virtual void OnOptionChanged(int option);
+	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	void AdvanceQueue(bool refresh = true);
 	bool TryStartNextTransfer();
@@ -189,7 +187,7 @@ protected:
 	bool ProcessFolderItems(int type = -1);
 	void ProcessUploadFolderItems();
 
-	void ProcessReply(t_EngineData* pEngineData, COperationNotification* pNotification);
+	void ProcessReply(t_EngineData* pEngineData, COperationNotification const& notification);
 	void SendNextCommand(t_EngineData& engineData);
 
 	enum ResetReason
@@ -206,12 +204,12 @@ protected:
 	void ResetEngine(t_EngineData& data, const enum ResetReason reason);
 	void DeleteEngines();
 
-	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true);
+	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true);
 
 	// Stops processing of given item
 	// Returns true on success, false if it would block
 	bool StopItem(CFileItem* item);
-	bool StopItem(CServerItem* pServerItem);
+	bool StopItem(CServerItem* pServerItem, bool updateSelections);
 
 	void CheckQueueState();
 	bool IncreaseErrorCount(t_EngineData& engineData);
@@ -226,7 +224,7 @@ protected:
 	void ActionAfterWarnUser(ActionAfterState s);
 #endif
 
-	void ProcessNotification(t_EngineData* pEngineData, CNotification* pNotification);
+	void ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<CNotification> && pNotification);
 
 	// Tries to refresh the current remote directory listing
 	// if there's an idle engine connected to the current server of
@@ -234,7 +232,7 @@ protected:
 	void TryRefreshListings();
 	CServer m_last_refresh_server;
 	CServerPath m_last_refresh_path;
-	CMonotonicTime m_last_refresh_listing_time;
+	fz::monotonic_clock m_last_refresh_listing_time;
 
 	// Called from Process Reply.
 	// After a disconnect, check if there's another idle engine that
@@ -258,7 +256,7 @@ protected:
 
 	void RemoveQueuedFolderItem(CFolderScanItem* pFolder);
 
-	CFolderProcessingThread *m_pFolderProcessingThread;
+	CFolderProcessingThread *m_pFolderProcessingThread{};
 
 	/*
 	 * Don't update status line positions if m_waitStatusLineUpdate is true.
@@ -279,13 +277,13 @@ protected:
 	enum ActionAfterState m_actionAfterState;
 	wxString m_actionAfterRunCommand;
 #if defined(__WXMSW__) || defined(__WXMAC__)
-	wxTimer* m_actionAfterTimer;
-	wxProgressDialog* m_actionAfterWarnDialog;
-	int m_actionAfterTimerCount;
-	int m_actionAfterTimerId;
+	wxTimer* m_actionAfterTimer{};
+	wxProgressDialog* m_actionAfterWarnDialog{};
+	int m_actionAfterTimerCount{};
+	int m_actionAfterTimerId{-1};
 #endif
 
-	wxLongLong m_totalQueueSize;
+	int64_t m_totalQueueSize;
 	int m_filesWithUnknownSize;
 
 	CMainFrame* m_pMainFrame;
@@ -308,7 +306,9 @@ protected:
 	void ReleaseExclusiveEngineLock(CFileZillaEngine* pEngine);
 
 #if WITH_LIBDBUS
-	CDesktopNotification* m_desktop_notification;
+	std::unique_ptr<CDesktopNotification> m_desktop_notification;
+#elif defined(__WXGTK__) || defined(__WXMSW__)
+	std::unique_ptr<wxNotificationMessage> m_desktop_notification;
 #endif
 
 	CQueueStorage m_queue_storage;
@@ -318,7 +318,7 @@ protected:
 	wxFileOffset GetCurrentSpeed(bool countDownload, bool countUpload);
 
 	DECLARE_EVENT_TABLE()
-	void OnEngineEvent(wxEvent &event);
+	void OnEngineEvent(wxFzEvent &event);
 	void OnFolderThreadComplete(wxCommandEvent& event);
 	void OnFolderThreadFiles(wxCommandEvent& event);
 	void OnChar(wxKeyEvent& event);
