@@ -17,9 +17,9 @@ public:
 	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
-	virtual bool CanStartComparison(wxString* pError);
+	virtual bool CanStartComparison();
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
+	virtual bool get_next_file(wxString& name, bool &dir, int64_t &size, fz::datetime& date);
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
@@ -44,7 +44,7 @@ protected:
 	virtual int OnGetItemImage(long item) const;
 
 	virtual bool ItemIsDir(int index) const;
-	virtual wxLongLong ItemGetSize(int index) const;
+	virtual int64_t ItemGetSize(int index) const;
 
 	bool IsItemValid(unsigned int item) const;
 	int GetItemIndex(unsigned int item) const;
@@ -94,7 +94,8 @@ protected:
 		CServerPath remote_path;
 		wxString link;
 		CLocalPath local_path;
-	} *m_pLinkResolveState;
+	};
+	std::unique_ptr<t_linkResolveState> m_pLinkResolveState;
 
 	CServerPath MenuMkdir();
 
