@@ -7,15 +7,14 @@
 #include "recursive_operation.h"
 #include "inputdialog.h"
 #include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include <wx/clipbrd.h>
 #include "queue.h"
 #include "QueueView.h"
 #include "themeprovider.h"
 #include "Options.h"
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
+#include <algorithm>
 
 class CItemData : public wxTreeItemData
 {
@@ -24,12 +23,14 @@ public:
 	CServerPath m_path;
 };
 
-class CRemoteTreeViewDropTarget : public wxDropTarget
+class CRemoteTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CRemoteTreeViewDropTarget(CRemoteTreeView* pRemoteTreeView)
-		: m_pRemoteTreeView(pRemoteTreeView), m_pFileDataObject(new wxFileDataObject()),
-		m_pRemoteDataObject(new CRemoteDataObject())
+		: CScrollableDropTarget<wxTreeCtrlEx>(pRemoteTreeView)
+		, m_pRemoteTreeView(pRemoteTreeView)
+		, m_pFileDataObject(new wxFileDataObject())
+		, m_pRemoteDataObject(new CRemoteDataObject())
 	{
 		m_pDataObject = new wxDataObjectComposite;
 		m_pDataObject->Add(m_pRemoteDataObject, true);
@@ -40,8 +41,7 @@ public:
 	void ClearDropHighlight()
 	{
 		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId())
-		{
+		if (dropHighlight != wxTreeItemId()) {
 			m_pRemoteTreeView->SetItemDropHighlight(dropHighlight, false);
 			m_pRemoteTreeView->m_dropHighlight = wxTreeItemId();
 		}
@@ -137,6 +137,9 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
+		if (!CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y)) {
+			return false;
+		}
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
@@ -150,21 +153,19 @@ public:
 		return true;
 	}
 
-	CServerPath DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint point)
 	{
 		wxTreeItemId hit = GetHit(point);
-		if (!hit)
-		{
+		if (!hit) {
 			ClearDropHighlight();
-			return CServerPath();
+			return wxTreeItemId();
 		}
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
 
-		if (path.empty())
-		{
+		if (path.empty()) {
 			ClearDropHighlight();
-			return CServerPath();
+			return wxTreeItemId();
 		}
 
 		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
@@ -174,11 +175,13 @@ public:
 		m_pRemoteTreeView->SetItemDropHighlight(hit, true);
 		m_pRemoteTreeView->m_dropHighlight = hit;
 
-		return path;
+		return hit;
 	}
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -187,8 +190,8 @@ public:
 			return def;
 		}
 
-		const CServerPath& path = DisplayDropHighlight(wxPoint(x, y));
-		if (path.empty())
+		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
+		if (!hit.IsOk())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -199,11 +202,13 @@ public:
 
 	virtual void OnLeave()
 	{
+		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -245,11 +250,13 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 #endif
 
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
 	CreateImageList();
 
+	UpdateSortMode();
+	RegisterOption(OPTION_FILELIST_NAMESORT);
+
 	m_busy = false;
 	m_pQueue = pQueue;
 	AddRoot(_T(""));
@@ -266,47 +273,29 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* v)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(pState->GetRemoteDir(), false);
-	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), v ? *reinterpret_cast<bool const*>(v) : false);
 	else if (notification == STATECHANGE_APPLYFILTER)
-		ApplyFilters();
+		ApplyFilters(false);
 }
 
 void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
 {
 	m_busy = true;
 
-	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
-	{
-	case 0:
-	default:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
-		break;
-	case 1:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
-		break;
-	case 2:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
-		break;
-	}
-
-	if (!pListing)
-	{
+	if (!pListing) {
 		m_ExpandAfterList = wxTreeItemId();
 		DeleteAllItems();
 		AddRoot(_T(""));
 		m_busy = false;
-		if (FindFocus() == this)
-		{
-			wxNavigationKeyEvent evt;
-			evt.SetFromTab(true);
-			evt.SetEventObject(this);
-			evt.SetDirection(true);
-			AddPendingEvent(evt);
+		if (FindFocus() == this) {
+			wxNavigationKeyEvent *evt = new wxNavigationKeyEvent();
+			evt->SetFromTab(true);
+			evt->SetEventObject(this);
+			evt->SetDirection(true);
+			QueueEvent(evt);
 		}
 		Enable(false);
 		m_contextMenuItem = wxTreeItemId();
@@ -379,61 +368,54 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 
 wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 {
-	std::list<wxString> pieces;
-	while (path.HasParent())
-	{
-		pieces.push_front(path.GetLastSegment());
+	std::vector<wxString> pieces;
+	pieces.reserve(path.SegmentCount() + 1);
+	while (path.HasParent()) {
+		pieces.push_back(path.GetLastSegment());
 		path = path.GetParent();
 	}
 	wxASSERT(!path.GetPath().empty());
-	pieces.push_front(path.GetPath());
+	pieces.push_back(path.GetPath());
 
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemId parent = root;
 
-	for (std::list<wxString>::const_iterator iter = pieces.begin(); iter != pieces.end(); ++iter)
-	{
-		if (iter != pieces.begin())
+	for (std::vector<wxString>::const_reverse_iterator iter = pieces.rbegin(); iter != pieces.rend(); ++iter) {
+		if (iter != pieces.rbegin())
 			path.AddSegment(*iter);
 
 		wxTreeItemIdValue cookie;
 		wxTreeItemId child = GetFirstChild(parent, cookie);
-		if (child && GetItemText(child).empty())
-		{
+		if (child && GetItemText(child).empty()) {
 			Delete(child);
 			child = wxTreeItemId();
 			if (parent != root)
 				ListExpand(parent);
 		}
-		for (child = GetFirstChild(parent, cookie); child; child = GetNextSibling(child))
-		{
+		for (child = GetFirstChild(parent, cookie); child; child = GetNextSibling(child)) {
 			const wxString& text = GetItemText(child);
 			if (text == *iter)
 				break;
 		}
-		if (!child)
-		{
+		if (!child) {
 			CDirectoryListing listing;
 
-			if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
-			{
+			if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
 				child = AppendItem(parent, *iter, 0, 2, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, false);
 			}
-			else
-			{
+			else {
 				child = AppendItem(parent, *iter, 1, 3, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, true);
 			}
 			SortChildren(parent);
 
-			std::list<wxString>::const_iterator nextIter = iter;
+			auto nextIter = iter;
 			++nextIter;
-			if (nextIter != pieces.end())
+			if (nextIter != pieces.rend())
 				DisplayItem(child, listing);
 		}
-		if (select && iter != pieces.begin())
-		{
+		if (select && iter != pieces.rbegin()) {
 #ifndef __WXMSW__
 			// Prevent CalculatePositions from being called
 			wxGenericTreeItem *anchor = m_anchor;
@@ -472,13 +454,11 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay /*=_T(""
 	GetSystemImageList()->Draw(index, dc, 0, 0, wxIMAGELIST_DRAW_TRANSPARENT);
 
 	// Load overlay
-	if (!overlay.empty())
-	{
+	if (!overlay.empty()) {
 		wxImage unknownIcon = wxArtProvider::GetBitmap(overlay, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)).ConvertToImage();
 
 		// Convert mask into alpha channel
-		if (unknownIcon.IsOk() && !unknownIcon.HasAlpha())
-		{
+		if (unknownIcon.IsOk() && !unknownIcon.HasAlpha()) {
 			wxASSERT(unknownIcon.HasMask());
 			unknownIcon.InitAlpha();
 		}
@@ -539,8 +519,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	const wxString path = listing.path.GetPath();
 
 	CFilterDialog filter;
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 		if (!listing[i].is_dir())
 			continue;
 
@@ -553,16 +532,14 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 		CDirectoryListing subListing;
 
-		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK)
-		{
+		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter))
 				AppendItem(child, _T(""), -1, -1);
 		}
-		else
-		{
+		else {
 			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
 			SetItemImages(child, true);
 		}
@@ -576,19 +553,17 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	wxTreeItemIdValue cookie;
 	wxTreeItemId child = GetFirstChild(parent, cookie);
-	if (!child || GetItemText(child).empty())
-	{
+	if (!child || GetItemText(child).empty()) {
 		DisplayItem(parent, listing);
 		return;
 	}
 
 	CFilterManager filter;
 
-	const wxString path = listing.path.GetPath();
+	wxString const path = listing.path.GetPath();
 
-	wxArrayString dirs;
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
+	std::vector<wxString> dirs;
+	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 		if (!listing[i].is_dir())
 			continue;
 
@@ -596,25 +571,21 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			dirs.push_back(listing[i].name);
 	}
 
-	dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
+	auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
+	std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
 
 	bool inserted = false;
 	child = GetLastChild(parent);
-	wxArrayString::reverse_iterator iter = dirs.rbegin();
-	while (child && iter != dirs.rend())
-	{
-		int cmp = GetItemText(child).CmpNoCase(*iter);
-		if (!cmp)
-			cmp = GetItemText(child).Cmp(*iter);
+	auto iter = dirs.rbegin();
+	while (child && iter != dirs.rend()) {
+		int cmp = sortFunc(GetItemText(child), *iter);
 
-		if (!cmp)
-		{
-			CServerPath path = listing.path;
-			path.AddSegment(*iter);
+		if (!cmp) {
+			CServerPath childPath = listing.path;
+			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-			{
+			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				if (!GetLastChild(child) && HasSubdirs(subListing, filter))
 					AppendItem(child, _T(""), -1, -1);
 				SetItemImages(child, false);
@@ -625,8 +596,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			child = GetPrevSibling(child);
 			++iter;
 		}
-		else if (cmp > 0)
-		{
+		else if (cmp > 0) {
 			// Child no longer exists
 			wxTreeItemId sel = GetSelection();
 			while (sel && sel != child)
@@ -636,34 +606,34 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 				Delete(child);
 			child = prev;
 		}
-		else if (cmp < 0)
-		{
+		else if (cmp < 0) {
 			// New directory
-			CServerPath path = listing.path;
-			path.AddSegment(*iter);
+			CServerPath childPath = listing.path;
+			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-			{
-				wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
-				SetItemImages(child, false);
+			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+				wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
+				if (childItem) {
+					SetItemImages(childItem, false);
 
-				if (HasSubdirs(subListing, filter))
-					AppendItem(child, _T(""), -1, -1);
+					if (HasSubdirs(subListing, filter)) {
+						AppendItem(childItem, _T(""), -1, -1);
+					}
+				}
 			}
-			else
-			{
-				wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
-				if (child)
-					SetItemImages(child, true);
+			else {
+				wxTreeItemId childItem = AppendItem(parent, *iter, 1, 3, 0);
+				if (childItem) {
+					SetItemImages(childItem, true);
+				}
 			}
 
 			++iter;
 			inserted = true;
 		}
 	}
-	while (child)
-	{
+	while (child) {
 		// Child no longer exists
 		wxTreeItemId sel = GetSelection();
 		while (sel && sel != child)
@@ -673,24 +643,26 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			Delete(child);
 		child = prev;
 	}
-	while (iter != dirs.rend())
-	{
-		CServerPath path = listing.path;
-		path.AddSegment(*iter);
+	while (iter != dirs.rend()) {
+		CServerPath childPath = listing.path;
+		childPath.AddSegment(*iter);
 
 		CDirectoryListing subListing;
-		if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-		{
-			wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
-			SetItemImages(child, false);
+		if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
+			if (childItem) {
+				SetItemImages(childItem, false);
 
-			if (HasSubdirs(subListing, filter))
-				AppendItem(child, _T(""), -1, -1);
+				if (HasSubdirs(subListing, filter)) {
+					AppendItem(childItem, _T(""), -1, -1);
+				}
+			}
 		}
-		else
-		{
-			wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
-			SetItemImages(child, true);
+		else {
+			wxTreeItemId childItem = AppendItem(parent, *iter, 1, 3, 0);
+			if (childItem) {
+				SetItemImages(childItem, true);
+			}
 		}
 
 		++iter;
@@ -701,25 +673,6 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		SortChildren(parent);
 }
 
-int CRemoteTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2)
-{
-	wxString label1 = GetItemText(item1);
-	wxString label2 = GetItemText(item2);
-
-	switch (m_nameSortMode)
-	{
-	case CFileListCtrlSortBase::namesort_casesensitive:
-		return CFileListCtrlSortBase::CmpCase(label1, label2);
-
-	default:
-	case CFileListCtrlSortBase::namesort_caseinsensitive:
-		return CFileListCtrlSortBase::CmpNoCase(label1, label2);
-
-	case CFileListCtrlSortBase::namesort_natural:
-		return CFileListCtrlSortBase::CmpNatural(label1, label2);
-	}
-}
-
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 {
 	if (m_busy)
@@ -768,8 +721,7 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	if (m_busy)
 		return;
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
@@ -821,8 +773,7 @@ CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 {
 	// Drag could result in recursive operation, don't allow at this point
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -831,7 +782,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	if (!item)
 		return;
 
-	const CServerPath& path = GetPathFromItem(item);
+	CServerPath path = GetPathFromItem(item);
 	if (path.empty() || !path.HasParent())
 		return;
 
@@ -842,9 +793,11 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	wxDataObjectComposite object;
 
-	const CServer* const pServer = m_pState->GetServer();
+	CServer const* pServer = m_pState->GetServer();
 	if (!pServer)
 		return;
+	CServer const server = *pServer;
+
 
 	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
@@ -854,9 +807,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	object.Add(pRemoteDataObject, true);
 
 #if FZ3_USESHELLEXT
-	CShellExtensionInterface* ext = CShellExtensionInterface::CreateInitialized();
-	if (ext)
-	{
+	std::unique_ptr<CShellExtensionInterface> ext = CShellExtensionInterface::CreateInitialized();
+	if (ext) {
 		const wxString& file = ext->GetDragDirectory();
 
 		wxASSERT(!file.empty());
@@ -880,46 +832,28 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	pDragDropManager->Release();
 
-	if (res != wxDragCopy)
-	{
-#if FZ3_USESHELLEXT
-		delete ext;
-		ext = 0;
-#endif
+	if (res != wxDragCopy) {
 		return;
 	}
 
 #if FZ3_USESHELLEXT
-	if (ext)
-	{
-		if (!pRemoteDataObject->DidSendData())
-		{
-			const CServer* const pServer = m_pState->GetServer();
-			if (!pServer || !m_pState->IsRemoteIdle())
-			{
+	if (ext) {
+		if (!pRemoteDataObject->DidSendData()) {
+			pServer = m_pState->GetServer();
+			if (!pServer || !m_pState->IsRemoteIdle() || *pServer != server) {
 				wxBell();
-				delete ext;
-				ext = 0;
 				return;
 			}
 
 			CLocalPath target(ext->GetTarget());
-			if (target.empty())
-			{
-				delete ext;
-				ext = 0;
+			if (target.empty()) {
+				ext.reset(); // Release extension before the modal message box
 				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
 				return;
 			}
 
 			m_pState->DownloadDroppedFiles(pRemoteDataObject, target);
-
-			delete ext;
-			ext = 0;
-			return;
 		}
-		delete ext;
-		ext = 0;
 	}
 #endif
 }
@@ -932,8 +866,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
-	if (!m_pState->IsRemoteIdle() || path.empty())
-	{
+	if (!m_pState->IsRemoteIdle() || path.empty()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
@@ -946,8 +879,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	else if (!path.HasParent())
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 
-	if (!m_pState->GetLocalDir().IsWriteable())
-	{
+	if (!m_pState->GetLocalDir().IsWriteable()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -978,17 +910,14 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 	bool cached = false;
 
 	// Obviously item needs to have a parent directory...
-	if (hasParent)
-	{
+	if (hasParent) {
 		const CServerPath& parentPath = path.GetParent();
 		CDirectoryListing listing;
 
 		// ... and it needs to be cached
 		cached = m_pState->m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
-		if (cached)
-		{
-			for (unsigned int i = 0; i < listing.GetCount(); i++)
-			{
+		if (cached) {
+			for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 				if (listing[i].name != name)
 					continue;
 
@@ -997,23 +926,20 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		}
 	}
 
-	if (!pChmodDlg->Create(this, 0, 1, name, permissions))
-	{
+	if (!pChmodDlg->Create(this, 0, 1, name, permissions)) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
 	}
 
-	if (pChmodDlg->ShowModal() != wxID_OK)
-	{
+	if (pChmodDlg->ShowModal() != wxID_OK) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
 	}
 
 	// State may have changed while chmod dialog was shown
-	if (!m_contextMenuItem || !m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle())
-	{
+	if (!m_contextMenuItem || !m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle()) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
@@ -1021,13 +947,10 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 
 	const int applyType = pChmodDlg->GetApplyType();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-
-	if (cached) // Implies hasParent
-	{
+	recursion_root root(hasParent ? path.GetParent() : path, !cached);
+	if (cached) { // Implies hasParent
 		// Change directory permissions
-		if (!applyType || applyType == 2)
-		{
+		if (!applyType || applyType == 2) {
 			wxString newPerms = pChmodDlg->GetPermissions(permissions, true);
 
 			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
@@ -1035,18 +958,18 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 
 		if (pChmodDlg->Recursive())
 			// Start recursion
-			pRecursiveOperation->AddDirectoryToVisit(path, _T(""), CLocalPath());
+			root.add_dir_to_visit(path, _T(""), CLocalPath());
 	}
-	else
-	{
+	else {
 		if (hasParent)
-			pRecursiveOperation->AddDirectoryToVisitRestricted(path.GetParent(), name, pChmodDlg->Recursive());
+			root.add_dir_to_visit_restricted(path.GetParent(), name, pChmodDlg->Recursive());
 		else
-			pRecursiveOperation->AddDirectoryToVisitRestricted(path, _T(""), pChmodDlg->Recursive());
+			root.add_dir_to_visit_restricted(path, _T(""), pChmodDlg->Recursive());
 	}
 
-	if (!cached || pChmodDlg->Recursive())
-	{
+	if (!cached || pChmodDlg->Recursive()) {
+		CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
 
 		CServerPath currentPath;
@@ -1054,14 +977,12 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		if (selected)
 			currentPath = GetPathFromItem(selected);
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, hasParent ? path.GetParent() : path, filter.GetActiveFilters(false), !cached, currentPath);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters(false), currentPath);
 	}
-	else
-	{
+	else {
 		pChmodDlg->Destroy();
 		const wxTreeItemId selected = GetSelection();
-		if (selected)
-		{
+		if (selected) {
 			CServerPath currentPath = GetPathFromItem(selected);
 			m_pState->ChangeRemoteDir(currentPath);
 		}
@@ -1071,8 +992,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 {
 	CLocalPath localDir = m_pState->GetLocalDir();
-	if (!localDir.IsWriteable())
-	{
+	if (!localDir.IsWriteable()) {
 		wxBell();
 		return;
 	}
@@ -1091,8 +1011,10 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 
+	recursion_root root(path, true);
+	root.add_dir_to_visit(path, _T(""), localDir);
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), localDir);
+	pRecursiveOperation->AddRecursionRoot(std::move(root));
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
@@ -1101,7 +1023,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const bool addOnly = event.GetId() == XRCID("ID_ADDTOQUEUE");
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, path, filter.GetActiveFilters(false), true, currentPath);
+	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, filter.GetActiveFilters(false), currentPath);
 }
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
@@ -1112,39 +1034,43 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 	if (!m_contextMenuItem)
 		return;
 
-	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.empty())
+	CServerPath const& pathToDelete = GetPathFromItem(m_contextMenuItem);
+	if (pathToDelete.empty())
 		return;
 
 	if (wxMessageBoxEx(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
 		return;
 
-	const bool hasParent = path.HasParent();
+	const bool hasParent = pathToDelete.HasParent();
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
 
+	recursion_root root;
 	CServerPath startDir;
-	if (hasParent)
-	{
+	if (hasParent) {
 		const wxString& name = GetItemText(m_contextMenuItem);
-		startDir = path.GetParent();
-		pRecursiveOperation->AddDirectoryToVisit(startDir, name);
+		startDir = pathToDelete.GetParent();
+		root = recursion_root(startDir, !hasParent);
+		root.add_dir_to_visit(startDir, name);
 	}
-	else
-	{
-		startDir = path;
-		pRecursiveOperation->AddDirectoryToVisit(startDir, _T(""));
+	else {
+		startDir = pathToDelete;
+		root = recursion_root(startDir, !hasParent);
+		root.add_dir_to_visit(startDir, _T(""));
 	}
+	pRecursiveOperation->AddRecursionRoot(std::move(root));
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (!currentPath.empty() && (path == currentPath || path.IsParentOf(currentPath, false)))
+	if (!currentPath.empty() && (pathToDelete == currentPath || pathToDelete.IsParentOf(currentPath, false))) {
 		currentPath = startDir;
+		m_pState->ChangeRemoteDir(startDir);
+	}
 
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, filter.GetActiveFilters(false), !hasParent, currentPath);
+	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(false), currentPath);
 }
 
 void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
@@ -1167,21 +1093,18 @@ void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 
 void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
 
 	const CServerPath& path = GetPathFromItem(event.GetItem());
-	if (path.empty())
-	{
+	if (path.empty()) {
 		event.Veto();
 		return;
 	}
 
-	if (!path.HasParent())
-	{
+	if (!path.HasParent()) {
 		event.Veto();
 		return;
 	}
@@ -1189,21 +1112,18 @@ void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 
 void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 {
-	if (event.IsEditCancelled())
-	{
+	if (event.IsEditCancelled()) {
 		event.Veto();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
 
 	CItemData* const pData = (CItemData*)GetItemData(event.GetItem());
-	if (pData)
-	{
+	if (pData) {
 		event.Veto();
 		return;
 	}
@@ -1213,8 +1133,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(event.GetItem());
 	const wxString& newName = event.GetLabel();
-	if (oldName == newName)
-	{
+	if (oldName == newName) {
 		event.Veto();
 		return;
 	}
@@ -1229,15 +1148,12 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	if (currentPath.empty())
 		return;
 
-	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false))
-	{
+	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false)) {
 		// Previously selected path was below renamed one, list the new one
 
 		std::list<wxString> subdirs;
-		while (currentPath != old_path)
-		{
-			if (!currentPath.HasParent())
-			{
+		while (currentPath != old_path) {
+			if (!currentPath.HasParent()) {
 				// Abort just in case
 				return;
 			}
@@ -1309,8 +1225,7 @@ CServerPath CRemoteTreeView::MenuMkdir()
 	// replace it with "New directory" later. This way we get the exact position of
 	// "New directory" and can preselect it in the dialog.
 	wxString tmpName = _T("25CF809E56B343b5A12D1F0466E3B37A49A9087FDCF8412AA9AF8D1E849D01CF");
-	if (newPath.AddSegment(tmpName))
-	{
+	if (newPath.AddSegment(tmpName)) {
 		wxString pathName = newPath.GetPath();
 		int pos = pathName.Find(tmpName);
 		wxASSERT(pos != -1);
@@ -1324,8 +1239,7 @@ CServerPath CRemoteTreeView::MenuMkdir()
 		return CServerPath();
 
 	newPath = path;
-	if (!newPath.ChangePath(dlg.GetValue()))
-	{
+	if (!newPath.ChangePath(dlg.GetValue())) {
 		wxBell();
 		return CServerPath();
 	}
@@ -1376,7 +1290,7 @@ struct _parents
 	CServerPath path;
 };
 
-void CRemoteTreeView::ApplyFilters()
+void CRemoteTreeView::ApplyFilters(bool resort)
 {
 	std::list<struct _parents> parents;
 
@@ -1398,6 +1312,10 @@ void CRemoteTreeView::ApplyFilters()
 		struct _parents parent = parents.back();
 		parents.pop_back();
 
+		if (resort) {
+			SortChildren(parent.item);
+		}
+
 		CDirectoryListing listing;
 		if (m_pState->m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
 			RefreshItem(parent.item, listing, false);
@@ -1407,7 +1325,7 @@ void CRemoteTreeView::ApplyFilters()
 				if (path.empty())
 					continue;
 
-				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, CDateTime())) {
+				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, fz::datetime())) {
 					wxTreeItemId sel = GetSelection();
 					while (sel && sel != child)
 						sel = GetItemParent(sel);
@@ -1475,3 +1393,28 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent&)
 	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
 }
+
+void CRemoteTreeView::UpdateSortMode()
+{
+	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	{
+	case 0:
+	default:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
+}
+
+void CRemoteTreeView::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILELIST_NAMESORT)) {
+		UpdateSortMode();
+		ApplyFilters(true);
+	}
+}
