@@ -1,14 +1,14 @@
 #ifndef __REMOTETREEVIEW_H__
 #define __REMOTETREEVIEW_H__
 
+#include <option_change_event_handler.h>
 #include "systemimagelist.h"
 #include "state.h"
 #include "filter.h"
 #include "treectrlex.h"
-#include "filelistctrl.h"
 
 class CQueueView;
-class CRemoteTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler
+class CRemoteTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
 {
 	DECLARE_CLASS(CRemoteTreeView)
 
@@ -30,13 +30,11 @@ protected:
 
 	bool HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter);
 
-	virtual int OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2);
-
 	CServerPath GetPathFromItem(const wxTreeItemId& item) const;
 
 	bool ListExpand(wxTreeItemId item);
 
-	void ApplyFilters();
+	void ApplyFilters(bool resort);
 
 	CQueueView* m_pQueue;
 
@@ -54,6 +52,10 @@ protected:
 
 	CServerPath MenuMkdir();
 
+	void UpdateSortMode();
+
+	virtual void OnOptionsChanged(changed_options_t const& options);
+
 	DECLARE_EVENT_TABLE()
 	void OnItemExpanding(wxTreeEvent& event);
 	void OnSelectionChanged(wxTreeEvent& event);
@@ -72,8 +74,6 @@ protected:
 	void OnMenuGeturl(wxCommandEvent&);
 
 	wxTreeItemId m_contextMenuItem;
-	enum CFileListCtrlSortBase::NameSortMode m_nameSortMode;
-
 };
 
 #endif
