@@ -4,10 +4,6 @@
 
 #include <wx/dcclient.h>
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 #define MAX_LINECOUNT 1000
 #define LINECOUNT_REMOVAL 10
 
@@ -17,7 +13,7 @@ EVT_MENU(XRCID("ID_CLEARALL"), CStatusView::OnClear)
 EVT_MENU(XRCID("ID_COPYTOCLIPBOARD"), CStatusView::OnCopy)
 END_EVENT_TABLE()
 
-class CFastTextCtrl : public wxNavigationEnabled<wxTextCtrl>
+class CFastTextCtrl final : public wxNavigationEnabled<wxTextCtrl>
 {
 public:
 	CFastTextCtrl(wxWindow* parent)
@@ -26,6 +22,8 @@ public:
 			wxNO_BORDER | wxVSCROLL | wxTE_MULTILINE |
 			wxTE_READONLY | wxTE_RICH | wxTE_RICH2 | wxTE_NOHIDESEL |
 			wxTAB_TRAVERSAL);
+
+		SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 	}
 #ifdef __WXMSW__
 	// wxTextCtrl::Remove is somewhat slow, this is a faster version
@@ -108,6 +106,11 @@ CStatusView::CStatusView(wxWindow* parent, wxWindowID id)
 	InitDefAttr();
 
 	m_shown = IsShown();
+
+	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+
+	RegisterOption(OPTION_LANGUAGE);
+	RegisterOption(OPTION_MESSAGELOG_TIMESTAMP);
 }
 
 CStatusView::~CStatusView()
@@ -122,12 +125,12 @@ void CStatusView::OnSize(wxSizeEvent &)
 	}
 }
 
-void CStatusView::AddToLog(CLogmsgNotification *pNotification)
+void CStatusView::AddToLog(CLogmsgNotification const& notification)
 {
-	AddToLog(pNotification->msgType, pNotification->msg, wxDateTime::Now());
+	AddToLog(notification.msgType, notification.msg, fz::datetime::now());
 }
 
-void CStatusView::AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time)
+void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz::datetime const& time)
 {
 	if (!m_shown) {
 		if (m_hiddenLines.size() >= MAX_LINECOUNT) {
@@ -183,10 +186,10 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, con
 		if (time != m_lastTime) {
 			m_lastTime = time;
 #ifndef __WXMAC__
-			m_lastTimeString = time.Format(_T("%H:%M:%S\t"));
+			m_lastTimeString = time.format(_T("%H:%M:%S\t"), fz::datetime::local);
 #else
 			// Tabs on OS X cannot be freely positioned
-			m_lastTimeString = time.Format(_T("%H:%M:%S "));
+			m_lastTimeString = time.format(_T("%H:%M:%S "), fz::datetime::local);
 #endif
 		}
 		prefix += m_lastTimeString;
@@ -255,8 +258,8 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, con
 void CStatusView::InitDefAttr()
 {
 	m_showTimestamps = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-	m_lastTime = wxDateTime::Now();
-	m_lastTimeString = m_lastTime.Format(_T("%H:%M:%S\t"));
+	m_lastTime = fz::datetime::now();
+	m_lastTimeString = m_lastTime.format(_T("%H:%M:%S\t"), fz::datetime::local);
 
 	// Measure withs of all types
 	wxClientDC dc(this);
@@ -266,7 +269,7 @@ void CStatusView::InitDefAttr()
 		wxCoord width = 0;
 		wxCoord height = 0;
 #ifndef __WXMAC__
-		dc.GetTextExtent(_T("88:88:88"), &width, &height);
+		dc.GetTextExtent(_T("88:88:88 "), &width, &height);
 #else
 		dc.GetTextExtent(_T("88:88:88 "), &width, &height);
 #endif
@@ -275,21 +278,21 @@ void CStatusView::InitDefAttr()
 
 	wxCoord width = 0;
 	wxCoord height = 0;
-	dc.GetTextExtent(_("Error:"), &width, &height);
+	dc.GetTextExtent(_("Error:") + _T(" "), &width, &height);
 	int maxPrefixWidth = width;
-	dc.GetTextExtent(_("Command:"), &width, &height);
+	dc.GetTextExtent(_("Command:") + _T(" "), &width, &height);
 	if (width > maxPrefixWidth)
 		maxPrefixWidth = width;
-	dc.GetTextExtent(_("Response:"), &width, &height);
+	dc.GetTextExtent(_("Response:") + _T(" "), &width, &height);
 	if (width > maxPrefixWidth)
 		maxPrefixWidth = width;
-	dc.GetTextExtent(_("Trace:"), &width, &height);
+	dc.GetTextExtent(_("Trace:") + _T(" "), &width, &height);
 	if (width > maxPrefixWidth)
 		maxPrefixWidth = width;
-	dc.GetTextExtent(_("Listing:"), &width, &height);
+	dc.GetTextExtent(_("Listing:") + _T(" "), &width, &height);
 	if (width > maxPrefixWidth)
 		maxPrefixWidth = width;
-	dc.GetTextExtent(_("Status:"), &width, &height);
+	dc.GetTextExtent(_("Status:") + _T(" "), &width, &height);
 	if (width > maxPrefixWidth)
 		maxPrefixWidth = width;
 
@@ -395,7 +398,11 @@ void CStatusView::OnContextMenu(wxContextMenuEvent&)
 	if (!pMenu)
 		return;
 
+	pMenu->Check(XRCID("ID_SHOW_DETAILED_LOG"), COptions::Get()->GetOptionVal(OPTION_LOGGING_SHOW_DETAILED_LOGS) != 0);
+
 	PopupMenu(pMenu);
+
+	COptions::Get()->SetOption(OPTION_LOGGING_SHOW_DETAILED_LOGS, pMenu->IsChecked(XRCID("ID_SHOW_DETAILED_LOG")) ? 1 : 0);
 	delete pMenu;
 }
 
@@ -449,3 +456,8 @@ bool CStatusView::Show(bool show /*=true*/)
 
 	return wxWindow::Show(show);
 }
+
+void CStatusView::OnOptionsChanged(changed_options_t const&)
+{
+	InitDefAttr();
+}
