@@ -13,9 +13,9 @@ public:
 
 protected:
 
-	DECLARE_EVENT_TABLE();
-	void OnOK(wxCommandEvent& event);
-	void OnCopy(wxCommandEvent& event);
+	DECLARE_EVENT_TABLE()
+	void OnOK(wxCommandEvent&);
+	void OnCopy(wxCommandEvent&);
 };
 
 #endif //__ABOUTDIALOG_H__
