@@ -5,6 +5,7 @@
 #include "ipcmutex.h"
 #include "themeprovider.h"
 #include "xmlfunctions.h"
+#include "xrc_helper.h"
 
 BEGIN_EVENT_TABLE(CNewBookmarkDialog, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CNewBookmarkDialog::OnOK)
@@ -36,33 +37,31 @@ int CNewBookmarkDialog::Run(const wxString &local_path, const CServerPath &remot
 	if (!Load(m_parent, _T("ID_NEWBOOKMARK")))
 		return wxID_CANCEL;
 
-	XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl)->ChangeValue(local_path);
+	xrc_call(*this, "ID_LOCALPATH", &wxTextCtrl::ChangeValue, local_path);
 	if (!remote_path.empty())
-		XRCCTRL(*this, "ID_REMOTEPATH", wxTextCtrl)->ChangeValue(remote_path.GetPath());
+		xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::ChangeValue, remote_path.GetPath());
 
 	if (!m_server)
-		XRCCTRL(*this, "ID_TYPE_SITE", wxRadioButton)->Enable(false);
+		xrc_call(*this, "ID_TYPE_SITE", &wxRadioButton::Enable, false);
 
 	return ShowModal();
 }
 
 void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 {
-	const bool global = XRCCTRL(*this, "ID_TYPE_GLOBAL", wxRadioButton)->GetValue();
+	bool const global = xrc_call(*this, "ID_TYPE_GLOBAL", &wxRadioButton::GetValue);
 
-	const wxString name = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
-	if (name.empty())
-	{
+	wxString const name = xrc_call(*this, "ID_NAME", &wxTextCtrl::GetValue);
+	if (name.empty()) {
 		wxMessageBoxEx(_("You need to enter a name for the bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
-	wxString local_path = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl)->GetValue();
-	wxString remote_path_raw = XRCCTRL(*this, "ID_REMOTEPATH", wxTextCtrl)->GetValue();
+	wxString const local_path = xrc_call(*this, "ID_LOCALPATH", &wxTextCtrl::GetValue);
+	wxString remote_path_raw = xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::GetValue);
 
 	CServerPath remote_path;
-	if (!remote_path_raw.empty())
-	{
+	if (!remote_path_raw.empty()) {
 		if (!global && m_server)
 			remote_path.SetType(m_server->GetType());
 		if (!remote_path.SetPath(remote_path_raw))
@@ -72,52 +71,46 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 		}
 	}
 
-	if (local_path.empty() && remote_path_raw.empty())
-	{
+	if (local_path.empty() && remote_path_raw.empty()) {
 		wxMessageBoxEx(_("You need to enter at least one path, empty bookmarks are not supported."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
-	bool sync = XRCCTRL(*this, "ID_SYNC", wxCheckBox)->GetValue();
-	if (sync && (local_path.empty() || remote_path_raw.empty()))
-	{
+	bool const sync = xrc_call(*this, "ID_SYNC", &wxCheckBox::GetValue);
+	if (sync && (local_path.empty() || remote_path_raw.empty())) {
 		wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
-	if (!global && m_server)
-	{
+	bool const comparison = xrc_call(*this, "ID_COMPARISON", &wxCheckBox::GetValue);
+
+	if (!global && m_server) {
 		std::list<wxString> bookmarks;
 
-		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks))
-		{
+		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks)) {
 			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
 				return;
 
 			m_site_path = CSiteManager::AddServer(*m_server);
-			if (m_site_path.empty())
-			{
+			if (m_site_path.empty()) {
 				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				EndModal(wxID_CANCEL);
 				return;
 			}
 		}
-		for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter)
-		{
-			if (*iter == name)
-			{
+		for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter) {
+			if (*iter == name) {
 				wxMessageBoxEx(_("A bookmark with the entered name already exists. Please enter an unused name."), _("New bookmark"), wxICON_EXCLAMATION, this);
 				return;
 			}
 		}
 
-		CSiteManager::AddBookmark(m_site_path, name, local_path, remote_path, sync);
+		CSiteManager::AddBookmark(m_site_path, name, local_path, remote_path, sync, comparison);
 
 		EndModal(wxID_OK);
 	}
-	else
-	{
-		if (!CBookmarksDialog::AddBookmark(name, local_path, remote_path, sync))
+	else {
+		if (!CBookmarksDialog::AddBookmark(name, local_path, remote_path, sync, comparison))
 			return;
 
 		EndModal(wxID_OK);
@@ -126,11 +119,10 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 
 void CNewBookmarkDialog::OnBrowse(wxCommandEvent&)
 {
-	wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
+	wxTextCtrl *const pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
 	wxDirDialog dlg(this, _("Choose the local directory"), pText->GetValue(), wxDD_NEW_DIR_BUTTON);
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		pText->ChangeValue(dlg.GetPath());
 	}
 }
@@ -139,18 +131,19 @@ class CBookmarkItemData : public wxTreeItemData
 {
 public:
 	CBookmarkItemData()
-		: m_sync(false)
 	{
 	}
 
-	CBookmarkItemData(const wxString& local_dir, const CServerPath& remote_dir, bool sync)
+	CBookmarkItemData(const wxString& local_dir, const CServerPath& remote_dir, bool sync, bool comparison)
 		: m_local_dir(local_dir), m_remote_dir(remote_dir), m_sync(sync)
+		, m_comparison(comparison)
 	{
 	}
 
 	wxString m_local_dir;
 	CServerPath m_remote_dir;
-	bool m_sync;
+	bool m_sync{};
+	bool m_comparison{};
 };
 
 CBookmarksDialog::CBookmarksDialog(wxWindow* parent, wxString& site_path, const CServer* server)
@@ -166,27 +159,26 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString name;
 		wxString local_dir;
 		wxString remote_dir_raw;
 		CServerPath remote_dir;
 
-		name = GetTextElement(pBookmark, "Name");
+		name = GetTextElement(bookmark, "Name");
 		if (name.empty())
 			continue;
 
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty())
-		{
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
 			if (!remote_dir.SetSafePath(remote_dir_raw))
 				continue;
 		}
@@ -197,9 +189,11 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 		if (local_dir.empty() || remote_dir.empty())
 			sync = false;
 		else
-			sync = GetTextElementBool(pBookmark, "SyncBrowsing");
+			sync = GetTextElementBool(bookmark, "SyncBrowsing");
 
-		CBookmarkItemData *data = new CBookmarkItemData(local_dir, remote_dir, sync);
+		bool const comparison = GetTextElementBool(bookmark, "DirectoryComparison");
+
+		CBookmarkItemData *data = new CBookmarkItemData(local_dir, remote_dir, sync, comparison);
 		m_pTree->AppendItem(m_bookmarks_global, name, 1, 1, data);
 	}
 
@@ -216,25 +210,23 @@ void CBookmarksDialog::LoadSiteSpecificBookmarks()
 	if (!CSiteManager::GetBookmarks(m_site_path, bookmarks))
 		return;
 
-	for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter)
-	{
+	for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter) {
 		wxString path = *iter;
 		path.Replace(_T("\\"), _T("\\\\"));
 		path.Replace(_T("/"), _T("\\/"));
 		path = m_site_path + _T("/") + path;
 
-		CSiteManagerItemData_Site* data = CSiteManager::GetSiteByPath(path);
+		std::unique_ptr<CSiteManagerItemData_Site> data = CSiteManager::GetSiteByPath(path);
 		if (data) {
-			CBookmarkItemData* new_data = new CBookmarkItemData(data->m_localDir, data->m_remoteDir, data->m_sync);
+			CBookmarkItemData* new_data = new CBookmarkItemData(data->m_localDir, data->m_remoteDir, data->m_sync, data->m_comparison);
 			m_pTree->AppendItem(m_bookmarks_site, *iter, 1, 1, new_data);
-			delete data;
 		}
 	}
 
 	m_pTree->SortChildren(m_bookmarks_site);
 }
 
-int CBookmarksDialog::Run(const wxString &local_path, const CServerPath &remote_path)
+int CBookmarksDialog::Run()
 {
 	if (!Load(m_parent, _T("ID_BOOKMARKS")))
 		return wxID_CANCEL;
@@ -256,8 +248,7 @@ int CBookmarksDialog::Run(const wxString &local_path, const CServerPath &remote_
 	m_bookmarks_global = m_pTree->AppendItem(root, _("Global bookmarks"), 0, 0);
 	LoadGlobalBookmarks();
 	m_pTree->Expand(m_bookmarks_global);
-	if (m_server)
-	{
+	if (m_server) {
 		m_bookmarks_site = m_pTree->AppendItem(root, _("Site-specific bookmarks"), 0, 0);
 		LoadSiteSpecificBookmarks();
 		m_pTree->Expand(m_bookmarks_site);
@@ -269,7 +260,7 @@ int CBookmarksDialog::Run(const wxString &local_path, const CServerPath &remote_
 	wxXmlResource::Get()->LoadPanel(pPanel, pBook, _T("ID_SITEMANAGER_BOOKMARK_PANEL"));
 	pBook->AddPage(pPanel, _("Bookmark"));
 
-	XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->GetContainingSizer()->GetItem((size_t)0)->SetMinSize(200, -1);
+	xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetContainingSizer)->GetItem((size_t)0)->SetMinSize(200, -1);
 
 	GetSizer()->Fit(this);
 
@@ -289,35 +280,37 @@ void CBookmarksDialog::SaveGlobalBookmarks()
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The global bookmarks could not be saved.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark");
-	while (pBookmark)
 	{
-		pDocument->RemoveChild(pBookmark);
-		pBookmark = pDocument->FirstChildElement("Bookmark");
+		auto bookmark = element.child("Bookmark");
+		while (bookmark) {
+			element.remove_child(bookmark);
+			bookmark = element.child("Bookmark");
+		}
 	}
 
 	wxTreeItemIdValue cookie;
-	for (wxTreeItemId child = m_pTree->GetFirstChild(m_bookmarks_global, cookie); child.IsOk(); child = m_pTree->GetNextChild(m_bookmarks_global, cookie))
-	{
+	for (wxTreeItemId child = m_pTree->GetFirstChild(m_bookmarks_global, cookie); child.IsOk(); child = m_pTree->GetNextChild(m_bookmarks_global, cookie)) {
 		CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(child);
 		wxASSERT(data);
 
-		TiXmlElement *pBookmark = pDocument->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
-		AddTextElement(pBookmark, "Name", m_pTree->GetItemText(child));
+		auto bookmark = element.append_child("Bookmark");
+		AddTextElement(bookmark, "Name", m_pTree->GetItemText(child));
 		if (!data->m_local_dir.empty())
-			AddTextElement(pBookmark, "LocalDir", data->m_local_dir);
+			AddTextElement(bookmark, "LocalDir", data->m_local_dir);
 		if (!data->m_remote_dir.empty())
-			AddTextElement(pBookmark, "RemoteDir", data->m_remote_dir.GetSafePath());
+			AddTextElement(bookmark, "RemoteDir", data->m_remote_dir.GetSafePath());
 		if (data->m_sync)
-			AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
+			AddTextElementRaw(bookmark, "SyncBrowsing", "1");
+		if (data->m_comparison)
+			AddTextElementRaw(bookmark, "DirectoryComparison", "1");
 	}
 
 	if (!file.Save(false)) {
@@ -335,17 +328,16 @@ void CBookmarksDialog::SaveSiteSpecificBookmarks()
 			return;
 
 	wxTreeItemIdValue cookie;
-	for (wxTreeItemId child = m_pTree->GetFirstChild(m_bookmarks_site, cookie); child.IsOk(); child = m_pTree->GetNextChild(m_bookmarks_site, cookie))
-	{
+	for (wxTreeItemId child = m_pTree->GetFirstChild(m_bookmarks_site, cookie); child.IsOk(); child = m_pTree->GetNextChild(m_bookmarks_site, cookie)) {
 		CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(child);
 		wxASSERT(data);
 
-		if (!CSiteManager::AddBookmark(m_site_path, m_pTree->GetItemText(child), data->m_local_dir, data->m_remote_dir, data->m_sync))
+		if (!CSiteManager::AddBookmark(m_site_path, m_pTree->GetItemText(child), data->m_local_dir, data->m_remote_dir, data->m_sync, data->m_comparison))
 			return;
 	}
 }
 
-void CBookmarksDialog::OnOK(wxCommandEvent& event)
+void CBookmarksDialog::OnOK(wxCommandEvent&)
 {
 	if (!Verify())
 		return;
@@ -357,7 +349,7 @@ void CBookmarksDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CBookmarksDialog::OnBrowse(wxCommandEvent& event)
+void CBookmarksDialog::OnBrowse(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item)
@@ -370,16 +362,17 @@ void CBookmarksDialog::OnBrowse(wxCommandEvent& event)
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl);
 
 	wxDirDialog dlg(this, _("Choose the local directory"), pText->GetValue(), wxDD_NEW_DIR_BUTTON);
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		pText->ChangeValue(dlg.GetPath());
 	}
 }
 
 void CBookmarksDialog::OnSelChanging(wxTreeEvent& event)
 {
-	if (!Verify())
-	{
+	if (m_is_deleting)
+		return;
+
+	if (!Verify()) {
 		event.Veto();
 		return;
 	}
@@ -387,7 +380,7 @@ void CBookmarksDialog::OnSelChanging(wxTreeEvent& event)
 	UpdateBookmark();
 }
 
-void CBookmarksDialog::OnSelChanged(wxTreeEvent& event)
+void CBookmarksDialog::OnSelChanged(wxTreeEvent&)
 {
 	DisplayBookmark();
 }
@@ -408,17 +401,14 @@ bool CBookmarksDialog::Verify()
 	else
 		server = 0;
 
-	const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
-	if (!remotePathRaw.empty())
-	{
+	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
+	if (!remotePathRaw.empty()) {
 		CServerPath remotePath;
 		if (server)
 			remotePath.SetType(server->GetType());
-		if (!remotePath.SetPath(remotePathRaw))
-		{
-			XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->SetFocus();
-			if (server)
-			{
+		if (!remotePath.SetPath(remotePathRaw)) {
+			xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::SetFocus);
+			if (server) {
 				wxString msg;
 				if (server->GetType() != DEFAULT)
 					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), server->GetNameFromServerType(server->GetType()));
@@ -432,18 +422,16 @@ bool CBookmarksDialog::Verify()
 		}
 	}
 
-	const wxString localPath = XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->GetValue();
+	wxString const localPath = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue);
 
-	if (remotePathRaw.empty() && localPath.empty())
-	{
-		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->SetFocus();
+	if (remotePathRaw.empty() && localPath.empty()) {
+		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::SetFocus);
 		wxMessageBoxEx(_("You need to enter at least one path, empty bookmarks are not supported."));
 		return false;
 	}
 
-	bool sync = XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->GetValue();
-	if (sync && (localPath.empty() || remotePathRaw.empty()))
-	{
+	bool const sync = xrc_call(*this, "ID_BOOKMARK_SYNC", &wxCheckBox::GetValue);
+	if (sync && (localPath.empty() || remotePathRaw.empty())) {
 		wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return false;
 	}
@@ -467,57 +455,56 @@ void CBookmarksDialog::UpdateBookmark()
 	else
 		server = 0;
 
-	const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
-	if (!remotePathRaw.empty())
-	{
+	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
+	if (!remotePathRaw.empty()) {
 		if (server)
 			data->m_remote_dir.SetType(server->GetType());
 		data->m_remote_dir.SetPath(remotePathRaw);
 	}
 
-	data->m_local_dir = XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->GetValue();
+	data->m_local_dir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue);
 
-	data->m_sync = XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->GetValue();
+	data->m_sync = xrc_call(*this, "ID_BOOKMARK_SYNC", &wxCheckBox::GetValue);
+	data->m_comparison = xrc_call(*this, "ID_BOOKMARK_COMPARISON", &wxCheckBox::GetValue);
 }
 
 void CBookmarksDialog::DisplayBookmark()
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item)
-	{
-		XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->ChangeValue(_T(""));
-		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->ChangeValue(_T(""));
-		XRCCTRL(*this, "ID_DELETE", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_RENAME", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_COPY", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook)->Enable(false);
+	if (!item) {
+		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_DELETE", &wxButton::Enable, false);
+		xrc_call(*this, "ID_RENAME", &wxButton::Enable, false);
+		xrc_call(*this, "ID_COPY", &wxButton::Enable, false);
+		xrc_call(*this, "ID_NOTEBOOK", &wxNotebook::Enable, false);
 		return;
 	}
 
 	CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(item);
-	if (!data)
-	{
-		XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->ChangeValue(_T(""));
-		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->ChangeValue(_T(""));
-		XRCCTRL(*this, "ID_DELETE", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_RENAME", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_COPY", wxButton)->Enable(false);
-		XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook)->Enable(false);
+	if (!data) {
+		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_DELETE", &wxButton::Enable, false);
+		xrc_call(*this, "ID_RENAME", &wxButton::Enable, false);
+		xrc_call(*this, "ID_COPY", &wxButton::Enable, false);
+		xrc_call(*this, "ID_NOTEBOOK", &wxNotebook::Enable, false);
 		return;
 	}
 
-	XRCCTRL(*this, "ID_DELETE", wxButton)->Enable(true);
-	XRCCTRL(*this, "ID_RENAME", wxButton)->Enable(true);
-	XRCCTRL(*this, "ID_COPY", wxButton)->Enable(true);
-	XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook)->Enable(true);
+	xrc_call(*this, "ID_DELETE", &wxButton::Enable, true);
+	xrc_call(*this, "ID_RENAME", &wxButton::Enable, true);
+	xrc_call(*this, "ID_COPY", &wxButton::Enable, true);
+	xrc_call(*this, "ID_NOTEBOOK", &wxNotebook::Enable, true);
 
-	XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->ChangeValue(data->m_remote_dir.GetPath());
-	XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->ChangeValue(data->m_local_dir);
+	xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, data->m_remote_dir.GetPath());
+	xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, data->m_local_dir);
 
-	XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetValue(data->m_sync);
+	xrc_call(*this, "ID_BOOKMARK_SYNC", &wxCheckBox::SetValue, data->m_sync);
+	xrc_call(*this, "ID_BOOKMARK_COMPARISON", &wxCheckBox::SetValue, data->m_comparison);
 }
 
-void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
+void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 {
 	if (!Verify())
 		return;
@@ -530,18 +517,15 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
 	if (m_pTree->GetItemData(item))
 		item = m_pTree->GetItemParent(item);
 
-	if (item == m_bookmarks_site)
-	{
+	if (item == m_bookmarks_site) {
 		std::list<wxString> bookmarks;
 
-		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks))
-		{
+		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks)) {
 			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
 				return;
 
 			m_site_path = CSiteManager::AddServer(*m_server);
-			if (m_site_path.empty())
-			{
+			if (m_site_path.empty()) {
 				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				return;
 			}
@@ -550,18 +534,15 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
 
 	wxString newName = _("New bookmark");
 	int index = 2;
-	for (;;)
-	{
+	for (;;) {
 		wxTreeItemId child;
 		wxTreeItemIdValue cookie;
 		child = m_pTree->GetFirstChild(item, cookie);
 		bool found = false;
-		while (child.IsOk())
-		{
+		while (child.IsOk()) {
 			wxString name = m_pTree->GetItemText(child);
 			int cmp = name.CmpNoCase(newName);
-			if (!cmp)
-			{
+			if (!cmp) {
 				found = true;
 				break;
 			}
@@ -580,7 +561,7 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
 	m_pTree->EditLabel(child);
 }
 
-void CBookmarksDialog::OnRename(wxCommandEvent& event)
+void CBookmarksDialog::OnRename(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
@@ -589,7 +570,7 @@ void CBookmarksDialog::OnRename(wxCommandEvent& event)
 	m_pTree->EditLabel(item);
 }
 
-void CBookmarksDialog::OnDelete(wxCommandEvent& event)
+void CBookmarksDialog::OnDelete(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
@@ -597,11 +578,13 @@ void CBookmarksDialog::OnDelete(wxCommandEvent& event)
 
 	wxTreeItemId parent = m_pTree->GetItemParent(item);
 
+	m_is_deleting = true;
 	m_pTree->Delete(item);
 	m_pTree->SelectItem(parent);
+	m_is_deleting = false;
 }
 
-void CBookmarksDialog::OnCopy(wxCommandEvent& event)
+void CBookmarksDialog::OnCopy(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item.IsOk())
@@ -610,7 +593,7 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 	if (!Verify())
 		return;
 
-	CBookmarkItemData* data = reinterpret_cast<CBookmarkItemData *>(m_pTree->GetItemData(item));
+	CBookmarkItemData* data = static_cast<CBookmarkItemData *>(m_pTree->GetItemData(item));
 	if (!data)
 		return;
 
@@ -618,21 +601,18 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 
 	wxTreeItemId parent = m_pTree->GetItemParent(item);
 
-	const wxString name = m_pTree->GetItemText(item);
-	wxString newName = wxString::Format(_("Copy of %s"), name);
+	const wxString oldName = m_pTree->GetItemText(item);
+	wxString newName = wxString::Format(_("Copy of %s"), oldName);
 	int index = 2;
-	for (;;)
-	{
+	for (;;) {
 		wxTreeItemId child;
 		wxTreeItemIdValue cookie;
 		child = m_pTree->GetFirstChild(parent, cookie);
 		bool found = false;
-		while (child.IsOk())
-		{
+		while (child.IsOk()) {
 			wxString name = m_pTree->GetItemText(child);
 			int cmp = name.CmpNoCase(newName);
-			if (!cmp)
-			{
+			if (!cmp) {
 				found = true;
 				break;
 			}
@@ -642,7 +622,7 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 		if (!found)
 			break;
 
-		newName = wxString::Format(_("Copy (%d) of %s"), index++, name);
+		newName = wxString::Format(_("Copy (%d) of %s"), index++, oldName);
 	}
 
 	CBookmarkItemData* newData = new CBookmarkItemData(*data);
@@ -656,17 +636,14 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 void CBookmarksDialog::OnBeginLabelEdit(wxTreeEvent& event)
 {
 	wxTreeItemId item = event.GetItem();
-	if (item != m_pTree->GetSelection())
-	{
-		if (!Verify())
-		{
+	if (item != m_pTree->GetSelection()) {
+		if (!Verify()) {
 			event.Veto();
 			return;
 		}
 	}
 
-	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
-	{
+	if (!item || item == m_bookmarks_global || item == m_bookmarks_site) {
 		event.Veto();
 		return;
 	}
@@ -678,17 +655,14 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 
 	wxTreeItemId item = event.GetItem();
-	if (item != m_pTree->GetSelection())
-	{
-		if (!Verify())
-		{
+	if (item != m_pTree->GetSelection()) {
+		if (!Verify()) {
 			event.Veto();
 			return;
 		}
 	}
 
-	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
-	{
+	if (!item || item == m_bookmarks_global || item == m_bookmarks_site) {
 		event.Veto();
 		return;
 	}
@@ -698,12 +672,10 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 	wxTreeItemId parent = m_pTree->GetItemParent(item);
 
 	wxTreeItemIdValue cookie;
-	for (wxTreeItemId child = m_pTree->GetFirstChild(parent, cookie); child.IsOk(); child = m_pTree->GetNextChild(parent, cookie))
-	{
+	for (wxTreeItemId child = m_pTree->GetFirstChild(parent, cookie); child.IsOk(); child = m_pTree->GetNextChild(parent, cookie)) {
 		if (child == item)
 			continue;
-		if (!name.CmpNoCase(m_pTree->GetItemText(child)))
-		{
+		if (!name.CmpNoCase(m_pTree->GetItemText(child))) {
 			wxMessageBoxEx(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
 			event.Veto();
 			return;
@@ -718,28 +690,26 @@ bool CBookmarksDialog::GetBookmarks(std::list<wxString> &bookmarks)
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString name;
 		wxString local_dir;
 		wxString remote_dir_raw;
 		CServerPath remote_dir;
 
-		name = GetTextElement(pBookmark, "Name");
+		name = GetTextElement(bookmark, "Name");
 		if (name.empty())
 			continue;
 
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty())
-		{
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
 			if (!remote_dir.SetSafePath(remote_dir_raw))
 				continue;
 		}
@@ -752,29 +722,27 @@ bool CBookmarksDialog::GetBookmarks(std::list<wxString> &bookmarks)
 	return true;
 }
 
-bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CServerPath &remote_dir, bool &sync)
+bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CServerPath &remote_dir, bool &sync, bool &comparison)
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString remote_dir_raw;
 
-		if (name != GetTextElement(pBookmark, "Name"))
+		if (name != GetTextElement(bookmark, "Name"))
 			continue;
 
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty())
-		{
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
 			if (!remote_dir.SetSafePath(remote_dir_raw))
 				return false;
 		}
@@ -784,8 +752,9 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 		if (local_dir.empty() || remote_dir_raw.empty())
 			sync = false;
 		else
-			sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+			sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
 
+		comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
 		return true;
 	}
 
@@ -793,7 +762,7 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 }
 
 
-bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
+bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
 	if (local_dir.empty() && remote_dir.empty())
 		return false;
@@ -803,40 +772,41 @@ bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_d
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The bookmark could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	TiXmlElement *pInsertBefore = 0;
-	TiXmlElement *pBookmark;
-	for (pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
+	pugi::xml_node bookmark, insertBefore;
+	for (bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString remote_dir_raw;
 
-		wxString old_name = GetTextElement(pBookmark, "Name");
+		wxString old_name = GetTextElement(bookmark, "Name");
 
 		if (!name.CmpNoCase(old_name)) {
 			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
-		if (name < old_name && !pInsertBefore)
-			pInsertBefore = pBookmark;
+		if (name < old_name && !insertBefore)
+			insertBefore = bookmark;
 	}
 
-	if (pInsertBefore)
-		pBookmark = pDocument->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
+	if (insertBefore)
+		bookmark = element.insert_child_before("Bookmark", insertBefore);
 	else
-		pBookmark = pDocument->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
-	AddTextElement(pBookmark, "Name", name);
+		bookmark = element.append_child("Bookmark");
+	AddTextElement(bookmark, "Name", name);
 	if (!local_dir.empty())
-		AddTextElement(pBookmark, "LocalDir", local_dir);
+		AddTextElement(bookmark, "LocalDir", local_dir);
 	if (!remote_dir.empty())
-		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
+		AddTextElement(bookmark, "RemoteDir", remote_dir.GetSafePath());
 	if (sync)
-		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
+		AddTextElementRaw(bookmark, "SyncBrowsing", "1");
+	if (comparison)
+		AddTextElementRaw(bookmark, "DirectoryComparison", "1");
 
 	if (!file.Save(false)) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the bookmark could not be added: %s"), file.GetFileName(), file.GetError());
