@@ -24,8 +24,7 @@ wxString CBuildInfo::GetBuildDateString()
 
 	wxString month = date.Left(pos);
 	int i = 0;
-	for (i = 0; i < 12; i++)
-	{
+	for (i = 0; i < 12; ++i) {
 		if (months[i] == month)
 			break;
 	}
@@ -53,10 +52,9 @@ wxString CBuildInfo::GetBuildTimeString()
 	return wxString(__TIME__, wxConvLocal);
 }
 
-wxDateTime CBuildInfo::GetBuildDate()
+fz::datetime CBuildInfo::GetBuildDate()
 {
-	wxDateTime date;
-	date.ParseDate(GetBuildDateString());
+	fz::datetime date(GetBuildDateString().ToStdWstring(), fz::datetime::utc);
 	return date;
 }
 
@@ -75,7 +73,7 @@ wxString CBuildInfo::GetCompiler()
 wxString CBuildInfo::GetCompilerFlags()
 {
 #ifndef USED_CXXFLAGS
-	return _T("");
+	return wxString();
 #else
 	wxString flags(USED_CXXFLAGS, wxConvLocal);
 	return flags;
@@ -89,10 +87,10 @@ wxString CBuildInfo::GetBuildType()
 	if (buildtype == _T("official") || buildtype == _T("nightly"))
 		return buildtype;
 #endif
-	return _T("");
+	return wxString();
 }
 
-wxLongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
+int64_t CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 {
 	// Crude conversion from version string into number for easy comparison
 	// Supported version formats:
@@ -115,31 +113,25 @@ wxLongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 	// which in turn corresponds to the simple 64-bit number 2254026754228227
 	// And these can be compared easily
 
-	wxASSERT(*version >= '0' && *version <= '9');
 	if (*version < '0' || *version > '9')
 		return -1;
 
-	wxLongLong v = 0;
-	int segment = 0;
+	int64_t v{};
+	int segment{};
+	int shifts{};
 
-	int shifts = 0;
-
-	for (; *version; version++)
-	{
-		if (*version == '.' || *version == '-' || *version == 'b')
-		{
+	for (; *version; ++version) {
+		if (*version == '.' || *version == '-' || *version == 'b') {
 			v += segment;
 			segment = 0;
 			v <<= 10;
 			shifts++;
 		}
-		if (*version == '-' && shifts < 4)
-		{
+		if (*version == '-' && shifts < 4) {
 			v <<= (4 - shifts) * 10;
 			shifts = 4;
 		}
-		else if (*version >= '0' && *version <= '9')
-		{
+		else if (*version >= '0' && *version <= '9') {
 			segment *= 10;
 			segment += *version - '0';
 		}
@@ -157,7 +149,7 @@ wxLongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 wxString CBuildInfo::GetHostname()
 {
 #ifndef USED_HOST
-	return _T("");
+	return wxString();
 #else
 	wxString flags(USED_HOST, wxConvLocal);
 	return flags;
@@ -167,7 +159,7 @@ wxString CBuildInfo::GetHostname()
 wxString CBuildInfo::GetBuildSystem()
 {
 #ifndef USED_BUILD
-	return _T("");
+	return wxString();
 #else
 	wxString flags(USED_BUILD, wxConvLocal);
 	return flags;
@@ -184,3 +176,72 @@ bool CBuildInfo::IsUnstable()
 
 	return false;
 }
+
+
+#if defined(__i386__) || defined(__x86_64__) || defined(_M_X64) || defined(_M_IX86)
+#define HAVE_CPUID 1
+#endif
+
+#if HAVE_CPUID
+
+#ifdef _MSC_VER
+namespace {
+	void cpuid(int f, int sub, int reg[4])
+	{
+		__cpuidex(reg, f, sub);
+	}
+}
+#else
+#include <cpuid.h>
+namespace {
+	void cpuid(int f, int sub, int reg[4])
+	{
+		__cpuid_count(f, sub, reg[0], reg[1], reg[2], reg[3]);
+	}
+}
+#endif
+#endif
+
+wxString CBuildInfo::GetCPUCaps(char separator)
+{
+	wxString ret;
+
+#if HAVE_CPUID
+	int reg[4];
+	cpuid(0, 0, reg);
+
+	int const max = reg[0];
+
+	// function (aka leave), subfunction (subleave), register, bit, description
+	std::tuple<int, int, int, int, wxString> const caps[] = {
+		std::make_tuple(1, 0, 3, 25, _T("sse")),
+		std::make_tuple(1, 0, 3, 26, _T("sse2")),
+		std::make_tuple(1, 0, 2, 0,  _T("sse3")),
+		std::make_tuple(1, 0, 2, 9,  _T("ssse3")),
+		std::make_tuple(1, 0, 2, 19, _T("sse4.1")),
+		std::make_tuple(1, 0, 2, 20, _T("sse4.2")),
+		std::make_tuple(1, 0, 2, 28, _T("avx")),
+		std::make_tuple(7, 0, 1, 5,  _T("avx2")),
+		std::make_tuple(1, 0, 2, 25, _T("aes")),
+		std::make_tuple(1, 0, 2, 1,  _T("pclmulqdq")),
+		std::make_tuple(1, 0, 2, 30, _T("rdrnd")),
+		std::make_tuple(7, 0, 1, 3,  _T("bmi2")),
+		std::make_tuple(7, 0, 1, 8,  _T("bmi2")),
+		std::make_tuple(7, 0, 1, 19, _T("adx"))
+	};
+
+	for (auto const& cap : caps) {
+		if (max >= std::get<0>(cap)) {
+			cpuid(std::get<0>(cap), std::get<1>(cap), reg);
+			if (reg[std::get<2>(cap)] & (1 << std::get<3>(cap))) {
+				if (!ret.empty()) {
+					ret += separator;
+				}
+				ret += std::get<4>(cap);
+			}
+		}
+	}
+#endif
+
+	return ret;
+}
