@@ -9,6 +9,7 @@
 #include "Options.h"
 #include "queue.h"
 #include "recursive_operation.h"
+#include "recursive_operation_status.h"
 #include "RemoteListView.h"
 #include "RemoteTreeView.h"
 #include "sitemanager.h"
@@ -29,16 +30,13 @@ EVT_MENU(XRCID("ID_TABCONTEXT_CLOSEOTHERS"), CContextControl::OnTabContextCloseO
 EVT_MENU(XRCID("ID_TABCONTEXT_NEW"), CContextControl::OnTabContextNew)
 END_EVENT_TABLE()
 
-CContextControl::CContextControl(CMainFrame* pMainFrame)
-	: CStateEventHandler(0),
-	m_tabs(0), m_right_clicked_tab(-1), m_pMainFrame(pMainFrame)
+CContextControl::CContextControl(CMainFrame& mainFrame)
+	: CStateEventHandler(0)
+	, m_mainFrame(mainFrame)
 {
-	m_current_context_controls = -1;
-	m_tabs = 0;
-
 	wxASSERT(!CContextManager::Get()->HandlerCount(STATECHANGE_CHANGEDCONTEXT));
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false);
 }
 
 CContextControl::~CContextControl()
@@ -53,54 +51,61 @@ void CContextControl::Create(wxWindow *parent)
 void CContextControl::CreateTab()
 {
 	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateTab"));
-#ifndef __WXMAC__
-	// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
-	// on OS X.
-	wxWindowUpdateLocker lock(this);
-#endif
 
-	CState* pState = 0;
+	{
+	#ifdef __WXMSW__
+		// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
+		// on OS X.
+		wxWindowUpdateLocker lock(this);
+	#endif
 
-	// See if we can reuse an existing context
-	for (size_t i = 0; i < m_context_controls.size(); i++) {
-		if (m_context_controls[i].tab_index != -1)
-			continue;
+		CState* pState = 0;
 
-		if (m_context_controls[i].pState->IsRemoteConnected() ||
-			!m_context_controls[i].pState->IsRemoteIdle())
-			continue;
+		// See if we can reuse an existing context
+		for (size_t i = 0; i < m_context_controls.size(); i++) {
+			if (m_context_controls[i].tab_index != -1)
+				continue;
 
-		pState = m_context_controls[i].pState;
-		m_context_controls.erase(m_context_controls.begin() + i);
-		if (m_current_context_controls > (int)i)
-			m_current_context_controls--;
-		break;
-	}
-	if (!pState) {
-		pState = CContextManager::Get()->CreateState(m_pMainFrame);
-		if (!pState->CreateEngine()) {
-			wxMessageBoxEx(_("Failed to initialize FTP engine"));
+			if (m_context_controls[i].pState->IsRemoteConnected() ||
+				!m_context_controls[i].pState->IsRemoteIdle())
+				continue;
+
+			pState = m_context_controls[i].pState;
+			m_context_controls.erase(m_context_controls.begin() + i);
+			if (m_current_context_controls > (int)i)
+				m_current_context_controls--;
+			break;
 		}
+		if (!pState) {
+			pState = CContextManager::Get()->CreateState(m_mainFrame);
+			if (!pState->CreateEngine()) {
+				wxMessageBoxEx(_("Failed to initialize FTP engine"));
+			}
+		}
+
+		// Restore last server and path
+		CServer last_server;
+		CServerPath last_path;
+		if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH)))
+			pState->SetLastServer(last_server, last_path);
+
+		CreateContextControls(pState);
+
+		pState->GetRecursiveOperationHandler()->SetQueue(m_mainFrame.GetQueue());
+
+		wxString const localDir = COptions::Get()->GetOption(OPTION_LASTLOCALDIR);
+		if (!pState->SetLocalDir(localDir)) {
+			wxString const homeDir = wxGetHomeDir();
+			if (!pState->SetLocalDir(homeDir)) {
+				pState->SetLocalDir(_T("/"));
+			}
+		}
+
+		CContextManager::Get()->SetCurrentContext(pState);
 	}
 
-	// Restore last server and path
-	CServer last_server;
-	CServerPath last_path;
-	if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH)))
-		pState->SetLastServer(last_server, last_path);
-
-	CreateContextControls(pState);
-
-	pState->GetRecursiveOperationHandler()->SetQueue(m_pMainFrame->GetQueue());
-
-	wxString localDir = COptions::Get()->GetOption(OPTION_LASTLOCALDIR);
-	if (!pState->SetLocalDir(localDir))
-		pState->SetLocalDir(_T("/"));
-
-	CContextManager::Get()->SetCurrentContext(pState);
-
-	if (!m_pMainFrame->RestoreSplitterPositions())
-		m_pMainFrame->SetDefaultSplitterPositions();
+	if (!m_mainFrame.RestoreSplitterPositions())
+		m_mainFrame.SetDefaultSplitterPositions();
 
 	if (m_tabs)
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
@@ -145,7 +150,7 @@ void CContextControl::CreateContextControls(CState* pState)
 #endif
 		}
 
-		m_pMainFrame->RememberSplitterPositions();
+		m_mainFrame.RememberSplitterPositions();
 		m_context_controls[m_current_context_controls].pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 		m_context_controls[m_current_context_controls].pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
@@ -167,15 +172,15 @@ void CContextControl::CreateContextControls(CState* pState)
 
 	context_controls.pLocalTreeViewPanel = new CView(context_controls.pLocalSplitter);
 	context_controls.pLocalListViewPanel = new CView(context_controls.pLocalSplitter);
-	context_controls.pLocalTreeView = new CLocalTreeView(context_controls.pLocalTreeViewPanel, -1, pState, m_pMainFrame->GetQueue());
-	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, pState, m_pMainFrame->GetQueue());
+	context_controls.pLocalTreeView = new CLocalTreeView(context_controls.pLocalTreeViewPanel, -1, pState, m_mainFrame.GetQueue());
+	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, pState, m_mainFrame.GetQueue());
 	context_controls.pLocalTreeViewPanel->SetWindow(context_controls.pLocalTreeView);
 	context_controls.pLocalListViewPanel->SetWindow(context_controls.pLocalListView);
 
 	context_controls.pRemoteTreeViewPanel = new CView(context_controls.pRemoteSplitter);
 	context_controls.pRemoteListViewPanel = new CView(context_controls.pRemoteSplitter);
-	context_controls.pRemoteTreeView = new CRemoteTreeView(context_controls.pRemoteTreeViewPanel, -1, pState, m_pMainFrame->GetQueue());
-	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, pState, m_pMainFrame->GetQueue());
+	context_controls.pRemoteTreeView = new CRemoteTreeView(context_controls.pRemoteTreeViewPanel, -1, pState, m_mainFrame.GetQueue());
+	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, pState, m_mainFrame.GetQueue());
 	context_controls.pRemoteTreeViewPanel->SetWindow(context_controls.pRemoteTreeView);
 	context_controls.pRemoteListViewPanel->SetWindow(context_controls.pRemoteListView);
 
@@ -186,14 +191,16 @@ void CContextControl::CreateContextControls(CState* pState)
 		pLocalFilelistStatusBar->Hide();
 	context_controls.pLocalListViewPanel->SetStatusBar(pLocalFilelistStatusBar);
 	context_controls.pLocalListView->SetFilelistStatusBar(pLocalFilelistStatusBar);
+	pLocalFilelistStatusBar->SetConnected(true);
 
 	CFilelistStatusBar* pRemoteFilelistStatusBar = new CFilelistStatusBar(context_controls.pRemoteListViewPanel);
 	if (!show_filelist_statusbars)
 		pRemoteFilelistStatusBar->Hide();
 	context_controls.pRemoteListViewPanel->SetStatusBar(pRemoteFilelistStatusBar);
 	context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
-	pRemoteFilelistStatusBar->SetConnected(false);
 
+	auto recursiveStatus = new CRecursiveOperationStatus(context_controls.pRemoteListViewPanel, context_controls.pState);
+	context_controls.pRemoteListViewPanel->SetFooter(recursiveStatus);
 
 	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
 	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
@@ -252,16 +259,16 @@ void CContextControl::CreateContextControls(CState* pState)
 			context_controls.pLocalSplitter->SetSashGravity(1.0);
 	}
 
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalListView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteListView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalTreeView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteTreeView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalViewHeader);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteViewHeader);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalListView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteListView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalTreeView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteTreeView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalViewHeader);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteViewHeader);
 
 	pState->GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
 
-	if (m_tabs) 	{
+	if (m_tabs) {
 		context_controls.tab_index = m_tabs->GetPageCount();
 		m_tabs->AddPage(context_controls.pViewSplitter, pState->GetTitle());
 
@@ -428,14 +435,12 @@ void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent&)
 void CContextControl::OnTabRightclick(wxAuiNotebookEvent& event)
 {
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TABCONTEXT"));
-	if (!pMenu)
-	{
+	if (!pMenu) {
 		wxBell();
 		return;
 	}
 
-	if (!m_tabs || m_tabs->GetPageCount() < 2)
-	{
+	if (!m_tabs || m_tabs->GetPageCount() < 2) {
 		pMenu->Enable(XRCID("ID_TABCONTEXT_CLOSE"), false);
 		pMenu->Enable(XRCID("ID_TABCONTEXT_CLOSEOTHERS"), false);
 	}
@@ -447,21 +452,19 @@ void CContextControl::OnTabRightclick(wxAuiNotebookEvent& event)
 	delete pMenu;
 }
 
-void CContextControl::OnTabContextClose(wxCommandEvent& event)
+void CContextControl::OnTabContextClose(wxCommandEvent&)
 {
 	if (m_right_clicked_tab == -1)
 		return;
 
 	// Need to defer event, wxAUI would write to free'd memory
 	// if we'd actually delete tab and potenially the notebook with it
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, m_right_clicked_tab);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_TAB_CLOSING_DEFERRED, m_right_clicked_tab));
 }
 
-void CContextControl::OnTabContextCloseOthers(wxCommandEvent& event)
+void CContextControl::OnTabContextCloseOthers(wxCommandEvent&)
 {
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, -m_right_clicked_tab - 1);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent (fzEVT_TAB_CLOSING_DEFERRED, -m_right_clicked_tab - 1));
 }
 
 void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
@@ -502,8 +505,7 @@ void CContextControl::OnTabClosing(wxAuiNotebookEvent& event)
 {
 	// Need to defer event, wxAUI would write to free'd memory
 	// if we'd actually delete tab and potenially the notebook with it
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, event.GetSelection());
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_TAB_CLOSING_DEFERRED, event.GetSelection()));
 
 	event.Veto();
 }
