@@ -9,14 +9,13 @@ public:
 	bool Load(wxWindow *pParent, const wxString& name);
 
 	bool SetChildLabel(int id, const wxString& label, unsigned long maxLength = 0);
+	bool SetChildLabel(char const* id, const wxString& label, unsigned long maxLength = 0);
 	wxString GetChildLabel(int id);
 
 	virtual int ShowModal();
 
 	bool ReplaceControl(wxWindow* old, wxWindow* wnd);
 
-	static int ShownDialogs() { return m_shown_dialogs; }
-
 	static bool CanShowPopupDialog();
 protected:
 	virtual void InitDialog();
