@@ -1,12 +1,15 @@
 #include <filezilla.h>
-#include "edithandler.h"
+#include "conditionaldialog.h"
 #include "dialogex.h"
+#include "edithandler.h"
 #include "filezillaapp.h"
 #include "file_utils.h"
-#include "queue.h"
 #include "Options.h"
+#include "queue.h"
 #include "window_state_manager.h"
-#include "local_filesys.h"
+#include "xrc_helper.h"
+
+#include <libfilezilla/local_filesys.hpp>
 
 class CChangedFileDialog : public wxDialogEx
 {
@@ -20,12 +23,12 @@ EVT_BUTTON(wxID_YES, CChangedFileDialog::OnYes)
 EVT_BUTTON(wxID_NO, CChangedFileDialog::OnNo)
 END_EVENT_TABLE()
 
-void CChangedFileDialog::OnYes(wxCommandEvent& event)
+void CChangedFileDialog::OnYes(wxCommandEvent&)
 {
 	EndDialog(wxID_YES);
 }
 
-void CChangedFileDialog::OnNo(wxCommandEvent& event)
+void CChangedFileDialog::OnNo(wxCommandEvent&)
 {
 	EndDialog(wxID_NO);
 }
@@ -69,7 +72,7 @@ CEditHandler* CEditHandler::Get()
 	return m_pEditHandler;
 }
 
-void CEditHandler::RemoveTemporaryFiles(const wxString& temp)
+void CEditHandler::RemoveTemporaryFiles(wxString const& temp)
 {
 	wxDir dir(temp);
 	if (!dir.IsOpened())
@@ -79,52 +82,61 @@ void CEditHandler::RemoveTemporaryFiles(const wxString& temp)
 	if (!dir.GetFirst(&file, _T("fz3temp-*"), wxDIR_DIRS))
 		return;
 
-	const wxChar& sep = wxFileName::GetPathSeparator();
-	do
-	{
-		const wxString lockfile = temp + file + sep + _("fz3temp-lockfile");
-		if (wxFileName::FileExists(lockfile))
-		{
-#ifndef __WXMSW__
-			int fd = open(lockfile.mb_str(), O_RDWR, 0);
-			if (fd >= 0)
-			{
-				// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-				struct flock f = {0};
-				f.l_type = F_WRLCK;
-				f.l_whence = SEEK_SET;
-				f.l_start = 0;
-				f.l_len = 1;
-				f.l_pid = getpid();
-				if (fcntl(fd, F_SETLK, &f)) {
-					// In use by other process
-					close(fd);
-					continue;
-				}
-				close(fd);
-			}
-#endif
-			{
-				wxLogNull log;
-				wxRemoveFile(lockfile);
-			}
-			if (wxFileName::FileExists(lockfile))
-				continue;
+	wxChar const& sep = wxFileName::GetPathSeparator();
+	do {
+		if (!m_localDir.empty() && temp + file + sep == m_localDir) {
+			// Don't delete own working directory
+			continue;
 		}
 
+		RemoveTemporaryFilesInSpecificDir(temp + file + sep);
+	} while (dir.GetNext(&file));
+}
+
+void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
+{
+	const wxString lockfile = temp + _("fz3temp-lockfile");
+	if (wxFileName::FileExists(lockfile)) {
+#ifndef __WXMSW__
+		int fd = open(lockfile.mb_str(), O_RDWR | O_CLOEXEC, 0);
+		if (fd >= 0)
+		{
+			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
+			struct flock f = {};
+			f.l_type = F_WRLCK;
+			f.l_whence = SEEK_SET;
+			f.l_start = 0;
+			f.l_len = 1;
+			f.l_pid = getpid();
+			if (fcntl(fd, F_SETLK, &f)) {
+				// In use by other process
+				close(fd);
+				return;
+			}
+			close(fd);
+		}
+#endif
 		{
 			wxLogNull log;
-			wxString file2;
-			wxDir dir2(temp + file);
-			bool res;
-			for ((res = dir2.GetFirst(&file2, _T(""), wxDIR_FILES)); res; res = dir2.GetNext(&file2)) {
-				wxRemoveFile(temp + file + sep + file2);
-			}
+			wxRemoveFile(lockfile);
 		}
+		if (wxFileName::FileExists(lockfile))
+			return;
+	}
+
+	wxLogNull log;
+
+	{
+		wxString file;
+		wxDir dir(temp);
+		bool res;
+		for ((res = dir.GetFirst(&file, _T(""), wxDIR_FILES)); res; res = dir.GetNext(&file)) {
+			wxRemoveFile(temp + file);
+		}
+	}
+
+	wxRmdir(temp);
 
-		wxLogNull log;
-		wxRmdir(temp + file + sep);
-	} while (dir.GetNext(&file));
 }
 
 wxString CEditHandler::GetLocalDirectory()
@@ -143,15 +155,12 @@ wxString CEditHandler::GetLocalDirectory()
 	if (dir.Last() != wxFileName::GetPathSeparator())
 		dir += wxFileName::GetPathSeparator();
 
-	RemoveTemporaryFiles(dir);
-
 	// On POSIX, the permissions of the created directory (700) ensure
 	// that this is a safe operation.
 	// On Windows, the user's profile directory and associated temp dir
 	// already has the correct permissions which get inherited.
 	int i = 1;
-	do
-	{
+	do {
 		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), i++);
 		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir))
 			continue;
@@ -163,8 +172,14 @@ wxString CEditHandler::GetLocalDirectory()
 		break;
 	} while (true);
 
+	// Defer deleting stale directories until after having created our own
+	// working directory.
+	// This avoids some strange errors where freshly deleted directories
+	// cannot be instantly recreated.
+	RemoveTemporaryFiles(dir);
+
 #ifdef __WXMSW__
-	m_lockfile_handle = ::CreateFile(m_localDir + _T("fz3temp-lockfile"), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
+	m_lockfile_handle = ::CreateFile((m_localDir + _T("fz3temp-lockfile")).wc_str(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
 	if (m_lockfile_handle == INVALID_HANDLE_VALUE)
 	{
 		wxRmdir(m_localDir);
@@ -172,11 +187,11 @@ wxString CEditHandler::GetLocalDirectory()
 	}
 #else
 	wxString file = m_localDir + _T("fz3temp-lockfile");
-	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR, 0600);
+	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
 	if (m_lockfile_descriptor >= 0)
 	{
 		// Lock 1 byte region in the lockfile.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = 0;
@@ -196,8 +211,7 @@ void CEditHandler::Release()
 	if (m_busyTimer.IsRunning())
 		m_busyTimer.Stop();
 
-	if (!m_localDir.empty())
-	{
+	if (!m_localDir.empty()) {
 #ifdef __WXMSW__
 		if (m_lockfile_handle != INVALID_HANDLE_VALUE)
 			CloseHandle(m_lockfile_handle);
@@ -208,13 +222,10 @@ void CEditHandler::Release()
 			close(m_lockfile_descriptor);
 #endif
 
-		{
-			wxLogNull log;
-			wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
-		}
-
+		wxLogNull log;
+		wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
 		RemoveAll(true);
-		wxRmdir(m_localDir);
+		RemoveTemporaryFilesInSpecificDir(m_localDir);
 	}
 
 	m_pEditHandler = 0;
@@ -242,37 +253,30 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandler::fileState state, const CServer* pServer /*=0*/) const
 {
 	int count = 0;
-	if (state == unknown)
-	{
+	if (state == unknown) {
 		wxASSERT(!pServer);
 		if (type != remote)
 			count += m_fileDataList[local].size();
 		if (type != local)
 			count += m_fileDataList[remote].size();
 	}
-	else
-	{
-		if (type != remote)
-		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-			{
-				if (iter->state != state)
+	else {
+		auto f = [state, pServer](decltype(m_fileDataList[0]) & items) {
+			int count = 0;
+			for (auto const& data : items) {
+				if (data.state != state)
 					continue;
 
-				if (!pServer || iter->server == *pServer)
-					count++;
+				if (!pServer || data.server == *pServer)
+					++count;
 			}
+			return count;
+		};
+		if (type != remote) {
+			count += f(m_fileDataList[local]);
 		}
-		if (type != local)
-		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-			{
-				if (iter->state != state)
-					continue;
-
-				if (!pServer || iter->server == *pServer)
-					count++;
-			}
+		if (type != local) {
+			count += f(m_fileDataList[remote]);
 		}
 	}
 
@@ -283,27 +287,24 @@ bool CEditHandler::AddFile(enum CEditHandler::fileType type, wxString& fileName,
 {
 	wxASSERT(type != none);
 
-	enum fileState state;
+	fileState state;
 	if (type == local)
 		state = GetFileState(fileName);
 	else
 		state = GetFileState(fileName, remotePath, server);
-	if (state != unknown)
-	{
+	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
 		return false;
 	}
 
 	t_fileData data;
-	if (type == remote)
-	{
+	if (type == remote) {
 		data.state = download;
 		data.name = fileName;
 		data.file = GetTemporaryFile(fileName);
 		fileName = data.file;
 	}
-	else
-	{
+	else {
 		data.file = fileName;
 		data.name = wxFileName(fileName).GetFullName();
 		data.state = edit;
@@ -363,18 +364,14 @@ bool CEditHandler::RemoveAll(bool force)
 {
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-		if (!force && (iter->state == download || iter->state == upload || iter->state == upload_and_remove))
-		{
+	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
+		if (!force && (iter->state == download || iter->state == upload || iter->state == upload_and_remove)) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (wxFileName::FileExists(iter->file))
-		{
-			if (!wxRemoveFile(iter->file))
-			{
+		if (wxFileName::FileExists(iter->file)) {
+			if (!wxRemoveFile(iter->file)) {
 				iter->state = removing;
 				keep.push_back(*iter);
 				continue;
@@ -384,13 +381,11 @@ bool CEditHandler::RemoveAll(bool force)
 	m_fileDataList[remote].swap(keep);
 	keep.clear();
 
-	for (auto iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (auto iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (force)
 			continue;
 
-		if (iter->state == upload || iter->state == upload_and_remove)
-		{
+		if (iter->state == upload || iter->state == upload_and_remove) {
 			keep.push_back(*iter);
 			continue;
 		}
@@ -409,24 +404,19 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 
 	std::list<t_fileData> keep;
 
-	for (auto iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-		if (iter->state != state)
-		{
+	for (auto iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
+		if (iter->state != state) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (pServer && iter->server != *pServer)
-		{
+		if (pServer && iter->server != *pServer) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (wxFileName::FileExists(iter->file))
-		{
-			if (!wxRemoveFile(iter->file))
-			{
+		if (wxFileName::FileExists(iter->file)) {
+			if (!wxRemoveFile(iter->file)) {
 				iter->state = removing;
 				keep.push_back(*iter);
 				continue;
@@ -441,8 +431,7 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (iter->file == fileName)
 			break;
 	}
@@ -453,8 +442,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (iter->file == fileName)
 			break;
 	}
@@ -465,8 +453,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
 		if (iter->name != fileName)
 			continue;
 
@@ -485,8 +472,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
 		if (iter->name != fileName)
 			continue;
 
@@ -502,7 +488,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
+void CEditHandler::FinishTransfer(bool, const wxString& fileName)
 {
 	auto iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
@@ -539,15 +525,13 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 	switch (iter->state)
 	{
 	case upload_and_remove:
-		if (successful)
-		{
+		if (successful) {
 			if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file))
 				iter->state = removing;
 			else
 				m_fileDataList[remote].erase(iter);
 		}
-		else
-		{
+		else {
 			if (!wxFileName::FileExists(iter->file))
 				m_fileDataList[remote].erase(iter);
 			else
@@ -561,8 +545,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 			m_fileDataList[remote].erase(iter);
 		break;
 	case download:
-		if (wxFileName::FileExists(iter->file))
-		{
+		if (wxFileName::FileExists(iter->file)) {
 			iter->state = edit;
 			if (StartEditing(remote, *iter))
 				break;
@@ -603,7 +586,7 @@ bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& da
 	wxASSERT(data.state == edit);
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(data.file, is_link, 0, &data.modificationTime, 0) != CLocalFileSystem::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file)
 		return false;
 
 	bool program_exists = false;
@@ -623,49 +606,44 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	if (insideCheckForModifications)
 		return;
 
-	if (emitEvent)
-	{
-		wxCommandEvent evt(fzEDIT_CHANGEDFILE);
-		AddPendingEvent(evt);
+	if (emitEvent) {
+		QueueEvent(new wxCommandEvent(fzEDIT_CHANGEDFILE));
 		return;
 	}
 
 	insideCheckForModifications = true;
 
-	for (int i = 0; i < 2; i++)
-	{
+	for (int i = 0; i < 2; ++i) {
 checkmodifications_loopbegin:
-		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
-		{
+		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter) {
 			if (iter->state != edit)
 				continue;
 
-			CDateTime mtime;
+			fz::datetime mtime;
 			bool is_link;
-			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
+			if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 
 				// Evil goto. Imo the next C++ standard needs a comefrom keyword.
 				goto checkmodifications_loopbegin;
 			}
 
-			if (!mtime.IsValid())
+			if (!mtime.empty())
 				continue;
 
-			if (iter->modificationTime.IsValid() && !iter->modificationTime.Compare(mtime))
+			if (iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
 
 			m_busyTimer.Stop();
-			if( !wxDialogEx::CanShowPopupDialog() ) {
+			if (!wxDialogEx::CanShowPopupDialog() ) {
 				m_busyTimer.Start(1000, true);
 				insideCheckForModifications = false;
 				return;
 			}
 			wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
-			if (pTopWindow && pTopWindow->IsIconized())
-			{
+			if (pTopWindow && pTopWindow->IsIconized()) {
 				pTopWindow->RequestUserAttention(wxUSER_ATTENTION_INFO);
 				insideCheckForModifications = false;
 				return;
@@ -676,30 +654,24 @@ checkmodifications_loopbegin:
 			if (res == -1)
 				continue;
 
-			if (res == wxID_YES)
-			{
+			if (res == wxID_YES) {
 				UploadFile(CEditHandler::fileType(i), iter, remove);
 				goto checkmodifications_loopbegin;
 			}
-			else if (remove)
-			{
-				if (i == remote)
-				{
-					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file))
-					{
+			else if (remove) {
+				if (i == static_cast<int>(remote)) {
+					if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file || wxRemoveFile(iter->file)) {
 						m_fileDataList[i].erase(iter);
 						goto checkmodifications_loopbegin;
 					}
 					iter->state = removing;
 				}
-				else
-				{
+				else {
 					m_fileDataList[i].erase(iter);
 					goto checkmodifications_loopbegin;
 				}
 			}
-			else if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file)
-			{
+			else if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
 			}
@@ -725,20 +697,19 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 
 	dlg.SetChildLabel(XRCID("ID_FILENAME"), iter->name);
 
-	if (type == local)
-	{
+	if (type == local) {
 		XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
 		XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
+
+		dlg.SetChildLabel("ID_DELETE", _T("&Finish editing"));
 	}
-	else
-	{
+	else {
 		wxString file = iter->file;
 		int pos = file.Find(wxFileName::GetPathSeparator(), true);
 		wxASSERT(pos != -1);
 		file = file.Mid(pos + 1);
 
-		if (file == iter->name)
-		{
+		if (file == iter->name) {
 			XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
 			XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
 		}
@@ -772,7 +743,7 @@ bool CEditHandler::UploadFile(const wxString& file, bool unedit)
 bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
 	wxCHECK(type != none, false);
-		
+
 	if (iter == m_fileDataList[type].end())
 		return false;
 
@@ -782,18 +753,17 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 
 	iter->state = unedit ? upload_and_remove : upload;
 
-	wxLongLong size;
-	CDateTime mtime;
+	int64_t size;
+	fz::datetime mtime;
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(iter->file, is_link, &size, &mtime, 0) != CLocalFileSystem::file)
-	{
+	if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, &size, &mtime, 0) != fz::local_filesys::file) {
 		m_fileDataList[type].erase(iter);
 		return false;
 	}
 
-	if (!mtime.IsValid())
-		mtime = CDateTime::Now();
+	if (!mtime.empty())
+		mtime = fz::datetime::now();
 
 	iter->modificationTime = mtime;
 
@@ -813,7 +783,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	return true;
 }
 
-void CEditHandler::OnTimerEvent(wxTimerEvent& event)
+void CEditHandler::OnTimerEvent(wxTimerEvent&)
 {
 #ifdef __WXMSW__
 	// Don't check for changes if mouse is captured,
@@ -865,33 +835,30 @@ wxString CEditHandler::CanOpen(enum CEditHandler::fileType type, const wxString&
 
 wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists)
 {
-	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT))
-	{
+	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT)) {
 		const wxString command = GetCustomOpenCommand(file, program_exists);
 		if (!command.empty())
 			return command;
 
-		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS))
-		{
-			const wxString command = GetSystemOpenCommand(file, program_exists);
-			if (!command.empty())
-				return command;
+		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS)) {
+			const wxString sysCommand = GetSystemOpenCommand(file, program_exists);
+			if (!sysCommand.empty())
+				return sysCommand;
 		}
 	}
 
 	wxString command = COptions::Get()->GetOption(OPTION_EDIT_DEFAULTEDITOR);
 	if (command.empty() || command[0] == '0')
 		return wxString(); // None set
-	else if (command[0] == '1')
-	{
+	else if (command[0] == '1') {
 		// Text editor
 		const wxString random = _T("5AC2EE515D18406 space aB77C2C60F1F88952.txt"); // Chosen by fair dice roll. Guaranteed to be random.
-		wxString command = GetSystemOpenCommand(random, program_exists);
-		if (command.empty() || !program_exists)
-			return command;
+		wxString sysCommand = GetSystemOpenCommand(random, program_exists);
+		if (sysCommand.empty() || !program_exists)
+			return sysCommand;
 
-		command.Replace(random, file);
-		return command;
+		sysCommand.Replace(random, file);
+		return sysCommand;
 	}
 	else if (command[0] == '2')
 		command = command.Mid(1);
@@ -967,7 +934,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 	return wxString();
 }
 
-void CEditHandler::OnChangedFileEvent(wxCommandEvent& event)
+void CEditHandler::OnChangedFileEvent(wxCommandEvent&)
 {
 	CheckForModifications();
 }
@@ -1046,20 +1013,19 @@ wxString CEditHandler::TruncateFilename(const wxString path, const wxString& nam
 
 bool CEditHandler::FilenameExists(const wxString& file)
 {
-	for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-#ifdef __WXMSW__
-		if (!iter->file.CmpNoCase(file))
-#else
-		if (iter->file == file)
-#endif
+	for (auto const& fileData : m_fileDataList[remote]) {
+		// Always ignore case, we don't know which type of filesystem the user profile
+		// is installed upon.
+		if (!fileData.file.CmpNoCase(file))
 			return true;
 	}
 
-	if (wxFileName::FileExists(file))
-	{
+	if (wxFileName::FileExists(file)) {
 		// Save to remove, it's not marked as edited anymore.
-		wxRemoveFile(file);
+		{
+			wxLogNull log;
+			wxRemoveFile(file);
+		}
 
 		if (wxFileName::FileExists(file))
 			return true;
@@ -1235,12 +1201,12 @@ void CEditHandlerStatusDialog::SetCtrlState()
 	XRCCTRL(*this, "ID_EDIT", wxWindow)->Enable(select);
 }
 
-void CEditHandlerStatusDialog::OnSelectionChanged(wxListEvent& event)
+void CEditHandlerStatusDialog::OnSelectionChanged(wxListEvent&)
 {
 	SetCtrlState();
 }
 
-void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
+void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
 	if (!pEditHandler)
@@ -1250,13 +1216,11 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
-		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed)
-		{
+		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
 			return;
 		}
@@ -1264,23 +1228,19 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
-		if (type == CEditHandler::local)
-		{
+		if (type == CEditHandler::local) {
 			pEditHandler->Remove(pData->file);
 			delete pData;
 			pListCtrl->DeleteItem(i);
 		}
-		else
-		{
-			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server))
-			{
+		else {
+			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
 				delete pData;
 				pListCtrl->DeleteItem(i);
 			}
@@ -1302,23 +1262,20 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
-		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed)
-		{
+		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
 			return;
 		}
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
@@ -1342,7 +1299,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	SetCtrlState();
 }
 
-void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
+void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
 	if (!pEditHandler)
@@ -1352,34 +1309,28 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
-		if (pData->state != CEditHandler::edit)
-		{
+		if (pData->state != CEditHandler::edit) {
 			wxBell();
 			return;
 		}
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
-		if (type == CEditHandler::local)
-		{
-			if (!pEditHandler->StartEditing(pData->file))
-			{
-				if (pEditHandler->Remove(pData->file))
-				{
+		if (type == CEditHandler::local) {
+			if (!pEditHandler->StartEditing(pData->file)) {
+				if (pEditHandler->Remove(pData->file)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
@@ -1387,12 +1338,9 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 					pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
 			}
 		}
-		else
-		{
-			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server))
-			{
-				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server))
-				{
+		else {
+			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server)) {
+				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
@@ -1451,15 +1399,16 @@ bool CNewAssociationDialog::Run(const wxString &file)
 	else
 		m_ext.clear();
 
-	wxStaticText *pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
-	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
+	wxStaticText *const pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
+	if (pDesc) {
+		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
+	}
 
 	bool program_exists = false;
 	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 	if (!program_exists)
 		cmd.clear();
-	if (!cmd.empty())
-	{
+	if (!cmd.empty()) {
 		wxString args;
 		if (!UnquoteCommand(cmd, args))
 			cmd.clear();
@@ -1468,17 +1417,17 @@ bool CNewAssociationDialog::Run(const wxString &file)
 	if (!PathExpand(cmd))
 		cmd.clear();
 
-	pDesc = XRCCTRL(*this, "ID_EDITOR_DESC", wxStaticText);
-	if (cmd.empty())
-	{
-		XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->SetValue(true);
-		XRCCTRL(*this, "ID_USE_EDITOR", wxRadioButton)->Enable(false);
-		pDesc->Hide();
+	if (cmd.empty()) {
+		xrc_call(*this, "ID_USE_CUSTOM", &wxRadioButton::SetValue, true);
+		xrc_call(*this, "ID_USE_EDITOR", &wxRadioButton::Enable, false);
+		xrc_call(*this, "ID_EDITOR_DESC", &wxStaticText::Hide);
 	}
-	else
-	{
-		XRCCTRL(*this, "ID_EDITOR_DESC_NONE", wxStaticText)->Hide();
-		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd));
+	else {
+		xrc_call(*this, "ID_EDITOR_DESC_NONE", &wxStaticText::Hide);
+		wxStaticText* const pEditorDesc = XRCCTRL(*this, "ID_EDITOR_DESC", wxStaticText);
+		if (pEditorDesc) {
+			pEditorDesc->SetLabel(wxString::Format(pEditorDesc->GetLabel(), cmd));
+		}
 	}
 
 	SetCtrlState();
@@ -1494,50 +1443,45 @@ bool CNewAssociationDialog::Run(const wxString &file)
 void CNewAssociationDialog::SetCtrlState()
 {
 	wxRadioButton* pCustom = wxDynamicCast(FindWindow(XRCID("ID_USE_CUSTOM")), wxRadioButton);
-	if (!pCustom)
-	{
+	if (!pCustom) {
 		// Return since it can get called before dialog got fully loaded
 		return;
 	}
 
-	const bool custom = pCustom->GetValue();
+	bool const custom = pCustom->GetValue();
 
-	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->Enable(custom);
-	XRCCTRL(*this, "ID_BROWSE", wxButton)->Enable(custom);
+	xrc_call(*this, "ID_CUSTOM", &wxTextCtrl::Enable, custom);
+	xrc_call(*this, "ID_BROWSE", &wxButton::Enable, custom);
 }
 
-void CNewAssociationDialog::OnRadioButton(wxCommandEvent& event)
+void CNewAssociationDialog::OnRadioButton(wxCommandEvent&)
 {
 	SetCtrlState();
 }
 
-void CNewAssociationDialog::OnOK(wxCommandEvent& event)
+void CNewAssociationDialog::OnOK(wxCommandEvent&)
 {
 	const bool custom = XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->GetValue();
 	const bool always = XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->GetValue();
 
-	if (custom)
-	{
+	if (custom) {
 		wxString cmd = XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->GetValue();
 		wxString editor = cmd;
 		wxString args;
-		if (!UnquoteCommand(editor, args) || editor.empty())
-		{
+		if (!UnquoteCommand(editor, args) || editor.empty()) {
 			wxMessageBoxEx(_("You need to enter a properly quoted command."), _("Cannot set file association"), wxICON_EXCLAMATION);
 			return;
 		}
-		if (!ProgramExists(editor))
-		{
+		if (!ProgramExists(editor)) {
 			wxMessageBoxEx(_("Selected editor does not exist."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 			return;
 		}
 
 		if (always)
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd);
-		else
-		{
+		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
-			if (associations.Last() != '\n')
+			if (!associations.empty() && associations.Last() != '\n')
 				associations += '\n';
 			if (m_ext.empty())
 				m_ext = _T("/");
@@ -1545,18 +1489,15 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
 		}
 	}
-	else
-	{
+	else {
 		if (always)
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("1"));
-		else
-		{
+		else {
 			bool program_exists = false;
 			wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 			if (!program_exists)
 				cmd.clear();
-			if (!cmd.empty())
-			{
+			if (!cmd.empty()) {
 				wxString args;
 				if (!UnquoteCommand(cmd, args))
 					cmd.clear();
@@ -1585,7 +1526,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
+void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 {
 	wxFileDialog dlg(this, _("Select default editor"), _T(""), _T(""),
 #ifdef __WXMSW__
@@ -1604,8 +1545,7 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 	if (editor.empty())
 		return;
 
-	if (!ProgramExists(editor))
-	{
+	if (!ProgramExists(editor)) {
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
 		wxMessageBoxEx(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
 		return;
@@ -1616,3 +1556,197 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
+
+bool CEditHandler::Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, int64_t size, wxWindow * parent)
+{
+	std::vector<FileData> data;
+	FileData d{fileName, size};
+	data.push_back(d);
+
+	return Edit(type, data, path, server, parent);
+}
+
+bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, CServer const& server, wxWindow * parent)
+{
+	if (type == CEditHandler::remote) {
+		wxString const& localDir = GetLocalDirectory();
+		if (localDir.empty()) {
+			wxMessageBoxEx(_("Could not get temporary directory to download file into."), _("Cannot edit file"), wxICON_STOP);
+			return false;
+		}
+	}
+
+	if (data.empty()) {
+		wxBell();
+		return false;
+	}
+
+	if (data.size() > 10) {
+		CConditionalDialog dlg(parent, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return false;
+	}
+
+	bool success = true;
+	int already_editing_action{};
+	for (auto const& file : data) {
+		if (!DoEdit(type, file, path, server, parent, data.size(), already_editing_action)) {
+			success = false;
+		}
+	}
+
+	return success;
+}
+
+bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, CServer const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
+{
+	bool dangerous = false;
+	bool program_exists = false;
+	wxString cmd = CanOpen(type, file.name, dangerous, program_exists);
+	if (cmd.empty()) {
+		CNewAssociationDialog dlg(parent);
+		if (!dlg.Run(file.name)) {
+			return false;
+		}
+		cmd = CanOpen(type, file.name, dangerous, program_exists);
+		if (cmd.empty()) {
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), file.name), _("Opening failed"), wxICON_EXCLAMATION);
+			return false;
+		}
+	}
+	if (!program_exists) {
+		wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), file.name, cmd);
+		wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+		return false;
+	}
+	if (dangerous) {
+		int res = wxMessageBoxEx(_("The selected file would be executed directly.\nThis can be dangerous and might damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
+		if (res != wxYES) {
+			wxBell();
+			return false;
+		}
+	}
+
+	fileState state;
+	if (type == local)
+		state = GetFileState(file.name);
+	else
+		state = GetFileState(file.name, path, server);
+	switch (state)
+	{
+	case CEditHandler::download:
+	case CEditHandler::upload:
+	case CEditHandler::upload_and_remove:
+	case CEditHandler::upload_and_remove_failed:
+		wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+		return false;
+	case CEditHandler::removing:
+		if (!Remove(file.name, path, server)) {
+			wxMessageBoxEx(_("A file with that name is still being edited. Please close it and try again."), _("Selected file is already opened"), wxICON_EXCLAMATION);
+			return false;
+		}
+		break;
+	case CEditHandler::edit:
+		{
+			int action = already_editing_action;
+			if (!action) {
+				wxDialogEx dlg;
+				if (!dlg.Load(parent, type == CEditHandler::local ? _T("ID_EDITEXISTING_LOCAL") : _T("ID_EDITEXISTING_REMOTE"))) {
+					wxBell();
+					return false;
+				}
+				dlg.SetChildLabel(XRCID("ID_FILENAME"), file.name);
+
+				int choices = COptions::Get()->GetOptionVal(OPTION_PERSISTENT_CHOICES);
+
+				if (fileCount < 2) {
+					xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::Hide);
+				}
+				else {
+					if (choices & edit_choices::edit_existing_always) {
+						xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::SetValue, true);
+					}
+				}
+
+				if (type == CEditHandler::remote && (choices & edit_choices::edit_existing_action)) {
+					xrc_call(dlg, "ID_RETRANSFER", &wxRadioButton::SetValue, true);
+				}
+
+				dlg.GetSizer()->Fit(&dlg);
+				int res = dlg.ShowModal();
+				if (res != wxID_OK && res != wxID_YES) {
+					wxBell();
+					action = -1;
+				}
+				else if (type == CEditHandler::local || xrc_call(dlg, "ID_REOPEN", &wxRadioButton::GetValue)) {
+					action = 1;
+					if (type == CEditHandler::remote) {
+						choices &= ~edit_choices::edit_existing_action;
+					}
+				}
+				else {
+					action = 2;
+					choices |= edit_choices::edit_existing_action;
+				}
+
+				bool always = xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::GetValue);
+				if (always) {
+					already_editing_action = action;
+					choices |= edit_choices::edit_existing_always;
+				}
+				else {
+					choices &= ~edit_choices::edit_existing_always;
+				}
+				COptions::Get()->SetOption(OPTION_PERSISTENT_CHOICES, choices);
+			}
+
+			if (action == -1) {
+				return false;
+			}
+			else if (action == 1) {
+				if (type == CEditHandler::local) {
+					StartEditing(file.name);
+				}
+				else {
+					StartEditing(file.name, path, server);
+				}
+				return true;
+			}
+			else {
+				if (!Remove(file.name, path, server)) {
+					wxMessageBoxEx(_("The selected file is still opened in some other program, please close it."), _("Selected file is still being edited"), wxICON_EXCLAMATION);
+					return false;
+				}
+			}
+		}
+		break;
+	default:
+		break;
+	}
+
+	wxString localFile = file.name;
+	if (!AddFile(type, localFile, path, server)) {
+		if( type == CEditHandler::local) {
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
+		}
+		else {
+			wxFAIL;
+			wxBell();
+			return false;
+		}
+	}
+
+	if (type == CEditHandler::remote) {
+		wxString localFileName;
+		CLocalPath localPath(localFile, &localFileName);
+
+		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : wxString(),
+			localPath, path, server, file.size, type, QueuePriority::high);
+		m_pQueue->QueueFile_Finish(true);
+	}
+
+	return true;
+}
