@@ -3,8 +3,18 @@
 
 #include "dialogex.h"
 
+#include <wx/timer.h>
+
 // Handles all aspects about remote file viewing/editing
 
+namespace edit_choices {
+enum type
+{
+	edit_existing_action = 0x1,
+	edit_existing_always = 0x2
+};
+}
+
 class CQueueView;
 class CEditHandler : protected wxEvtHandler
 {
@@ -20,7 +30,7 @@ public:
 		removing
 	};
 
-	enum fileType
+	enum fileType : signed char
 	{
 		none = -1,
 		local,
@@ -43,6 +53,21 @@ public:
 	// pServer may be set only if state isn't unknown
 	int GetFileCount(enum fileType type, enum fileState state, const CServer* pServer = 0) const;
 
+	// Starts editing the given file, queues it if needed. For local files, fileName must include local path.
+	// Can be used to edit files already being added, user is prompted for action.
+	bool Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, int64_t size, wxWindow* parent);
+
+	class FileData final {
+	public:
+		FileData() = default;
+		FileData(wxString const& n, int64_t s)
+			: name(n), size(s) {}
+
+		wxString name;
+		int64_t size{};
+	};
+	bool Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, CServer const& server, wxWindow* parent);
+
 	// Adds the file that doesn't exist yet. (Has to be in unknown state)
 	// The initial state will be download
 	bool AddFile(enum fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server);
@@ -77,12 +102,12 @@ public:
 		wxString name; // The name of the file
 		wxString file; // The actual local filename
 		fileState state;
-		CDateTime modificationTime;
+		fz::datetime modificationTime;
 		CServerPath remotePath;
 		CServer server;
 	};
 
-	const std::list<t_fileData>& GetFiles(enum fileType type) const { wxASSERT(type != none); return m_fileDataList[type]; }
+	const std::list<t_fileData>& GetFiles(enum fileType type) const { wxASSERT(type != none); return m_fileDataList[(type == local) ? 0 : 1]; }
 
 	bool UploadFile(const wxString& file, bool unedit);
 	bool UploadFile(const wxString& file, const CServerPath& remotePath, const CServer& server, bool unedit);
@@ -92,6 +117,8 @@ public:
 	wxString GetOpenCommand(const wxString& file, bool& program_exists);
 
 protected:
+	bool DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, CServer const& server, wxWindow* parent, size_t fileCount, int & already_editing_action);
+
 	CEditHandler();
 	virtual ~CEditHandler() {}
 
@@ -119,7 +146,8 @@ protected:
 	wxTimer m_timer;
 	wxTimer m_busyTimer;
 
-	void RemoveTemporaryFiles(const wxString& temp);
+	void RemoveTemporaryFiles(wxString const& temp);
+	void RemoveTemporaryFilesInSpecificDir(wxString const& temp);
 
 	wxString GetTemporaryFile(wxString name);
 	wxString TruncateFilename(const wxString path, const wxString& name, int max);
