@@ -260,7 +260,7 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	m_focusItem = -1;
 #endif
 
-#if defined __WXGTK__ && !defined(__WXGTK3__)
+#if defined(__WXGTK__) && !defined(__WXGTK3__)
 	m_gtkEventCallbackProxy.reset(new CGtkEventCallbackProxy<CFileData>(this));
 
 	GtkWidget* widget = GetMainWindow()->GetConnectWidget();
@@ -269,6 +269,11 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 
 	m_genericTypes[genericTypes::file] = _("File");
 	m_genericTypes[genericTypes::directory] = _("Directory");
+
+	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+#ifndef __WXMSW__
+	GetMainWindow()->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+#endif
 }
 
 template<class CFileData> CFileListCtrl<CFileData>::~CFileListCtrl()
@@ -279,8 +284,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 {
 	CancelLabelEdit();
 
-	if (column != -1)
-	{
+	if (column != -1) {
 		if (column != m_sortColumn)
 		{
 			const int oldVisibleColumn = GetColumnVisibleIndex(m_sortColumn);
@@ -295,8 +299,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		direction = m_sortDirection;
 
 	int newVisibleColumn = GetColumnVisibleIndex(column);
-	if (newVisibleColumn == -1)
-	{
+	if (newVisibleColumn == -1) {
 		newVisibleColumn = 0;
 		column = 0;
 	}
@@ -305,24 +308,29 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 
 	// Remember which files are selected
 	bool *selected = 0;
-	int focused = -1;
-	if (updateSelections)
-	{
-		selected = new bool[m_fileData.size()];
-		memset(selected, 0, sizeof(bool) * m_fileData.size());
+	int focused_item = -1;
+	unsigned int focused_index{};
 
+	if (updateSelections) {
 #ifndef __WXMSW__
 		// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
 		if (GetSelectedItemCount())
 #endif
 		{
-			int item = -1;
-			while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-				selected[m_indexMapping[item]] = 1;
+			int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+			if (item != -1) {
+				selected = new bool[m_fileData.size()];
+				memset(selected, 0, sizeof(bool) * m_fileData.size());
+			
+				do {
+					selected[m_indexMapping[item]] = 1;
+					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+				} while (item != -1);
+			}
 		}
-		focused = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-		if (focused != -1)
-			focused = m_indexMapping[focused];
+		focused_item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+		if (focused_item != -1)
+			focused_index = m_indexMapping[focused_item];
 	}
 
 	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
@@ -338,9 +346,8 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 			++start;
 		std::reverse(start, m_indexMapping.end());
 
-		if (updateSelections)
-		{
-			SortList_UpdateSelections(selected, focused);
+		if (updateSelections) {
+			SortList_UpdateSelections(selected, focused_item, focused_index);
 			delete [] selected;
 		}
 
@@ -351,10 +358,8 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	m_sortColumn = column;
 
 	const unsigned int minsize = m_hasParent ? 3 : 2;
-	if (m_indexMapping.size() < minsize)
-	{
-		if (updateSelections)
-			delete [] selected;
+	if (m_indexMapping.size() < minsize) {
+		delete [] selected;
 		return;
 	}
 
@@ -365,33 +370,34 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	std::sort(start, m_indexMapping.end(), object);
 	object.Destroy();
 
-	if (updateSelections)
-	{
-		SortList_UpdateSelections(selected, focused);
+	if (updateSelections) {
+		SortList_UpdateSelections(selected, focused_item, focused_index);
 		delete [] selected;
 	}
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelections(bool* selections, int focus)
+template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelections(bool* selections, int focused_item, unsigned int focused_index)
 {
-	for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
-	{
-		const int state = GetItemState(i, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
-		const bool selected = (state & wxLIST_STATE_SELECTED) != 0;
-		const bool focused = (state & wxLIST_STATE_FOCUSED) != 0;
+	if (focused_item >= 0) {
+		if (m_indexMapping[focused_item] != focused_index) {
+			SetItemState(focused_item, 0, wxLIST_STATE_FOCUSED);
 
-		int item = m_indexMapping[i];
-		if (selections[item] != selected)
-			SetSelection(i, selections[item]);
-		if (focused)
-		{
-			if (item != focus)
-				SetItemState(i, 0, wxLIST_STATE_FOCUSED);
+			for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
+				if (m_indexMapping[i] == focused_index) {
+					SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+				}
+			}
 		}
-		else
-		{
-			if (item == focus)
-				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+	}
+	
+	if (selections) {
+		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
+			const int state = GetItemState(i, wxLIST_STATE_SELECTED);
+			const bool selected = (state & wxLIST_STATE_SELECTED) != 0;
+
+			int item = m_indexMapping[i];
+			if (selections[item] != selected)
+				SetSelection(i, selections[item]);
 		}
 	}
 }
@@ -449,8 +455,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 	if (col == -1)
 		return;
 
-	if (IsComparing())
-	{
+	if (IsComparing()) {
 #ifdef __WXMSW__
 		ReleaseCapture();
 		Refresh();
@@ -481,7 +486,7 @@ wxString GetExt(const wxString& file)
 	wxString ret;
 
 	int pos = file.Find('.', true);
-	if (pos > 0 && file[pos + 1] != 0) // Does neither starts nor end with dot
+	if (pos > 0 && (static_cast<size_t>(pos) + 1) < file.size()) // Does neither starts nor end with dot
 		ret = file.Mid(pos + 1);
 
 	return ret;
@@ -509,7 +514,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 
 	SHFILEINFO shFinfo;
 	memset(&shFinfo, 0, sizeof(SHFILEINFO));
-	if (SHGetFileInfo(name,
+	if (SHGetFileInfo(name.wc_str(),
 		dir ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
 		&shFinfo,
 		sizeof(shFinfo),
@@ -609,9 +614,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnPostScroll()
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 {
+	if (m_originalIndexMapping.empty()) {
+		return;
+	}
+
 	ComparisonRememberSelections();
 
-	wxASSERT(!m_originalIndexMapping.empty());
 	m_indexMapping.clear();
 	m_indexMapping.swap(m_originalIndexMapping);
 
@@ -627,8 +635,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 
 template<class CFileData> void CFileListCtrl<CFileData>::CompareAddFile(t_fileEntryFlags flags)
 {
-	if (flags == fill)
-	{
+	if (flags == fill) {
 		m_indexMapping.push_back(m_fileData.size() - 1);
 		return;
 	}
@@ -647,8 +654,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 		return;
 
 	int focus = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-	if (focus != -1)
-	{
+	if (focus != -1) {
 		SetItemState(focus, 0, wxLIST_STATE_FOCUSED);
 		int index = m_indexMapping[focus];
 		if (m_fileData[index].comparison_flags == fill)
@@ -835,12 +841,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnFocusChanged(wxListEv
 	const int focusItem = event.GetIndex();
 
 	// Need to defer processing, as focus it set before selection by wxWidgets internally
-	wxCommandEvent evt;
-	evt.SetEventType(fz_EVT_FILELIST_FOCUSCHANGE);
-	evt.SetInt(m_focusItem);
-	evt.SetExtraLong((long)focusItem);
+	wxCommandEvent *evt = new wxCommandEvent();
+	evt->SetEventType(fz_EVT_FILELIST_FOCUSCHANGE);
+	evt->SetInt(m_focusItem);
+	evt->SetExtraLong((long)focusItem);
 	m_pending_focus_processing++;
-	AddPendingEvent(evt);
+	QueueEvent(evt);
 
 	m_focusItem = focusItem;
 }
@@ -936,9 +942,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnLeftDown(wxMouseEvent
 	// Left clicks in the whitespace around the items deselect everything
 	// but does not change focus. Defer event.
 	event.Skip();
-	wxCommandEvent evt;
-	evt.SetEventType(fz_EVT_DEFERRED_MOUSEEVENT);
-	AddPendingEvent(evt);
+	wxCommandEvent *evt = new wxCommandEvent();
+	evt->SetEventType(fz_EVT_DEFERRED_MOUSEEVENT);
+	QueueEvent(evt);
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent&)
