@@ -5,6 +5,7 @@
 #include "systemimagelist.h"
 #include "listingcomparison.h"
 
+#include <cstring>
 #include <memory>
 
 class CQueueView;
@@ -63,12 +64,25 @@ public:
 				return false;\
 		}
 
-	static int CmpCase(const wxString& str1, const wxString& str2)
+	static int CmpCase(std::wstring const& str1, std::wstring const& str2)
+	{
+		return str1.compare(str2);
+	}
+
+	static int CmpCase(wxString const& str1, wxString const& str2)
 	{
 		return str1.Cmp(str2);
 	}
 
-	static int CmpNoCase(const wxString& str1, const wxString& str2)
+	static int CmpNoCase(std::wstring const& str1, std::wstring const& str2)
+	{
+		int cmp = fz::stricmp(str1, str2);
+		if (cmp)
+			return cmp;
+		return str1.compare(str2);
+	}
+
+	static int CmpNoCase(wxString const& str1, wxString const& str2)
 	{
 		int cmp = str1.CmpNoCase(str2);
 		if (cmp)
@@ -76,15 +90,22 @@ public:
 		return str1.Cmp(str2);
 	}
 
-	static int CmpNatural(const wxString& str1, const wxString& str2)
+	static int CmpNatural(wxString const& str1, wxString const& str2)
 	{
-		wxString::const_iterator p1 = str1.begin();
-		wxString::const_iterator p2 = str2.begin();
+		return CmpNatural(static_cast<wxChar const*>(str1.c_str()), static_cast<wxChar const*>(str2.c_str()));
+	}
 
+	static int CmpNatural(std::wstring const& str1, std::wstring const& str2)
+	{
+		return CmpNatural(str1.c_str(), str2.c_str());
+	}
+
+	static int CmpNatural(wxChar const* p1, wxChar const* p2)
+	{
 		int res = 0;
 		int zeroCount = 0;
 		bool isNumber = false;
-		for (; p1 != str1.end() && p2 != str2.end(); ++p1, ++p2) {
+		for (; *p1 && *p2; ++p1, ++p2) {
 			int diff = static_cast<int>(wxTolower(*p1)) - static_cast<int>(wxTolower(*p2));
 			if (isNumber) {
 				if (res == 0)
@@ -98,9 +119,9 @@ public:
 					break;
 			} else if (wxIsdigit(*p1) && wxIsdigit(*p2)) {
 				zeroCount = 0;
-				for(; *p1 == '0' && p1+1 != str1.end() && wxIsdigit(*(p1+1)); ++p1)
+				for(; *p1 == '0' && *(p1+1) && wxIsdigit(*(p1+1)); ++p1)
 					zeroCount++;
-				for(; *p2 == '0' && p2+1 != str2.end() && wxIsdigit(*(p2+1)); ++p2)
+				for(; *p2 == '0' && *(p2+1) && wxIsdigit(*(p2+1)); ++p2)
 					zeroCount--;
 				res = *p1 - *p2;
 				isNumber = true;
@@ -111,13 +132,13 @@ public:
 		if (res == 0 && isNumber)
 			res = zeroCount;
 
-		if (p1 == str1.end() && p2 == str2.end())
+		if (!*p1 && !*p2)
 			return res;
 		if (!isNumber || res == 0)
-			 return p1 == str1.end() ? -1 : 1;
-		if (p1 != str1.end() && wxIsdigit(*p1))
+			 return !*p1 ? -1 : 1;
+		if (*p1 && wxIsdigit(*p1))
 			return 1;       //more digits
-		if (p2 != str2.end() && wxIsdigit(*p2))
+		if (*p2 && wxIsdigit(*p2))
 			return -1;      //fewer digits
 		return res;         //same length, compare first different digit in the sequence
 	}
@@ -214,7 +235,7 @@ public:
 
 	inline int CmpSize(const value_type &data1, const value_type &data2) const
 	{
-		const wxLongLong diff = data1.size - data2.size;
+		int64_t const diff = data1.size - data2.size;
 		if (diff < 0)
 			return -1;
 		else if (diff > 0)
@@ -559,7 +580,7 @@ protected:
 #endif
 
 private:
-	void SortList_UpdateSelections(bool* selections, int focus);
+	void SortList_UpdateSelections(bool* selections, int focused_item, unsigned int focused_index);
 
 	// If this is set to true, don't process selection changed events
 	bool m_insideSetSelection;
