@@ -4,12 +4,13 @@
 #include "filezillaapp.h"
 #include "inputdialog.h"
 #include "ipcmutex.h"
-#include "local_filesys.h"
 #include "Mainfrm.h"
 #include "Options.h"
 #include "state.h"
 #include "xmlfunctions.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 #include <wx/regex.h>
 
 bool CFilterManager::m_loaded = false;
@@ -125,12 +126,12 @@ void CFilterDialog::OnOkOrApply(wxCommandEvent& event)
 	}
 }
 
-void CFilterDialog::OnCancel(wxCommandEvent& event)
+void CFilterDialog::OnCancel(wxCommandEvent&)
 {
 	EndModal(wxID_CANCEL);
 }
 
-void CFilterDialog::OnEdit(wxCommandEvent& event)
+void CFilterDialog::OnEdit(wxCommandEvent&)
 {
 	CFilterEditDialog dlg;
 	if (!dlg.Create(this, m_filters, m_filterSets))
@@ -146,17 +147,16 @@ void CFilterDialog::OnEdit(wxCommandEvent& event)
 	DisplayFilters();
 }
 
-void CFilterDialog::SaveFilter(TiXmlElement* pElement, const CFilter& filter)
+void CFilterDialog::SaveFilter(pugi::xml_node& element, const CFilter& filter)
 {
-	AddTextElement(pElement, "Name", filter.name);
-	AddTextElement(pElement, "ApplyToFiles", filter.filterFiles ? _T("1") : _T("0"));
-	AddTextElement(pElement, "ApplyToDirs", filter.filterDirs ? _T("1") : _T("0"));
-	AddTextElement(pElement, "MatchType", (filter.matchType == CFilter::any) ? _T("Any") : ((filter.matchType == CFilter::none) ? _T("None") : _T("All")));
-	AddTextElement(pElement, "MatchCase", filter.matchCase ? _T("1") : _T("0"));
+	AddTextElement(element, "Name", filter.name);
+	AddTextElement(element, "ApplyToFiles", filter.filterFiles ? _T("1") : _T("0"));
+	AddTextElement(element, "ApplyToDirs", filter.filterDirs ? _T("1") : _T("0"));
+	AddTextElement(element, "MatchType", (filter.matchType == CFilter::any) ? _T("Any") : ((filter.matchType == CFilter::none) ? _T("None") : _T("All")));
+	AddTextElement(element, "MatchCase", filter.matchCase ? _T("1") : _T("0"));
 
-	TiXmlElement* pConditions = pElement->LinkEndChild(new TiXmlElement("Conditions"))->ToElement();
-	for (std::vector<CFilterCondition>::const_iterator conditionIter = filter.filters.begin(); conditionIter != filter.filters.end(); ++conditionIter)
-	{
+	auto xConditions = element.append_child("Conditions");
+	for (std::vector<CFilterCondition>::const_iterator conditionIter = filter.filters.begin(); conditionIter != filter.filters.end(); ++conditionIter) {
 		const CFilterCondition& condition = *conditionIter;
 
 		int type;
@@ -185,22 +185,21 @@ void CFilterDialog::SaveFilter(TiXmlElement* pElement, const CFilter& filter)
 			continue;
 		}
 
-		TiXmlElement* pCondition = pConditions->LinkEndChild(new TiXmlElement("Condition"))->ToElement();
-		AddTextElement(pCondition, "Type", type);
+		auto xCondition = xConditions.append_child("Condition");
+		AddTextElement(xCondition, "Type", type);
 
-		if (condition.type == filter_size)
-		{
+		if (condition.type == filter_size) {
 			// Backwards compatibility sucks
 			int v = condition.condition;
 			if (v == 2)
 				v = 3;
 			else if (v > 2)
 				--v;
-			AddTextElement(pCondition, "Condition", v);
+			AddTextElement(xCondition, "Condition", v);
 		}
 		else
-			AddTextElement(pCondition, "Condition", condition.condition);
-		AddTextElement(pCondition, "Value", condition.strValue);
+			AddTextElement(xCondition, "Condition", condition.condition);
+		AddTextElement(xCondition, "Value", condition.strValue);
 	}
 }
 
@@ -209,55 +208,47 @@ void CFilterDialog::SaveFilters()
 	CInterProcessMutex mutex(MUTEX_FILTERS);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("filters")));
-	TiXmlElement* pDocument = xml.Load();
-	if (!pDocument) {
+	auto element = xml.Load();
+	if (!element) {
 		wxString msg = xml.GetError() + _T("\n\n") + _("Any changes made to the filters could not be saved.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	TiXmlElement *pFilters = pDocument->FirstChildElement("Filters");
-	while (pFilters)
-	{
-		pDocument->RemoveChild(pFilters);
-		pFilters = pDocument->FirstChildElement("Filters");
+	auto xFilters = element.child("Filters");
+	while (xFilters) {
+		element.remove_child(xFilters);
+		xFilters = element.child("Filters");
 	}
 
-	pFilters = pDocument->LinkEndChild(new TiXmlElement("Filters"))->ToElement();
+	xFilters = element.append_child("Filters");
 
-	for (std::vector<CFilter>::const_iterator iter = m_globalFilters.begin(); iter != m_globalFilters.end(); ++iter)
-	{
-		const CFilter& filter = *iter;
-
-		TiXmlElement* pElement = new TiXmlElement("Filter");
-		SaveFilter(pElement, filter);
-		pFilters->LinkEndChild(pElement);
+	for (auto const& filter : m_globalFilters) {
+		auto xFilter = xFilters.append_child("Filter");
+		SaveFilter(xFilter, filter);
 	}
 
-	TiXmlElement *pSets = pDocument->FirstChildElement("Sets");
-	while (pSets)
-	{
-		pDocument->RemoveChild(pSets);
-		pSets = pDocument->FirstChildElement("Sets");
+	auto xSets = element.child("Sets");
+	while (xSets) {
+		element.remove_child(xSets);
+		xSets = element.child("Sets");
 	}
 
-	pSets = pDocument->LinkEndChild(new TiXmlElement("Sets"))->ToElement();
-	SetTextAttribute(pSets, "Current", wxString::Format(_T("%d"), m_currentFilterSet));
+	xSets = element.append_child("Sets");
+	SetAttributeInt(xSets, "Current", m_currentFilterSet);
 
-	for (std::vector<CFilterSet>::const_iterator iter = m_globalFilterSets.begin(); iter != m_globalFilterSets.end(); ++iter)
-	{
-		const CFilterSet& set = *iter;
-		TiXmlElement* pSet = pSets->LinkEndChild(new TiXmlElement("Set"))->ToElement();
+	for (auto const& set : m_globalFilterSets) {
+		auto xSet = xSets.append_child("Set");
 
-		if (iter != m_globalFilterSets.begin())
-			AddTextElement(pSet, "Name", set.name);
+		if (!set.name.empty()) {
+			AddTextElement(xSet, "Name", set.name);
+		}
 
-		for (unsigned int i = 0; i < set.local.size(); i++)
-		{
-			TiXmlElement* pItem = pSet->LinkEndChild(new TiXmlElement("Item"))->ToElement();
-			AddTextElement(pItem, "Local", set.local[i] ? _T("1") : _T("0"));
-			AddTextElement(pItem, "Remote", set.remote[i] ? _T("1") : _T("0"));
+		for (unsigned int i = 0; i < set.local.size(); ++i) {
+			auto xItem = xSet.append_child("Item");
+			AddTextElement(xItem, "Local", set.local[i] ? _T("1") : _T("0"));
+			AddTextElement(xItem, "Remote", set.remote[i] ? _T("1") : _T("0"));
 		}
 	}
 
@@ -274,8 +265,7 @@ void CFilterDialog::DisplayFilters()
 	pLocalFilters->Clear();
 	pRemoteFilters->Clear();
 
-	for (unsigned int i = 0; i < m_filters.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_filters.size(); ++i) {
 		const CFilter& filter = m_filters[i];
 
 		const bool localOnly = filter.IsLocalFilter();
@@ -309,18 +299,15 @@ void CFilterDialog::OnFilterSelect(wxCommandEvent& event)
 
 	const CFilter& filter = m_filters[item];
 	const bool localOnly = filter.IsLocalFilter();
-	if (localOnly && event.GetEventObject() != pLocal)
-	{
+	if (localOnly && event.GetEventObject() != pLocal) {
 		pRemote->Check(item, false);
 		wxMessageBoxEx(_("Selected filter only works for local files."), _("Cannot select filter"), wxICON_INFORMATION);
 		return;
 	}
 
 
-	if (m_shiftClick)
-	{
-		if (event.GetEventObject() == pLocal)
-		{
+	if (m_shiftClick) {
+		if (event.GetEventObject() == pLocal) {
 			if (!localOnly)
 				pRemote->Check(item, pLocal->IsChecked(event.GetSelection()));
 		}
@@ -328,8 +315,7 @@ void CFilterDialog::OnFilterSelect(wxCommandEvent& event)
 			pLocal->Check(item, pRemote->IsChecked(event.GetSelection()));
 	}
 
-	if (m_currentFilterSet)
-	{
+	if (m_currentFilterSet) {
 		m_filterSets[0] = m_filterSets[m_currentFilterSet];
 		m_currentFilterSet = 0;
 		wxChoice* pChoice = XRCCTRL(*this, "ID_SETS", wxChoice);
@@ -342,7 +328,7 @@ void CFilterDialog::OnFilterSelect(wxCommandEvent& event)
 	m_filterSets[0].remote[item] = remoteChecked;
 }
 
-void CFilterDialog::OnSaveAs(wxCommandEvent& event)
+void CFilterDialog::OnSaveAs(wxCommandEvent&)
 {
 	CInputDialog dlg;
 	dlg.Create(this, _("Enter name for filterset"), _("Please enter a unique name for this filter set"));
@@ -350,8 +336,7 @@ void CFilterDialog::OnSaveAs(wxCommandEvent& event)
 		return;
 
 	wxString name = dlg.GetValue();
-	if (name.empty())
-	{
+	if (name.empty()) {
 		wxMessageBoxEx(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
 		return;
 	}
@@ -365,14 +350,12 @@ void CFilterDialog::OnSaveAs(wxCommandEvent& event)
 		set = m_filterSets[0];
 
 	int pos = pChoice->FindString(name);
-	if (pos != wxNOT_FOUND)
-	{
+	if (pos != wxNOT_FOUND) {
 		if (wxMessageBoxEx(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
 			return;
 	}
 
-	if (pos == wxNOT_FOUND)
-	{
+	if (pos == wxNOT_FOUND) {
 		pos = m_filterSets.size();
 		m_filterSets.push_back(set);
 		pChoice->Append(name);
@@ -390,15 +373,14 @@ void CFilterDialog::OnSaveAs(wxCommandEvent& event)
 	GetSizer()->Fit(this);
 }
 
-void CFilterDialog::OnRename(wxCommandEvent& event)
+void CFilterDialog::OnRename(wxCommandEvent&)
 {
 	wxChoice* pChoice = XRCCTRL(*this, "ID_SETS", wxChoice);
 	int old_pos = pChoice->GetSelection();
 	if (old_pos == -1)
 		return;
 
-	if (!old_pos)
-	{
+	if (!old_pos) {
 		wxMessageBoxEx(_("This filter set cannot be renamed."));
 		return;
 	}
@@ -413,21 +395,18 @@ void CFilterDialog::OnRename(wxCommandEvent& event)
 
 	wxString name = dlg.GetValue();
 
-	if (name == pChoice->GetStringSelection())
-	{
+	if (name == pChoice->GetStringSelection()) {
 		// Nothing changed
 		return;
 	}
 
-	if (name.empty())
-	{
+	if (name.empty()) {
 		wxMessageBoxEx(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
 		return;
 	}
 
 	int pos = pChoice->FindString(name);
-	if (pos != wxNOT_FOUND)
-	{
+	if (pos != wxNOT_FOUND) {
 		if (wxMessageBoxEx(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
 			return;
 	}
@@ -438,8 +417,7 @@ void CFilterDialog::OnRename(wxCommandEvent& event)
 	m_filterSets.erase(m_filterSets.begin() + old_pos);
 
 	pos = pChoice->FindString(name);
-	if (pos == wxNOT_FOUND)
-	{
+	if (pos == wxNOT_FOUND) {
 		pos = m_filterSets.size();
 		m_filterSets.push_back(set);
 		pChoice->Append(name);
@@ -455,15 +433,14 @@ void CFilterDialog::OnRename(wxCommandEvent& event)
 	GetSizer()->Fit(this);
 }
 
-void CFilterDialog::OnDeleteSet(wxCommandEvent& event)
+void CFilterDialog::OnDeleteSet(wxCommandEvent&)
 {
 	wxChoice* pChoice = XRCCTRL(*this, "ID_SETS", wxChoice);
 	int pos = pChoice->GetSelection();
 	if (pos == -1)
 		return;
 
-	if (!pos)
-	{
+	if (!pos) {
 		wxMessageBoxEx(_("This filter set cannot be removed."));
 		return;
 	}
@@ -496,36 +473,30 @@ void CFilterDialog::OnChangeAll(wxCommandEvent& event)
 	bool local;
 	std::vector<bool>* pValues;
 	wxCheckListBox* pListBox;
-	if (event.GetId() == XRCID("ID_LOCAL_ENABLEALL") || event.GetId() == XRCID("ID_LOCAL_DISABLEALL"))
-	{
+	if (event.GetId() == XRCID("ID_LOCAL_ENABLEALL") || event.GetId() == XRCID("ID_LOCAL_DISABLEALL")) {
 		pListBox = XRCCTRL(*this, "ID_LOCALFILTERS", wxCheckListBox);
 		pValues = &m_filterSets[0].local;
 		local = true;
 	}
-	else
-	{
+	else {
 		pListBox = XRCCTRL(*this, "ID_REMOTEFILTERS", wxCheckListBox);
 		pValues = &m_filterSets[0].remote;
 		local = false;
 	}
 
-	if (m_currentFilterSet)
-	{
+	if (m_currentFilterSet) {
 		m_filterSets[0] = m_filterSets[m_currentFilterSet];
 		m_currentFilterSet = 0;
 		wxChoice* pChoice = XRCCTRL(*this, "ID_SETS", wxChoice);
 		pChoice->SetSelection(0);
 	}
 
-	for (size_t i = 0; i < pListBox->GetCount(); i++)
-	{
-		if (!local && (m_filters[i].IsLocalFilter()))
-		{
+	for (size_t i = 0; i < pListBox->GetCount(); ++i) {
+		if (!local && (m_filters[i].IsLocalFilter())) {
 			pListBox->Check(i, false);
 			(*pValues)[i] = false;
 		}
-		else
-		{
+		else {
 			pListBox->Check(i, check);
 			(*pValues)[i] = check;
 		}
@@ -545,8 +516,7 @@ CFilterManager::CFilterManager()
 {
 	LoadFilters();
 
-	if (m_globalFilterSets.empty())
-	{
+	if (m_globalFilterSets.empty()) {
 		CFilterSet set;
 		set.local.resize(m_globalFilters.size(), false);
 		set.remote.resize(m_globalFilters.size(), false);
@@ -569,8 +539,7 @@ bool CFilterManager::HasActiveFilters(bool ignore_disabled /*=false*/)
 		return false;
 
 	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
-	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_globalFilters.size(); ++i) {
 		if (set.local[i])
 			return true;
 
@@ -598,7 +567,7 @@ bool CFilterManager::HasSameLocalAndRemoteFilters() const
 	return true;
 }
 
-bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const
+bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
 {
 	if (m_filters_disabled)
 		return false;
@@ -608,16 +577,13 @@ bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path
 	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
 
 	// Check active filters
-	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-	{
-		if (local)
-		{
+	for (unsigned int i = 0; i < m_globalFilters.size(); ++i) {
+		if (local) {
 			if (set.local[i])
 				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes, date))
 					return true;
 		}
-		else
-		{
+		else {
 			if (set.remote[i])
 				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes, date))
 					return true;
@@ -627,9 +593,9 @@ bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path
 	return false;
 }
 
-bool CFilterManager::FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const
+bool CFilterManager::FilenameFiltered(std::vector<CFilter> const& filters, const wxString& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const
 {
-	for( auto const& filter : filters ) {
+	for (auto const& filter : filters) {
 		if (FilenameFilteredByFilter(filter, name, path, dir, size, attributes, date))
 			return true;
 	}
@@ -709,7 +675,7 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 	return match;
 }
 
-bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes, CDateTime const& date)
+bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date)
 {
 	if (dir && !filter.filterDirs)
 		return false;
@@ -836,8 +802,8 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 #endif //__WXMSW__
 			break;
 		case filter_date:
-			if (date.IsValid()) {
-				int cmp = date.Compare( condition.date );
+			if (date.empty()) {
+				int cmp = date.compare( condition.date );
 				switch (condition.condition)
 				{
 				case 0: // Before
@@ -859,21 +825,19 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 			wxFAIL_MSG(_T("Unhandled filter type"));
 			break;
 		}
-		if (match)
-		{
+		if (match) {
 			if (filter.matchType == CFilter::any)
 				return true;
 			else if (filter.matchType == CFilter::none)
 				return false;
 		}
-		else
-		{
+		else {
 			if (filter.matchType == CFilter::all)
 				return false;
 		}
 	}
 
-	if (filter.matchType != CFilter::any)
+	if (filter.matchType != CFilter::any || filter.filters.empty())
 		return true;
 
 	return false;
@@ -900,36 +864,35 @@ bool CFilterManager::CompileRegexes(CFilter& filter)
 
 bool CFilterManager::CompileRegexes()
 {
-	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-		CompileRegexes(m_globalFilters[i]);
+	for (auto & filter : m_globalFilters) {
+		CompileRegexes(filter);
+	}
 	return true;
 }
 
-bool CFilterManager::LoadFilter(TiXmlElement* pElement, CFilter& filter)
+bool CFilterManager::LoadFilter(pugi::xml_node& element, CFilter& filter)
 {
-	filter.name = GetTextElement(pElement, "Name");
-	filter.filterFiles = GetTextElement(pElement, "ApplyToFiles") == _T("1");
-	filter.filterDirs = GetTextElement(pElement, "ApplyToDirs") == _T("1");
+	filter.name = GetTextElement(element, "Name");
+	filter.filterFiles = GetTextElement(element, "ApplyToFiles") == _T("1");
+	filter.filterDirs = GetTextElement(element, "ApplyToDirs") == _T("1");
 
-	wxString type = GetTextElement(pElement, "MatchType");
-	if (type == _T("Any"))
+	wxString const matchType = GetTextElement(element, "MatchType");
+	if (matchType == _T("Any"))
 		filter.matchType = CFilter::any;
-	else if (type == _T("None"))
+	else if (matchType == _T("None"))
 		filter.matchType = CFilter::none;
 	else
 		filter.matchType = CFilter::all;
-	filter.matchCase = GetTextElement(pElement, "MatchCase") == _T("1");
+	filter.matchCase = GetTextElement(element, "MatchCase") == _T("1");
 
-	TiXmlElement *pConditions = pElement->FirstChildElement("Conditions");
-	if (!pConditions)
+	auto xConditions = element.child("Conditions");
+	if (!xConditions)
 		return false;
 
-	for (TiXmlElement *pCondition = pConditions->FirstChildElement("Condition"); pCondition; pCondition = pCondition->NextSiblingElement("Condition"))
-	{
+	for (auto xCondition = xConditions.child("Condition"); xCondition; xCondition = xCondition.next_sibling("Condition")) {
 		CFilterCondition condition;
-		int type = GetTextElementInt(pCondition, "Type", 0);
-		switch (type)
-		{
+		int const type = GetTextElementInt(xCondition, "Type", 0);
+		switch (type) {
 		case 0:
 			condition.type = filter_name;
 			break;
@@ -951,39 +914,34 @@ bool CFilterManager::LoadFilter(TiXmlElement* pElement, CFilter& filter)
 		default:
 			continue;
 		}
-		condition.condition = GetTextElementInt(pCondition, "Condition", 0);
-		if (condition.type == filter_size)
-		{
+		condition.condition = GetTextElementInt(xCondition, "Condition", 0);
+		if (condition.type == filter_size) {
 			if (condition.value == 3)
 				condition.value = 2;
 			else if (condition.value >= 2)
 				++condition.value;
 		}
-		condition.strValue = GetTextElement(pCondition, "Value");
+		condition.strValue = GetTextElement(xCondition, "Value");
 		condition.matchCase = filter.matchCase;
 		if (condition.strValue.empty())
 			continue;
 
-		// TODO: 64bit filesize
-		if (condition.type == filter_size)
-		{
-			unsigned long tmp;
-			condition.strValue.ToULong(&tmp);
+		if (condition.type == filter_size) {
+			unsigned long long tmp;
+			condition.strValue.ToULongLong(&tmp);
 			condition.value = tmp;
 		}
-		else if (condition.type == filter_attributes || condition.type == filter_permissions)
-		{
+		else if (condition.type == filter_attributes || condition.type == filter_permissions) {
 			if (condition.strValue == _T("0"))
 				condition.value = 0;
 			else
 				condition.value = 1;
 		}
-		else if (condition.type == filter_date)
-		{
-			wxDateTime t;
-			if (!t.ParseFormat(condition.strValue, _T("%Y-%m-%d")) || !t.IsValid())
+		else if (condition.type == filter_date) {
+			condition.date = fz::datetime(condition.strValue.ToStdWstring(), fz::datetime::local);
+			if (!condition.date.empty()) {
 				continue;
-			condition.date = CDateTime(t, CDateTime::days);
+			}
 		}
 
 		filter.filters.push_back(condition);
@@ -1002,60 +960,55 @@ void CFilterManager::LoadFilters()
 	CInterProcessMutex mutex(MUTEX_FILTERS);
 
 	wxString file(wxGetApp().GetSettingsFile(_T("filters")));
-	if (CLocalFileSystem::GetSize(file) < 1) {
+	if (fz::local_filesys::get_size(fz::to_native(file)) < 1) {
 		file = wxGetApp().GetResourceDir().GetPath() + _T("defaultfilters.xml");
 	}
 
 	CXmlFile xml(file);
-	TiXmlElement* pDocument = xml.Load();
-	if (!pDocument) {
+	auto element = xml.Load();
+	if (!element) {
 		wxString msg = xml.GetError() + _T("\n\n") + _("Any changes made to the filters will not be saved.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	TiXmlElement *pFilters = pDocument->FirstChildElement("Filters");
-
-	if (!pFilters)
+	auto xFilters = element.child("Filters");
+	if (!xFilters)
 		return;
 
-	TiXmlElement *pFilter = pFilters->FirstChildElement("Filter");
-	while (pFilter)
-	{
+	auto xFilter = xFilters.child("Filter");
+	while (xFilter) {
 		CFilter filter;
 
-		bool loaded = LoadFilter(pFilter, filter);
+		bool loaded = LoadFilter(xFilter, filter);
 
 		if (loaded && !filter.name.empty() && !filter.filters.empty())
 			m_globalFilters.push_back(filter);
 
-		pFilter = pFilter->NextSiblingElement("Filter");
+		xFilter = xFilter.next_sibling("Filter");
 	}
 
 	CompileRegexes();
 
-	TiXmlElement* pSets = pDocument->FirstChildElement("Sets");
-	if (!pSets)
+	auto xSets = element.child("Sets");
+	if (!xSets)
 		return;
 
-	for (TiXmlElement* pSet = pSets->FirstChildElement("Set"); pSet; pSet = pSet->NextSiblingElement("Set"))
-	{
+	for (auto xSet = xSets.child("Set"); xSet; xSet = xSet.next_sibling("Set")) {
 		CFilterSet set;
-		TiXmlElement* pItem = pSet->FirstChildElement("Item");
-		while (pItem)
-		{
-			wxString local = GetTextElement(pItem, "Local");
-			wxString remote = GetTextElement(pItem, "Remote");
+		auto xItem = xSet.child("Item");
+		while (xItem) {
+			wxString local = GetTextElement(xItem, "Local");
+			wxString remote = GetTextElement(xItem, "Remote");
 			set.local.push_back(local == _T("1") ? true : false);
 			set.remote.push_back(remote == _T("1") ? true : false);
 
-			pItem = pItem->NextSiblingElement("Item");
+			xItem = xItem.next_sibling("Item");
 		}
 
-		if (!m_globalFilterSets.empty())
-		{
-			set.name = GetTextElement(pSet, "Name");
+		if (!m_globalFilterSets.empty()) {
+			set.name = GetTextElement(xSet, "Name");
 			if (set.name.empty())
 				continue;
 		}
@@ -1064,19 +1017,15 @@ void CFilterManager::LoadFilters()
 			m_globalFilterSets.push_back(set);
 	}
 
-	wxString attribute = GetTextAttribute(pSets, "Current");
-	unsigned long value;
-	if (attribute.ToULong(&value))
-	{
-		if (value < m_globalFilterSets.size())
-			m_globalCurrentFilterSet = value;
+	int value = GetAttributeInt(xSets, "Current");
+	if (value >= 0 && static_cast<size_t>(value) < m_globalFilterSets.size()) {
+		m_globalCurrentFilterSet = value;
 	}
 }
 
 void CFilterManager::ToggleFilters()
 {
-	if (m_filters_disabled)
-	{
+	if (m_filters_disabled) {
 		m_filters_disabled = false;
 		return;
 	}
@@ -1085,9 +1034,9 @@ void CFilterManager::ToggleFilters()
 		m_filters_disabled = true;
 }
 
-std::list<CFilter> CFilterManager::GetActiveFilters(bool local)
+std::vector<CFilter> CFilterManager::GetActiveFilters(bool local)
 {
-	std::list<CFilter> filters;
+	std::vector<CFilter> filters;
 
 	if (m_filters_disabled)
 		return filters;
@@ -1095,15 +1044,12 @@ std::list<CFilter> CFilterManager::GetActiveFilters(bool local)
 	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
 
 	// Check active filters
-	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-	{
-		if (local)
-		{
+	for (unsigned int i = 0; i < m_globalFilters.size(); ++i) {
+		if (local) {
 			if (set.local[i])
 				filters.push_back(m_globalFilters[i]);
 		}
-		else
-		{
+		else {
 			if (set.remote[i])
 				filters.push_back(m_globalFilters[i]);
 		}
