@@ -30,7 +30,7 @@ CFilterEditDialog::~CFilterEditDialog()
 	}
 }
 
-void CFilterEditDialog::OnOK(wxCommandEvent& event)
+void CFilterEditDialog::OnOK(wxCommandEvent&)
 {
 	if (!Validate())
 		return;
@@ -52,7 +52,7 @@ void CFilterEditDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CFilterEditDialog::OnCancel(wxCommandEvent& event)
+void CFilterEditDialog::OnCancel(wxCommandEvent&)
 {
 	EndModal(wxID_CANCEL);
 }
@@ -168,7 +168,7 @@ void CFilterEditDialog::OnNew(wxCommandEvent&)
 	OnFilterSelect(evt);
 }
 
-void CFilterEditDialog::OnDelete(wxCommandEvent& event)
+void CFilterEditDialog::OnDelete(wxCommandEvent&)
 {
 	int item = m_pFilterListCtrl->GetSelection();
 	if (item == -1)
@@ -190,7 +190,7 @@ void CFilterEditDialog::OnDelete(wxCommandEvent& event)
 	SetCtrlState(false);
 }
 
-void CFilterEditDialog::OnRename(wxCommandEvent& event)
+void CFilterEditDialog::OnRename(wxCommandEvent&)
 {
 	const wxString& oldName = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
 	wxTextEntryDialog *pDlg = new wxTextEntryDialog(this, _("Please enter a new name for the filter."), _("Enter filter name"), oldName);
@@ -223,7 +223,7 @@ void CFilterEditDialog::OnRename(wxCommandEvent& event)
 	m_pFilterListCtrl->Select(m_currentSelection);
 }
 
-void CFilterEditDialog::OnCopy(wxCommandEvent& event)
+void CFilterEditDialog::OnCopy(wxCommandEvent&)
 {
 	if (m_currentSelection == -1)
 		return;
@@ -274,11 +274,10 @@ void CFilterEditDialog::OnCopy(wxCommandEvent& event)
 	OnFilterSelect(evt);
 }
 
-void CFilterEditDialog::OnFilterSelect(wxCommandEvent& event)
+void CFilterEditDialog::OnFilterSelect(wxCommandEvent&)
 {
 	int item = m_pFilterListCtrl->GetSelection();
-	if (item == -1)
-	{
+	if (item == -1) {
 		m_currentSelection = -1;
 		SetCtrlState(false);
 		return;
@@ -289,8 +288,7 @@ void CFilterEditDialog::OnFilterSelect(wxCommandEvent& event)
 	if (item == m_currentSelection)
 		return;
 
-	if (m_currentSelection != -1)
-	{
+	if (m_currentSelection != -1) {
 		wxASSERT((unsigned int)m_currentSelection < m_filters.size());
 
 		if (!Validate())
@@ -334,16 +332,14 @@ bool CFilterEditDialog::Validate()
 		return true;
 
 	wxString error;
-	if (!ValidateFilter(error))
-	{
+	if (!ValidateFilter(error)) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
 		wxMessageBoxEx(error, _("Filter validation failed"), wxICON_ERROR, this);
 		return false;
 	}
 
 	wxString name = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
-	if (name.empty())
-	{
+	if (name.empty()) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
 		XRCCTRL(*this, "ID_NAME", wxTextCtrl)->SetFocus();
 		wxMessageBoxEx(_("Need to enter filter name"), _("Filter validation failed"), wxICON_ERROR, this);
@@ -351,8 +347,7 @@ bool CFilterEditDialog::Validate()
 	}
 
 	int pos = m_pFilterListCtrl->FindString(name);
-	if (pos != wxNOT_FOUND && pos != m_currentSelection)
-	{
+	if (pos != wxNOT_FOUND && pos != m_currentSelection) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
 		XRCCTRL(*this, "ID_NAME", wxTextCtrl)->SetFocus();
 		wxMessageBoxEx(_("Filter name already exists"), _("Filter validation failed"), wxICON_ERROR, this);
