@@ -15,13 +15,12 @@ CInterProcessMutex::CInterProcessMutex(enum t_ipcMutexType mutexType, bool initi
 	m_locked = false;
 #ifdef __WXMSW__
 	// Create mutex_
-	hMutex = ::CreateMutex(0, false, wxString::Format(_T("FileZilla 3 Mutex Type %d"), mutexType));
+	hMutex = ::CreateMutex(0, false, wxString::Format(_T("FileZilla 3 Mutex Type %d"), mutexType).wc_str());
 #else
-	if (!m_instanceCount)
-	{
+	if (!m_instanceCount) {
 		// Open file only if this is the first instance
 		wxFileName fn(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("lockfile"));
-		m_fd = open(fn.GetFullPath().mb_str(), O_CREAT | O_RDWR, 0644);
+		m_fd = open(fn.GetFullPath().mb_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0644);
 	}
 	m_instanceCount++;
 #endif
@@ -56,7 +55,7 @@ bool CInterProcessMutex::Lock()
 	if (m_fd >= 0)
 	{
 		// Lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = m_type;
@@ -100,7 +99,7 @@ int CInterProcessMutex::TryLock()
 	if (m_fd >= 0)
 	{
 		// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = m_type;
@@ -139,7 +138,7 @@ void CInterProcessMutex::Unlock()
 	if (m_fd >= 0)
 	{
 		// Unlock region specified by m_type.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_UNLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = m_type;
