@@ -4,10 +4,6 @@
 
 #include <wx/dcclient.h>
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 DEFINE_EVENT_TYPE(fzEVT_UPDATE_LED_TOOLTIP)
 
 BEGIN_EVENT_TABLE(CLed, wxWindow)
@@ -20,6 +16,7 @@ END_EVENT_TABLE()
 #define LED_ON 0
 
 CLed::CLed(wxWindow *parent, unsigned int index)
+	: m_index(index ? 1 : 0)
 {
 #ifdef __WXGTK__
 	SetBackgroundStyle(wxBG_STYLE_TRANSPARENT);
@@ -27,17 +24,10 @@ CLed::CLed(wxWindow *parent, unsigned int index)
 
 	Create(parent, -1, wxDefaultPosition, wxSize(11, 11));
 
-	if (index == 1)
-		m_index = 1;
-	else
-		m_index = 0;
-
 	m_ledState = LED_OFF;
 
 	m_timer.SetOwner(this);
 
-	m_loaded = false;
-
 	wxImage image;
 	if (!image.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("leds.png"), wxBITMAP_TYPE_PNG))
 		return;
@@ -48,11 +38,6 @@ CLed::CLed(wxWindow *parent, unsigned int index)
 	m_loaded = true;
 }
 
-CLed::~CLed()
-{
-	m_timer.Stop();
-}
-
 void CLed::OnPaint(wxPaintEvent&)
 {
 	wxPaintDC dc(this);
@@ -65,8 +50,7 @@ void CLed::OnPaint(wxPaintEvent&)
 
 void CLed::Set()
 {
-	if (m_ledState != LED_ON)
-	{
+	if (m_ledState != LED_ON) {
 		m_ledState = LED_ON;
 		Refresh();
 	}
@@ -74,8 +58,7 @@ void CLed::Set()
 
 void CLed::Unset()
 {
-	if (m_ledState != LED_OFF)
-	{
+	if (m_ledState != LED_OFF) {
 		m_ledState = LED_OFF;
 		Refresh();
 	}
@@ -86,14 +69,11 @@ void CLed::OnTimer(wxTimerEvent& event)
 	if (!m_timer.IsRunning())
 		return;
 
-	if (event.GetId() != m_timer.GetId())
-	{
-		event.Skip();
+	if (event.GetId() != m_timer.GetId()) {
 		return;
 	}
 
-	if (!CFileZillaEngine::IsActive((enum CFileZillaEngine::_direction)m_index))
-	{
+	if (!CFileZillaEngine::IsActive((enum CFileZillaEngine::_direction)m_index)) {
 		Unset();
 		m_timer.Stop();
 	}
