@@ -46,7 +46,7 @@ wxListCtrlEx::wxListCtrlEx(wxWindow *parent,
 						   const wxString& name)
 {
 	Create(parent, id, pos, size, style, validator, name);
-	
+
 #ifndef __WXMSW__
 	m_editing = false;
 #else
@@ -97,8 +97,7 @@ void wxListCtrlEx::OnPreEmitPostScrollEvent()
 
 void wxListCtrlEx::EmitPostScrollEvent()
 {
-	wxCommandEvent evt(fzEVT_POSTSCROLL, wxID_ANY);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_POSTSCROLL, wxID_ANY));
 }
 
 void wxListCtrlEx::OnScrollEvent(wxScrollWinEvent& event)
@@ -121,9 +120,21 @@ void wxListCtrlEx::OnSelectionChanged(wxListEvent& event)
 
 void wxListCtrlEx::ScrollTopItem(int item)
 {
+	if (!GetItemCount()) {
+		return;
+	}
+
+	if (item < 0) {
+		item = 0;
+	}
+	else if (item >= GetItemCount()) {
+		item = GetItemCount() - 1;
+	}
+	
 	const int current = GetTopItem();
 
 	int delta = item - current;
+
 	if (!delta)
 		return;
 
@@ -140,12 +151,12 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 	wxASSERT(character);
 
 	// Keyboard navigation within items
-	wxDateTime now = wxDateTime::UNow();
-	if (m_prefixSearch_lastKeyPress.IsValid())
-	{
-		wxTimeSpan span = now - m_prefixSearch_lastKeyPress;
-		if (span.GetSeconds() >= 1)
+	fz::datetime now = fz::datetime::now();
+	if (m_prefixSearch_lastKeyPress.empty()) {
+		fz::duration span = now - m_prefixSearch_lastKeyPress;
+		if (span.get_seconds() >= 1) {
 			m_prefixSearch_prefix = _T("");
+		}
 	}
 	m_prefixSearch_lastKeyPress = now;
 
@@ -163,8 +174,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 	}
 
 	bool beep = false;
-	if (item != -1)
-	{
+	if (item != -1) {
 		wxString text = GetItemText(item, 0);
 		if (text.Length() >= m_prefixSearch_prefix.Length() && !m_prefixSearch_prefix.CmpNoCase(text.Left(m_prefixSearch_prefix.Length())))
 			beep = true;
@@ -427,34 +437,28 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 	delete [] m_pVisibleColumnMapping;
 	m_pVisibleColumnMapping = new unsigned int[m_columnInfo.size()];
 
-	if (visibilityOptionId != -1)
-	{
+	if (visibilityOptionId != -1) {
 		wxString visibleColumns = COptions::Get()->GetOption(visibilityOptionId);
-		if (visibleColumns.Len() >= m_columnInfo.size())
-		{
-			for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-			{
+		if (visibleColumns.Len() >= m_columnInfo.size()) {
+			for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 				if (!m_columnInfo[i].fixed)
 					m_columnInfo[i].shown = visibleColumns[i] == '1';
 			}
 		}
 	}
 
-	if (sortOptionId != -1)
-	{
+	if (sortOptionId != -1) {
 		wxString strorder = COptions::Get()->GetOption(sortOptionId);
 		wxStringTokenizer tokens(strorder, _T(","));
 
 		unsigned int count = tokens.CountTokens();
-		if (count == m_columnInfo.size())
-		{
+		if (count == m_columnInfo.size()) {
 			unsigned long *order = new unsigned long[count];
 			bool *order_set = new bool[count];
 			memset(order_set, 0, sizeof(bool) * count);
 
 			unsigned int i = 0;
-			while (tokens.HasMoreTokens())
-			{
+			while (tokens.HasMoreTokens()) {
 				if (!tokens.GetNextToken().ToULong(&order[i]))
 					break;
 				if (order[i] >= count || order_set[order[i]])
@@ -462,24 +466,23 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 				order_set[order[i]] = true;
 				i++;
 			}
-			if (i == count)
-			{
+			if (i == count) {
 				bool valid = true;
-				for (unsigned int j = 0; j < m_columnInfo.size(); j++)
-				{
+				for (size_t j = 0; j < m_columnInfo.size(); ++j) {
 					if (!m_columnInfo[j].fixed)
 						continue;
 
-					if (j != order[j])
-					{
+					if (j != order[j]) {
 						valid = false;
 						break;
 					}
 				}
 
-				if (valid)
-					for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-						m_columnInfo[i].order = order[i];
+				if (valid) {
+					for (size_t j = 0; j < m_columnInfo.size(); ++j) {
+						m_columnInfo[j].order = order[j];
+					}
+				}
 			}
 
 			delete [] order;
@@ -532,30 +535,23 @@ bool wxListCtrlEx::ReadColumnWidths(unsigned int optionId)
 		return true;
 	}
 
-	const unsigned int count = m_columnInfo.size();
+	size_t const columnCount = m_columnInfo.size();
 
 	wxString savedWidths = COptions::Get()->GetOption(optionId);
 	wxStringTokenizer tokens(savedWidths, _T(" "));
-	if (tokens.CountTokens() < count)
-		return false;
 
-	unsigned long* newWidths = new unsigned long[count];
-	for (unsigned int i = 0; i < count; i++)
-	{
-		wxString token = tokens.GetNextToken();
-		if (!token.ToULong(newWidths + i) || newWidths[i] > 5000)
-		{
-			delete [] newWidths;
-			return false;
+	size_t const tokenCount = tokens.CountTokens();
+
+	for (size_t i = 0; i < columnCount; ++i) {
+		unsigned long newWidth{};
+		if (i < tokenCount) {
+			wxString token = tokens.GetNextToken();
+			if (token.ToULong(&newWidth) && newWidth < 10000) {
+				m_columnInfo[i].width = newWidth;
+			}
 		}
-		else if (newWidths[i] < MIN_COLUMN_WIDTH)
-			newWidths[i] = MIN_COLUMN_WIDTH;
 	}
 
-	for (unsigned int i = 0; i < count; i++)
-		m_columnInfo[i].width = newWidths[i];
-
-	delete [] newWidths;
 	return true;
 }
 
@@ -564,13 +560,11 @@ void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
 	const unsigned int count = m_columnInfo.size();
 
 	wxString widths;
-	for (unsigned int i = 0; i < count; i++)
-	{
+	for (unsigned int i = 0; i < count; ++i) {
 		int width = 0;
 
 		bool found = false;
-		for (int j = 0; j < GetColumnCount(); j++)
-		{
+		for (int j = 0; j < GetColumnCount(); ++j) {
 			if (m_pVisibleColumnMapping[j] != i)
 				continue;
 
@@ -587,7 +581,7 @@ void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
 }
 
 
-void wxListCtrlEx::AddColumn(const wxString& name, int align, int initialWidth, bool fixed /*=false*/)
+void wxListCtrlEx::AddColumn(const wxString& name, int align, int initialWidth, bool fixed)
 {
 	wxASSERT(!GetColumnCount());
 
@@ -609,30 +603,27 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 	if (m_columnInfo[col].order == before)
 		return;
 
-	for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 		if (i == col)
 			continue;
 
 		t_columnInfo& info = m_columnInfo[i];
 		if (info.order > col)
-			info.order--;
+			--info.order;
 		if (info.order >= before)
-			info.order++;
+			++info.order;
 	}
 
 	t_columnInfo& info = m_columnInfo[col];
 
-	if (info.shown)
-	{
+	if (info.shown) {
 		int icon = -1;
 		// Remove old column
-		for (unsigned int i = 0; i < (unsigned int)GetColumnCount(); i++)
-		{
+		for (unsigned int i = 0; i < (unsigned int)GetColumnCount(); ++i) {
 			if (m_pVisibleColumnMapping[i] != col)
 				continue;
 
-			for (unsigned int j = i + 1; j < (unsigned int)GetColumnCount(); j++)
+			for (unsigned int j = i + 1; j < (unsigned int)GetColumnCount(); ++j)
 				m_pVisibleColumnMapping[j - 1] = m_pVisibleColumnMapping[j];
 			info.width = GetColumnWidth(i);
 
@@ -644,15 +635,15 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 
 		// Insert new column
 		unsigned int pos = 0;
-		for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-		{
+		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			if (i == col)
 				continue;
-			t_columnInfo& info = m_columnInfo[i];
-			if (info.shown && info.order < before)
-				pos++;
+			t_columnInfo& info2 = m_columnInfo[i];
+			if (info2.shown && info2.order < before) {
+				++pos;
+			}
 		}
-		for (unsigned int i = (int)GetColumnCount(); i > pos; i--)
+		for (unsigned int i = (int)GetColumnCount(); i > pos; --i)
 			m_pVisibleColumnMapping[i] = m_pVisibleColumnMapping[i - 1];
 		m_pVisibleColumnMapping[pos] = col;
 
@@ -691,7 +682,7 @@ public:
 	DECLARE_EVENT_TABLE()
 
 protected:
-	void OnUp(wxCommandEvent& event)
+	void OnUp(wxCommandEvent&)
 	{
 		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
 		int sel = pListBox->GetSelection();
@@ -714,7 +705,7 @@ protected:
 		OnSelChanged(evt);
 	}
 
-	void OnDown(wxCommandEvent& event)
+	void OnDown(wxCommandEvent&)
 	{
 		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
 		int sel = pListBox->GetSelection();
@@ -739,7 +730,7 @@ protected:
 		OnSelChanged(evt);
 	}
 
-	void OnSelChanged(wxCommandEvent& event)
+	void OnSelChanged(wxCommandEvent&)
 	{
 		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
 		int sel = pListBox->GetSelection();
@@ -769,7 +760,10 @@ void wxListCtrlEx::ShowColumnEditor()
 {
 	CColumnEditDialog dlg;
 
-	dlg.Load(this, _T("ID_COLUMN_SETUP"));
+	if (!dlg.Load(this, _T("ID_COLUMN_SETUP"))) {
+		wxBell();
+		return;
+	}
 
 	wxCheckListBox* pListBox = XRCCTRL(dlg, "ID_ACTIVE", wxCheckListBox);
 
@@ -912,9 +906,9 @@ void wxListCtrlEx::InitHeaderSortImageList()
 	wxBitmap bmp;
 
 	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.up = m_pImageList->Add(bmp);
+	m_header_icon_index.up = GetSystemImageList()->Add(bmp);
 	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.down = m_pImageList->Add(bmp);
+	m_header_icon_index.down = GetSystemImageList()->Add(bmp);
 #endif
 }
 
@@ -1053,12 +1047,12 @@ void wxListCtrlEx::OnEndLabelEdit(wxListEvent& event)
 		event.Veto();
 }
 
-bool wxListCtrlEx::OnBeginRename(const wxListEvent& event)
+bool wxListCtrlEx::OnBeginRename(const wxListEvent&)
 {
 	return false;
 }
 
-bool wxListCtrlEx::OnAcceptRename(const wxListEvent& event)
+bool wxListCtrlEx::OnAcceptRename(const wxListEvent&)
 {
 	return false;
 }
@@ -1121,21 +1115,25 @@ bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 		break;
 	case HDN_ITEMCHANGINGA:
 	case HDN_ITEMCHANGINGW:
-		if (m_columnDragging)
-		{
-			if (nmHDR->pitem->mask & HDI_WIDTH && nmHDR->pitem->cxy < MIN_COLUMN_WIDTH)
-			{
+		if (m_columnDragging) {
+			if (nmHDR->pitem->mask & HDI_WIDTH && nmHDR->pitem->cxy < MIN_COLUMN_WIDTH) {
 				*result = 1;
 				return true;
 			}
-			else
-			{
+			else {
 				*result = 0;
 				return false;
 			}
 		}
 		else
 			return false;
+	case HDN_DIVIDERDBLCLICK:
+		{
+			wxListEvent event(wxEVT_LIST_COL_END_DRAG, GetId());
+			event.SetEventObject(this);
+			AddPendingEvent(event);
+		}
+		break;
 	}
 
 	return wxListCtrl::MSWOnNotify(idCtrl, lParam, result);
@@ -1153,7 +1151,7 @@ bool wxListCtrlEx::HasSelection() const
 }
 
 
-wxRect wxListCtrlEx::GetListRect() const
+wxRect wxListCtrlEx::GetActualClientRect() const
 {
 	wxRect windowRect = GetMainWindow()->GetClientRect();
 #ifdef __WXMSW__
@@ -1166,139 +1164,3 @@ wxRect wxListCtrlEx::GetListRect() const
 
 	return windowRect;
 }
-
-
-CListCtrlDropTarget::CListCtrlDropTarget(wxListCtrlEx* pListCtrl)
-	: m_pListCtrl(pListCtrl)
-	, m_count()
-{
-	m_timer.SetOwner(this);
-}
-
-
-bool CListCtrlDropTarget::OnDrop(wxCoord, wxCoord)
-{
-	m_timer.Stop();
-	return true;
-}
-
-
-wxDragResult CListCtrlDropTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
-{
-	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
-	{
-		m_timer.Start(100, true);
-		m_count = 0;
-	}
-	return def;
-}
-
-
-
-void CListCtrlDropTarget::OnLeave()
-{
-	m_timer.Stop();
-}
-
-
-wxDragResult CListCtrlDropTarget::OnEnter(wxCoord x, wxCoord y, wxDragResult def)
-{
-	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
-	{
-		m_timer.Start(100, true);
-		m_count = 0;
-	}
-	return def;
-}
-
-
-bool CListCtrlDropTarget::IsScroll(wxPoint p) const
-{
-	return IsTopScroll(p) || IsBottomScroll(p);
-}
-
-
-bool CListCtrlDropTarget::IsTopScroll(wxPoint p) const
-{
-	wxRect itemRect;
-	if (!m_pListCtrl->GetItemRect(m_pListCtrl->GetTopItem(), itemRect))
-		return false;
-
-	wxRect windowRect = m_pListCtrl->GetListRect();
-
-	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
-		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
-	}
-
-	if (p.y < 0 || p.y >= itemRect.GetBottom())
-		return false;
-
-	if (p.x < 0 || p.x > windowRect.GetWidth())
-		return false;
-
-	int top = m_pListCtrl->GetTopItem();
-	if( top <= 0 ) {
-		return false;
-	}
-
-	return true;
-}
-
-
-bool CListCtrlDropTarget::IsBottomScroll(wxPoint p) const
-{
-	wxRect itemRect;
-	if (!m_pListCtrl->GetItemRect(0, itemRect))
-		return false;
-
-	wxRect windowRect = m_pListCtrl->GetListRect();
-	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
-		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
-	}
-
-	if (p.y > windowRect.GetBottom() || p.y < windowRect.GetBottom() - itemRect.GetHeight())
-		return false;
-
-	if (p.x < 0 || p.x > windowRect.GetWidth())
-		return false;
-
-	int bottom = m_pListCtrl->GetTopItem() + m_pListCtrl->GetCountPerPage();
-	if (bottom >= m_pListCtrl->GetItemCount())
-		return false;
-
-	return true;
-}
-
-
-void CListCtrlDropTarget::OnTimer(wxTimerEvent& /*event*/)
-{
-	wxPoint p = wxGetMousePosition();
-	wxWindow* ctrl = m_pListCtrl->GetMainWindow();
-	p = ctrl->ScreenToClient(p);
-
-	if (IsTopScroll(p))
-	{
-		int top = m_pListCtrl->GetTopItem();
-		m_pListCtrl->EnsureVisible(top - 1);
-	}
-	else if (IsBottomScroll(p))
-	{
-		int top = m_pListCtrl->GetTopItem();
-		m_pListCtrl->EnsureVisible(top + m_pListCtrl->GetCountPerPage());
-	}
-	else
-	{
-		m_timer.Stop();
-		return;
-	}
-
-	DisplayDropHighlight(p);
-
-	if (m_count < 90)
-		++m_count;
-	m_timer.Start(100 - m_count, true);
-}
-
-BEGIN_EVENT_TABLE(CListCtrlDropTarget, wxEvtHandler)
-EVT_TIMER(wxID_ANY, CListCtrlDropTarget::OnTimer)
-END_EVENT_TABLE()
