@@ -12,6 +12,8 @@ typedef wxListCtrl wxListCtrlExBase;
 class wxListCtrlEx : public wxListCtrlExBase, public CSystemImageList
 {
 public:
+	typedef int Item;
+
 	wxListCtrlEx(wxWindow *parent,
 		wxWindowID id = wxID_ANY,
 		const wxPoint& pos = wxDefaultPosition,
@@ -60,7 +62,31 @@ public:
 	bool HasSelection() const;
 
 	// Returns the true rect of the area containing the actual list items
-	wxRect GetListRect() const;
+	wxRect GetActualClientRect() const;
+
+	bool Valid(int i) const { return i >= 0 && i < GetItemCount(); }
+	int GetFirstItem() const { return GetItemCount() ? 0 : -1; }
+	int GetLastItem() const { return GetItemCount() - 1; }
+	int GetBottomItem() const {
+		if (GetItemCount()) {
+			return wxMin(GetLastItem(), GetTopItem() + GetCountPerPage() - 1);
+		}
+		return -1;
+	}
+
+	int GetNextItemSimple(int i) const {
+		if (!Valid(++i)) {
+			i = -1;
+		}
+		return i;
+	}
+
+	int GetPrevItemSimple(int i) const {
+		if (!Valid(--i)) {
+			i = -1;
+		}
+		return i;
+	}
 
 protected:
 	virtual void OnPostScroll();
@@ -103,7 +129,7 @@ private:
 	void OnColumnDragging(wxListEvent& event);
 
 	bool m_prefixSearch_enabled{};
-	wxDateTime m_prefixSearch_lastKeyPress;
+	fz::datetime m_prefixSearch_lastKeyPress;
 	wxString m_prefixSearch_prefix;
 
 	bool ReadColumnWidths(unsigned int optionId);
@@ -146,35 +172,4 @@ private:
 	wxListCtrlEx& m_listCtrl;
 };
 
-class CListCtrlDropTarget : public wxEvtHandler, public wxDropTarget
-{
-public:
-	CListCtrlDropTarget(wxListCtrlEx* pListCtrl);
-
-	virtual bool OnDrop(wxCoord x, wxCoord y);
-
-	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);
-
-	virtual void OnLeave();
-
-	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def);
-
-	virtual void DisplayDropHighlight(wxPoint) = 0;
-
-protected:
-	bool IsScroll(wxPoint p) const;
-	bool IsTopScroll(wxPoint p) const;
-	bool IsBottomScroll(wxPoint p) const;
-
-	void OnTimer(wxTimerEvent& /*event*/);
-
-protected:
-	wxListCtrlEx *m_pListCtrl;
-
-	wxTimer m_timer;
-	int m_count;
-
-	DECLARE_EVENT_TABLE()
-};
-
 #endif //__LISTCTRLEX_H__
