@@ -23,10 +23,10 @@ CMenuBar::CMenuBar()
 
 CMenuBar::~CMenuBar()
 {
-	for (auto menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); ++menu_iter)
-	{
-		for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter)
-			delete iter->second;
+	for (auto hidden_menu : m_hidden_items) {
+		for (auto hidden_item : hidden_menu.second) {
+			delete hidden_item.second;
+		}
 	}
 
 	m_pMainFrame->Disconnect(wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(CMenuBar::OnMenuEvent), 0, this);
@@ -114,13 +114,13 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 
 	pMainFrame->Connect(wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(CMenuBar::OnMenuEvent), 0, menubar);
 
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_REMOTE_IDLE, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SERVER, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SYNC_BROWSE, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_COMPARISON, true, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_REMOTE_IDLE, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SYNC_BROWSE, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_COMPARISON, true);
 
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_QUEUEPROCESSING, false, false);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_CHANGEDCONTEXT, false, false);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_QUEUEPROCESSING, false);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_CHANGEDCONTEXT, false);
 
 	menubar->RegisterOption(OPTION_ASCIIBINARY);
 	menubar->RegisterOption(OPTION_PRESERVE_TIMESTAMPS);
@@ -241,10 +241,11 @@ void CMenuBar::ClearBookmarks()
 void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState)
+		return;
 
 	std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.find(event.GetId());
-	if (iter != m_bookmark_menu_id_map_site.end())
-	{
+	if (iter != m_bookmark_menu_id_map_site.end()) {
 		// We hit a site-specific bookmark
 		CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
 		CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
@@ -258,84 +259,74 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 		name.Replace(_T("/"), _T("\\/"));
 		name = controls->site_bookmarks->path + _T("/") + name;
 
-		CSiteManagerItemData_Site *pData = CSiteManager::GetSiteByPath(name);
+		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteByPath(name);
 		if (!pData)
 			return;
 
-		if (!pState)
-			return;
-
 		pState->SetSyncBrowse(false);
-		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle())
-		{
+		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
 			const CServer* pServer = pState->GetServer();
-			if (!pServer || *pServer != pData->m_server)
-			{
-				m_pMainFrame->ConnectToSite(pData);
+			if (!pServer || *pServer != pData->m_server) {
+				m_pMainFrame->ConnectToSite(*pData);
 				pData->m_localDir.clear(); // So not to set again below
 			}
 			else
 				pState->ChangeRemoteDir(pData->m_remoteDir);
 		}
-		if (!pData->m_localDir.empty())
-		{
+		if (!pData->m_localDir.empty()) {
 			bool set = pState->SetLocalDir(pData->m_localDir);
 
-			if (set && pData->m_sync)
-			{
+			if (set && pData->m_sync) {
 				wxASSERT(!pData->m_remoteDir.empty());
 				pState->SetSyncBrowse(true, pData->m_remoteDir);
 			}
 		}
 
-		delete pData;
-
 		return;
 	}
 
 	std::map<int, wxString>::const_iterator iter2 = m_bookmark_menu_id_map_global.find(event.GetId());
-	if (iter2 != m_bookmark_menu_id_map_global.end())
-	{
+	if (iter2 != m_bookmark_menu_id_map_global.end()) {
 		// We hit a global bookmark
 		wxString local_dir;
 		CServerPath remote_dir;
 		bool sync;
-		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync))
+		bool comparison;
+		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync, comparison))
 			return;
 
 		pState->SetSyncBrowse(false);
-		if (!remote_dir.empty() && pState->IsRemoteIdle())
-		{
+		if (!remote_dir.empty() && pState->IsRemoteIdle(true)) {
 			const CServer* pServer = pState->GetServer();
-			if (pServer)
-			{
+			if (pServer) {
 				CServerPath current_remote_path = pState->GetRemotePath();
-				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType())
-				{
+				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType()) {
 					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
 					return;
 				}
 				pState->ChangeRemoteDir(remote_dir);
 			}
 		}
-		if (!local_dir.empty())
-		{
+		if (!local_dir.empty()) {
 			bool set = pState->SetLocalDir(local_dir);
 
-			if (set && sync)
-			{
+			if (set && sync) {
 				wxASSERT(!remote_dir.empty());
 				pState->SetSyncBrowse(true, remote_dir);
 			}
 		}
 
+		if (comparison) {
+			pState->GetComparisonManager()->CompareListings();
+		}
+
 		return;
 	}
 
 	event.Skip();
 }
 
-void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	switch (notification)
 	{
@@ -370,11 +361,9 @@ void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications no
 	}
 }
 
-void CMenuBar::OnOptionChanged(int option)
+void CMenuBar::OnOptionsChanged(changed_options_t const& options)
 {
-	switch (option)
-	{
-	case OPTION_ASCIIBINARY:
+	if (options.test(OPTION_ASCIIBINARY)) {
 		switch (COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY))
 		{
 		default:
@@ -387,53 +376,47 @@ void CMenuBar::OnOptionChanged(int option)
 			Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
 			break;
 		}
-		break;
-	case OPTION_PRESERVE_TIMESTAMPS:
+	}
+	if (options.test(OPTION_PRESERVE_TIMESTAMPS)) {
 		Check(XRCID("ID_MENU_TRANSFER_PRESERVETIMES"), COptions::Get()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS) != 0);
-		break;
-	case OPTION_SHOW_TREE_LOCAL:
+	}
+	if (options.test(OPTION_SHOW_TREE_LOCAL)) {
 		Check(XRCID("ID_VIEW_LOCALTREE"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL) != 0);
-		break;
-	case OPTION_SHOW_TREE_REMOTE:
+	}
+	if (options.test(OPTION_SHOW_TREE_REMOTE)) {
 		Check(XRCID("ID_VIEW_REMOTETREE"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE) != 0);
-		break;
-	case OPTION_SHOW_QUICKCONNECT:
+	}
+	if (options.test(OPTION_SHOW_QUICKCONNECT)) {
 		Check(XRCID("ID_VIEW_QUICKCONNECT"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT) != 0);
-		break;
-	case OPTION_TOOLBAR_HIDDEN:
+	}
+	if (options.test(OPTION_TOOLBAR_HIDDEN)) {
 		Check(XRCID("ID_VIEW_TOOLBAR"), COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) == 0);
-		break;
-	case OPTION_SHOW_MESSAGELOG:
+	}
+	if (options.test(OPTION_SHOW_MESSAGELOG)) {
 		Check(XRCID("ID_VIEW_MESSAGELOG"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
-		break;
-	case OPTION_SHOW_QUEUE:
+	}
+	if (options.test(OPTION_SHOW_QUEUE)) {
 		Check(XRCID("ID_VIEW_QUEUE"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE) != 0);
-		break;
-	case OPTION_COMPARE_HIDEIDENTICAL:
+	}
+	if (options.test(OPTION_COMPARE_HIDEIDENTICAL)) {
 		Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
-		break;
-	case OPTION_COMPARISONMODE:
+	}
+	if (options.test(OPTION_COMPARISONMODE)) {
 		if (COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE) != 1)
 			Check(XRCID("ID_COMPARE_SIZE"), true);
 		else
 			Check(XRCID("ID_COMPARE_DATE"), true);
-		break;
-	case OPTION_MESSAGELOG_POSITION:
+	}
+	if (options.test(OPTION_MESSAGELOG_POSITION)) {
 		if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2)
 			HideItem(XRCID("ID_VIEW_MESSAGELOG"));
-		else
-		{
+		else {
 			ShowItem(XRCID("ID_VIEW_MESSAGELOG"));
 			Check(XRCID("ID_VIEW_MESSAGELOG"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
 		}
-		break;
-	case OPTION_SPEEDLIMIT_ENABLE:
-	case OPTION_SPEEDLIMIT_INBOUND:
-	case OPTION_SPEEDLIMIT_OUTBOUND:
+	}
+	if (options.test(OPTION_SPEEDLIMIT_ENABLE) || options.test(OPTION_SPEEDLIMIT_INBOUND) || options.test(OPTION_SPEEDLIMIT_OUTBOUND)) {
 		UpdateSpeedLimitMenuItem();
-		break;
-	default:
-		break;
 	}
 }
 
@@ -463,7 +446,6 @@ void CMenuBar::UpdateMenubarState()
 	Enable(XRCID("ID_CANCEL"), pServer && !idle);
 	Enable(XRCID("ID_MENU_SERVER_CMD"), pServer && idle);
 	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), pServer != 0);
-	Enable(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
 	Enable(XRCID("ID_MENU_SERVER_SEARCH"), pServer && idle);
 
@@ -473,16 +455,14 @@ void CMenuBar::UpdateMenubarState()
 	bool canReconnect;
 	if (pServer || !idle)
 		canReconnect = false;
-	else
-	{
+	else {
 		CServer tmp;
 		canReconnect = !pState->GetLastServer().GetHost().empty();
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
 	wxMenuItem* pItem = FindItem(XRCID("ID_MENU_TRANSFER_TYPE"));
-	if (!pServer || CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol()))
-	{
+	if (!pServer || CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol())) {
 		pItem->Enable(true);
 	}
 	else
