@@ -30,7 +30,7 @@ protected:
 	void UpdateSpeedLimitMenuItem();
 
 	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
-	virtual void OnOptionChanged(int option);
+	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	DECLARE_DYNAMIC_CLASS(CMenuBar)
 
