@@ -9,10 +9,6 @@
 
 CQueueItem::CQueueItem(CQueueItem* parent)
 	: m_parent(parent)
-	, m_visibleOffspring(0)
-	, m_indent(0)
-	, m_maxCachedIndex(-1)
-	, m_removed_at_front(0)
 {
 }
 
@@ -30,84 +26,42 @@ void CQueueItem::SetPriority(QueuePriority priority)
 
 void CQueueItem::AddChild(CQueueItem* item)
 {
-	if (m_removed_at_front)
-	{
+	wxASSERT(GetType() != QueueItemType::Folder);
+	wxASSERT(GetType() != QueueItemType::Status);
+
+	if (m_removed_at_front) {
 		m_children.erase(m_children.begin(), m_children.begin() + m_removed_at_front);
 		m_removed_at_front = 0;
 	}
-	item->m_indent = m_indent + 1;
 	m_children.push_back(item);
 
-	m_visibleOffspring += 1 + item->m_visibleOffspring;
-	m_maxCachedIndex = -1;
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_visibleOffspring += 1 + item->m_visibleOffspring;
-		parent->m_maxCachedIndex = -1;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_visibleOffspring += 1 + item->GetChildrenCount(true);
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+		}
 		parent = parent->GetParent();
 	}
 }
 
-CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
+
+CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive)
 {
-	std::vector<CQueueItem*>::iterator iter;
-	if (!recursive)
-	{
+	if (!recursive) {
 		if (item + m_removed_at_front >= m_children.size())
 			return 0;
-		iter = m_children.begin();
-		iter += item + m_removed_at_front;
-		return *iter;
+		return *(m_children.begin() + item + m_removed_at_front);
 	}
 
-	if ((int)item <= m_maxCachedIndex)
-	{
-		// Index is cached
-		iter = m_children.begin() + m_removed_at_front;
-		iter += m_lookupCache[item].child;
-		item -= m_lookupCache[item].index;
-		if (!item)
-			return *iter;
-		else
-			return (*iter)->GetChild(item - 1);
-	}
-
-	int index;
-	int child;
-	iter = m_children.begin() + m_removed_at_front;
-	if (m_maxCachedIndex == -1)
-	{
-		child = 0;
-		index = 0;
-	}
-	else
-	{
-		// Start with loop with the last cached item index
-		iter += m_lookupCache[m_maxCachedIndex].child + 1;
-		item -= m_maxCachedIndex + 1;
-		index = m_maxCachedIndex + 1;
-		child = m_lookupCache[m_maxCachedIndex].child + 1;
-	}
-
-	for (; iter != m_children.end(); ++iter, ++child)
-	{
+	std::vector<CQueueItem*>::iterator iter = m_children.begin() + m_removed_at_front;
+	for (; iter != m_children.end(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
-			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
-				m_lookupCache.resize(m_visibleOffspring);
-			for (unsigned int k = index; k <= index + count; k++)
-			{
-				m_lookupCache[k].child = child;
-				m_lookupCache[k].index = index;
-			}
-			m_maxCachedIndex = index + count;
+		if (item > count) {
 			item -= count + 1;
-			index += count + 1;
 			continue;
 		}
 
@@ -116,29 +70,36 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 	return 0;
 }
 
-unsigned int CQueueItem::GetChildrenCount(bool recursive)
+unsigned int CQueueItem::GetChildrenCount(bool recursive) const
 {
-	if (!recursive)
-		return m_children.size() - m_removed_at_front;
+	unsigned int count{};
+	if (!recursive) {
+		count = m_children.size() - m_removed_at_front;
+	}
+	else {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
+			count += 1 + (*iter)->GetChildrenCount(true);
+		}
+	}
 
-	return m_visibleOffspring;
+	return count;
 }
 
-bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
-	int oldVisibleOffspring = m_visibleOffspring;
+	int const oldVisibleOffspring = GetChildrenCount(true);
+	int visibleOffspring = oldVisibleOffspring;
+
 	bool deleted = false;
-	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		if (*iter == pItem)
-		{
-			m_visibleOffspring -= 1;
-			m_visibleOffspring -= pItem->m_visibleOffspring;
+
+	auto doRemove = [&](std::vector<CQueueItem*>::iterator iter) {
+		if (*iter == pItem) {
+			visibleOffspring -= 1;
+			visibleOffspring -= pItem->GetChildrenCount(true);
 			if (destroy)
 				delete pItem;
 
-			if (iter - m_children.begin() - m_removed_at_front <= 10)
-			{
+			if (iter - m_children.begin() - m_removed_at_front <= 10) {
 				m_removed_at_front++;
 				unsigned int end = iter - m_children.begin();
 				int c = 0;
@@ -149,20 +110,17 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 				m_children.erase(iter);
 
 			deleted = true;
-			break;
+			return;
 		}
 
-		int visibleOffspring = (*iter)->m_visibleOffspring;
-		if ((*iter)->RemoveChild(pItem, destroy))
-		{
-			m_visibleOffspring -= visibleOffspring - (*iter)->m_visibleOffspring;
-			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front))
-			{
-				m_visibleOffspring -= 1;
+		int childVisibleOffspring = (*iter)->GetChildrenCount(true);
+		if ((*iter)->RemoveChild(pItem, destroy)) {
+			visibleOffspring -= childVisibleOffspring - (*iter)->GetChildrenCount(true);
+			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front)) {
+				visibleOffspring -= 1;
 				delete *iter;
 
-				if (iter - m_children.begin() - m_removed_at_front <= 10)
-				{
+				if (iter - m_children.begin() - m_removed_at_front <= 10) {
 					m_removed_at_front++;
 					unsigned int end = iter - m_children.begin();
 					for (int i = end; i >= m_removed_at_front; i--)
@@ -173,60 +131,46 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 			}
 
 			deleted = true;
-			break;
+		}
+	};
+
+	if (forward) {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter ) {
+			doRemove(iter);
+			if (deleted) {
+				break;
+			}
+		}
+	}
+	else {
+		for (auto iter = m_children.rbegin(); iter != m_children.rend() - m_removed_at_front; ++iter ) {
+			doRemove(iter.base() - 1);
+			if (deleted) {
+				break;
+			}
 		}
 	}
 
 	if (!deleted)
 		return false;
 
-	m_maxCachedIndex = -1;
+	if (GetType() == QueueItemType::Server) {
+		static_cast<CServerItem*>(this)->m_visibleOffspring = visibleOffspring;
+	}
 
 	// Propagate new children count to parent
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+			static_cast<CServerItem*>(parent)->m_visibleOffspring -= oldVisibleOffspring - visibleOffspring;
+		}
 		parent = parent->GetParent();
 	}
 
 	return true;
 }
 
-bool CQueueItem::TryRemoveAll()
-{
-	const int oldVisibleOffspring = m_visibleOffspring;
-	std::vector<CQueueItem*>::iterator iter;
-	std::vector<CQueueItem*> keepChildren;
-	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		CQueueItem* pItem = *iter;
-		if (pItem->TryRemoveAll())
-			delete pItem;
-		else
-		{
-			keepChildren.push_back(pItem);
-			m_visibleOffspring++;
-			m_visibleOffspring += pItem->GetChildrenCount(true);
-		}
-	}
-	m_children = keepChildren;
-	m_maxCachedIndex = -1;
-	m_removed_at_front = 0;
-
-	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
-
-	return m_children.empty();
-}
-
 CQueueItem* CQueueItem::GetTopLevelItem()
 {
 	if (!m_parent)
@@ -234,8 +178,7 @@ CQueueItem* CQueueItem::GetTopLevelItem()
 
 	CQueueItem* newParent = m_parent;
 	CQueueItem* parent = 0;
-	while (newParent)
-	{
+	while (newParent) {
 		parent = newParent;
 		newParent = newParent->GetParent();
 	}
@@ -277,33 +220,12 @@ int CQueueItem::GetItemIndex() const
 	return index + pParent->GetItemIndex();
 }
 
-const wxString& CQueueItem::GetIndent() const
-{
-	wxASSERT(m_indent < 5);
-	static const wxString indents[5] =
-	{
-		_T(""),
-		_T("  "),
-		_T("    "),
-		_T("      "),
-		_T("        ")
-	};
-	return indents[m_indent];
-}
-
 CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
 					 const wxString& sourceFile, const wxString& targetFile,
-					 const CLocalPath& localPath, const CServerPath& remotePath, wxLongLong size)
+					 const CLocalPath& localPath, const CServerPath& remotePath, int64_t size)
 	: CQueueItem(parent)
-	, m_errorCount(0)
-	, m_edit(CEditHandler::none)
-	, m_pEngineData(0)
-	, m_defaultFileExistsAction(CFileExistsNotification::unknown)
-	, m_onetime_action(CFileExistsNotification::unknown)
-	, flags(0)
-	, m_priority(QueuePriority::normal)
 	, m_sourceFile(sourceFile)
-	, m_targetFile(targetFile)
+	, m_targetFile(targetFile.empty() ? CSparseOptional<std::wstring>() : CSparseOptional<std::wstring>(to_wstring(targetFile)))
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
 	, m_size(size)
@@ -325,7 +247,7 @@ void CFileItem::SetPriority(QueuePriority priority)
 
 	if (m_parent)
 	{
-		CServerItem* parent = reinterpret_cast<CServerItem*>(m_parent);
+		CServerItem* parent = static_cast<CServerItem*>(m_parent);
 		parent->SetChildPriority(this, m_priority, priority);
 	}
 	m_priority = priority;
@@ -343,37 +265,36 @@ QueuePriority CFileItem::GetPriority() const
 
 void CFileItem::SetActive(const bool active)
 {
-	if (active && !IsActive())
-	{
+	if (active && !IsActive()) {
+		wxASSERT(!GetChildrenCount(false));
 		AddChild(new CStatusItem);
 		flags |= flag_active;
 	}
-	else if (!active && IsActive())
-	{
+	else if (!active && IsActive()) {
 		CQueueItem* pItem = GetChild(0, false);
 		RemoveChild(pItem);
 		flags &= ~flag_active;
 	}
 }
 
-void CFileItem::SaveItem(TiXmlElement* pElement) const
+void CFileItem::SaveItem(pugi::xml_node& element) const
 {
-	if (m_edit != CEditHandler::none)
+	if (m_edit != CEditHandler::none || !element)
 		return;
 
-	TiXmlElement *file = pElement->LinkEndChild(new TiXmlElement("File"))->ToElement();
+	auto file = element.append_child("File");
 
 	AddTextElement(file, "LocalFile", m_localPath.GetPath() + GetLocalFile());
 	AddTextElement(file, "RemoteFile", GetRemoteFile());
 	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
 	if (m_size != -1)
-		AddTextElement(file, "Size", m_size.ToString());
+		AddTextElement(file, "Size", m_size);
 	if (m_errorCount)
 		AddTextElement(file, "ErrorCount", m_errorCount);
 	if (m_priority != QueuePriority::normal)
 		AddTextElement(file, "Priority", static_cast<int>(m_priority));
-	AddTextElementRaw(file, "DataType", m_transferSettings.binary ? "1" : "0");
+	AddTextElementRaw(file, "DataType", Ascii() ? "0" : "1");
 	if (m_defaultFileExistsAction != CFileExistsNotification::unknown)
 		AddTextElement(file, "OverwriteAction", m_defaultFileExistsAction);
 }
@@ -387,15 +308,41 @@ bool CFileItem::TryRemoveAll()
 	return false;
 }
 
-void CFileItem::SetTargetFile(const wxString &file)
+void CFileItem::SetTargetFile(wxString const& file)
 {
-	wxASSERT(!file.empty());
-	if (file != m_sourceFile)
-		m_targetFile = file;
+	if (!file.empty() && file != m_sourceFile)
+		m_targetFile = CSparseOptional<std::wstring>(to_wstring(file));
 	else
 		m_targetFile.clear();
 }
 
+void CFileItem::SetStatusMessage(CFileItem::Status status)
+{
+	m_status = status;
+}
+
+wxString const& CFileItem::GetStatusMessage() const
+{
+	static wxString statusTexts[] = {
+		wxString(),
+		_("Incorrect password"),
+		_("Timeout"),
+		_("Disconnecting from previous server"),
+		_("Disconnected from server"),
+		_("Connecting"),
+		_("Connection attempt failed"),
+		_("Interrupted by user"),
+		_("Waiting for browsing connection"),
+		_("Waiting for password"),
+		_("Could not write to local file"),
+		_("Could not start transfer"),
+		_("Transferring"),
+		_("Creating directory")
+	};
+
+	return statusTexts[m_status];
+}
+
 CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath)
 	: CFileItem(parent, queued, true, wxEmptyString, wxEmptyString, localPath, CServerPath(), -1)
 {
@@ -406,20 +353,17 @@ CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CServerPath& re
 {
 }
 
-void CFolderItem::SaveItem(TiXmlElement* pElement) const
+void CFolderItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *file = new TiXmlElement("Folder");
+	auto file = element.append_child("Folder");
 
 	if (Download())
 		AddTextElement(file, "LocalFile", GetLocalPath().GetPath() + GetLocalFile());
-	else
-	{
+	else {
 		AddTextElement(file, "RemoteFile", GetRemoteFile());
 		AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	}
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
-
-	pElement->LinkEndChild(file);
 }
 
 void CFolderItem::SetActive(const bool active)
@@ -453,9 +397,22 @@ wxString CServerItem::GetName() const
 void CServerItem::AddChild(CQueueItem* pItem)
 {
 	CQueueItem::AddChild(pItem);
+	m_maxCachedIndex = -1;
+	m_visibleOffspring += 1 + pItem->GetChildrenCount(true);
 	if (pItem->GetType() == QueueItemType::File ||
 		pItem->GetType() == QueueItemType::Folder)
 		AddFileItemToList((CFileItem*)pItem);
+
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+}
+
+unsigned int CServerItem::GetChildrenCount(bool recursive) const
+{
+	if (!recursive)
+		return m_children.size() - m_removed_at_front;
+
+	return m_visibleOffspring;
 }
 
 void CServerItem::AddFileItemToList(CFileItem* pItem)
@@ -466,13 +423,23 @@ void CServerItem::AddFileItemToList(CFileItem* pItem)
 	m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())].push_back(pItem);
 }
 
-void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
+void CServerItem::RemoveFileItemFromList(CFileItem* pItem, bool forward)
 {
-	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
-		if (*iter == pItem) {
-			fileList.erase(iter);
-			return;
+	std::deque<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
+	if (forward) {
+		for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter);
+				return;
+			}
+		}
+	}
+	else {
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter.base() - 1);
+				return;
+			}
 		}
 	}
 	wxFAIL_MSG(_T("File item not deleted from m_fileList"));
@@ -498,8 +465,68 @@ void CServerItem::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteA
 	}
 }
 
+CQueueItem* CServerItem::GetChild(unsigned int item, bool recursive)
+{
+	std::vector<CQueueItem*>::iterator iter;
+	if (!recursive) {
+		if (item + m_removed_at_front >= m_children.size())
+			return 0;
+		iter = m_children.begin();
+		iter += item + m_removed_at_front;
+		return *iter;
+	}
+
+	if ((int)item <= m_maxCachedIndex) {
+		// Index is cached
+		iter = m_children.begin() + m_removed_at_front;
+		iter += m_lookupCache[item].child;
+		item -= m_lookupCache[item].index;
+		if (!item)
+			return *iter;
+		else
+			return (*iter)->GetChild(item - 1);
+	}
+
+	int index;
+	int child;
+	iter = m_children.begin() + m_removed_at_front;
+	if (m_maxCachedIndex == -1) {
+		child = 0;
+		index = 0;
+	}
+	else {
+		// Start with loop with the last cached item index
+		iter += m_lookupCache[m_maxCachedIndex].child + 1;
+		item -= m_maxCachedIndex + 1;
+		index = m_maxCachedIndex + 1;
+		child = m_lookupCache[m_maxCachedIndex].child + 1;
+	}
+
+	for (; iter != m_children.end(); ++iter, ++child) {
+		if (!item)
+			return *iter;
+
+		unsigned int count = (*iter)->GetChildrenCount(true);
+		if (item > count) {
+			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
+				m_lookupCache.resize(m_visibleOffspring);
+			for (unsigned int k = index; k <= index + count; ++k) {
+				m_lookupCache[k].child = child;
+				m_lookupCache[k].index = index;
+			}
+			m_maxCachedIndex = index + count;
+			item -= count + 1;
+			index += count + 1;
+			continue;
+		}
+
+		return (*iter)->GetChild(item - 1);
+	}
+	return 0;
+}
+
 namespace {
-CFileItem* DoGetIdleChild(std::list<CFileItem*> const* fileList, TransferDirection direction)
+CFileItem* DoGetIdleChild(std::deque<CFileItem*> const* fileList, TransferDirection direction)
 {
 	int i = 0;
 	for (i = static_cast<int>(QueuePriority::count) - 1; i >= 0; --i) {
@@ -532,37 +559,43 @@ CFileItem* CServerItem::GetIdleChild(bool immediateOnly, TransferDirection direc
 	return item;
 }
 
-bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
 	if (!pItem)
 		return false;
 
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
-		CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
-		RemoveFileItemFromList(pFileItem);
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
+		CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
+		RemoveFileItemFromList(pFileItem, forward);
 	}
 
-	return CQueueItem::RemoveChild(pItem, destroy);
+	bool removed = CQueueItem::RemoveChild(pItem, destroy, forward);
+	if (removed) {
+		m_maxCachedIndex = -1;
+	}
+	
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+
+	return removed;
 }
 
 void CServerItem::QueueImmediateFiles()
 {
 	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
-		std::list<CFileItem*> activeList;
-		std::list<CFileItem*>& fileList = m_fileList[1][i];
-		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+		std::deque<CFileItem*> activeList;
+		std::deque<CFileItem*>& fileList = m_fileList[1][i];
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
 			CFileItem* item = *iter;
 			wxASSERT(!item->queued());
 			if (item->IsActive())
 				activeList.push_front(item);
-			else
-			{
+			else {
 				item->set_queued(true);
 				m_fileList[0][i].push_front(item);
 			}
 		}
-		fileList = activeList;
+		std::swap(fileList, activeList);
 	}
 }
 
@@ -571,9 +604,8 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	if (pItem->queued())
 		return;
 
-	std::list<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter)
-	{
+	std::deque<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
+	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -585,29 +617,23 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	wxASSERT(false);
 }
 
-void CServerItem::SaveItem(TiXmlElement* pElement) const
+void CServerItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *server = new TiXmlElement("Server");
+	auto server = element.append_child("Server");
 	SetServer(server, m_server);
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
+	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter)
 		(*iter)->SaveItem(server);
-
-	pElement->LinkEndChild(server);
 }
 
-wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
+int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
 {
-	wxLongLong totalSize = 0;
-	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i)
-	{
-		for (int j = 0; j < 2; j++)
-		{
-			const std::list<CFileItem*>& fileList = m_fileList[j][i];
-			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter)
-			{
-				const CFileItem* item = *iter;
-				wxLongLong size = item->GetSize();
+	int64_t totalSize = 0;
+	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
+		for (int j = 0; j < 2; ++j) {
+			const std::deque<CFileItem*>& fileList = m_fileList[j][i];
+			for (auto const& item : fileList) {
+				int64_t size = item->GetSize();
 				if (size >= 0)
 					totalSize += size;
 				else
@@ -616,8 +642,7 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 		}
 	}
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File ||
 			(*iter)->GetType() == QueueItemType::Folder)
 			queuedFiles++;
@@ -630,38 +655,35 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 
 bool CServerItem::TryRemoveAll()
 {
+	wxASSERT(!GetParent());
+
 	const int oldVisibleOffspring = m_visibleOffspring;
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
 	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		CQueueItem* pItem = *iter;
-		if (pItem->TryRemoveAll())
-		{
-			if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-			{
-				CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
-				RemoveFileItemFromList(pFileItem);
+		if (pItem->TryRemoveAll()) {
+			if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
+				CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
+				RemoveFileItemFromList(pFileItem, true);
 			}
 			delete pItem;
 		}
-		else
-		{
+		else {
 			keepChildren.push_back(pItem);
 			m_visibleOffspring++;
 			m_visibleOffspring += pItem->GetChildrenCount(true);
 		}
 	}
-	m_children = keepChildren;
+	std::swap(m_children, keepChildren);
 	m_removed_at_front = 0;
 
-	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
+	m_maxCachedIndex = -1;
+
+	wxASSERT(oldVisibleOffspring >= m_visibleOffspring);
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()));
+	(void)oldVisibleOffspring;
 
 	return m_children.empty();
 }
@@ -675,7 +697,7 @@ void CServerItem::DetachChildren()
 	m_maxCachedIndex = -1;
 	m_removed_at_front = 0;
 
-	for (int i = 0; i < 2; i++)
+	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); j++)
 			m_fileList[i][j].clear();
 }
@@ -683,8 +705,7 @@ void CServerItem::DetachChildren()
 void CServerItem::SetPriority(QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File)
 			((CFileItem*)(*iter))->SetPriorityRaw(priority);
 		else
@@ -693,8 +714,10 @@ void CServerItem::SetPriority(QueuePriority priority)
 
 	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); ++j) {
-			if (j != static_cast<int>(priority))
-				m_fileList[i][static_cast<int>(priority)].splice(m_fileList[i][static_cast<int>(priority)].end(), m_fileList[i][j]);
+			if (j != static_cast<int>(priority)) {
+				std::move(m_fileList[i][j].begin(), m_fileList[i][j].end(), std::back_inserter(m_fileList[i][static_cast<int>(priority)]));
+				m_fileList[i][j].clear();
+			}
 		}
 }
 
@@ -702,8 +725,7 @@ void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority,
 {
 	int i = pItem->queued() ? 0 : 1;
 
-	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter)
-	{
+	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -717,11 +739,6 @@ void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority,
 
 CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath)
 	: CQueueItem(parent)
-	, m_remove(false)
-	, m_active(false)
-	, m_count(0)
-	, m_defaultFileExistsAction(CFileExistsNotification::unknown)
-	, m_dir_is_empty(false)
 	, m_queued(queued)
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
@@ -748,6 +765,7 @@ EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
 EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
 EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
+EVT_MENU(XRCID("ID_EXPORT"), CQueueViewBase::OnExport)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
@@ -786,17 +804,14 @@ CQueueViewBase::~CQueueViewBase()
 		delete *iter;
 }
 
-CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item)
+CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item) const
 {
-	std::vector<CServerItem*>::iterator iter;
-	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (auto iter = m_serverList.cbegin(); iter != m_serverList.cend(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
+		if (item > count) {
 			item -= count + 1;
 			continue;
 		}
@@ -848,18 +863,18 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 	{
 	case QueueItemType::Server:
 		{
-			CServerItem* pServerItem = reinterpret_cast<CServerItem*>(pItem);
+			CServerItem* pServerItem = static_cast<CServerItem*>(pItem);
 			if (!column)
 				return pServerItem->GetName();
 		}
 		break;
 	case QueueItemType::File:
 		{
-			CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
+			CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
-				return pFileItem->GetIndent() + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
+				return _T("  ") + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
 			case colDirection:
 				if (pFileItem->Download())
 					if (pFileItem->queued())
@@ -876,7 +891,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				return pFileItem->GetRemotePath().FormatFilename(pFileItem->GetRemoteFile());
 			case colSize:
 				{
-					const wxLongLong& size = pFileItem->GetSize();
+					auto const& size = pFileItem->GetSize();
 					if (size >= 0)
 						return CSizeFormat::Format(size);
 					else
@@ -900,7 +915,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				break;
 			case colTransferStatus:
 			case colErrorReason:
-				return pFileItem->m_statusMessage;
+				return pFileItem->GetStatusMessage();
 			case colTime:
 				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
@@ -910,7 +925,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 		break;
 	case QueueItemType::FolderScan:
 		{
-			CFolderScanItem* pFolderItem = reinterpret_cast<CFolderScanItem*>(pItem);
+			CFolderScanItem* pFolderItem = static_cast<CFolderScanItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
@@ -941,12 +956,12 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 		break;
 	case QueueItemType::Folder:
 		{
-			CFileItem* pFolderItem = reinterpret_cast<CFolderItem*>(pItem);
+			CFileItem* pFolderItem = static_cast<CFolderItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
 				if (pFolderItem->Download())
-					return pFolderItem->GetIndent() + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
+					return _T("  ") + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
 				break;
 			case colDirection:
 				if (pFolderItem->Download())
@@ -987,7 +1002,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				break;
 			case colTransferStatus:
 			case colErrorReason:
-				return pFolderItem->m_statusMessage;
+				return pFolderItem->GetStatusMessage();
 			case colTime:
 				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
@@ -1083,16 +1098,14 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 		return;
 #endif
 
-	std::list<int> itemsToSelect;
+	std::deque<int> itemsToSelect;
 
 	// Go through all selected items
 	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
-	while (item != -1)
-	{
+	while (item != -1) {
 		// Select new items preceding to current one
-		while (!itemsToSelect.empty() && itemsToSelect.front() < item)
-		{
+		while (!itemsToSelect.empty() && itemsToSelect.front() < item) {
 			SetItemState(itemsToSelect.front(), wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 			itemsToSelect.pop_front();
 		}
@@ -1105,8 +1118,9 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToSelect.begin(); iter != itemsToSelect.end(); ++iter)
-		SetItemState(*iter, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+	for (auto const& sel : itemsToSelect) {
+		SetItemState(sel, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+	}
 }
 
 void CQueueViewBase::UpdateSelections_ItemRemoved(int removed)
@@ -1162,15 +1176,13 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 
 	SetItemState(removed, 0, wxLIST_STATE_SELECTED);
 
-	std::list<int> itemsToUnselect;
+	std::deque<int> itemsToUnselect;
 
 	int item = GetNextItem(removed - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
-	while (item != -1)
-	{
+	while (item != -1) {
 		// Unselect new items preceding to current one
-		while (!itemsToUnselect.empty() && itemsToUnselect.front() < item - count)
-		{
+		while (!itemsToUnselect.empty() && itemsToUnselect.front() < item - count) {
 			SetItemState(itemsToUnselect.front(), 0, wxLIST_STATE_SELECTED);
 			itemsToUnselect.pop_front();
 		}
@@ -1186,8 +1198,9 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToUnselect.begin(); iter != itemsToUnselect.end(); ++iter)
-		SetItemState(*iter, 0, wxLIST_STATE_SELECTED);
+	for (auto const& unsel : itemsToUnselect) {
+		SetItemState(unsel, 0, wxLIST_STATE_SELECTED);
+	}
 }
 
 void CQueueViewBase::AddQueueColumn(ColumnId id)
@@ -1266,15 +1279,13 @@ void CQueueViewBase::CommitChanges()
 
 void CQueueViewBase::DisplayNumberQueuedFiles()
 {
-	if (m_filecount_delay_timer.IsRunning())
-	{
+	if (m_filecount_delay_timer.IsRunning()) {
 		m_fileCountChanged = true;
 		return;
 	}
 
 	wxString str;
-	if (m_fileCount > 0)
-	{
+	if (m_fileCount > 0) {
 		if (!m_folderScanCount)
 			str.Printf(m_title + _T(" (%d)"), m_fileCount);
 		else
@@ -1305,28 +1316,24 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 		m_insertionStart = newIndex;
 	m_insertionCount++;
 
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		m_fileCount++;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan)
-	{
+	else if (pItem->GetType() == QueueItemType::FolderScan) {
 		m_folderScanCount++;
 		m_folderScanCountChanged = true;
 	}
 }
 
-bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount /*=true*/, bool updateSelections /*=true*/)
+bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount, bool updateSelections, bool forward)
 {
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		wxASSERT(m_fileCount > 0);
 		m_fileCount--;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan)
-	{
+	else if (pItem->GetType() == QueueItemType::FolderScan) {
 		wxASSERT(m_folderScanCount > 0);
 		m_folderScanCount--;
 		m_folderScanCountChanged = true;
@@ -1340,16 +1347,14 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	CQueueItem* topLevelItem = pItem->GetTopLevelItem();
 
 	int count = topLevelItem->GetChildrenCount(true);
-	topLevelItem->RemoveChild(pItem, destroy);
+	topLevelItem->RemoveChild(pItem, destroy, forward);
 
 	bool didRemoveParent;
 
 	int oldCount = m_itemCount;
-	if (!topLevelItem->GetChild(0))
-	{
+	if (!topLevelItem->GetChild(0)) {
 		std::vector<CServerItem*>::iterator iter;
-		for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-		{
+		for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 			if (*iter == topLevelItem)
 				break;
 		}
@@ -1366,8 +1371,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 
 		didRemoveParent = true;
 	}
-	else
-	{
+	else {
 		count -= topLevelItem->GetChildrenCount(true);
 
 		if (updateSelections)
@@ -1380,8 +1384,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 		didRemoveParent = false;
 	}
 
-	if (updateItemCount)
-	{
+	if (updateItemCount) {
 		if (m_fileCountChanged || m_folderScanCountChanged)
 			DisplayNumberQueuedFiles();
 		if (oldCount > m_itemCount)
@@ -1435,7 +1438,7 @@ void CQueueViewBase::OnChar(wxKeyEvent& event)
 	m_pQueue->SetSelection(selection);
 }
 
-void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
+void CQueueViewBase::OnEndColumnDrag(wxListEvent&)
 {
 	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); i++)
 	{
@@ -1458,8 +1461,9 @@ void CQueueViewBase::OnTimer(wxTimerEvent& event)
 		return;
 	}
 
-	if (m_fileCountChanged || m_folderScanCountChanged)
+	if (m_fileCountChanged || m_folderScanCountChanged) {
 		DisplayNumberQueuedFiles();
+	}
 }
 
 void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
@@ -1475,6 +1479,35 @@ void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
 		event.Skip();
 }
 
+void CQueueViewBase::WriteToFile(pugi::xml_node element) const
+{
+	auto queue = element.child("Queue");
+	if (!queue) {
+		queue = element.append_child("Queue");
+	}
+
+	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
+		(*iter)->SaveItem(queue);
+}
+
+void CQueueViewBase::OnExport(wxCommandEvent&)
+{
+	wxFileDialog dlg(m_parent, _("Select file for exported queue"), wxString(),
+		_T("FileZilla.xml"), _T("XML files (*.xml)|*.xml"),
+		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+
+	if (dlg.ShowModal() != wxID_OK)
+		return;
+
+	CXmlFile xml(dlg.GetPath());
+
+	auto exportRoot = xml.CreateEmpty();
+
+	WriteToFile(exportRoot);
+
+	xml.Save(true);
+}
+
 // ------
 // CQueue
 // ------
