@@ -4,8 +4,9 @@
 #include "aui_notebook_ex.h"
 #include "listctrlex.h"
 #include "edithandler.h"
+#include "optional.h"
 
-enum class QueuePriority {
+enum class QueuePriority : char {
 	lowest,
 	low,
 	normal,
@@ -30,7 +31,7 @@ enum class TransferDirection
 	upload
 };
 
-class TiXmlElement;
+namespace pugi { class xml_node; }
 class CQueueItem
 {
 public:
@@ -39,27 +40,24 @@ public:
 	virtual void SetPriority(QueuePriority priority);
 
 	virtual void AddChild(CQueueItem* pItem);
-	unsigned int GetChildrenCount(bool recursive);
-	CQueueItem* GetChild(unsigned int item, bool recursive = true);
+	virtual unsigned int GetChildrenCount(bool recursive) const;
+	virtual CQueueItem* GetChild(unsigned int item, bool recursive = true);
 	CQueueItem* GetParent() { return m_parent; }
 	const CQueueItem* GetParent() const { return m_parent; }
 	void SetParent(CQueueItem* parent) { m_parent = parent; }
-	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true); // Removes a child item with is somewhere in the tree of children.
-	virtual bool TryRemoveAll(); // Removes a inactive childrens, queues active children for removal.
-								 // Returns true if item can be removed itself
+
+	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true, bool forward = true); // Removes a child item with is somewhere in the tree of children.
+	virtual bool TryRemoveAll() = 0; // Removes a inactive childrens, queues active children for removal. Returns true if item itself can be removed
 	CQueueItem* GetTopLevelItem();
 	const CQueueItem* GetTopLevelItem() const;
 	int GetItemIndex() const; // Return the visible item index relative to the topmost parent item.
-	virtual void SaveItem(TiXmlElement*) const {}
+	virtual void SaveItem(pugi::xml_node&) const {}
 
 	virtual QueueItemType GetType() const = 0;
 
-	wxDateTime GetTime() const { return m_time; }
-	void UpdateTime() { m_time = wxDateTime::Now(); }
+	fz::datetime GetTime() const { return m_time; }
+	void UpdateTime() { m_time = fz::datetime::now(); }
 
-	const wxString& GetIndent() const;
-
-	const std::vector<CQueueItem*>& GetChildren() const { return m_children; }
 	int GetRemovedAtFront() const { return m_removed_at_front; }
 
 protected:
@@ -67,21 +65,9 @@ protected:
 
 	CQueueItem* m_parent;
 
-	int m_visibleOffspring; // Visible offspring over all sublevels
-	int m_indent;
-
-	int m_maxCachedIndex;
-
-	struct t_cacheItem
-	{
-		int index;
-		int child;
-	};
-	std::vector<t_cacheItem> m_lookupCache;
-
 	friend class CServerItem;
 
-	wxDateTime m_time;
+	fz::datetime m_time;
 
 private:
 	std::vector<CQueueItem*> m_children;
@@ -89,7 +75,7 @@ private:
 	// Number of items removed at front of list
 	// Increased instead of calling slow m_children.erase(0),
 	// resetted on insert.
-	int m_removed_at_front;
+	int m_removed_at_front{};
 };
 
 class CFileItem;
@@ -104,20 +90,23 @@ public:
 	wxString GetName() const;
 
 	virtual void AddChild(CQueueItem* pItem);
+	virtual unsigned int GetChildrenCount(bool recursive) const;
+	virtual CQueueItem* GetChild(unsigned int item, bool recursive = true);
 
 	CFileItem* GetIdleChild(bool immadiateOnly, TransferDirection direction);
-	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true); // Removes a child item with is somewhere in the tree of children
-	wxLongLong GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
+
+	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true, bool forward = true); // Removes a child item with is somewhere in the tree of children
+	virtual bool TryRemoveAll();
+
+	int64_t GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
 
 	void QueueImmediateFiles();
 	void QueueImmediateFile(CFileItem* pItem);
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
 	void SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction);
 
-	virtual bool TryRemoveAll();
-
 	void DetachChildren();
 
 	virtual void SetPriority(QueuePriority priority);
@@ -126,16 +115,30 @@ public:
 
 	int m_activeCount;
 
+	const std::vector<CQueueItem*>& GetChildren() const { return m_children; }
+
 protected:
 	void AddFileItemToList(CFileItem* pItem);
-	void RemoveFileItemFromList(CFileItem* pItem);
+	void RemoveFileItemFromList(CFileItem* pItem, bool forward);
 
 	CServer m_server;
 
 	// array of item lists, sorted by priority. Used by scheduler to find
 	// next file to transfer
 	// First index specifies whether the item is queued (0) or immediate (1)
-	std::list<CFileItem*> m_fileList[2][static_cast<int>(QueuePriority::count)];
+	std::deque<CFileItem*> m_fileList[2][static_cast<int>(QueuePriority::count)];
+
+	friend class CQueueItem;
+
+	int m_visibleOffspring{}; // Visible offspring over all sublevels
+	int m_maxCachedIndex{-1};
+
+	struct t_cacheItem
+	{
+		int index;
+		int child;
+	};
+	std::vector<t_cacheItem> m_lookupCache;
 };
 
 struct t_EngineData;
@@ -145,7 +148,7 @@ class CFileItem : public CQueueItem
 public:
 	CFileItem(CServerItem* parent, bool queued, bool download,
 		const wxString& sourceFile, const wxString& targetFile,
-		const CLocalPath& localPath, const CServerPath& remotePath, wxLongLong size);
+		const CLocalPath& localPath, const CServerPath& remotePath, int64_t size);
 
 	virtual ~CFileItem();
 
@@ -153,14 +156,14 @@ public:
 	void SetPriorityRaw(QueuePriority priority);
 	QueuePriority GetPriority() const;
 
-	const wxString& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile.empty() ? m_sourceFile : m_targetFile); }
-	const wxString& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile.empty() ? m_sourceFile : m_targetFile); }
-	const wxString& GetSourceFile() const { return m_sourceFile; }
-	const wxString& GetTargetFile() const { return m_targetFile; }
-	const CLocalPath& GetLocalPath() const { return m_localPath; }
-	const CServerPath& GetRemotePath() const { return m_remotePath; }
-	const wxLongLong& GetSize() const { return m_size; }
-	void SetSize(wxLongLong size) { m_size = size; }
+	std::wstring const& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	std::wstring const& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	std::wstring const& GetSourceFile() const { return m_sourceFile; }
+	CSparseOptional<std::wstring> const& GetTargetFile() const { return m_targetFile; }
+	CLocalPath const& GetLocalPath() const { return m_localPath; }
+	CServerPath const& GetRemotePath() const { return m_remotePath; }
+	int64_t GetSize() const { return m_size; }
+	void SetSize(int64_t size) { m_size = size; }
 	inline bool Download() const { return flags & flag_download; }
 
 	inline bool queued() const { return (flags & flag_queued) != 0; }
@@ -186,23 +189,55 @@ public:
 	bool IsActive() const { return (flags & flag_active) != 0; }
 	virtual void SetActive(bool active);
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
-	virtual bool TryRemoveAll(); // Removes a inactive childrens, queues active children for removal.
+	virtual bool TryRemoveAll(); // Removes inactive children, queues active children for removal.
 								 // Returns true if item can be removed itself
 
-	void SetTargetFile(const wxString &file);
+	void SetTargetFile(wxString const& file);
 
-	int m_errorCount;
-	CEditHandler::fileType m_edit;
+	enum Status : char {
+		none,
+		incorrect_password,
+		timeout,
+		disconnecting,
+		disconnected,
+		connecting,
+		connection_failed,
+		interrupted,
+		wait_browsing,
+		wait_password,
+		local_file_unwriteable,
+		could_not_start,
+		transferring,
+		creating_dir
+	};
 
-	wxString m_statusMessage;
+	wxString const& GetStatusMessage() const;
+	void SetStatusMessage(Status status);
 
-	CFileTransferCommand::t_transferSettings m_transferSettings;
+	unsigned char m_errorCount{};
+	CEditHandler::fileType m_edit{CEditHandler::none};
+	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction{CFileExistsNotification::unknown};
+	CFileExistsNotification::OverwriteAction m_onetime_action{CFileExistsNotification::unknown};
+	QueuePriority m_priority{QueuePriority::normal};
 
-	t_EngineData* m_pEngineData;
+protected:
+	enum : unsigned char
+	{
+		flag_download = 0x01,
+		flag_active = 0x02,
+		flag_made_progress = 0x04,
+		flag_queued = 0x08,
+		flag_remove = 0x10,
+		flag_ascii = 0x20
+	};
+	unsigned char flags{};
+	Status m_status{};
+
+public:
+	t_EngineData* m_pEngineData{};
 
-	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
 
 	inline bool made_progress() const { return (flags & flag_made_progress) != 0; }
 	inline void set_made_progress(bool made_progress)
@@ -213,26 +248,24 @@ public:
 			flags &= ~flag_made_progress;
 	}
 
-	CFileExistsNotification::OverwriteAction m_onetime_action;
+	bool Ascii() const { return (flags & flag_ascii) != 0; }
+
+	void SetAscii(bool ascii)
+	{
+		if (ascii) {
+			flags |= flag_ascii;
+		}
+		else {
+			flags &= ~flag_ascii;
+		}
+	}
 
 protected:
-	enum
-	{
-		flag_download = 0x01,
-		flag_active = 0x02,
-		flag_made_progress = 0x04,
-		flag_queued = 0x08,
-		flag_remove = 0x10
-	};
-	int flags;
-
-	QueuePriority m_priority;
-
-	wxString m_sourceFile;
-	wxString m_targetFile;
-	const CLocalPath m_localPath;
-	const CServerPath m_remotePath;
-	wxLongLong m_size;
+	std::wstring const m_sourceFile;
+	CSparseOptional<std::wstring> m_targetFile;
+	CLocalPath const m_localPath;
+	CServerPath const m_remotePath;
+	int64_t m_size{};
 };
 
 class CFolderItem : public CFileItem
@@ -243,9 +276,11 @@ public:
 
 	virtual QueueItemType GetType() const { return QueueItemType::Folder; }
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
 	virtual void SetActive(bool active);
+
+	virtual bool TryRemoveAll() { return true; }
 };
 
 class CFolderScanItem : public CQueueItem
@@ -266,14 +301,14 @@ public:
 
 	wxString m_statusMessage;
 
-	volatile bool m_remove;
-	bool m_active;
+	volatile bool m_remove{};
+	bool m_active{};
 
-	int m_count;
+	int m_count{};
 
-	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
+	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction{CFileExistsNotification::unknown};
 
-	bool m_dir_is_empty;
+	bool m_dir_is_empty{};
 	CLocalPath m_current_local_path;
 	CServerPath m_current_remote_path;
 
@@ -291,6 +326,8 @@ public:
 	virtual ~CStatusItem() {}
 
 	virtual QueueItemType GetType() const { return QueueItemType::Status; }
+
+	virtual bool TryRemoveAll() { return true; }
 };
 
 class CQueue;
@@ -317,7 +354,7 @@ public:
 	CServerItem* CreateServerItem(const CServer& server);
 
 	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem);
-	virtual bool RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount = true, bool updateSelections = true);
+	virtual bool RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true);
 
 	// Has to be called after adding or removing items. Also updates
 	// item count and selections.
@@ -327,6 +364,8 @@ public:
 
 	int GetFileCount() const { return m_fileCount; }
 
+	void WriteToFile(pugi::xml_node element) const;
+
 protected:
 
 	void CreateColumns(std::list<ColumnId> const& extraColumns = std::list<ColumnId>());
@@ -336,7 +375,7 @@ protected:
 	CServerItem* GetServerItem(const CServer& server);
 
 	// Gets item with given index
-	CQueueItem* GetQueueItem(unsigned int item);
+	CQueueItem* GetQueueItem(unsigned int item) const;
 
 	// Get index for given queue item
 	int GetItemIndex(const CQueueItem* item);
@@ -385,6 +424,7 @@ protected:
 	void OnEndColumnDrag(wxListEvent& event);
 	void OnTimer(wxTimerEvent& event);
 	void OnKeyDown(wxKeyEvent& event);
+	void OnExport(wxCommandEvent&);
 };
 
 class CQueueView;
