@@ -10,7 +10,7 @@
 
 #define INVALID_DATA -1
 
-enum _column_type
+enum class Column_type
 {
 	text,
 	integer
@@ -25,7 +25,7 @@ enum _column_flags
 struct _column
 {
 	const wxChar* const name;
-	_column_type type;
+	Column_type type;
 	unsigned int flags;
 };
 
@@ -53,22 +53,22 @@ namespace server_table_column_names
 }
 
 _column server_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("host"), text, not_null },
-	{ _T("port"), integer, 0 },
-	{ _T("user"), text, 0 },
-	{ _T("password"), text, 0 },
-	{ _T("account"), text, 0 },
-	{ _T("protocol"), integer, 0 },
-	{ _T("type"), integer, 0 },
-	{ _T("logontype"), integer, 0 },
-	{ _T("timezone_offset"), integer, 0 },
-	{ _T("transfer_mode"), text, 0 },
-	{ _T("max_connections"), integer, 0 },
-	{ _T("encoding"), text, 0 },
-	{ _T("bypass_proxy"), integer, 0 },
-	{ _T("post_login_commands"), text, 0 },
-	{ _T("name"), text, 0 }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("host"), Column_type::text, not_null },
+	{ _T("port"), Column_type::integer, 0 },
+	{ _T("user"), Column_type::text, 0 },
+	{ _T("password"), Column_type::text, 0 },
+	{ _T("account"), Column_type::text, 0 },
+	{ _T("protocol"), Column_type::integer, 0 },
+	{ _T("type"), Column_type::integer, 0 },
+	{ _T("logontype"), Column_type::integer, 0 },
+	{ _T("timezone_offset"), Column_type::integer, 0 },
+	{ _T("transfer_mode"), Column_type::text, 0 },
+	{ _T("max_connections"), Column_type::integer, 0 },
+	{ _T("encoding"), Column_type::text, 0 },
+	{ _T("bypass_proxy"), Column_type::integer, 0 },
+	{ _T("post_login_commands"), Column_type::text, 0 },
+	{ _T("name"), Column_type::text, 0 }
 };
 
 namespace file_table_column_names
@@ -91,18 +91,18 @@ namespace file_table_column_names
 }
 
 _column file_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("server"), integer, not_null },
-	{ _T("source_file"), text, 0 },
-	{ _T("target_file"), text, 0 },
-	{ _T("local_path"), integer, 0 },
-	{ _T("remote_path"), integer, 0 },
-	{ _T("download"), integer, not_null },
-	{ _T("size"), integer, 0 },
-	{ _T("error_count"), integer, 0 },
-	{ _T("priority"), integer, 0 },
-	{ _T("ascii_file"), integer, 0 },
-	{ _T("default_exists_action"), integer, 0 }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("server"), Column_type::integer, not_null },
+	{ _T("source_file"), Column_type::text, 0 },
+	{ _T("target_file"), Column_type::text, 0 },
+	{ _T("local_path"), Column_type::integer, 0 },
+	{ _T("remote_path"), Column_type::integer, 0 },
+	{ _T("download"), Column_type::integer, not_null },
+	{ _T("size"), Column_type::integer, 0 },
+	{ _T("error_count"), Column_type::integer, 0 },
+	{ _T("priority"), Column_type::integer, 0 },
+	{ _T("ascii_file"), Column_type::integer, 0 },
+	{ _T("default_exists_action"), Column_type::integer, 0 }
 };
 
 namespace path_table_column_names
@@ -115,8 +115,8 @@ namespace path_table_column_names
 }
 
 _column path_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("path"), text, not_null }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("path"), Column_type::text, not_null }
 };
 
 struct fast_equal
@@ -153,30 +153,30 @@ public:
 	sqlite3_stmt* PrepareInsertStatement(const wxString& name, const _column*, unsigned int count);
 
 	bool SaveServer(const CServerItem& item);
-	bool SaveFile(wxLongLong server, const CFileItem& item);
-	bool SaveDirectory(wxLongLong server, const CFolderItem& item);
+	bool SaveFile(const CFileItem& item);
+	bool SaveDirectory(const CFolderItem& item);
 
-	wxLongLong_t SaveLocalPath(const CLocalPath& path);
-	wxLongLong_t SaveRemotePath(const CServerPath& path);
+	int64_t SaveLocalPath(const CLocalPath& path);
+	int64_t SaveRemotePath(const CServerPath& path);
 
 	void ReadLocalPaths();
 	void ReadRemotePaths();
 
-	const CLocalPath& GetLocalPath(wxLongLong_t id) const;
-	const CServerPath& GetRemotePath(wxLongLong_t id) const;
+	const CLocalPath& GetLocalPath(int64_t id) const;
+	const CServerPath& GetRemotePath(int64_t id) const;
 
 	bool Bind(sqlite3_stmt* statement, int index, int value);
-	bool Bind(sqlite3_stmt* statement, int index, wxLongLong_t value);
+	bool Bind(sqlite3_stmt* statement, int index, int64_t value);
 	bool Bind(sqlite3_stmt* statement, int index, const wxString& value);
 	bool Bind(sqlite3_stmt* statement, int index, const char* const value);
 	bool BindNull(sqlite3_stmt* statement, int index);
 
 	wxString GetColumnText(sqlite3_stmt* statement, int index, bool shrink = true);
-	wxLongLong_t GetColumnInt64(sqlite3_stmt* statement, int index, wxLongLong_t def = 0);
+	int64_t GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def = 0);
 	int GetColumnInt(sqlite3_stmt* statement, int index, int def = 0);
 
-	wxLongLong_t ParseServerFromRow(CServer& server);
-	wxLongLong_t ParseFileFromRow(CFileItem** pItem);
+	int64_t ParseServerFromRow(CServer& server);
+	int64_t ParseFileFromRow(CFileItem** pItem);
 
 	bool MigrateSchema();
 
@@ -199,11 +199,11 @@ public:
 	// Caches to speed up saving and loading
 	void ClearCaches();
 
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash, fast_equal> localPaths_;
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash> remotePaths_; // No need for fast_equal as GetSafePath returns unshared string anyhow
+	std::unordered_map<wxString, int64_t, wxStringHash, fast_equal> localPaths_;
+	std::unordered_map<wxString, int64_t, wxStringHash> remotePaths_; // No need for fast_equal as GetSafePath returns unshared string anyhow
 
-	std::map<wxLongLong_t, CLocalPath> reverseLocalPaths_;
-	std::map<wxLongLong_t, CServerPath> reverseRemotePaths_;
+	std::map<int64_t, CLocalPath> reverseLocalPaths_;
+	std::map<int64_t, CServerPath> reverseRemotePaths_;
 };
 
 
@@ -218,7 +218,7 @@ void CQueueStorage::Impl::ReadLocalPaths()
 		res = sqlite3_step(selectLocalPathQuery_);
 		if (res == SQLITE_ROW)
 		{
-			wxLongLong_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
+			int64_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
 			wxString localPathRaw = GetColumnText(selectLocalPathQuery_, path_table_column_names::path);
 			CLocalPath localPath;
 			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw))
@@ -242,7 +242,7 @@ void CQueueStorage::Impl::ReadRemotePaths()
 		res = sqlite3_step(selectRemotePathQuery_);
 		if (res == SQLITE_ROW)
 		{
-			wxLongLong_t id = GetColumnInt64(selectRemotePathQuery_, path_table_column_names::id);
+			int64_t id = GetColumnInt64(selectRemotePathQuery_, path_table_column_names::id);
 			wxString remotePathRaw = GetColumnText(selectRemotePathQuery_, path_table_column_names::path);
 			CServerPath remotePath;
 			if (id > 0 && !remotePathRaw.empty() && remotePath.SetSafePath(remotePathRaw))
@@ -255,31 +255,31 @@ void CQueueStorage::Impl::ReadRemotePaths()
 }
 
 
-const CLocalPath& CQueueStorage::Impl::GetLocalPath(wxLongLong_t id) const
+const CLocalPath& CQueueStorage::Impl::GetLocalPath(int64_t id) const
 {
-	std::map<wxLongLong_t, CLocalPath>::const_iterator it = reverseLocalPaths_.find(id);
+	std::map<int64_t, CLocalPath>::const_iterator it = reverseLocalPaths_.find(id);
 	if (it != reverseLocalPaths_.end())
 		return it->second;
 
-	static const CLocalPath empty;
+	static CLocalPath const empty{};
 	return empty;
 }
 
 
-const CServerPath& CQueueStorage::Impl::GetRemotePath(wxLongLong_t id) const
+const CServerPath& CQueueStorage::Impl::GetRemotePath(int64_t id) const
 {
-	std::map<wxLongLong_t, CServerPath>::const_iterator it = reverseRemotePaths_.find(id);
+	std::map<int64_t, CServerPath>::const_iterator it = reverseRemotePaths_.find(id);
 	if (it != reverseRemotePaths_.end())
 		return it->second;
 
-	static const CServerPath empty;
+	static CServerPath const empty{};
 	return empty;
 }
 
 
 static int int_callback(void* p, int n, char** v, char**)
 {
-	int* i = reinterpret_cast<int*>(p);
+	int* i = static_cast<int*>(p);
 	if (!i || !n || !v || !*v)
 		return -1;
 
@@ -311,9 +311,9 @@ void CQueueStorage::Impl::ClearCaches()
 }
 
 
-wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
+int64_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 {
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash, fast_equal>::const_iterator it = localPaths_.find(path.GetPath());
+	std::unordered_map<wxString, int64_t, wxStringHash, fast_equal>::const_iterator it = localPaths_.find(path.GetPath());
 	if (it != localPaths_.end())
 		return it->second;
 
@@ -328,7 +328,7 @@ wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 
 	if (res == SQLITE_DONE)
 	{
-		wxLongLong_t id = sqlite3_last_insert_rowid(db_);
+		int64_t id = sqlite3_last_insert_rowid(db_);
 		localPaths_[path.GetPath()] = id;
 		return id;
 	}
@@ -337,14 +337,14 @@ wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 }
 
 
-wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
+int64_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 {
-	wxString safePath = path.GetSafePath();
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash>::const_iterator it = remotePaths_.find(safePath);
+	wxString const& safePath = path.GetSafePath();
+	std::unordered_map<wxString, int64_t, wxStringHash>::const_iterator it = remotePaths_.find(safePath);
 	if (it != remotePaths_.end())
 		return it->second;
 
-	Bind(insertRemotePathQuery_, path_table_column_names::path, path.GetSafePath());
+	Bind(insertRemotePathQuery_, path_table_column_names::path, safePath);
 
 	int res;
 	do {
@@ -353,9 +353,8 @@ wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 
 	sqlite3_reset(insertRemotePathQuery_);
 
-	if (res == SQLITE_DONE)
-	{
-		wxLongLong_t id = sqlite3_last_insert_rowid(db_);
+	if (res == SQLITE_DONE) {
+		int64_t id = sqlite3_last_insert_rowid(db_);
 		remotePaths_[safePath] = id;
 		return id;
 	}
@@ -367,12 +366,11 @@ wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 wxString CQueueStorage::Impl::CreateColumnDefs(_column* columns, size_t count)
 {
 	wxString query = _T("(");
-	for (unsigned int i = 0; i < count; ++i)
-	{
+	for (unsigned int i = 0; i < count; ++i) {
 		if (i)
 			query += _T(", ");
 		query += columns[i].name;
-		if (columns[i].type == integer)
+		if (columns[i].type == Column_type::integer)
 			query += _T(" INTEGER");
 		else
 			query += _T(" TEXT");
@@ -539,7 +537,7 @@ bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int value)
 }
 
 
-bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, wxLongLong_t value)
+bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int64_t value)
 {
 	int res = sqlite3_bind_int64(statement, index, value);
 	return res == SQLITE_OK;
@@ -550,7 +548,7 @@ bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, wxLongLong_t
 extern "C" {
 static void custom_free(void* v)
 {
-	char* s = reinterpret_cast<char*>(v);
+	char* s = static_cast<char*>(v);
 	delete [] s;
 }
 }
@@ -593,20 +591,16 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(server.GetType()));
 
 	enum LogonType logonType = server.GetLogonType();
-	if (server.GetLogonType() != ANONYMOUS)
-	{
+	if (server.GetLogonType() != ANONYMOUS) {
 		Bind(insertServerQuery_, server_table_column_names::user, server.GetUser());
 
-		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT)
-		{
-			if (kiosk_mode)
-			{
+		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT) {
+			if (kiosk_mode) {
 				logonType = ASK;
 				BindNull(insertServerQuery_, server_table_column_names::password);
 				BindNull(insertServerQuery_, server_table_column_names::account);
 			}
-			else
-			{
+			else {
 				Bind(insertServerQuery_, server_table_column_names::password, server.GetPass());
 
 				if (server.GetLogonType() == ACCOUNT)
@@ -615,14 +609,12 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 					BindNull(insertServerQuery_, server_table_column_names::account);
 			}
 		}
-		else
-		{
+		else {
 			BindNull(insertServerQuery_, server_table_column_names::password);
 			BindNull(insertServerQuery_, server_table_column_names::account);
 		}
 	}
-	else
-	{
+	else {
 			BindNull(insertServerQuery_, server_table_column_names::user);
 			BindNull(insertServerQuery_, server_table_column_names::password);
 			BindNull(insertServerQuery_, server_table_column_names::account);
@@ -659,15 +651,11 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 		break;
 	}
 
-	const enum ServerProtocol protocol = server.GetProtocol();
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
 		const std::vector<wxString>& postLoginCommands = server.GetPostLoginCommands();
-		if (!postLoginCommands.empty())
-		{
+		if (!postLoginCommands.empty()) {
 			wxString commands;
-			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter)
-			{
+			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter) {
 				if (!commands.empty())
 					commands += _T("\n");
 				commands += *iter;
@@ -694,38 +682,37 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	sqlite3_reset(insertServerQuery_);
 
 	bool ret = res == SQLITE_DONE;
-	if (ret)
-	{
+	if (ret) {
 		sqlite3_int64 serverId = sqlite3_last_insert_rowid(db_);
-		Bind(insertFileQuery_, file_table_column_names::server, serverId);
+		Bind(insertFileQuery_, file_table_column_names::server, static_cast<int64_t>(serverId));
 
 		const std::vector<CQueueItem*>& children = item.GetChildren();
-		for (std::vector<CQueueItem*>::const_iterator it = children.begin() + item.GetRemovedAtFront(); it != children.end(); ++it)
-		{
-			CQueueItem* item = *it;
-			if (item->GetType() == QueueItemType::File)
-				ret &= SaveFile(serverId, *reinterpret_cast<CFileItem*>(item));
-			else if (item->GetType() == QueueItemType::Folder)
-				ret &= SaveDirectory(serverId, *reinterpret_cast<CFolderItem*>(item));
+		for (std::vector<CQueueItem*>::const_iterator it = children.begin() + item.GetRemovedAtFront(); it != children.end(); ++it) {
+			CQueueItem & childItem = **it;
+			if (childItem.GetType() == QueueItemType::File)
+				ret &= SaveFile(static_cast<CFileItem&>(childItem));
+			else if (childItem.GetType() == QueueItemType::Folder)
+				ret &= SaveDirectory(static_cast<CFolderItem&>(childItem));
 		}
 	}
 	return ret;
 }
 
 
-bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
+bool CQueueStorage::Impl::SaveFile(const CFileItem& file)
 {
 	if (file.m_edit != CEditHandler::none)
 		return true;
 
 	Bind(insertFileQuery_, file_table_column_names::source_file, file.GetSourceFile());
-	if (file.GetTargetFile().empty())
-		BindNull(insertFileQuery_, file_table_column_names::target_file);
+	auto const& targetFile = file.GetTargetFile();
+	if (targetFile)
+		Bind(insertFileQuery_, file_table_column_names::target_file, *targetFile);
 	else
-		Bind(insertFileQuery_, file_table_column_names::target_file, file.GetTargetFile());
+		BindNull(insertFileQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = SaveLocalPath(file.GetLocalPath());
-	wxLongLong_t remotePathId = SaveRemotePath(file.GetRemotePath());
+	int64_t localPathId = SaveLocalPath(file.GetLocalPath());
+	int64_t remotePathId = SaveRemotePath(file.GetRemotePath());
 	if (localPathId == -1 || remotePathId == -1)
 		return false;
 
@@ -734,7 +721,7 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 
 	Bind(insertFileQuery_, file_table_column_names::download, file.Download() ? 1 : 0);
 	if (file.GetSize() != -1)
-		Bind(insertFileQuery_, file_table_column_names::size, file.GetSize().GetValue());
+		Bind(insertFileQuery_, file_table_column_names::size, file.GetSize());
 	else
 		BindNull(insertFileQuery_, file_table_column_names::size);
 	if (file.m_errorCount)
@@ -742,7 +729,7 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 	else
 		BindNull(insertFileQuery_, file_table_column_names::error_count);
 	Bind(insertFileQuery_, file_table_column_names::priority, static_cast<int>(file.GetPriority()));
-	Bind(insertFileQuery_, file_table_column_names::ascii_file, file.m_transferSettings.binary ? 0 : 1);
+	Bind(insertFileQuery_, file_table_column_names::ascii_file, file.Ascii() ? 1 : 0);
 
 	if (file.m_defaultFileExistsAction != CFileExistsNotification::unknown)
 		Bind(insertFileQuery_, file_table_column_names::default_exists_action, file.m_defaultFileExistsAction);
@@ -760,7 +747,7 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 }
 
 
-bool CQueueStorage::Impl::SaveDirectory(wxLongLong server, const CFolderItem& directory)
+bool CQueueStorage::Impl::SaveDirectory(const CFolderItem& directory)
 {
 	if (directory.Download())
 		BindNull(insertFileQuery_, file_table_column_names::source_file);
@@ -768,8 +755,8 @@ bool CQueueStorage::Impl::SaveDirectory(wxLongLong server, const CFolderItem& di
 		Bind(insertFileQuery_, file_table_column_names::source_file, directory.GetSourceFile());
 	BindNull(insertFileQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = directory.Download() ? SaveLocalPath(directory.GetLocalPath()) : -1;
-	wxLongLong_t remotePathId = directory.Download() ? -1 : SaveRemotePath(directory.GetRemotePath());
+	int64_t localPathId = directory.Download() ? SaveLocalPath(directory.GetLocalPath()) : -1;
+	int64_t remotePathId = directory.Download() ? -1 : SaveRemotePath(directory.GetRemotePath());
 	if (localPathId == -1 && remotePathId == -1)
 		return false;
 
@@ -804,11 +791,11 @@ wxString CQueueStorage::Impl::GetColumnText(sqlite3_stmt* statement, int index,
 
 #ifdef __WXMSW__
 	(void)shrink;
-	const wxChar* text = reinterpret_cast<const wxChar*>(sqlite3_column_text16(statement, index));
+	const wxChar* text = static_cast<const wxChar*>(sqlite3_column_text16(statement, index));
 	if (text)
 		ret = text;
 #else
-	const char* text = reinterpret_cast<const char*>(sqlite3_column_text16(statement, index));
+	const char* text = static_cast<const char*>(sqlite3_column_text16(statement, index));
 	int len = sqlite3_column_bytes16(statement, index);
 	if (text)
 	{
@@ -825,7 +812,7 @@ wxString CQueueStorage::Impl::GetColumnText(sqlite3_stmt* statement, int index,
 	return ret;
 }
 
-wxLongLong_t CQueueStorage::Impl::GetColumnInt64(sqlite3_stmt* statement, int index, wxLongLong_t def)
+int64_t CQueueStorage::Impl::GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def)
 {
 	if (sqlite3_column_type(statement, index) == SQLITE_NULL)
 		return def;
@@ -841,7 +828,7 @@ int CQueueStorage::Impl::GetColumnInt(sqlite3_stmt* statement, int index, int de
 		return sqlite3_column_int(statement, index);
 }
 
-wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
+int64_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 {
 	server = CServer();
 
@@ -856,11 +843,10 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 	if (!server.SetHost(host, port))
 		return INVALID_DATA;
 
-	int protocol = GetColumnInt(selectServersQuery_, server_table_column_names::protocol);
+	int const protocol = GetColumnInt(selectServersQuery_, server_table_column_names::protocol);
 	if (protocol < 0 || protocol > MAX_VALUE)
 		return INVALID_DATA;
-
-	server.SetProtocol((enum ServerProtocol)protocol);
+	server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetColumnInt(selectServersQuery_, server_table_column_names::type);
 	if (type < 0 || type >= SERVERTYPE_MAX)
@@ -923,8 +909,7 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 			return INVALID_DATA;
 	}
 
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
 		std::vector<wxString> postLoginCommands;
 
 		wxString commands = GetColumnText(selectServersQuery_, server_table_column_names::post_login_commands);
@@ -956,13 +941,13 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 }
 
 
-wxLongLong_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
+int64_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 {
 	wxString sourceFile = GetColumnText(selectFilesQuery_, file_table_column_names::source_file);
 	wxString targetFile = GetColumnText(selectFilesQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::local_path, false);
-	wxLongLong_t remotePathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::remote_path, false);
+	int64_t localPathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::local_path, false);
+	int64_t remotePathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::remote_path, false);
 
 	const CLocalPath& localPath(GetLocalPath(localPathId));
 	const CServerPath& remotePath(GetRemotePath(remotePathId));
@@ -985,25 +970,24 @@ wxLongLong_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 	}
 	else
 	{
-		wxLongLong size = GetColumnInt64(selectFilesQuery_, file_table_column_names::size);
-		int errorCount = GetColumnInt(selectFilesQuery_, file_table_column_names::error_count);
+		int64_t size = GetColumnInt64(selectFilesQuery_, file_table_column_names::size);
+		unsigned char errorCount = static_cast<unsigned char>(GetColumnInt(selectFilesQuery_, file_table_column_names::error_count));
 		int priority = GetColumnInt(selectFilesQuery_, file_table_column_names::priority, static_cast<int>(QueuePriority::normal));
 
-		bool binary = GetColumnInt(selectFilesQuery_, file_table_column_names::ascii_file) == 0;
+		bool ascii = GetColumnInt(selectFilesQuery_, file_table_column_names::ascii_file) != 0;
 		int overwrite_action = GetColumnInt(selectFilesQuery_, file_table_column_names::default_exists_action, CFileExistsNotification::unknown);
 
 		if (sourceFile.empty() || localPath.empty() ||
 			remotePath.empty() ||
 			size < -1 ||
-			priority < 0 || priority >= static_cast<int>(QueuePriority::count) ||
-			errorCount < 0)
+			priority < 0 || priority >= static_cast<int>(QueuePriority::count))
 		{
 			return INVALID_DATA;
 		}
 
 		CFileItem* fileItem = new CFileItem(0, true, download, sourceFile, targetFile, localPath, remotePath, size);
 		*pItem = fileItem;
-		fileItem->m_transferSettings.binary = binary;
+		fileItem->SetAscii(ascii);
 		fileItem->SetPriorityRaw(QueuePriority(priority));
 		fileItem->m_errorCount = errorCount;
 
@@ -1048,8 +1032,7 @@ bool CQueueStorage::SaveQueue(std::vector<CServerItem*> const& queue)
 	d_->ClearCaches();
 
 	bool ret = true;
-	if (sqlite3_exec(d_->db_, "BEGIN TRANSACTION", 0, 0, 0) == SQLITE_OK)
-	{
+	if (sqlite3_exec(d_->db_, "BEGIN TRANSACTION", 0, 0, 0) == SQLITE_OK) {
 		for (std::vector<CServerItem*>::const_iterator it = queue.begin(); it != queue.end(); ++it)
 			ret &= d_->SaveServer(**it);
 
@@ -1064,9 +1047,9 @@ bool CQueueStorage::SaveQueue(std::vector<CServerItem*> const& queue)
 	return ret;
 }
 
-wxLongLong_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
+int64_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
 {
-	wxLongLong_t ret = -1;
+	int64_t ret = -1;
 
 	if (d_->selectServersQuery_)
 	{
@@ -1114,9 +1097,9 @@ wxLongLong_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
 }
 
 
-wxLongLong_t CQueueStorage::GetFile(CFileItem** pItem, wxLongLong_t server)
+int64_t CQueueStorage::GetFile(CFileItem** pItem, int64_t server)
 {
-	wxLongLong_t ret = -1;
+	int64_t ret = -1;
 	*pItem = 0;
 
 	if (d_->selectFilesQuery_)
