@@ -9,11 +9,6 @@ public:
 	static void SetMostRecentServer(const CServer& server);
 	static const std::list<CServer> GetMostRecentServers(bool lockMutex = true);
 	static void Clear();
-
-protected:
-
-	static CXmlFile m_XmlFile;
-	static std::list<CServer> m_mostRecentServers;
 };
 
 #endif //__RECENTSERVERLIST_H__
