@@ -5,22 +5,44 @@
 #include "filter.h"
 #include "Options.h"
 #include "queue.h"
-#include "local_filesys.h"
 
-CRecursiveOperation::CNewDir::CNewDir()
+recursion_root::recursion_root(CServerPath const& start_dir, bool allow_parent)
+	: m_startDir(start_dir)
+	, m_allowParent(allow_parent)
 {
-	recurse = true;
-	second_try = false;
-	link = 0;
-	doVisit = true;
+	wxASSERT_MSG(!start_dir.empty(), _T("Empty startDir in recursion_root constructor"));
 }
 
+void recursion_root::add_dir_to_visit(CServerPath const& path, wxString const& subdir, CLocalPath const& localDir, bool is_link)
+{
+	new_dir dirToVisit;
+
+	dirToVisit.localDir = localDir;
+	dirToVisit.parent = path;
+	dirToVisit.subdir = subdir;
+	dirToVisit.link = is_link ? 2 : 0;
+	m_dirsToVisit.push_back(dirToVisit);
+}
+
+void recursion_root::add_dir_to_visit_restricted(CServerPath const& path, wxString const& restrict, bool recurse)
+{
+	new_dir dirToVisit;
+	dirToVisit.parent = path;
+	dirToVisit.recurse = recurse;
+	if (!restrict.empty()) {
+		dirToVisit.restrict = CSparseOptional<wxString>(restrict);
+	}
+	m_dirsToVisit.push_back(dirToVisit);
+}
+
+
+
 CRecursiveOperation::CRecursiveOperation(CState* pState)
 	: CStateEventHandler(pState),
 	  m_operationMode(recursive_none)
 {
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR, false);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR, false);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_OTHER);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
 }
 
 CRecursiveOperation::~CRecursiveOperation()
@@ -32,26 +54,30 @@ CRecursiveOperation::~CRecursiveOperation()
 	}
 }
 
-void CRecursiveOperation::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* data2)
+void CRecursiveOperation::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void* data2)
 {
-	if (notification == STATECHANGE_REMOTE_LINKNOTDIR)
-	{
+	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
+		std::shared_ptr<CDirectoryListing> const& listing = *reinterpret_cast<std::shared_ptr<CDirectoryListing> const*>(data2);
+		ProcessDirectoryListing(listing.get());
+	}
+	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR) {
 		wxASSERT(data2);
 		LinkIsNotDir();
 	}
-	else
-	{
-		wxASSERT(pState);
-		wxASSERT(notification == STATECHANGE_REMOTE_DIR);
-		ProcessDirectoryListing(pState->GetRemoteDir().get());
+}
+
+void CRecursiveOperation::AddRecursionRoot(recursion_root && root)
+{
+	if (!root.empty()) {
+		recursion_roots_.push_back(std::forward<recursion_root>(root));
 	}
 }
 
-void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const CServerPath& startDir, const std::list<CFilter>& filters, bool allowParent /*=false*/, const CServerPath& finalDir /*=CServerPath()*/)
+void CRecursiveOperation::StartRecursiveOperation(OperationMode mode, std::vector<CFilter> const& filters, CServerPath const& finalDir)
 {
 	wxCHECK_RET(m_operationMode == recursive_none, _T("StartRecursiveOperation called with m_operationMode != recursive_none"));
 	wxCHECK_RET(m_pState->IsRemoteConnected(), _T("StartRecursiveOperation while disconnected"));
-	wxCHECK_RET(!startDir.empty(), _T("Empty startDir in StartRecursiveOperation"));
+	wxCHECK_RET(!finalDir.empty(), _T("Empty final dir in recursive operation"));
 
 	if (mode == recursive_chmod && !m_pChmodDlg)
 		return;
@@ -59,94 +85,80 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 	if ((mode == recursive_download || mode == recursive_addtoqueue || mode == recursive_download_flatten || mode == recursive_addtoqueue_flatten) && !m_pQueue)
 		return;
 
-	if (m_dirsToVisit.empty())
-	{
+	if (recursion_roots_.empty()) {
 		// Nothing to do in this case
 		return;
 	}
 
+	m_processedFiles = 0;
+	m_processedDirectories = 0;
+
 	m_operationMode = mode;
 	m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
-
-	m_startDir = startDir;
-
-	if (finalDir.empty())
-		m_finalDir = startDir;
-	else
-		m_finalDir = finalDir;
-
-	m_allowParent = allowParent;
+	m_pState->NotifyHandlers(STATECHANGE_RECURSION_STATUS);
 
 	m_filters = filters;
 
 	NextOperation();
 }
 
-void CRecursiveOperation::AddDirectoryToVisit(const CServerPath& path, const wxString& subdir, const CLocalPath& localDir /*=CLocalPath()*/, bool is_link /*=false*/)
-{
-	CNewDir dirToVisit;
-
-	dirToVisit.localDir = localDir;
-	dirToVisit.parent = path;
-	dirToVisit.subdir = subdir;
-	dirToVisit.link = is_link ? 2 : 0;
-	m_dirsToVisit.push_back(dirToVisit);
-}
-
-void CRecursiveOperation::AddDirectoryToVisitRestricted(const CServerPath& path, const wxString& restrict, bool recurse)
-{
-	CNewDir dirToVisit;
-	dirToVisit.parent = path;
-	dirToVisit.recurse = recurse;
-	dirToVisit.restrict = restrict;
-	m_dirsToVisit.push_back(dirToVisit);
-}
-
 bool CRecursiveOperation::NextOperation()
 {
 	if (m_operationMode == recursive_none)
 		return false;
 
-	while (!m_dirsToVisit.empty())
-	{
-		const CNewDir& dirToVisit = m_dirsToVisit.front();
-		if (m_operationMode == recursive_delete && !dirToVisit.doVisit)
-		{
-			m_pState->m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(dirToVisit.parent, dirToVisit.subdir));
-			m_dirsToVisit.pop_front();
-			continue;
+	while (!recursion_roots_.empty()) {
+		auto & root = recursion_roots_.front();
+		while (!root.m_dirsToVisit.empty()) {
+			const recursion_root::new_dir& dirToVisit = root.m_dirsToVisit.front();
+			if (m_operationMode == recursive_delete && !dirToVisit.doVisit) {
+				m_pState->m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(dirToVisit.parent, dirToVisit.subdir), CCommandQueue::recursiveOperation);
+				root.m_dirsToVisit.pop_front();
+				continue;
+			}
+
+			CListCommand* cmd = new CListCommand(dirToVisit.parent, dirToVisit.subdir, dirToVisit.link ? LIST_FLAG_LINK : 0);
+			m_pState->m_pCommandQueue->ProcessCommand(cmd, CCommandQueue::recursiveOperation);
+			return true;
 		}
 
-		CListCommand* cmd = new CListCommand(dirToVisit.parent, dirToVisit.subdir, dirToVisit.link ? LIST_FLAG_LINK : 0);
-		m_pState->m_pCommandQueue->ProcessCommand(cmd);
-		return true;
+		recursion_roots_.pop_front();
+	}
+
+	if (m_operationMode == recursive_delete && !m_finalDir.empty()) {
+		// After a deletion we cannot refresh if inside the deleted directories. Navigate user out if it
+		auto curPath = m_pState->GetRemotePath();
+		if (!curPath.empty() && (curPath == m_finalDir || m_finalDir.IsParentOf(curPath, false))) {
+			StopRecursiveOperation();
+			m_pState->ChangeRemoteDir(m_finalDir, wxString(), LIST_FLAG_REFRESH);
+			return false;
+		}
 	}
 
 	StopRecursiveOperation();
-	m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(m_finalDir));
+	m_pState->RefreshRemote();
 	return false;
 }
 
-bool CRecursiveOperation::BelowRecursionRoot(const CServerPath& path, CNewDir &dir)
+bool CRecursiveOperation::BelowRecursionRoot(const CServerPath& path, recursion_root::new_dir &dir)
 {
-	if (!dir.start_dir.empty())
-	{
+	if (!dir.start_dir.empty()) {
 		if (path.IsSubdirOf(dir.start_dir, false))
 			return true;
 		else
 			return false;
 	}
 
-	if (path.IsSubdirOf(m_startDir, false))
+	auto & root = recursion_roots_.front();
+	if (path.IsSubdirOf(root.m_startDir, false))
 		return true;
 
 	// In some cases (chmod from tree for example) it is neccessary to list the
 	// parent first
-	if (path == m_startDir && m_allowParent)
+	if (path == root.m_startDir && root.m_allowParent)
 		return true;
 
-	if (dir.link == 2)
-	{
+	if (dir.link == 2) {
 		dir.start_dir = path;
 		return true;
 	}
@@ -159,73 +171,66 @@ extern wxString StripVMSRevision(const wxString& name);
 
 void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDirectoryListing)
 {
-	if (!pDirectoryListing)
-	{
+	if (!pDirectoryListing) {
 		StopRecursiveOperation();
 		return;
 	}
 
-	if (m_operationMode == recursive_none)
+	if (m_operationMode == recursive_none || recursion_roots_.empty())
 		return;
 
-	if (pDirectoryListing->failed())
-	{
+	if (pDirectoryListing->failed()) {
 		// Ignore this.
 		// It will get handled by the failed command in ListingFailed
 		return;
 	}
 
-	wxASSERT(!m_dirsToVisit.empty());
+	auto & root = recursion_roots_.front();
+	wxASSERT(!root.m_dirsToVisit.empty());
 
-	if (!m_pState->IsRemoteConnected() || m_dirsToVisit.empty())
-	{
+	if (!m_pState->IsRemoteConnected() || root.m_dirsToVisit.empty()) {
 		StopRecursiveOperation();
 		return;
 	}
 
-	CNewDir dir = m_dirsToVisit.front();
-	m_dirsToVisit.pop_front();
+	recursion_root::new_dir dir = root.m_dirsToVisit.front();
+	root.m_dirsToVisit.pop_front();
 
-	if (!BelowRecursionRoot(pDirectoryListing->path, dir))
-	{
+	if (!BelowRecursionRoot(pDirectoryListing->path, dir)) {
 		NextOperation();
 		return;
 	}
 
-	if (m_operationMode == recursive_delete && dir.doVisit && !dir.subdir.empty())
-	{
+	if (m_operationMode == recursive_delete && dir.doVisit && !dir.subdir.empty()) {
 		// After recursing into directory to delete its contents, delete directory itself
 		// Gets handled in NextOperation
-		CNewDir dir2 = dir;
+		recursion_root::new_dir dir2 = dir;
 		dir2.doVisit = false;
-		m_dirsToVisit.push_front(dir2);
+		root.m_dirsToVisit.push_front(dir2);
 	}
 
-	if (dir.link && !dir.recurse)
-	{
+	if (dir.link && !dir.recurse) {
 		NextOperation();
 		return;
 	}
 
 	// Check if we have already visited the directory
-	if (!m_visitedDirs.insert(pDirectoryListing->path).second)
-	{
+	if (!root.m_visitedDirs.insert(pDirectoryListing->path).second) {
 		NextOperation();
 		return;
 	}
 
+	++m_processedDirectories;
+
 	const CServer* pServer = m_pState->GetServer();
 	wxASSERT(pServer);
 
-	if (!pDirectoryListing->GetCount())
-	{
-		if (m_operationMode == recursive_download)
-		{
+	if (!pDirectoryListing->GetCount()) {
+		if (m_operationMode == recursive_download) {
 			wxFileName::Mkdir(dir.localDir.GetPath(), 0777, wxPATH_MKDIR_FULL);
 			m_pState->RefreshLocalFile(dir.localDir.GetPath());
 		}
-		else if (m_operationMode == recursive_addtoqueue)
-		{
+		else if (m_operationMode == recursive_addtoqueue) {
 			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), *pServer, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
@@ -234,31 +239,31 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 	CFilterManager filter;
 
 	// Is operation restricted to a single child?
-	bool restrict = !dir.restrict.empty();
+	bool const restrict = static_cast<bool>(dir.restrict);
 
-	std::list<wxString> filesToDelete;
+	std::deque<wxString> filesToDelete;
 
 	const wxString path = pDirectoryListing->path.GetPath();
 
 	bool added = false;
 
-	for (int i = pDirectoryListing->GetCount() - 1; i >= 0; --i)
-	{
+	for (int i = pDirectoryListing->GetCount() - 1; i >= 0; --i) {
 		const CDirentry& entry = (*pDirectoryListing)[i];
 
-		if (restrict)
-		{
-			if (entry.name != dir.restrict)
+		if (restrict) {
+			if (entry.name != *dir.restrict)
 				continue;
 		}
-		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, false, 0, entry.time))
+		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, 0, entry.time))
 			continue;
 
-		if (entry.is_dir() && (!entry.is_link() || m_operationMode != recursive_delete))
-		{
-			if (dir.recurse)
-			{
-				CNewDir dirToVisit;
+		if (!entry.is_dir()) {
+			++m_processedFiles;
+		}
+
+		if (entry.is_dir() && (!entry.is_link() || m_operationMode != recursive_delete)) {
+			if (dir.recurse) {
+				recursion_root::new_dir dirToVisit;
 				dirToVisit.parent = pDirectoryListing->path;
 				dirToVisit.subdir = entry.name;
 				dirToVisit.localDir = dir.localDir;
@@ -266,16 +271,14 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 
 				if (m_operationMode == recursive_download || m_operationMode == recursive_addtoqueue)
 					dirToVisit.localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(entry.name));
-				if (entry.is_link())
-				{
+				if (entry.is_link()) {
 					dirToVisit.link = 1;
 					dirToVisit.recurse = false;
 				}
-				m_dirsToVisit.push_front(dirToVisit);
+				root.m_dirsToVisit.push_front(dirToVisit);
 			}
 		}
-		else
-		{
+		else {
 			switch (m_operationMode)
 			{
 			case recursive_download:
@@ -310,8 +313,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			}
 		}
 
-		if (m_operationMode == recursive_chmod && m_pChmodDlg)
-		{
+		if (m_operationMode == recursive_chmod && m_pChmodDlg) {
 			const int applyType = m_pChmodDlg->GetApplyType();
 			if (!applyType ||
 				(!entry.is_dir() && applyType == 1) ||
@@ -320,7 +322,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 				char permissions[9];
 				bool res = m_pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
 				wxString newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
-				m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms));
+				m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms), CCommandQueue::recursiveOperation);
 			}
 		}
 	}
@@ -328,7 +330,9 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue && m_operationMode != recursive_addtoqueue_flatten);
 
 	if (m_operationMode == recursive_delete && !filesToDelete.empty())
-		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(pDirectoryListing->path, filesToDelete));
+		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(pDirectoryListing->path, std::move(filesToDelete)), CCommandQueue::recursiveOperation);
+
+	m_pState->NotifyHandlers(STATECHANGE_RECURSION_STATUS);
 
 	NextOperation();
 }
@@ -345,16 +349,14 @@ void CRecursiveOperation::SetChmodDialog(CChmodDialog* pChmodDialog)
 
 void CRecursiveOperation::StopRecursiveOperation()
 {
-	if (m_operationMode != recursive_none)
-	{
+	if (m_operationMode != recursive_none) {
 		m_operationMode = recursive_none;
 		m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+		m_pState->NotifyHandlers(STATECHANGE_RECURSION_STATUS);
 	}
-	m_dirsToVisit.clear();
-	m_visitedDirs.clear();
+	recursion_roots_.clear();
 
-	if (m_pChmodDlg)
-	{
+	if (m_pChmodDlg) {
 		m_pChmodDlg->Destroy();
 		m_pChmodDlg = 0;
 	}
@@ -362,7 +364,7 @@ void CRecursiveOperation::StopRecursiveOperation()
 
 void CRecursiveOperation::ListingFailed(int error)
 {
-	if (m_operationMode == recursive_none)
+	if (m_operationMode == recursive_none || recursion_roots_.empty())
 		return;
 
 	if( (error & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
@@ -371,18 +373,16 @@ void CRecursiveOperation::ListingFailed(int error)
 		return;
 	}
 
-	wxASSERT(!m_dirsToVisit.empty());
-	if (m_dirsToVisit.empty())
-		return;
+	auto & root = recursion_roots_.front();
+	wxCHECK_RET(!root.m_dirsToVisit.empty(), _T("Empty dirs to visit"));
 
-	CNewDir dir = m_dirsToVisit.front();
-	m_dirsToVisit.pop_front();
-	if ((error & FZ_REPLY_CRITICALERROR) != FZ_REPLY_CRITICALERROR && !dir.second_try)
-	{
+	recursion_root::new_dir dir = root.m_dirsToVisit.front();
+	root.m_dirsToVisit.pop_front();
+	if ((error & FZ_REPLY_CRITICALERROR) != FZ_REPLY_CRITICALERROR && !dir.second_try) {
 		// Retry, could have been a temporary socket creating failure
 		// (e.g. hitting a blocked port) or a disconnect (e.g. no-filetransfer-timeout)
 		dir.second_try = true;
-		m_dirsToVisit.push_front(dir);
+		root.m_dirsToVisit.push_front(dir);
 	}
 
 	NextOperation();
@@ -405,36 +405,31 @@ bool CRecursiveOperation::ChangeOperationMode(enum OperationMode mode)
 
 void CRecursiveOperation::LinkIsNotDir()
 {
-	if (m_operationMode == recursive_none)
+	if (m_operationMode == recursive_none || recursion_roots_.empty())
 		return;
 
-	wxASSERT(!m_dirsToVisit.empty());
-	if (m_dirsToVisit.empty())
-		return;
+	auto & root = recursion_roots_.front();
+	wxCHECK_RET(!root.m_dirsToVisit.empty(), _T("Empty dirs to visit"));
 
-	CNewDir dir = m_dirsToVisit.front();
-	m_dirsToVisit.pop_front();
+	recursion_root::new_dir dir = root.m_dirsToVisit.front();
+	root.m_dirsToVisit.pop_front();
 
 	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	if (!pServer) {
 		NextOperation();
 		return;
 	}
 
-	if (m_operationMode == recursive_delete)
-	{
-		if (!dir.subdir.empty())
-		{
-			std::list<wxString> files;
+	if (m_operationMode == recursive_delete) {
+		if (!dir.subdir.empty()) {
+			std::deque<wxString> files;
 			files.push_back(dir.subdir);
-			m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(dir.parent, files));
+			m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(dir.parent, std::move(files)), CCommandQueue::recursiveOperation);
 		}
 		NextOperation();
 		return;
 	}
-	else if (m_operationMode != recursive_list )
-	{
+	else if (m_operationMode != recursive_list) {
 		CLocalPath localPath = dir.localDir;
 		wxString localFile = dir.subdir;
 		if (m_operationMode != recursive_addtoqueue_flatten && m_operationMode != recursive_download_flatten)
