@@ -12,6 +12,7 @@
 #include "sizeformatting.h"
 #include "timeformatting.h"
 #include "window_state_manager.h"
+#include "xrc_helper.h"
 
 class CSearchFileData : public CGenericFileData, public CDirentry
 {
@@ -19,6 +20,34 @@ public:
 	CServerPath path;
 };
 
+template<>
+inline int DoCmpName(CSearchFileData const& data1, CSearchFileData const& data2, CFileListCtrlSortBase::NameSortMode const nameSortMode)
+{
+	int res;
+	switch (nameSortMode)
+	{
+	case CFileListCtrlSortBase::namesort_casesensitive:
+		res = CFileListCtrlSortBase::CmpCase(data1.name, data2.name);
+		break;
+	default:
+	case CFileListCtrlSortBase::namesort_caseinsensitive:
+		res = CFileListCtrlSortBase::CmpNoCase(data1.name, data2.name);
+		break;
+	case CFileListCtrlSortBase::namesort_natural:
+		res = CFileListCtrlSortBase::CmpNatural(data1.name, data2.name);
+		break;
+	}
+
+	if (!res) {
+		if (data1.path < data2.path)
+			res = -1;
+		else if (data2.path < data1.path)
+			res = 1;
+	}
+
+	return res;
+}
+
 class CSearchDialogFileList : public CFileListCtrl<CSearchFileData>
 {
 	friend class CSearchDialog;
@@ -29,7 +58,7 @@ public:
 protected:
 	virtual bool ItemIsDir(int index) const;
 
-	virtual wxLongLong ItemGetSize(int index) const;
+	virtual int64_t ItemGetSize(int index) const;
 
 	CFileListCtrl<CSearchFileData>::CSortComparisonObject GetSortComparisonObject();
 
@@ -43,9 +72,9 @@ protected:
 #endif
 
 private:
-	virtual bool CanStartComparison(wxString*) { return false; }
+	virtual bool CanStartComparison() { return false; }
 	virtual void StartComparison() {}
-	virtual bool GetNextFile(wxString&, bool &, wxLongLong &, CDateTime&) { return false; }
+	virtual bool get_next_file(wxString&, bool &, int64_t&, fz::datetime&) { return false; }
 	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags) {}
 	virtual void FinishComparison() {}
 	virtual void ScrollTopItem(int) {}
@@ -91,7 +120,7 @@ bool CSearchDialogFileList::ItemIsDir(int index) const
 	return m_fileData[index].is_dir();
 }
 
-wxLongLong CSearchDialogFileList::ItemGetSize(int index) const
+int64_t CSearchDialogFileList::ItemGetSize(int index) const
 {
 	return m_fileData[index].size;
 }
@@ -101,8 +130,7 @@ CFileListCtrl<CSearchFileData>::CSortComparisonObject CSearchDialogFileList::Get
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -116,10 +144,9 @@ CFileListCtrl<CSearchFileData>::CSortComparisonObject CSearchDialogFileList::Get
 		else if (m_sortColumn == 6)
 			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortNamePath<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -133,7 +160,7 @@ CFileListCtrl<CSearchFileData>::CSortComparisonObject CSearchDialogFileList::Get
 		else if (m_sortColumn == 6)
 			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortNamePath<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 }
 
@@ -148,18 +175,15 @@ wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 		return entry.name;
 	else if (column == 1)
 		return m_fileData[index].path.GetPath();
-	else if (column == 2)
-	{
+	else if (column == 2) {
 		if (entry.is_dir() || entry.size < 0)
 			return wxString();
 		else
 			return CSizeFormat::Format(entry.size);
 	}
-	else if (column == 3)
-	{
+	else if (column == 3) {
 		CSearchFileData& data = m_fileData[index];
-		if (data.fileType.empty())
-		{
+		if (data.fileType.empty()) {
 			if (data.path.GetType() == VMS)
 				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
 			else
@@ -201,6 +225,7 @@ EVT_BUTTON(XRCID("ID_START"), CSearchDialog::OnSearch)
 EVT_BUTTON(XRCID("ID_STOP"), CSearchDialog::OnStop)
 EVT_CONTEXT_MENU(CSearchDialog::OnContextMenu)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DOWNLOAD"), CSearchDialog::OnDownload)
+EVT_MENU(XRCID("ID_MENU_SEARCH_EDIT"), CSearchDialog::OnEdit)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DELETE"), CSearchDialog::OnDelete)
 EVT_CHAR_HOOK(CSearchDialog::OnCharHook)
 END_EVENT_TABLE()
@@ -208,17 +233,13 @@ END_EVENT_TABLE()
 CSearchDialog::CSearchDialog(wxWindow* parent, CState* pState, CQueueView* pQueue)
 	: CStateEventHandler(pState)
 	, m_parent(parent)
-	, m_results(0)
 	, m_pQueue(pQueue)
-	, m_pWindowStateManager(0)
-	, m_searching(false)
 {
 }
 
 CSearchDialog::~CSearchDialog()
 {
-	if (m_pWindowStateManager)
-	{
+	if (m_pWindowStateManager) {
 		m_pWindowStateManager->Remember(OPTION_SEARCH_SIZE);
 		delete m_pWindowStateManager;
 	}
@@ -232,6 +253,7 @@ bool CSearchDialog::Load()
 	/* XRCed complains if adding a status bar to a dialog, so do it here instead */
 	CFilelistStatusBar* pStatusBar = new CFilelistStatusBar(this);
 	pStatusBar->SetEmptyString(_("No search results"));
+	pStatusBar->SetConnected(true);
 
 	GetSizer()->Add(pStatusBar, 0, wxGROW);
 
@@ -245,7 +267,7 @@ bool CSearchDialog::Load()
 
 	const CServerPath path = m_pState->GetRemotePath();
 	if (!path.empty())
-		XRCCTRL(*this, "ID_PATH", wxTextCtrl)->ChangeValue(path.GetPath());
+		xrc_call(*this, "ID_PATH", &wxTextCtrl::ChangeValue, path.GetPath());
 
 	SetCtrlState();
 
@@ -256,7 +278,10 @@ bool CSearchDialog::Load()
 
 	LoadConditions();
 	EditFilter(m_search_filter);
-	XRCCTRL(*this, "ID_CASE", wxCheckBox)->SetValue(m_search_filter.matchCase);
+
+	xrc_call(*this, "ID_CASE", &wxCheckBox::SetValue, m_search_filter.matchCase);
+	xrc_call(*this, "ID_FIND_FILES", &wxCheckBox::SetValue, m_search_filter.filterFiles);
+	xrc_call(*this, "ID_FIND_DIRS", &wxCheckBox::SetValue, m_search_filter.filterDirs);
 
 	return true;
 }
@@ -266,53 +291,48 @@ void CSearchDialog::Run()
 	m_original_dir = m_pState->GetRemotePath();
 	m_local_target = m_pState->GetLocalDir();
 
-	m_pState->BlockHandlers(STATECHANGE_REMOTE_DIR);
-	m_pState->BlockHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
-	m_pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR, false);
-	m_pState->RegisterHandler(this, STATECHANGE_REMOTE_IDLE, false);
+	m_pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_OTHER, m_pState->GetRecursiveOperationHandler());
+	m_pState->RegisterHandler(this, STATECHANGE_REMOTE_IDLE, m_pState->GetRecursiveOperationHandler());
 
 	ShowModal();
 
 	SaveConditions();
 
 	m_pState->UnregisterHandler(this, STATECHANGE_REMOTE_IDLE);
-	m_pState->UnregisterHandler(this, STATECHANGE_REMOTE_DIR);
-	m_pState->UnblockHandlers(STATECHANGE_REMOTE_DIR);
-	m_pState->UnblockHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
+	m_pState->UnregisterHandler(this, STATECHANGE_REMOTE_DIR_OTHER);
 
-	if (m_searching)
-	{
-		if (!m_pState->IsRemoteIdle())
-		{
+	if (m_searching) {
+		if (!m_pState->IsRemoteIdle()) {
 			m_pState->m_pCommandQueue->Cancel();
 			m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 		}
 		if (!m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
-	else
-	{
+	else {
 		if (m_pState->IsRemoteIdle() && !m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
 }
 
-void CSearchDialog::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CSearchDialog::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* data2)
 {
-	if (notification == STATECHANGE_REMOTE_DIR)
-		ProcessDirectoryListing();
-	else if (notification == STATECHANGE_REMOTE_IDLE)
-	{
+	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
+		auto recursiveOperation = m_pState->GetRecursiveOperationHandler();
+		if (recursiveOperation && recursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_list) {
+			std::shared_ptr<CDirectoryListing> const& listing = *reinterpret_cast<std::shared_ptr<CDirectoryListing> const*>(data2);
+			ProcessDirectoryListing(listing);
+		}
+	}
+	else if (notification == STATECHANGE_REMOTE_IDLE) {
 		if (pState->IsRemoteIdle())
 			m_searching = false;
 		SetCtrlState();
 	}
 }
 
-void CSearchDialog::ProcessDirectoryListing()
+void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> const& listing)
 {
-	std::shared_ptr<CDirectoryListing> listing = m_pState->GetRemoteDir();
-
 	if (!listing || listing->failed())
 		return;
 
@@ -326,11 +346,10 @@ void CSearchDialog::ProcessDirectoryListing()
 	m_results->m_fileData.reserve(m_results->m_fileData.size() + listing->GetCount());
 	m_results->m_indexMapping.reserve(m_results->m_indexMapping.size() + listing->GetCount());
 
-	for (unsigned int i = 0; i < listing->GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < listing->GetCount(); ++i) {
 		const CDirentry& entry = (*listing)[i];
 
-		if (!m_search_filter.filters.empty() && !CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.time))
+		if (!CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.time))
 			continue;
 
 		CSearchFileData data;
@@ -346,17 +365,16 @@ void CSearchDialog::ProcessDirectoryListing()
 			m_results->GetFilelistStatusBar()->AddFile(entry.size);
 	}
 
-	m_results->SetItemCount(old_count + added);
-
-	m_results->SortList(-1, -1, true);
-
-	m_results->RefreshListOnly(false);
+	if (added) {
+		m_results->SetItemCount(old_count + added);
+		m_results->SortList(-1, -1, true);
+		m_results->RefreshListOnly(false);
+	}
 }
 
-void CSearchDialog::OnSearch(wxCommandEvent& event)
+void CSearchDialog::OnSearch(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -364,14 +382,12 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 	CServerPath path;
 
 	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	if (!pServer) {
 		wxMessageBoxEx(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	path.SetType(pServer->GetType());
-	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.empty())
-	{
+	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.empty()) {
 		wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
@@ -380,18 +396,18 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 
 	// Prepare filter
 	wxString error;
-	if (!ValidateFilter(error, true))
-	{
+	if (!ValidateFilter(error, true)) {
 		wxMessageBoxEx(wxString::Format(_("Invalid search conditions: %s"), error), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	m_search_filter = GetFilter();
-	if (!CFilterManager::CompileRegexes(m_search_filter))
-	{
+	if (!CFilterManager::CompileRegexes(m_search_filter)) {
 		wxMessageBoxEx(_("Invalid regular expression in search conditions."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
-	m_search_filter.matchCase = XRCCTRL(*this, "ID_CASE", wxCheckBox)->GetValue();
+	m_search_filter.matchCase = xrc_call(*this, "ID_CASE", &wxCheckBox::GetValue);
+	m_search_filter.filterFiles = xrc_call(*this, "ID_FIND_FILES", &wxCheckBox::GetValue);
+	m_search_filter.filterDirs = xrc_call(*this, "ID_FIND_DIRS", &wxCheckBox::GetValue);
 
 	// Delete old results
 	m_results->ClearSelection();
@@ -405,15 +421,16 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 
 	// Start
 	m_searching = true;
-	m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisitRestricted(path, _T(""), true);
-	std::list<CFilter> filters; // Empty, recurse into everything
-	m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(CRecursiveOperation::recursive_list, path, filters, true);
+	recursion_root root(path, true);
+	root.add_dir_to_visit_restricted(path, _T(""), true);
+	m_pState->GetRecursiveOperationHandler()->AddRecursionRoot(std::move(root));
+	std::vector<CFilter> const filters; // Empty, recurse into everything
+	m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(CRecursiveOperation::recursive_list, filters, path);
 }
 
-void CSearchDialog::OnStop(wxCommandEvent& event)
+void CSearchDialog::OnStop(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		m_pState->m_pCommandQueue->Cancel();
 		m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 	}
@@ -440,6 +457,7 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 	if (!m_pState->IsRemoteIdle()) {
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_DELETE"), false);
+		pMenu->Enable(XRCID("ID_MENU_SEARCH_EDIT"), false);
 	}
 
 	PopupMenu(pMenu);
@@ -461,8 +479,7 @@ public:
 			desc.Printf(wxPLURAL("Selected %d file for transfer.", "Selected %d files for transfer.", count_files), count_files);
 		else if (!count_files)
 			desc.Printf(wxPLURAL("Selected %d directory with its contents for transfer.", "Selected %d directories with their contents for transfer.", count_dirs), count_dirs);
-		else
-		{
+		else {
 			wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
 			wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
 			desc.Printf(_("Selected %s and %s for transfer."), files, dirs);
@@ -489,7 +506,7 @@ EVT_BUTTON(XRCID("ID_BROWSE"), CSearchDownloadDialog::OnBrowse)
 EVT_BUTTON(XRCID("wxID_OK"), CSearchDownloadDialog::OnOK)
 END_EVENT_TABLE()
 
-void CSearchDownloadDialog::OnBrowse(wxCommandEvent& event)
+void CSearchDownloadDialog::OnBrowse(wxCommandEvent&)
 {
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
@@ -498,19 +515,17 @@ void CSearchDownloadDialog::OnBrowse(wxCommandEvent& event)
 		pText->ChangeValue(dlg.GetPath());
 }
 
-void CSearchDownloadDialog::OnOK(wxCommandEvent& event)
+void CSearchDownloadDialog::OnOK(wxCommandEvent&)
 {
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
 	CLocalPath path(pText->GetValue());
-	if (path.empty())
-	{
+	if (path.empty()) {
 		wxMessageBoxEx(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	if (!path.IsWriteable())
-	{
+	if (!path.IsWriteable()) {
 		wxMessageBoxEx(_("You have to enter a writable local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
@@ -518,79 +533,50 @@ void CSearchDownloadDialog::OnOK(wxCommandEvent& event)
 	EndDialog(wxID_OK);
 }
 
-void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<CServerPath> &selected_dirs)
+void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::deque<CServerPath> &selected_dirs)
 {
+	std::deque<CServerPath> dirs;
+
 	int sel = -1;
-	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		if (sel > (int)m_results->m_indexMapping.size())
 			continue;
 		int index = m_results->m_indexMapping[sel];
 
-		if (m_results->m_fileData[index].is_dir())
-		{
+		if (m_results->m_fileData[index].is_dir()) {
 			CServerPath path = m_results->m_fileData[index].path;
 			path.ChangePath(m_results->m_fileData[index].name);
 			if (path.empty())
 				continue;
 
-			bool replaced = false;
-			std::list<CServerPath>::iterator iter = selected_dirs.begin();
-			std::list<CServerPath>::iterator prev;
-
-			// Make sure that selected_dirs does not contain
-			// any directories that are in a parent-child relationship
-			// Resolve by only keeping topmost parents
-			while (iter != selected_dirs.end())
-			{
-				if (*iter == path)
-				{
-					replaced = true;
-					break;
-				}
-
-				if (iter->IsParentOf(path, false))
-				{
-					replaced = true;
-					break;
-				}
-
-				if (iter->IsSubdirOf(path, false))
-				{
-					if (!replaced)
-					{
-						*iter = path;
-						replaced = true;
-					}
-					else
-					{
-						prev = iter++;
-						selected_dirs.erase(prev);
-						continue;
-					}
-				}
-				++iter;
-			}
-			if (!replaced)
-				selected_dirs.push_back(path);
+			dirs.push_back(path);
 		}
 		else
 			selected_files.push_back(index);
 	}
 
+	// Make sure that selected_dirs does not contain
+	// any directories that are in a parent-child relationship
+	// Resolve by only keeping topmost parents
+	std::sort(dirs.begin(), dirs.end());
+	for (CServerPath const& path : dirs) {
+		if (!selected_dirs.empty() && (path.IsSubdirOf(selected_dirs.back(), false) || path == selected_dirs.back())) {
+			continue;
+		}
+		selected_dirs.push_back(path);
+	}
+
 	// Now in a second phase filter out all files that are also in a directory
 	std::list<int> selected_files_new;
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter)
-	{
-		CServerPath path = m_results->m_fileData[*iter].path;
-		std::list<CServerPath>::const_iterator path_iter;
-		for (path_iter = selected_dirs.begin(); path_iter != selected_dirs.end(); ++path_iter)
-		{
+	for (auto const& sel_file : selected_files) {
+		CServerPath const& path = m_results->m_fileData[sel_file].path;
+		std::deque<CServerPath>::iterator path_iter;
+		for (path_iter = selected_dirs.begin(); path_iter != selected_dirs.end(); ++path_iter) {
 			if (*path_iter == path || path_iter->IsParentOf(path, false))
 				break;
 		}
 		if (path_iter == selected_dirs.end())
-			selected_files_new.push_back(*iter);
+			selected_files_new.push_back(sel_file);
 	}
 	selected_files.swap(selected_files_new);
 
@@ -605,19 +591,13 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		return;
 
 	// Find all selected files and directories
-	std::list<CServerPath> selected_dirs;
+	std::deque<CServerPath> selected_dirs;
 	std::list<int> selected_files;
 	ProcessSelection(selected_files, selected_dirs);
 
 	if (selected_files.empty() && selected_dirs.empty())
 		return;
 
-	if (selected_dirs.size() > 1)
-	{
-		wxMessageBoxEx(_("Downloading multiple unrelated directories is not yet supported"), _("Downloading search results"), wxICON_EXCLAMATION);
-		return;
-	}
-
 	CSearchDownloadDialog dlg;
 	if (!dlg.Run(this, m_local_target.GetPath(), selected_files.size(), selected_dirs.size()))
 		return;
@@ -625,16 +605,14 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 	wxTextCtrl *pText = XRCCTRL(dlg, "ID_LOCALPATH", wxTextCtrl);
 
 	CLocalPath path(pText->GetValue());
-	if (path.empty() || !path.IsWriteable())
-	{
+	if (path.empty() || !path.IsWriteable()) {
 		wxBell();
 		return;
 	}
 	m_local_target = path;
 
-	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	CServer const* pServer = m_pState->GetServer();
+	if (!pServer) {
 		wxBell();
 		return;
 	}
@@ -642,26 +620,24 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 	bool start = XRCCTRL(dlg, "ID_QUEUE_START", wxRadioButton)->GetValue();
 	bool flatten = XRCCTRL(dlg, "ID_PATHS_FLATTEN", wxRadioButton)->GetValue();
 
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter)
-	{
-		const CDirentry& entry = m_results->m_fileData[*iter];
+	for (auto const& sel : selected_files) {
+		const CDirentry& entry = m_results->m_fileData[sel];
 
 		CLocalPath target_path = path;
-		if (!flatten)
-		{
+		if (!flatten) {
 			// Append relative path to search root to local target path
-			CServerPath remote_path = m_results->m_fileData[*iter].path;
+			CServerPath remote_path = m_results->m_fileData[sel].path;
 			std::list<wxString> segments;
-			while (m_search_root.IsParentOf(remote_path, false) && remote_path.HasParent())
-			{
+			while (m_search_root.IsParentOf(remote_path, false) && remote_path.HasParent()) {
 				segments.push_front(remote_path.GetLastSegment());
 				remote_path = remote_path.GetParent();
 			}
-			for (std::list<wxString>::const_iterator segment_iter = segments.begin(); segment_iter != segments.end(); ++segment_iter)
-				target_path.AddSegment(*segment_iter);
+			for (auto const& segment : segments) {
+				target_path.AddSegment(segment);
+			}
 		}
 
-		CServerPath remote_path = m_results->m_fileData[*iter].path;
+		CServerPath remote_path = m_results->m_fileData[sel].path;
 		wxString localName = CQueueView::ReplaceInvalidCharacters(entry.name);
 		if (!entry.is_dir() && remote_path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 			localName = StripVMSRevision(localName);
@@ -678,15 +654,69 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 	else
 		mode = start ? CRecursiveOperation::recursive_download : CRecursiveOperation::recursive_addtoqueue;
 
-	for (std::list<CServerPath>::const_iterator iter = selected_dirs.begin(); iter != selected_dirs.end(); ++iter)
-	{
+	for (auto const& dir : selected_dirs) {
 		CLocalPath target_path = path;
-		if (!flatten && iter->HasParent())
-			target_path.AddSegment(iter->GetLastSegment());
+		if (!flatten && dir.HasParent())
+			target_path.AddSegment(dir.GetLastSegment());
 
-		m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(*iter, _T(""), target_path, false);
-		std::list<CFilter> filters; // Empty, recurse into everything
-		m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(mode, *iter, filters, true, m_original_dir);
+		recursion_root root(dir, true);
+		root.add_dir_to_visit(dir, _T(""), target_path, false);
+		m_pState->GetRecursiveOperationHandler()->AddRecursionRoot(std::move(root));
+	}
+	std::vector<CFilter> const filters; // Empty, recurse into everything
+	m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(mode, filters, m_original_dir);
+}
+
+void CSearchDialog::OnEdit(wxCommandEvent&)
+{
+	if (!m_pState->IsRemoteIdle())
+		return;
+
+	// Find all selected files and directories
+	std::deque<CServerPath> selected_dirs;
+	std::list<int> selected_files;
+	ProcessSelection(selected_files, selected_dirs);
+
+	if (selected_files.empty() && selected_dirs.empty())
+		return;
+
+	if (!selected_dirs.empty()) {
+		wxMessageBoxEx(_("Editing directories is not supported"), _("Editing search results"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	CEditHandler* pEditHandler = CEditHandler::Get();
+	if (!pEditHandler) {
+		wxBell();
+		return;
+	}
+
+	const wxString& localDir = pEditHandler->GetLocalDirectory();
+	if (localDir.empty()) {
+		wxMessageBoxEx(_("Could not get temporary directory to download file into."), _("Cannot edit file"), wxICON_STOP);
+		return;
+	}
+
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer) {
+		wxBell();
+		return;
+	}
+
+	if (selected_files.size() > 10) {
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return;
+	}
+
+	for (auto const& item : selected_files) {
+		const CDirentry& entry = m_results->m_fileData[item];
+		const CServerPath path = m_results->m_fileData[item].path;
+
+		pEditHandler->Edit(CEditHandler::remote, entry.name, path, *pServer, entry.size, this);
 	}
 }
 
@@ -696,26 +726,19 @@ void CSearchDialog::OnDelete(wxCommandEvent&)
 		return;
 
 	// Find all selected files and directories
-	std::list<CServerPath> selected_dirs;
+	std::deque<CServerPath> selected_dirs;
 	std::list<int> selected_files;
 	ProcessSelection(selected_files, selected_dirs);
 
 	if (selected_files.empty() && selected_dirs.empty())
 		return;
 
-	if (selected_dirs.size() > 1)
-	{
-		wxMessageBoxEx(_("Deleting multiple unrelated directories is not yet supported"), _("Deleting directories"), wxICON_EXCLAMATION);
-		return;
-	}
-
 	wxString question;
 	if (selected_dirs.empty())
 		question.Printf(wxPLURAL("Really delete %d file from the server?", "Really delete %d files from the server?", selected_files.size()), selected_files.size());
 	else if (selected_files.empty())
 		question.Printf(wxPLURAL("Really delete %d directory with its contents from the server?", "Really delete %d directories with their contents from the server?", selected_dirs.size()), selected_dirs.size());
-	else
-	{
+	else {
 		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", selected_files.size()), selected_files.size());
 		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", selected_dirs.size()), selected_dirs.size());
 		question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
@@ -724,34 +747,30 @@ void CSearchDialog::OnDelete(wxCommandEvent&)
 	if (wxMessageBoxEx(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
 		return;
 
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter)
-	{
-		const CDirentry& entry = m_results->m_fileData[*iter];
-		std::list<wxString> files_to_delete;
+	for (auto const& file : selected_files) {
+		CDirentry const& entry = m_results->m_fileData[file];
+		std::deque<wxString> files_to_delete;
 		files_to_delete.push_back(entry.name);
-		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_results->m_fileData[*iter].path, files_to_delete));
+		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_results->m_fileData[file].path, std::move(files_to_delete)));
 	}
 
-	for (std::list<CServerPath>::const_iterator iter = selected_dirs.begin(); iter != selected_dirs.end(); ++iter)
-	{
-		CServerPath path = *iter;
-		if (!path.HasParent())
-			m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(path, _T(""));
-		else
-		{
-			m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(path.GetParent(), path.GetLastSegment());
+	for (auto path : selected_dirs) {
+		wxString segment;
+		if (path.HasParent()) {
+			segment = path.GetLastSegment();
 			path = path.GetParent();
 		}
-
-		std::list<CFilter> filters; // Empty, recurse into everything
-		m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(CRecursiveOperation::recursive_delete, path, filters, !path.HasParent(), m_original_dir);
+		recursion_root root(path, !path.HasParent());
+		root.add_dir_to_visit(path, segment);
+		m_pState->GetRecursiveOperationHandler()->AddRecursionRoot(std::move(root));
 	}
+	std::vector<CFilter> const filters; // Empty, recurse into everything
+	m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filters, m_original_dir);
 }
 
 void CSearchDialog::OnCharHook(wxKeyEvent& event)
 {
-	if (IsEscapeKey(event))
-	{
+	if (IsEscapeKey(event)) {
 		EndDialog(wxID_CANCEL);
 		return;
 	}
@@ -778,37 +797,37 @@ void CSearchDialog::LoadConditions()
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
-	TiXmlElement* pFilter = pDocument->FirstChildElement("Filter");
-	if (!pFilter)
+	auto filter = document.child("Filter");
+	if (!filter)
 		return;
 
-	if (!CFilterManager::LoadFilter(pFilter, m_search_filter))
+	if (!CFilterManager::LoadFilter(filter, m_search_filter))
 		m_search_filter = CFilter();
 }
 
 void CSearchDialog::SaveConditions()
 {
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
-	
+
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
-	TiXmlElement* pFilter;
-	while ((pFilter = pDocument->FirstChildElement("Filter")))
-		pDocument->RemoveChild(pFilter);
-	pFilter = pDocument->LinkEndChild(new TiXmlElement("Filter"))->ToElement();
+	pugi::xml_node filter;
+	while ((filter = document.child("Filter")))
+		document.remove_child(filter);
+	filter = document.append_child("Filter");
 
-	CFilterDialog::SaveFilter(pFilter, m_search_filter);
+	CFilterDialog::SaveFilter(filter, m_search_filter);
 
 	file.Save(true);
 }
