@@ -20,7 +20,7 @@ public:
 	void Run();
 
 protected:
-	void ProcessDirectoryListing();
+	void ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> const& listing);
 
 	void SetCtrlState();
 
@@ -28,27 +28,28 @@ protected:
 	void LoadConditions();
 
 	wxWindow* m_parent;
-	CSearchDialogFileList *m_results;
+	CSearchDialogFileList *m_results{};
 	CQueueView* m_pQueue;
 
 	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 
-	CWindowStateManager* m_pWindowStateManager;
+	CWindowStateManager* m_pWindowStateManager{};
 
 	CFilter m_search_filter;
 
-	bool m_searching;
+	bool m_searching{};
 
 	CServerPath m_original_dir;
 	CLocalPath m_local_target;
 
-	void ProcessSelection(std::list<int> &selected_files, std::list<CServerPath> &selected_dirs);
+	void ProcessSelection(std::list<int> &selected_files, std::deque<CServerPath> &selected_dirs);
 
 	DECLARE_EVENT_TABLE()
 	void OnSearch(wxCommandEvent& event);
 	void OnStop(wxCommandEvent& event);
 	void OnContextMenu(wxContextMenuEvent& event);
 	void OnDownload(wxCommandEvent&);
+	void OnEdit(wxCommandEvent&);
 	void OnDelete(wxCommandEvent&);
 	void OnCharHook(wxKeyEvent& event);
 
