@@ -71,17 +71,17 @@ bool COptionsPageDateFormatting::Validate()
 		if (dateformat.empty()) {
 			return DisplayError(_T("ID_CUSTOM_DATEFORMAT"), _("Please enter a custom date format."));
 		}
-		if (!CDateTime::VerifyFormat(dateformat)) {
+		if (!fz::datetime::verify_format(dateformat.ToStdWstring())) {
 			return DisplayError(_T("ID_CUSTOM_DATEFORMAT"), _("The custom date format is invalid or contains unsupported format specifiers."));
 		}
 	}
-		
+
 	if (GetRCheck(XRCID("ID_TIMEFORMAT_CUSTOM"))) {
 		wxString const timeformat = XRCCTRL(*this, "ID_CUSTOM_TIMEFORMAT", wxTextCtrl)->GetValue();
 		if (timeformat.empty()) {
 			return DisplayError(_T("ID_CUSTOM_TIMEFORMAT"), _("Please enter a custom time format."));
 		}
-		if (!CDateTime::VerifyFormat(timeformat)) {
+		if (!fz::datetime::verify_format(timeformat.ToStdWstring())) {
 			return DisplayError(_T("ID_CUSTOM_TIMEFORMAT"), _("The custom time format is invalid or contains unsupported format specifiers."));
 		}
 	}
@@ -89,7 +89,7 @@ bool COptionsPageDateFormatting::Validate()
 	return true;
 }
 
-void COptionsPageDateFormatting::OnRadioChanged(wxCommandEvent& event)
+void COptionsPageDateFormatting::OnRadioChanged(wxCommandEvent&)
 {
 	SetCtrlState();
 }
