@@ -1,6 +1,7 @@
 #include <filezilla.h>
 
 #include "../Options.h"
+#include "file_utils.h"
 #include "settingsdialog.h"
 #include "optionspage.h"
 #include "optionspage_edit.h"
@@ -62,74 +63,11 @@ bool COptionsPageEdit::SavePage()
 	return true;
 }
 
-bool UnquoteCommand(wxString& command, wxString& arguments, bool is_dde = false)
-{
-	arguments = _T("");
-
-	if (command == _T(""))
-		return true;
-
-	wxChar inQuotes = 0;
-	wxString file;
-	for (unsigned int i = 0; i < command.Len(); i++)
-	{
-		const wxChar& c = command[i];
-		if (c == '"' || c == '\'')
-		{
-			if (!inQuotes)
-				inQuotes = c;
-			else if (c != inQuotes)
-				file += c;
-			else if (command[i + 1] == c)
-			{
-				file += c;
-				i++;
-			}
-			else
-				inQuotes = false;
-		}
-		else if (command[i] == ' ' && !inQuotes)
-		{
-			arguments = command.Mid(i + 1);
-			arguments.Trim(false);
-			break;
-		}
-		else if (is_dde && !inQuotes && (command[i] == ',' || command[i] == '#'))
-		{
-			arguments = command.Mid(i + 1);
-			arguments.Trim(false);
-			break;
-		}
-		else
-			file += command[i];
-	}
-	if (inQuotes)
-		return false;
-
-	command = file;
-
-	return true;
-}
-
-bool ProgramExists(const wxString& editor)
-{
-	if (wxFileName::FileExists(editor))
-		return true;
-
-#ifdef __WXMAC__
-	if (editor.Right(4) == _T(".app") && wxFileName::DirExists(editor))
-		return true;
-#endif
-
-	return false;
-}
-
 bool COptionsPageEdit::Validate()
 {
 	const bool custom = GetRCheck(XRCID("ID_DEFAULT_CUSTOM"));
 	wxString editor;
-	if (custom)
-	{
+	if (custom) {
 		bool failure = false;
 
 		editor = GetText(XRCID("ID_EDITOR"));
@@ -137,13 +75,12 @@ bool COptionsPageEdit::Validate()
 		editor.Trim(false);
 		SetText(XRCID("EDITOR"), editor, failure);
 
-		if (editor != _T(""))
-		{
+		if (!editor.empty()) {
 			wxString args;
 			if (!UnquoteCommand(editor, args))
 				return DisplayError(_T("ID_EDITOR"), _("Default editor not properly quoted."));
 
-			if (editor == _T(""))
+			if (editor.empty())
 				return DisplayError(_T("ID_EDITOR"), _("Empty quoted string."));
 
 			if (!ProgramExists(editor))
@@ -151,8 +88,7 @@ bool COptionsPageEdit::Validate()
 		}
 	}
 
-	if (GetRCheck(XRCID("ID_USEDEFAULT")))
-	{
+	if (GetRCheck(XRCID("ID_USEDEFAULT"))) {
 		if (GetRCheck(XRCID("ID_DEFAULT_NONE")) ||
 			(custom && editor.empty()))
 		{
@@ -163,9 +99,9 @@ bool COptionsPageEdit::Validate()
 	return true;
 }
 
-void COptionsPageEdit::OnBrowseEditor(wxCommandEvent& event)
+void COptionsPageEdit::OnBrowseEditor(wxCommandEvent&)
 {
-	wxFileDialog dlg(this, _("Select default editor"), _T(""), _T(""),
+	wxFileDialog dlg(this, _("Select default editor"), wxString(), wxString(),
 #ifdef __WXMSW__
 		_T("Executable file (*.exe)|*.exe"),
 #elif __WXMAC__
@@ -179,13 +115,13 @@ void COptionsPageEdit::OnBrowseEditor(wxCommandEvent& event)
 		return;
 
 	wxString editor = dlg.GetPath();
-	if (editor == _T(""))
+	if (editor.empty())
 		return;
 
 	if (!ProgramExists(editor))
 	{
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
-		wxMessageBox(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
@@ -206,7 +142,7 @@ void COptionsPageEdit::SetCtrlState()
 	XRCCTRL(*this, "ID_USEDEFAULT", wxRadioButton)->Enable(!GetRCheck(XRCID("ID_DEFAULT_NONE")) || GetRCheck(XRCID("ID_USEDEFAULT")));
 }
 
-void COptionsPageEdit::OnRadioButton(wxCommandEvent& event)
+void COptionsPageEdit::OnRadioButton(wxCommandEvent&)
 {
 	SetCtrlState();
 }
