@@ -6,6 +6,7 @@
 #include "optionspage_themes.h"
 #include "../themeprovider.h"
 
+#include <wx/dcclient.h>
 #include <wx/scrolwin.h>
 
 BEGIN_EVENT_TABLE(COptionsPageThemes, COptionsPage)
@@ -24,18 +25,9 @@ public:
 		m_extra_padding = 0;
 	}
 
-	virtual ~CIconPreview()
-	{
-		for (std::list<wxBitmap*>::iterator iter = m_icons.begin(); iter != m_icons.end(); ++iter)
-			delete *iter;
-	}
-
 	void LoadIcons(const wxString& theme, const wxSize& iconSize)
 	{
 		m_iconSize = iconSize;
-		for (std::list<wxBitmap*>::iterator iter = m_icons.begin(); iter != m_icons.end(); ++iter)
-			delete *iter;
-
 		m_icons = CThemeProvider::GetAllImages(theme, m_iconSize);
 	}
 
@@ -47,19 +39,16 @@ public:
 
 		wxSize size = GetClientSize();
 
-		if (!m_icons.empty())
-		{
+		if (!m_icons.empty()) {
 			int icons_per_line = wxMax(1, (size.GetWidth() - BORDER) / (m_iconSize.GetWidth() + BORDER));
 
 			// Number of lines and line height
 			int lines = (m_icons.size() - 1) / icons_per_line + 1;
 			int vheight = lines * (m_iconSize.GetHeight() + BORDER) + BORDER;
-			if (vheight > size.GetHeight())
-			{
+			if (vheight > size.GetHeight()) {
 				// Scroll bar would appear, need to adjust width
 				size.SetHeight(vheight);
 				SetVirtualSize(size);
-				SetVirtualSizeHints(size, size);
 				SetScrollRate(0, m_iconSize.GetHeight() + BORDER);
 
 				wxSize size2 = GetClientSize();
@@ -73,20 +62,18 @@ public:
 			}
 
 			// Calculate extra padding
-			if (icons_per_line > 1)
-			{
+			if (icons_per_line > 1) {
 				int extra = size.GetWidth() - BORDER - icons_per_line * (m_iconSize.GetWidth() + BORDER);
 				m_extra_padding = extra / (icons_per_line - 1);
 			}
 		}
 		SetVirtualSize(size);
-		SetVirtualSizeHints(size, size);
 		SetScrollRate(0, m_iconSize.GetHeight() + BORDER);
 	}
 
 protected:
 	DECLARE_EVENT_TABLE()
-	virtual void OnPaint(wxPaintEvent& event)
+	virtual void OnPaint(wxPaintEvent&)
 	{
 		CalcSize();
 
@@ -95,8 +82,7 @@ protected:
 
 		wxSize size = GetClientSize();
 
-		if (m_icons.empty())
-		{
+		if (m_icons.empty()) {
 			dc.SetFont(GetFont());
 			wxString text = _("No images available");
 			wxCoord w, h;
@@ -108,20 +94,17 @@ protected:
 		int x = BORDER;
 		int y = BORDER;
 
-		for (std::list<wxBitmap*>::iterator iter = m_icons.begin(); iter != m_icons.end(); ++iter)
-		{
-			wxBitmap* bitmap = *iter;
-			dc.DrawBitmap(*bitmap, x, y, true);
+		for( auto const& bmp : m_icons ) {
+			dc.DrawBitmap(*bmp, x, y, true);
 			x += m_iconSize.GetWidth() + BORDER + m_extra_padding;
-			if ((x + m_iconSize.GetWidth() + BORDER) > size.GetWidth())
-			{
+			if ((x + m_iconSize.GetWidth() + BORDER) > size.GetWidth()) {
 				x = BORDER;
 				y += m_iconSize.GetHeight() + BORDER;
 			}
 		}
 	}
 
-	std::list<wxBitmap*> m_icons;
+	std::vector<std::unique_ptr<wxBitmap>> m_icons;
 	wxSize m_iconSize;
 	bool m_sizeInitialized;
 	int m_extra_padding;
@@ -167,15 +150,19 @@ bool COptionsPageThemes::Validate()
 bool COptionsPageThemes::DisplayTheme(const wxString& theme)
 {
 	wxString name, author, mail;
-	if (!CThemeProvider::GetThemeData(theme, name, author, mail))
+	if (!CThemeProvider::GetThemeData(theme, name, author, mail)) {
 		return false;
-	if (name == _T(""))
+	}
+	if (name.empty()) {
 		return false;
+	}
 
-	if (author == _T(""))
+	if (author.empty()) {
 		author = _("N/a");
-	if (mail == _T(""))
+	}
+	if (mail.empty()) {
 		mail = _("N/a");
+	}
 
 	bool failure = false;
 	SetStaticText(XRCID("ID_AUTHOR"), author, failure);
@@ -192,11 +179,7 @@ bool COptionsPageThemes::DisplayTheme(const wxString& theme)
 
 	pPreview->DeleteAllPages();
 
-	std::list<wxString> sizes = CThemeProvider::GetThemeSizes(theme);
-	for (std::list<wxString>::const_iterator iter = sizes.begin(); iter != sizes.end(); ++iter)
-	{
-		const wxString& size = *iter;
-
+	for( auto const& size : CThemeProvider::GetThemeSizes(theme) ) {
 		int pos = size.Find('x');
 		if (pos <= 0 || pos >= (int)size.Len() - 1)
 			continue;
@@ -233,7 +216,7 @@ bool COptionsPageThemes::DisplayTheme(const wxString& theme)
 	return !failure;
 }
 
-void COptionsPageThemes::OnThemeChange(wxCommandEvent& event)
+void COptionsPageThemes::OnThemeChange(wxCommandEvent&)
 {
 	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
 
@@ -256,29 +239,27 @@ bool COptionsPageThemes::OnDisplayedFirstTime()
 		!XRCCTRL(*this, "ID_EMAIL", wxStaticText))
 		return false;
 
-	std::list<wxString> themes = CThemeProvider::GetThemes();
+	auto const themes = CThemeProvider::GetThemes();
 	if (themes.empty())
 		return false;
 
-	wxString theme = m_pOptions->GetOption(OPTION_THEME);
+	wxString activeTheme = m_pOptions->GetOption(OPTION_THEME);
 	wxString firstName;
-	for (std::list<wxString>::const_iterator iter = themes.begin(); iter != themes.end(); ++iter)
-	{
+	for (auto const& theme : themes) {
 		wxString name, author, mail;
-		if (!CThemeProvider::GetThemeData(*iter, name, author, mail))
+		if (!CThemeProvider::GetThemeData(theme, name, author, mail))
 			continue;
-		if (firstName == _T(""))
+		if (firstName.empty())
 			firstName = name;
-		int n = pTheme->Append(name, new wxStringClientData(*iter));
-		if (*iter == theme)
+		int n = pTheme->Append(name, new wxStringClientData(theme));
+		if (theme == activeTheme)
 			pTheme->SetSelection(n);
 	}
 	if (pTheme->GetSelection() == wxNOT_FOUND)
 		pTheme->SetSelection(pTheme->FindString(firstName));
+	activeTheme = ((wxStringClientData*)pTheme->GetClientObject(pTheme->GetSelection()))->GetData();
 
-	theme = ((wxStringClientData*)pTheme->GetClientObject(pTheme->GetSelection()))->GetData();
-
-	if (!DisplayTheme(theme))
+	if (!DisplayTheme(activeTheme))
 		failure = true;
 
 	pTheme->GetContainingSizer()->Layout();
