@@ -9,9 +9,10 @@ public:
 	virtual wxString GetResourceName() { return _T("ID_SETTINGS_UPDATECHECK"); }
 	virtual bool LoadPage();
 	virtual bool SavePage();
+	virtual bool Validate();
 
 protected:
-	void OnRunUpdateCheck(wxCommandEvent& event);
+	void OnRunUpdateCheck(wxCommandEvent&);
 
 	DECLARE_EVENT_TABLE()
 };
