@@ -6,68 +6,71 @@
 #include "Options.h"
 #include "xmlfunctions.h"
 
-std::map<int, CSiteManagerItemData_Site*> CSiteManager::m_idMap;
+std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> CSiteManager::m_idMap;
 
-bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler)
+bool CSiteManager::Load(CSiteManagerXmlHandler& handler)
 {
-	wxASSERT(pElement);
-	wxASSERT(pHandler);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	auto document = file.Load();
+	if (!document) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return false;
+	}
 
-	for (TiXmlElement* pChild = pElement->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
-	{
-		if (!strcmp(pChild->Value(), "Folder"))
-		{
-			wxString name = GetTextElement_Trimmed(pChild);
+	auto element = document.child("Servers");
+	if (!element) {
+		return false;
+	}
+
+	return Load(element, handler);
+}
+
+bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
+{
+	wxASSERT(element);
+
+	for (auto child = element.first_child(); child; child = child.next_sibling()) {
+		if (!strcmp(child.name(), "Folder")) {
+			wxString name = GetTextElement_Trimmed(child);
 			if (name.empty())
 				continue;
 
-			const bool expand = GetTextAttribute(pChild, "expanded") != _T("0");
-			if (!pHandler->AddFolder(name, expand))
+			const bool expand = GetTextAttribute(child, "expanded") != _T("0");
+			if (!handler.AddFolder(name, expand))
 				return false;
-			Load(pChild, pHandler);
-			if (!pHandler->LevelUp())
+			Load(child, handler);
+			if (!handler.LevelUp())
 				return false;
 		}
-		else if (!strcmp(pChild->Value(), "Server"))
-		{
-			CSiteManagerItemData_Site* data = ReadServerElement(pChild);
+		else if (!strcmp(child.name(), "Server")) {
+			std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(child);
 
-			if (data)
-			{
-				pHandler->AddSite(data);
+			if (data) {
+				handler.AddSite(std::move(data));
 
 				// Bookmarks
-				for (TiXmlElement* pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-				{
-					TiXmlHandle handle(pBookmark);
-
-					wxString name = GetTextElement_Trimmed(pBookmark, "Name");
+				for (auto bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
+					wxString name = GetTextElement_Trimmed(bookmark, "Name");
 					if (name.empty())
 						continue;
 
-					CSiteManagerItemData* data = new CSiteManagerItemData(CSiteManagerItemData::BOOKMARK);
+					auto bookmarkData = std::make_unique<CSiteManagerItemData>(CSiteManagerItemData::BOOKMARK);
 
-					TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
-					if (localDir)
-						data->m_localDir = GetTextElement(pBookmark, "LocalDir");
-
-					TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
-					if (remoteDir)
-						data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
-
-					if (data->m_localDir.empty() && data->m_remoteDir.empty())
-					{
-						delete data;
+					bookmarkData->m_localDir = GetTextElement(bookmark, "LocalDir");
+					bookmarkData->m_remoteDir.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
+					if (bookmarkData->m_localDir.empty() && bookmarkData->m_remoteDir.empty()) {
 						continue;
 					}
 
-					if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
-						data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+					if (!bookmarkData->m_localDir.empty() && !bookmarkData->m_remoteDir.empty())
+						bookmarkData->m_sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
 
-					pHandler->AddBookmark(name, data);
+					bookmarkData->m_comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
+
+					handler.AddBookmark(name, std::move(bookmarkData));
 				}
 
-				if (!pHandler->LevelUp())
+				if (!handler.LevelUp())
 					return false;
 			}
 		}
@@ -76,32 +79,23 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 	return true;
 }
 
-CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElement)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
 	CServer server;
-	if (!::GetServer(pElement, server))
+	if (!::GetServer(element, server))
 		return 0;
 	if (server.GetName().empty())
 		return 0;
 
-	CSiteManagerItemData_Site* data = new CSiteManagerItemData_Site(server);
-
-	TiXmlHandle handle(pElement);
-
-	TiXmlText* comments = handle.FirstChildElement("Comments").FirstChild().Text();
-	if (comments)
-		data->m_comments = ConvLocal(comments->Value());
-
-	TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
-	if (localDir)
-		data->m_localDir = ConvLocal(localDir->Value());
-
-	TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
-	if (remoteDir)
-		data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
+	auto data = std::make_unique<CSiteManagerItemData_Site>(server);
+	data->m_comments = GetTextElement(element, "Comments");
+	data->m_localDir = GetTextElement(element, "LocalDir");
+	data->m_remoteDir.SetSafePath(GetTextElement(element, "RemoteDir"));
 
 	if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
-		data->m_sync = GetTextElementBool(pElement, "SyncBrowsing", false);
+		data->m_sync = GetTextElementBool(element, "SyncBrowsing", false);
+
+	data->m_comparison = GetTextElementBool(element, "DirectoryComparison", false);
 
 	return data;
 }
@@ -109,7 +103,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 class CSiteManagerXmlHandler_Menu : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, CSiteManagerItemData_Site*> *idMap, bool predefined)
+	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *idMap, bool predefined)
 		: m_pMenu(pMenu), m_idMap(idMap)
 	{
 		m_added_site = false;
@@ -154,7 +148,7 @@ public:
 		return true;
 	}
 
-	virtual bool AddSite(CSiteManagerItemData_Site* data)
+	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site> data)
 	{
 		wxString newName(data->m_server.GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
@@ -163,16 +157,15 @@ public:
 
 		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
 
-		(*m_idMap)[pItem->GetId()] = data;
+		(*m_idMap)[pItem->GetId()] = std::move(data);
 
 		m_added_site = true;
 
 		return true;
 	}
 
-	virtual bool AddBookmark(const wxString& name, CSiteManagerItemData* data)
+	virtual bool AddBookmark(const wxString&, std::unique_ptr<CSiteManagerItemData>)
 	{
-		delete data;
 		return true;
 	}
 
@@ -213,7 +206,7 @@ public:
 protected:
 	wxMenu* m_pMenu;
 
-	std::map<int, CSiteManagerItemData_Site*> *m_idMap;
+	std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *m_idMap;
 
 	std::list<wxMenu*> m_parents;
 	std::list<wxString> m_childNames;
@@ -224,7 +217,34 @@ protected:
 	std::list<wxString> m_paths;
 };
 
-wxMenu* CSiteManager::GetSitesMenu()
+
+class CSiteManagerXmlHandler_Stats : public CSiteManagerXmlHandler
+{
+public:
+	virtual bool AddFolder(const wxString&, bool)
+	{
+		++directories_;
+		return true;
+	}
+
+	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site>)
+	{
+		++sites_;
+		return true;
+	}
+
+	virtual bool AddBookmark(const wxString&, std::unique_ptr<CSiteManagerItemData>)
+	{
+		++bookmarks_;
+		return true;
+	}
+
+	unsigned int sites_{};
+	unsigned int directories_{};
+	unsigned int bookmarks_{};
+};
+
+std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 {
 	ClearIdMap();
 
@@ -232,53 +252,23 @@ wxMenu* CSiteManager::GetSitesMenu()
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	wxMenu* predefinedSites = GetSitesMenu_Predefined(m_idMap);
+	auto predefinedSites = GetSitesMenu_Predefined(m_idMap);
 
-	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument)
-	{
-		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	auto pMenu = std::make_unique<wxMenu>();
+	CSiteManagerXmlHandler_Menu handler(pMenu.get(), &m_idMap, false);
 
-		if (!predefinedSites)
-			return predefinedSites;
-
-		wxMenu *pMenu = new wxMenu;
-		wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
-		pItem->Enable(false);
-		return pMenu;
+	bool res = Load(handler);
+	if (!res || !pMenu->GetMenuItemCount()) {
+		pMenu.reset();
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
-	{
-		if (predefinedSites)
-			return predefinedSites;
-
-		wxMenu *pMenu = new wxMenu;
-		wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
-		pItem->Enable(false);
-		return pMenu;
-	}
-
-	wxMenu* pMenu = new wxMenu;
-	CSiteManagerXmlHandler_Menu handler(pMenu, &m_idMap, false);
-
-	bool res = Load(pElement, &handler);
-	if (!res || !pMenu->GetMenuItemCount())
-	{
-		delete pMenu;
-		pMenu = 0;
-	}
-
-	if (pMenu)
-	{
+	if (pMenu) {
 		if (!predefinedSites)
 			return pMenu;
 
-		wxMenu* pRootMenu = new wxMenu;
-		pRootMenu->AppendSubMenu(predefinedSites, _("Predefined Sites"));
-		pRootMenu->AppendSubMenu(pMenu, _("My Sites"));
+		auto pRootMenu = std::make_unique<wxMenu>();
+		pRootMenu->AppendSubMenu(predefinedSites.release(), _("Predefined Sites"));
+		pRootMenu->AppendSubMenu(pMenu.release(), _("My Sites"));
 
 		return pRootMenu;
 	}
@@ -286,7 +276,7 @@ wxMenu* CSiteManager::GetSitesMenu()
 	if (predefinedSites)
 		return predefinedSites;
 
-	pMenu = new wxMenu;
+	pMenu = std::make_unique<wxMenu>();
 	wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
 	pItem->Enable(false);
 	return pMenu;
@@ -294,60 +284,57 @@ wxMenu* CSiteManager::GetSitesMenu()
 
 void CSiteManager::ClearIdMap()
 {
-	for (auto iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
-		delete iter->second;
-
 	m_idMap.clear();
 }
 
-wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, CSiteManagerItemData_Site*> &idMap)
+bool CSiteManager::LoadPredefined(CSiteManagerXmlHandler& handler)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
 	if (defaultsDir.empty())
-		return 0;
+		return false;
 
 	wxString const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	CXmlFile file(name);
 
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument)
-		return 0;
+	auto document = file.Load();
+	if (!document)
+		return false;
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
-		return 0;
+	auto element = document.child("Servers");
+	if (!element)
+		return false;
 
-	wxMenu* pMenu = new wxMenu;
-	CSiteManagerXmlHandler_Menu handler(pMenu, &idMap, true);
+	if (!Load(element, handler)) {
+		return false;
+	}
 
-	if (!Load(pElement, &handler))
-	{
-		delete pMenu;
+	return true;
+}
+
+std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu_Predefined(std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> &idMap)
+{
+	auto pMenu = std::make_unique<wxMenu>();
+	CSiteManagerXmlHandler_Menu handler(pMenu.get(), &idMap, true);
+
+	if (!LoadPredefined(handler)) {
 		return 0;
 	}
 
-	if (!pMenu->GetMenuItemCount())
-	{
-		delete pMenu;
+	if (!pMenu->GetMenuItemCount()) {
 		return 0;
 	}
 
 	return pMenu;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteById(int id)
 {
 	auto iter = m_idMap.find(id);
 
-	CSiteManagerItemData_Site *pData;
-	if (iter != m_idMap.end())
-	{
-		pData = iter->second;
-		iter->second = 0;
+	std::unique_ptr<CSiteManagerItemData_Site> pData;
+	if (iter != m_idMap.end()) {
+		pData = std::move(iter->second);
 	}
-	else
-		pData = 0;
-
 	ClearIdMap();
 
 	return pData;
@@ -417,11 +404,10 @@ wxString CSiteManager::BuildPath(wxChar root, std::list<wxString> const& segment
 	return ret;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString sitePath)
 {
 	wxChar c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0' && c != '1')
-	{
+	if (c != '0' && c != '1') {
 		wxMessageBoxEx(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
 		return 0;
 	}
@@ -445,71 +431,53 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
 
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return 0;
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement) {
+	auto element = document.child("Servers");
+	if (!element) {
 		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments) || segments.empty())
-	{
+	if (!UnescapeSitePath(sitePath, segments) || segments.empty()) {
 		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
-	TiXmlElement* pChild = GetElementByPath(pElement, segments);
-	if (!pChild)
-	{
+	auto child = GetElementByPath(element, segments);
+	if (!child) {
 		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
-	TiXmlElement* pBookmark;
-	if (!strcmp(pChild->Value(), "Bookmark"))
-	{
-		pBookmark = pChild;
-		pChild = pChild->Parent()->ToElement();
+	pugi::xml_node bookmark;
+	if (!strcmp(child.name(), "Bookmark")) {
+		bookmark = child;
+		child = child.parent();
 		segments.pop_back();
 	}
-	else
-		pBookmark = 0;
 
-	CSiteManagerItemData_Site* data = ReadServerElement(pChild);
+	std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(child);
 
-	if (!data)
-	{
+	if (!data) {
 		wxMessageBoxEx(_("Could not read server item."), _("Invalid site path"));
 		return 0;
 	}
 
-	if (pBookmark)
-	{
-		TiXmlHandle handle(pBookmark);
-
-		wxString localPath;
-		CServerPath remotePath;
-		TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
-		if (localDir)
-			localPath = ConvLocal(localDir->Value());
-		TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
-		if (remoteDir)
-			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
-		if (!localPath.empty() && !remotePath.empty())
-		{
-			data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+	if (bookmark) {
+		data->m_localDir = GetTextElement(bookmark, "LocalDir");
+		data->m_remoteDir.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
+		if (!data->m_localDir.empty() && !data->m_remoteDir.empty()) {
+			data->m_sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
 		}
 		else
 			data->m_sync = false;
-
-		data->m_localDir = localPath;
-		data->m_remoteDir = remotePath;
+		data->m_comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
 	}
 
 	data->m_path = BuildPath( c, segments );
@@ -543,15 +511,15 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		}
 		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
-	
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+
+	auto document = file.Load();
+	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return false;
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return false;
 
 	std::list<wxString> segments;
@@ -560,32 +528,23 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		return 0;
 	}
 
-	TiXmlElement* pChild = GetElementByPath(pElement, segments);
+	auto child = GetElementByPath(element, segments);
 
-	if (pChild && !strcmp(pChild->Value(), "Bookmark"))
-		pChild = pChild->Parent()->ToElement();
+	if (child && !strcmp(child.name(), "Bookmark"))
+		child = child.parent();
 
-	if (!pChild || strcmp(pChild->Value(), "Server"))
+	if (!child || strcmp(child.name(), "Server"))
 		return 0;
 
 	// Bookmarks
-	for (TiXmlElement* pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
-		TiXmlHandle handle(pBookmark);
-
-		wxString name = GetTextElement_Trimmed(pBookmark, "Name");
+	for (auto bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
+		wxString name = GetTextElement_Trimmed(bookmark, "Name");
 		if (name.empty())
 			continue;
 
-		wxString localPath;
+		wxString localPath = GetTextElement(bookmark, "LocalDir");
 		CServerPath remotePath;
-		TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
-		if (localDir)
-			localPath = ConvLocal(localDir->Value());
-		TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
-		if (remoteDir)
-			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
-
+		remotePath.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
 		if (localPath.empty() && remotePath.empty())
 			continue;
 
@@ -602,22 +561,21 @@ wxString CSiteManager::AddServer(CServer server)
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return wxString();
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return wxString();
 
 	std::list<wxString> names;
-	for (TiXmlElement* pChild = pElement->FirstChildElement("Server"); pChild; pChild = pChild->NextSiblingElement("Server"))
-	{
-		wxString name = GetTextElement(pChild, "Name");
+	for (auto child = element.child("Server"); child; child = child.next_sibling("Server")) {
+		wxString name = GetTextElement(child, "Name");
 		if (name.empty())
 			continue;
 
@@ -627,11 +585,9 @@ wxString CSiteManager::AddServer(CServer server)
 	wxString name = _("New site");
 	int i = 1;
 
-	for (;;)
-	{
+	for (;;) {
 		std::list<wxString>::const_iterator iter;
-		for (iter = names.begin(); iter != names.end(); ++iter)
-		{
+		for (iter = names.begin(); iter != names.end(); ++iter) {
 			if (*iter == name)
 				break;
 		}
@@ -643,13 +599,9 @@ wxString CSiteManager::AddServer(CServer server)
 
 	server.SetName(name);
 
-	TiXmlElement* pServer = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
-	SetServer(pServer, server);
-
-	wxScopedCharBuffer utf8 = name.utf8_str();
-	if (utf8) {
-		pServer->LinkEndChild(new TiXmlText(utf8));
-	}
+	auto xServer = element.append_child("Server");
+	SetServer(xServer, server);
+	AddTextElement(xServer, name);
 
 	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
@@ -663,39 +615,36 @@ wxString CSiteManager::AddServer(CServer server)
 	return _T("0/") + EscapeSegment(name);
 }
 
-TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxString> const& segments)
+pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::list<wxString> const& segments)
 {
-	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
-	{
+	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it) {
 		const wxString & segment = *it;
 
-		TiXmlElement* pChild;
-		for (pChild = pNode->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
-		{
-			const char* value = pChild->Value();
-			if (strcmp(value, "Server") && strcmp(value, "Folder") && strcmp(value, "Bookmark"))
+		pugi::xml_node child;
+		for (child = node.first_child(); child; child = child.next_sibling()) {
+			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark"))
 				continue;
 
-			wxString name = GetTextElement_Trimmed(pChild, "Name");
+			wxString name = GetTextElement_Trimmed(child, "Name");
 			if (name.empty())
-				name = GetTextElement_Trimmed(pChild);
+				name = GetTextElement_Trimmed(child);
 			if (name.empty())
 				continue;
 
 			if (name == segment)
 				break;
 		}
-		if (!pChild)
-			return 0;
+		if (!child)
+			return pugi::xml_node();
 
-		pNode = pChild;
+		node = child;
 		continue;
 	}
 
-	return pNode;
+	return node;
 }
 
-bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
+bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
 	if (local_dir.empty() && remote_dir.empty())
 		return false;
@@ -711,63 +660,58 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmark could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return false;
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
-	{
+	if (!UnescapeSitePath(sitePath, segments)) {
 		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
-	TiXmlElement* pChild = GetElementByPath(pElement, segments);
-	if (!pChild || strcmp(pChild->Value(), "Server"))
-	{
+	auto child = GetElementByPath(element, segments);
+	if (!child || strcmp(child.name(), "Server")) {
 		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	// Bookmarks
-	TiXmlElement *pInsertBefore = 0;
-	TiXmlElement* pBookmark;
-	for (pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
-		TiXmlHandle handle(pBookmark);
-
-		wxString old_name = GetTextElement_Trimmed(pBookmark, "Name");
+	pugi::xml_node insertBefore, bookmark;
+	for (bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
+		wxString old_name = GetTextElement_Trimmed(bookmark, "Name");
 		if (old_name.empty())
 			continue;
 
-		if (name == old_name)
-		{
+		if (name == old_name) {
 			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
-		if (name < old_name && !pInsertBefore)
-			pInsertBefore = pBookmark;
+		if (name < old_name && !insertBefore)
+			insertBefore = bookmark;
 	}
 
-	if (pInsertBefore)
-		pBookmark = pChild->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
+	if (insertBefore)
+		bookmark = child.insert_child_before("Bookmark", insertBefore);
 	else
-		pBookmark = pChild->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
-	AddTextElement(pBookmark, "Name", name);
+		bookmark = child.append_child("Bookmark");
+	AddTextElement(bookmark, "Name", name);
 	if (!local_dir.empty())
-		AddTextElement(pBookmark, "LocalDir", local_dir);
+		AddTextElement(bookmark, "LocalDir", local_dir);
 	if (!remote_dir.empty())
-		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
+		AddTextElement(bookmark, "RemoteDir", remote_dir.GetSafePath());
 	if (sync)
-		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
+		AddTextElementRaw(bookmark, "SyncBrowsing", "1");
+	if (comparison)
+		AddTextElementRaw(bookmark, "DirectoryComparison", "1");
 
 	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
@@ -793,16 +737,16 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
-	TiXmlElement *pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmarks could not be cleared.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return false;
 
 	std::list<wxString> segments;
@@ -811,16 +755,16 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 		return 0;
 	}
 
-	TiXmlElement* pChild = GetElementByPath(pElement, segments);
-	if (!pChild || strcmp(pChild->Value(), "Server")) {
+	auto child = GetElementByPath(element, segments);
+	if (!child || strcmp(child.name(), "Server")) {
 		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
-	TiXmlElement *pBookmark = pChild->FirstChildElement("Bookmark");
-	while (pBookmark) {
-		pChild->RemoveChild(pBookmark);
-		pBookmark = pChild->FirstChildElement("Bookmark");
+	auto bookmark = child.child("Bookmark");
+	while (bookmark) {
+		child.remove_child(bookmark);
+		bookmark = child.child("Bookmark");
 	}
 
 	if (!file.Save(false)) {
@@ -833,3 +777,11 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 
 	return true;
 }
+
+bool CSiteManager::HasSites()
+{
+	CSiteManagerXmlHandler_Stats handler;
+	Load(handler);
+
+	return handler.sites_ > 0;
+}
