@@ -3,6 +3,8 @@
 
 #include <wx/treectrl.h>
 
+#include "xmlfunctions.h"
+
 class CSiteManagerItemData : public wxTreeItemData
 {
 public:
@@ -15,7 +17,6 @@ public:
 	CSiteManagerItemData(enum type item_type)
 		: m_type(item_type)
 	{
-		m_sync = false;
 	}
 
 	virtual ~CSiteManagerItemData()
@@ -27,7 +28,8 @@ public:
 
 	enum type m_type;
 
-	bool m_sync;
+	bool m_sync{};
+	bool m_comparison{};
 
 	wxString m_path;
 };
@@ -57,14 +59,13 @@ public:
 
 	// Adds a folder and descents
 	virtual bool AddFolder(const wxString& name, bool expanded) = 0;
-	virtual bool AddSite(CSiteManagerItemData_Site* data) = 0;
-	virtual bool AddBookmark(const wxString& name, CSiteManagerItemData* data) = 0;
+	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site> data) = 0;
+	virtual bool AddBookmark(const wxString& name, std::unique_ptr<CSiteManagerItemData> data) = 0;
 
 	// Go up a level
-	virtual bool LevelUp() = 0; // *Ding*
+	virtual bool LevelUp() { return true; } // *Ding*
 };
 
-class TiXmlElement;
 class CSiteManagerXmlHandler;
 class CSiteManagerDialog;
 class CSiteManager
@@ -72,32 +73,37 @@ class CSiteManager
 	friend class CSiteManagerDialog;
 public:
 	// This function also clears the Id map
-	static CSiteManagerItemData_Site* GetSiteById(int id);
-	static CSiteManagerItemData_Site* GetSiteByPath(wxString sitePath);
+	static std::unique_ptr<CSiteManagerItemData_Site> GetSiteById(int id);
+	static std::unique_ptr<CSiteManagerItemData_Site> GetSiteByPath(wxString sitePath);
 
 	static bool GetBookmarks(wxString sitePath, std::list<wxString> &bookmarks);
 
 	static wxString AddServer(CServer server);
-	static bool AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync);
+	static bool AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison);
 	static bool ClearBookmarks(wxString sitePath);
 
-	static wxMenu* GetSitesMenu();
+	static std::unique_ptr<wxMenu> GetSitesMenu();
 	static void ClearIdMap();
 
 	static bool UnescapeSitePath(wxString path, std::list<wxString>& result);
 	static wxString EscapeSegment( wxString segment );
 
-protected:
-	static bool Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler);
-	static CSiteManagerItemData_Site* ReadServerElement(TiXmlElement *pElement);
+	static bool HasSites();
 
-	static TiXmlElement* GetElementByPath(TiXmlElement* pNode, std::list<wxString> const& segments);
+protected:
+	static bool Load(CSiteManagerXmlHandler& pHandler);
+	static bool Load(pugi::xml_node element, CSiteManagerXmlHandler& pHandler);
+	static std::unique_ptr<CSiteManagerItemData_Site> ReadServerElement(pugi::xml_node element);
+
+	static pugi::xml_node GetElementByPath(pugi::xml_node node, std::list<wxString> const& segments);
 	static wxString BuildPath(wxChar root, std::list<wxString> const& segments);
 
-	static std::map<int, CSiteManagerItemData_Site*> m_idMap;
+	static std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> m_idMap;
 
 	// The map maps event id's to sites
-	static wxMenu* GetSitesMenu_Predefined(std::map<int, CSiteManagerItemData_Site*> &idMap);
+	static std::unique_ptr<wxMenu> GetSitesMenu_Predefined(std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> &idMap);
+
+	static bool LoadPredefined(CSiteManagerXmlHandler& handler);
 };
 
 #endif //__SITEMANAGER_H__
