@@ -7,9 +7,10 @@
 #include "queue.h"
 #include "filezillaapp.h"
 #include "recursive_operation.h"
-#include "statusbar.h"
-#include "local_filesys.h"
 #include "listingcomparison.h"
+#include "xrc_helper.h"
+
+#include <libfilezilla/local_filesys.hpp>
 
 CContextManager CContextManager::m_the_context_manager;
 
@@ -23,23 +24,20 @@ CContextManager* CContextManager::Get()
 	return &m_the_context_manager;
 }
 
-CState* CContextManager::CreateState(CMainFrame* pMainFrame)
+CState* CContextManager::CreateState(CMainFrame &mainFrame)
 {
-	wxASSERT(pMainFrame);
-
-	CState* pState = new CState(pMainFrame);
+	CState* pState = new CState(mainFrame);
 
 	m_contexts.push_back(pState);
 
-	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0, false);
+	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0);
 
 	return pState;
 }
 
 void CContextManager::DestroyState(CState* pState)
 {
-	for (unsigned int i = 0; i < m_contexts.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
 		if (m_contexts[i] != pState)
 			continue;
 
@@ -50,13 +48,13 @@ void CContextManager::DestroyState(CState* pState)
 		{
 			if (i >= m_contexts.size())
 				m_current_context--;
-			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 		}
 
 		break;
 	}
 
-	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0);
 	delete pState;
 }
 
@@ -71,41 +69,39 @@ void CContextManager::SetCurrentContext(CState* pState)
 			continue;
 
 		m_current_context = i;
-		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 	}
 }
 
 void CContextManager::DestroyAllStates()
 {
 	m_current_context = -1;
-	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 
 	while (!m_contexts.empty())
 	{
 		CState* pState = m_contexts.back();
 		m_contexts.pop_back();
 
-		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0);
 		delete pState;
 	}
 }
 
-void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable)
+void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 
-	std::list<t_handler> &handlers = m_handlers[notification];
-	std::list<t_handler>::const_iterator iter;
-	for (iter = handlers.begin(); iter != handlers.end(); ++iter)
-	{
-		if (iter->pHandler == pHandler)
+	auto &handlers = m_handlers[notification];
+	for (auto const& it : handlers) {
+		if (it.pHandler == pHandler) {
 			return;
+		}
 	}
 
 	t_handler handler;
 	handler.pHandler = pHandler;
-	handler.blockable = blockable;
 	handler.current_only = current_only;
 	handlers.push_back(handler);
 }
@@ -115,28 +111,21 @@ void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_sta
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
 
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-		{
-			std::list<t_handler> &handlers = m_handlers[i];
-			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
-			{
-				if (iter->pHandler == pHandler)
-				{
+	if (notification == STATECHANGE_NONE) {
+		for (int i = 0; i < STATECHANGE_MAX; ++i) {
+			auto &handlers = m_handlers[i];
+			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
+				if (iter->pHandler == pHandler) {
 					handlers.erase(iter);
 					break;
 				}
 			}
 		}
 	}
-	else
-	{
-		std::list<t_handler> &handlers = m_handlers[notification];
-		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
-		{
-			if (iter->pHandler == pHandler)
-			{
+	else {
+		auto &handlers = m_handlers[notification];
+		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
+			if (iter->pHandler == pHandler) {
 				handlers.erase(iter);
 				return;
 			}
@@ -150,20 +139,16 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
-	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
-	{
-		if (blocked && iter->blockable)
+	auto const& handlers = m_handlers[notification];
+	for (auto const& handler : handlers) {
+		if (handler.current_only && pState != GetCurrentContext())
 			continue;
 
-		if (iter->current_only && pState != GetCurrentContext())
-			continue;
-
-		iter->pHandler->OnStateChange(pState, notification, data, data2);
+		handler.pHandler->OnStateChange(pState, notification, data, data2);
 	}
 }
 
@@ -183,24 +168,17 @@ void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notific
 
 void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
-	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
-		iter->pHandler->OnStateChange(0, notification, data, data2);
+	auto const& handlers = m_handlers[notification];
+	for (auto const& handler : handlers) {
+		handler.pHandler->OnStateChange(0, notification, data, data2);
+	}
 }
 
-CState::CState(CMainFrame* pMainFrame)
+CState::CState(CMainFrame &mainFrame)
+	: m_mainFrame(mainFrame)
 {
-	memset(m_blocked, 0, sizeof(m_blocked));
-
-	m_pMainFrame = pMainFrame;
-
-	m_pDirectoryListing = 0;
-	m_pServer = 0;
 	m_title = _("Not connected");
-	m_successful_connect = 0;
 
-	m_pEngine = 0;
-	m_pCommandQueue = 0;
 	m_pComparisonManager = new CComparisonManager(this);
 
 	m_pRecursiveOperation = new CRecursiveOperation(this);
@@ -209,18 +187,12 @@ CState::CState(CMainFrame* pMainFrame)
 	m_sync_browse.compare = false;
 
 	m_localDir.SetPath(CLocalPath::path_separator);
-
-	m_pCertificate = 0;
-	m_pSftpEncryptionInfo = 0;
 }
 
 CState::~CState()
 {
 	delete m_pServer;
 
-	delete m_pCertificate;
-	delete m_pSftpEncryptionInfo;
-
 	delete m_pComparisonManager;
 	delete m_pCommandQueue;
 	delete m_pEngine;
@@ -242,14 +214,9 @@ CLocalPath CState::GetLocalDir() const
 	return m_localDir;
 }
 
+
 bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPreviousSubdir)
 {
-	if (m_sync_browse.is_changing)
-	{
-		wxMessageBoxEx(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
-		return false;
-	}
-
 	CLocalPath p(m_localDir);
 #ifdef __WXMSW__
 	if (dir == _T("..") && !p.HasParent() && p.HasLogicalParent())
@@ -263,43 +230,49 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPrev
 	if (!p.ChangePath(dir))
 		return false;
 
-	if (!p.Exists(error))
+	return SetLocalDir(p, error, rememberPreviousSubdir);
+}
+
+bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPreviousSubdir)
+{
+	if (m_sync_browse.is_changing) {
+		wxMessageBoxEx(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
+		return false;
+	}
+
+	if (!dir.Exists(error))
 		return false;
 
-	if (!m_sync_browse.local_root.empty())
-	{
+	if (!m_sync_browse.local_root.empty()) {
 		wxASSERT(m_pServer);
 
-		if (p != m_sync_browse.local_root && !p.IsSubdirOf(m_sync_browse.local_root))
-		{
+		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
-					p.GetPath(),
+					dir.GetPath(),
 					m_sync_browse.local_root.GetPath());
 			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle())
-		{
+		else if (!IsRemoteIdle(true)) {
 			wxString msg(_("A remote operation is in progress and synchronized browsing is enabled.\nDisable synchronized browsing and continue changing the local directory?"));
 			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
 			SetSyncBrowse(false);
 		}
-		else
-		{
-			CServerPath remote_path = GetSynchronizedDirectory(p);
-			if (remote_path.empty())
-			{
+		else {
+			CServerPath remote_path = GetSynchronizedDirectory(dir);
+			if (remote_path.empty()) {
 				SetSyncBrowse(false);
 				wxString msg = wxString::Format(_("Could not obtain corresponding remote directory for the local directory '%s'.\nSynchronized browsing has been disabled."),
-					p.GetPath());
+					dir.GetPath());
 				wxMessageBoxEx(msg, _("Synchronized browsing"));
 				return false;
 			}
 
 			m_sync_browse.is_changing = true;
 			m_sync_browse.compare = m_pComparisonManager->IsComparing();
+			m_sync_browse.target_path = remote_path;
 			CListCommand *pCommand = new CListCommand(remote_path);
 			m_pCommandQueue->ProcessCommand(pCommand);
 
@@ -307,9 +280,9 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPrev
 		}
 	}
 
-	if (p == m_localDir.GetParent() && rememberPreviousSubdir) {
+	if (dir == m_localDir.GetParent() && rememberPreviousSubdir) {
 #ifdef __WXMSW__
-		if (p.GetPath() == _T("\\")) {
+		if (dir.GetPath() == _T("\\")) {
 			m_previouslyVisitedLocalSubdir = m_localDir.GetPath();
 			m_previouslyVisitedLocalSubdir.RemoveLast();
 		}
@@ -321,7 +294,7 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPrev
 		m_previouslyVisitedLocalSubdir = _T("");
 
 
-	m_localDir = p;
+	m_localDir = dir;
 
 	COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
 
@@ -337,16 +310,15 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 		if (modified)
 			return false;
 
-		if (m_pDirectoryListing)
-		{
+		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
-			NotifyHandlers(STATECHANGE_REMOTE_DIR);
+			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
 		}
 		m_previouslyVisitedRemoteSubdir = _T("");
 		return true;
 	}
 
-	wxASSERT(pDirectoryListing->m_firstListTime.IsValid());
+	wxASSERT(pDirectoryListing->m_firstListTime);
 
 	if (pDirectoryListing && m_pDirectoryListing &&
 		pDirectoryListing->path == m_pDirectoryListing->path.GetParent())
@@ -372,15 +344,11 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	m_pDirectoryListing = pDirectoryListing;
 
-	if (!modified)
-		NotifyHandlers(STATECHANGE_REMOTE_DIR);
-	else
-		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
+	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
 
 	if (m_sync_browse.is_changing && !modified) {
 		m_sync_browse.is_changing = false;
-		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false))
-		{
+		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			SetSyncBrowse(false);
 			wxString msg = wxString::Format(_("Current remote directory (%s) is not below the synchronization root (%s).\nSynchronized browsing has been disabled."),
 					m_pDirectoryListing->path.GetPath(),
@@ -477,10 +445,9 @@ void CState::LocalDirCreated(const CLocalPath& path)
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
 }
 
-void CState::SetServer(const CServer* server)
+void CState::SetServer(const CServer* server, CServerPath const& path)
 {
-	if (m_pServer)
-	{
+	if (m_pServer) {
 		if (server && *server == *m_pServer &&
 			server->GetName() == m_pServer->GetName() &&
 			server->MaximumMultipleConnections() == m_pServer->MaximumMultipleConnections())
@@ -491,15 +458,16 @@ void CState::SetServer(const CServer* server)
 
 		SetRemoteDir(0);
 		delete m_pServer;
-		delete m_pCertificate;
-		m_pCertificate = 0;
-		delete m_pSftpEncryptionInfo;
-		m_pSftpEncryptionInfo = 0;
+		m_pCertificate.reset();
+		m_pSftpEncryptionInfo.reset();
 	}
-	if (server)
-	{
-		if (m_last_server != *server)
+	if (server) {
+		if (!path.empty()) {
+			m_last_path = path;
+		}
+		else if (m_last_server != *server) {
 			m_last_path.clear();
+		}
 		m_last_server = *server;
 
 		m_pServer = new CServer(*server);
@@ -510,8 +478,7 @@ void CState::SetServer(const CServer* server)
 		else
 			m_title = server->FormatServer();
 	}
-	else
-	{
+	else {
 		m_pServer = 0;
 		m_title = _("Not connected");
 	}
@@ -543,7 +510,7 @@ bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPa
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
-	SetServer(&server);
+	SetServer(&server, path);
 
 	return true;
 }
@@ -571,10 +538,10 @@ bool CState::CreateEngine()
 	if (m_pEngine)
 		return true;
 
-	m_pEngine = new CFileZillaEngine();
-	m_pEngine->Init(m_pMainFrame, COptions::Get());
+	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext());
+	m_pEngine->Init(&m_mainFrame);
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame, this);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, this);
 
 	return true;
 }
@@ -587,26 +554,32 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool blockable /*=true*/)
+void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, CStateEventHandler* insertBefore)
 {
 	wxASSERT(pHandler);
 	wxASSERT(pHandler->m_pState == this);
 	if (pHandler->m_pState != this)
 		return;
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
+	wxASSERT(pHandler != insertBefore);
 
-	std::list<t_handler> &handlers = m_handlers[notification];
-	std::list<t_handler>::const_iterator iter;
-	for (iter = handlers.begin(); iter != handlers.end(); ++iter)
-	{
-		if (iter->pHandler == pHandler)
+	
+	auto &handlers = m_handlers[notification];
+	auto insertionPoint = handlers.end();
+
+	for (auto it = handlers.begin(); it != handlers.end(); ++it) {
+		if (it->pHandler == insertBefore) {
+			insertionPoint = it;
+		}
+		if (it->pHandler == pHandler) {
+			wxASSERT(insertionPoint == handlers.end());
 			return;
+		}
 	}
 
 	t_handler handler;
 	handler.pHandler = pHandler;
-	handler.blockable = blockable;
-	handlers.push_back(handler);
+	handlers.insert(insertionPoint, handler);
 }
 
 void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
@@ -614,28 +587,21 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
 
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-		{
-			std::list<t_handler> &handlers = m_handlers[i];
-			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
-			{
-				if (iter->pHandler == pHandler)
-				{
+	if (notification == STATECHANGE_NONE) {
+		for (int i = 0; i < STATECHANGE_MAX; ++i) {
+			auto &handlers = m_handlers[i];
+			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
+				if (iter->pHandler == pHandler) {
 					handlers.erase(iter);
 					break;
 				}
 			}
 		}
 	}
-	else
-	{
-		std::list<t_handler> &handlers = m_handlers[notification];
-		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
-		{
-			if (iter->pHandler == pHandler)
-			{
+	else {
+		auto &handlers = m_handlers[notification];
+		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
+			if (iter->pHandler == pHandler) {
 				handlers.erase(iter);
 				return;
 			}
@@ -643,47 +609,16 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 }
 
-void CState::BlockHandlers(enum t_statechange_notifications notification)
-{
-	wxASSERT(notification != STATECHANGE_MAX);
-
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-			m_blocked[i] = true;
-	}
-	else
-		m_blocked[notification] = true;
-}
-
-void CState::UnblockHandlers(enum t_statechange_notifications notification)
-{
-	wxASSERT(notification != STATECHANGE_MAX);
-
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-			m_blocked[i] = false;
-	}
-	else
-		m_blocked[notification] = false;
-
-}
-
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
-	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
-	{
-		if (m_blocked[notification] && iter->blockable)
-			continue;
-
-		iter->pHandler->OnStateChange(this, notification, data, data2);
+	auto const& handlers = m_handlers[notification];
+	for (auto const& handler : handlers) {
+		handler.pHandler->OnStateChange(this, notification, data, data2);
 	}
 
-	CContextManager::Get()->NotifyHandlers(this, notification, data, data2, m_blocked[notification]);
+	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
 CStateEventHandler::CStateEventHandler(CState* pState)
@@ -721,26 +656,23 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 
 	const wxArrayString& files = pFileDataObject->GetFilenames();
 
-	for (unsigned int i = 0; i < files.Count(); i++)
-	{
-		wxLongLong size;
+	for (unsigned int i = 0; i < files.Count(); ++i) {
+		int64_t size;
 		bool is_link;
-		CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(files[i], is_link, &size, 0, 0);
-		if (type == CLocalFileSystem::file)
-		{
+		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(files[i]), is_link, &size, 0, 0);
+		if (type == fz::local_filesys::file) {
 			wxString localFile;
 			const CLocalPath localPath(files[i], &localFile);
-			m_pMainFrame->GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
-			m_pMainFrame->GetQueue()->QueueFile_Finish(!queueOnly);
+			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
+			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
-		else if (type == CLocalFileSystem::dir)
-		{
+		else if (type == fz::local_filesys::dir) {
 			CLocalPath localPath(files[i]);
 			if (localPath.HasParent())
 			{
 				CServerPath target = path;
 				target.AddSegment(localPath.GetLastSegment());
-				m_pMainFrame->GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
+				m_mainFrame.GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
 			}
 		}
 	}
@@ -755,15 +687,15 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 #ifdef __WXMSW__
 	int len = 1;
 
-	for (unsigned int i = 0; i < files.Count(); i++)
+	for (unsigned int i = 0; i < files.Count(); ++i) {
 		len += files[i].Len() + 1;
+	}
 
 	// SHFILEOPSTRUCT's pTo and pFrom accept null-terminated lists
 	// of null-terminated filenames.
 	wxChar* from = new wxChar[len];
 	wxChar* p = from;
-	for (unsigned int i = 0; i < files.Count(); i++)
-	{
+	for (unsigned int i = 0; i < files.Count(); ++i) {
 		wxStrcpy(p, files[i]);
 		p += files[i].Len() + 1;
 	}
@@ -777,22 +709,20 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	op.pFrom = from;
 	op.pTo = to;
 	op.wFunc = copy ? FO_COPY : FO_MOVE;
-	op.hwnd = (HWND)m_pMainFrame->GetHandle();
+	op.hwnd = (HWND)m_mainFrame.GetHandle();
 	SHFileOperation(&op);
 
 	delete [] to;
 	delete [] from;
 #else
 	wxString error;
-	for (unsigned int i = 0; i < files.Count(); i++)
-	{
-		const wxString& file(files[i]);
+	for (unsigned int i = 0; i < files.Count(); ++i) {
+		wxString const& file(files[i]);
 
-		wxLongLong size;
+		int64_t size;
 		bool is_link;
-		CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(file, is_link, &size, 0, 0);
-		if (type == CLocalFileSystem::file)
-		{
+		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(file), is_link, &size, 0, 0);
+		if (type == fz::local_filesys::file) {
 			wxString name;
 			CLocalPath sourcePath(file, &name);
 			if (name.empty())
@@ -806,21 +736,18 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 			else
 				wxRenameFile(file, target);
 		}
-		else if (type == CLocalFileSystem::dir)
-		{
+		else if (type == fz::local_filesys::dir) {
 			CLocalPath sourcePath(file);
 			if (sourcePath == path || sourcePath.GetParent() == path)
 				continue;
-			if (sourcePath.IsParentOf(path))
-			{
+			if (sourcePath.IsParentOf(path)) {
 				error = _("A directory cannot be dragged into one of its subdirectories.");
 				continue;
 			}
 
 			if (copy)
 				RecursiveCopy(sourcePath, path);
-			else
-			{
+			else {
 				if (!sourcePath.HasParent())
 					continue;
 				wxRenameFile(file, path.GetPath() + sourcePath.GetLastSegment());
@@ -856,28 +783,24 @@ bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 	dirsToVisit.push_back(last_segment + CLocalPath::path_separator);
 
 	// Process any subdirs which still have to be visited
-	while (!dirsToVisit.empty())
-	{
+	while (!dirsToVisit.empty()) {
 		wxString dirname = dirsToVisit.front();
 		dirsToVisit.pop_front();
 		wxMkdir(target.GetPath() + dirname);
 
-		CLocalFileSystem fs;
-		if (!fs.BeginFindFiles(source.GetPath() + dirname, false))
+		fz::local_filesys fs;
+		if (!fs.begin_find_files(fz::to_native(source.GetPath() + dirname), false))
 			continue;
 
 		bool is_dir, is_link;
-		wxString file;
-		while (fs.GetNextFile(file, is_link, is_dir, 0, 0, 0))
-		{
-			if (file.empty())
-			{
+		fz::native_string file;
+		while (fs.get_next_file(file, is_link, is_dir, 0, 0, 0)) {
+			if (file.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (is_dir)
-			{
+			if (is_dir) {
 				if (is_link)
 					continue;
 
@@ -897,41 +820,44 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	bool hasDirs = false;
 	bool hasFiles = false;
 	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		if (iter->dir)
 			hasDirs = true;
 		else
 			hasFiles = true;
 	}
 
-	if (hasDirs)
-	{
+	if (hasDirs) {
 		if (!IsRemoteConnected() || !IsRemoteIdle())
 			return false;
 	}
 
 	if (hasFiles)
-		m_pMainFrame->GetQueue()->QueueFiles(queueOnly, path, *pRemoteDataObject);
+		m_mainFrame.GetQueue()->QueueFiles(queueOnly, path, *pRemoteDataObject);
 
 	if (!hasDirs)
 		return true;
 
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	recursion_root root(pRemoteDataObject->GetServerPath(), false);
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		if (!iter->dir)
 			continue;
 
 		CLocalPath newPath(path);
 		newPath.AddSegment(CQueueView::ReplaceInvalidCharacters(iter->name));
-		m_pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, newPath, iter->link);
+		root.add_dir_to_visit(pRemoteDataObject->GetServerPath(), iter->name, newPath, iter->link);
 	}
 
-	if (m_pComparisonManager->IsComparing())
-		m_pComparisonManager->ExitComparisonMode();
+	if (!root.empty()) {
+		if (m_pComparisonManager->IsComparing())
+			m_pComparisonManager->ExitComparisonMode();
 
-	CFilterManager filter;
-	m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, pRemoteDataObject->GetServerPath(), filter.GetActiveFilters(false));
+		m_pRecursiveOperation->AddRecursionRoot(std::move(root));
+
+		CFilterManager filter;
+		m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download,
+			filter.GetActiveFilters(false), pRemoteDataObject->GetServerPath());
+	}
 
 	return true;
 }
@@ -944,24 +870,47 @@ bool CState::IsRemoteConnected() const
 	return m_pServer != 0;
 }
 
-bool CState::IsRemoteIdle() const
+bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
 		return false;
 
 	if (!m_pCommandQueue)
 		return true;
 
-	return m_pCommandQueue->Idle();
+	return m_pCommandQueue->Idle(ignore_recursive ? CCommandQueue::normal : CCommandQueue::any);
 }
 
-void CState::ListingFailed(int error)
+void CState::ListingFailed(int)
 {
-	m_sync_browse.is_changing = false;
+	if (m_sync_browse.is_changing && !m_sync_browse.target_path.empty()) {
+		wxDialogEx dlg;
+		if (!dlg.Load(0, _T("ID_SYNCBROWSE_NONEXISTING"))) {
+			SetSyncBrowse(false);
+			return;
+		}
 
-	// Let the recursive operation handler know if a LIST command failed,
-	// so that it may issue the next command in recursive operations.
-	m_pRecursiveOperation->ListingFailed(error);
+		xrc_call(dlg, "ID_SYNCBROWSE_NONEXISTING_LABEL", &wxStaticText::SetLabel, wxString::Format(_("The remote directory '%s' does not exist."), m_sync_browse.target_path.GetPath()));
+		xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::SetLabel, _("Create &missing remote directory and enter it"));
+		xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the local directory"));
+		dlg.GetSizer()->Fit(&dlg);
+		if (dlg.ShowModal() == wxID_OK) {
+
+			if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
+				m_pCommandQueue->ProcessCommand(new CMkdirCommand(m_sync_browse.target_path));
+				m_pCommandQueue->ProcessCommand(new CListCommand(m_sync_browse.target_path));
+				m_sync_browse.target_path.clear();
+				return;
+			}
+			else {
+				CLocalPath local = GetSynchronizedDirectory(m_sync_browse.target_path);
+				SetSyncBrowse(false);
+				SetLocalDir(local);
+			}
+		}
+	}
+
+	m_sync_browse.is_changing = false;
 }
 
 void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
@@ -976,18 +925,15 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 	if (!m_pServer || !m_pCommandQueue)
 		return false;
 
-	if (!m_sync_browse.local_root.empty())
-	{
+	if (!m_sync_browse.local_root.empty()) {
 		CServerPath p(path);
-		if (!subdir.empty() && !p.ChangePath(subdir))
-		{
+		if (!subdir.empty() && !p.ChangePath(subdir)) {
 			wxString msg = wxString::Format(_("Could not get full remote path."));
 			wxMessageBoxEx(msg, _("Synchronized browsing"));
 			return false;
 		}
 
-		if (p != m_sync_browse.remote_root && !p.IsSubdirOf(m_sync_browse.remote_root, false))
-		{
+		if (p != m_sync_browse.remote_root && !p.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			wxString msg = wxString::Format(_("The remote directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath(),
 					m_sync_browse.remote_root.GetPath());
@@ -995,35 +941,58 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle() && !ignore_busy)
-		{
+		else if (!IsRemoteIdle(true) && !ignore_busy) {
 			wxString msg(_("Another remote operation is already in progress, cannot change directory now."));
 			wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
 			return false;
 		}
-		else
-		{
+		else {
 			wxString error;
 			CLocalPath local_path = GetSynchronizedDirectory(p);
-			if (local_path.empty())
-			{
+			if (local_path.empty()) {
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath());
 				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 					return false;
 				SetSyncBrowse(false);
 			}
-			else if (!local_path.Exists(&error))
-			{
+			else if (!local_path.Exists(&error)) {
 				wxString msg = error + _T("\n") + _("Disable synchronized browsing and continue changing the remote directory?");
-				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+
+				wxDialogEx dlg;
+				if (!dlg.Load(0, _T("ID_SYNCBROWSE_NONEXISTING"))) {
 					return false;
-				SetSyncBrowse(false);
+				}
+				xrc_call(dlg, "ID_SYNCBROWSE_NONEXISTING_LABEL", &wxStaticText::SetLabel, wxString::Format(_("The local directory '%s' does not exist."), local_path.GetPath()));
+				xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::SetLabel, _("Create &missing local directory and enter it"));
+				xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the remote directory"));
+				dlg.GetSizer()->Fit(&dlg);
+				if (dlg.ShowModal() != wxID_OK) {
+					return false;
+				}
+
+				if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
+					{
+						wxLogNull log;
+						wxMkdir(local_path.GetPath());
+					}
+
+					if (!local_path.Exists(&error)) {
+						wxMessageBox(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
+						return false;
+					}
+					m_sync_browse.is_changing = true;
+					m_sync_browse.compare = m_pComparisonManager->IsComparing();
+					m_sync_browse.target_path.clear();
+				}
+				else {
+					SetSyncBrowse(false);
+				}
 			}
-			else
-			{
+			else {
 				m_sync_browse.is_changing = true;
 				m_sync_browse.compare = m_pComparisonManager->IsComparing();
+				m_sync_browse.target_path.clear();
 			}
 		}
 	}
@@ -1039,8 +1008,7 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 	if (enable != m_sync_browse.local_root.empty())
 		return enable;
 
-	if (!enable)
-	{
+	if (!enable) {
 		wxASSERT(assumed_remote_root.empty());
 		m_sync_browse.local_root.clear();
 		m_sync_browse.remote_root.clear();
@@ -1050,8 +1018,7 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 		return false;
 	}
 
-	if (!m_pDirectoryListing && assumed_remote_root.empty())
-	{
+	if (!m_pDirectoryListing && assumed_remote_root.empty()) {
 		NotifyHandlers(STATECHANGE_SYNC_BROWSE);
 		return false;
 	}
@@ -1061,8 +1028,7 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 
 	if (assumed_remote_root.empty())
 		m_sync_browse.remote_root = m_pDirectoryListing->path;
-	else
-	{
+	else {
 		m_sync_browse.remote_root = assumed_remote_root;
 		m_sync_browse.is_changing = true;
 		m_sync_browse.compare = false;
@@ -1082,8 +1048,7 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 CLocalPath CState::GetSynchronizedDirectory(CServerPath remote_path)
 {
 	std::list<wxString> segments;
-	while (remote_path.HasParent() && remote_path != m_sync_browse.remote_root)
-	{
+	while (remote_path.HasParent() && remote_path != m_sync_browse.remote_root) {
 		segments.push_front(remote_path.GetLastSegment());
 		remote_path = remote_path.GetParent();
 	}
@@ -1101,8 +1066,7 @@ CLocalPath CState::GetSynchronizedDirectory(CServerPath remote_path)
 CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 {
 	std::list<wxString> segments;
-	while (local_path.HasParent() && local_path != m_sync_browse.local_root)
-	{
+	while (local_path.HasParent() && local_path != m_sync_browse.local_root) {
 		wxString last;
 		local_path.MakeParent(&last);
 		segments.push_front(last);
@@ -1122,7 +1086,7 @@ bool CState::RefreshRemote()
 	if (!m_pCommandQueue)
 		return false;
 
-	if (!IsRemoteConnected() || !IsRemoteIdle())
+	if (!IsRemoteConnected() || !IsRemoteIdle(true))
 		return false;
 
 	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
@@ -1130,28 +1094,26 @@ bool CState::RefreshRemote()
 
 bool CState::GetSecurityInfo(CCertificateNotification *& pInfo)
 {
-	pInfo = m_pCertificate;
+	pInfo = m_pCertificate.get();
 	return m_pCertificate != 0;
 }
 
 bool CState::GetSecurityInfo(CSftpEncryptionNotification *& pInfo)
 {
-	pInfo = m_pSftpEncryptionInfo;
+	pInfo = m_pSftpEncryptionInfo.get();
 	return m_pSftpEncryptionInfo != 0;
 }
 
 void CState::SetSecurityInfo(CCertificateNotification const& info)
 {
-	delete m_pCertificate;
-	m_pCertificate = new CCertificateNotification(info);
-	delete m_pSftpEncryptionInfo;
-	m_pSftpEncryptionInfo = 0;
+	m_pSftpEncryptionInfo.reset();
+	m_pCertificate = std::make_unique<CCertificateNotification>(info);
+	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
 
 void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 {
-	delete m_pCertificate;
-	m_pCertificate = 0;
-	delete m_pSftpEncryptionInfo;
-	m_pSftpEncryptionInfo = new CSftpEncryptionNotification(info);
+	m_pCertificate.reset();
+	m_pSftpEncryptionInfo = std::make_unique<CSftpEncryptionNotification>(info);
+	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
