@@ -5,6 +5,8 @@
 #include "Options.h"
 #include "sizeformatting.h"
 
+#include <algorithm>
+
 BEGIN_EVENT_TABLE(CStatusLineCtrl, wxWindow)
 EVT_PAINT(CStatusLineCtrl::OnPaint)
 EVT_TIMER(wxID_ANY, CStatusLineCtrl::OnTimer)
@@ -18,15 +20,10 @@ bool CStatusLineCtrl::m_initialized = false;
 #define PROGRESSBAR_WIDTH 102
 
 CStatusLineCtrl::CStatusLineCtrl(CQueueView* pParent, const t_EngineData* const pEngineData, const wxRect& initialPosition)
-	: m_pEngineData(pEngineData)
+	: m_pParent(pParent)
+	, m_pEngineData(pEngineData)
+	, m_gcLastTimeStamp(wxDateTime::Now())
 {
-	m_mdc = 0;
-	m_pPreviousStatusText = 0;
-	m_last_elapsed_seconds = 0;
-	m_last_left = 0;
-	m_last_bar_split = -1;
-	m_last_permill = -1;
-
 	wxASSERT(pEngineData);
 
 	Create(pParent->GetMainWindow(), wxID_ANY, initialPosition.GetPosition(), initialPosition.GetSize());
@@ -38,44 +35,39 @@ CStatusLineCtrl::CStatusLineCtrl(CQueueView* pParent, const t_EngineData* const
 
 	m_transferStatusTimer.SetOwner(this);
 
-	m_pParent = pParent;
-	status_valid_ = false;
-	m_lastOffset = -1;
+	InitFieldOffsets();
 
-	m_gcLastTimeStamp = wxDateTime::Now();
-	m_gcLastOffset = -1;
-	m_gcLastSpeed = -1;
-
-	SetTransferStatus(0);
+	ClearTransferStatus();
+}
 
+void CStatusLineCtrl::InitFieldOffsets()
+{
+	if (m_initialized)
+		return;
+	m_initialized = true;
 
 	// Calculate field widths so that the contents fit under every language.
-	if (!m_initialized)
-	{
-		m_initialized = true;
-		wxClientDC dc(this);
-		dc.SetFont(GetFont());
+	wxClientDC dc(this);
+	dc.SetFont(GetFont());
 
-		wxCoord w, h;
-		wxTimeSpan elapsed(100, 0, 0);
-		dc.GetTextExtent(elapsed.Format(_("%H:%M:%S elapsed")), &w, &h);
-		m_textHeight = h;
-		m_fieldOffsets[0] = 50 + w;
+	wxCoord w, h;
+	wxTimeSpan elapsed(100, 0, 0);
+	// @translator: This is a date/time formatting specifier. See https://wiki.filezilla-project.org/Date_and_Time_formatting
+	dc.GetTextExtent(elapsed.Format(_("%H:%M:%S elapsed")), &w, &h);
+	m_textHeight = h;
+	m_fieldOffsets[0] = 50 + w;
 
-		dc.GetTextExtent(elapsed.Format(_("%H:%M:%S left")), &w, &h);
-		m_fieldOffsets[1] = m_fieldOffsets[0] + 20 + w;
+	// @translator: This is a date/time formatting specifier. See https://wiki.filezilla-project.org/Date_and_Time_formatting
+	dc.GetTextExtent(elapsed.Format(_("%H:%M:%S left")), &w, &h);
+	m_fieldOffsets[1] = m_fieldOffsets[0] + 20 + w;
 
-		m_fieldOffsets[2] = m_fieldOffsets[1] + 20;
-		m_fieldOffsets[3] = m_fieldOffsets[2] + PROGRESSBAR_WIDTH + 20;
-	}
+	m_fieldOffsets[2] = m_fieldOffsets[1] + 20;
+	m_fieldOffsets[3] = m_fieldOffsets[2] + PROGRESSBAR_WIDTH + 20;
 }
 
 CStatusLineCtrl::~CStatusLineCtrl()
 {
-	delete m_mdc;
-	delete m_pPreviousStatusText;
-
-	if (status_valid_ && status_.totalSize >= 0)
+	if (!status_.empty() && status_.totalSize >= 0)
 		m_pEngineData->pItem->SetSize(status_.totalSize);
 
 	if (m_transferStatusTimer.IsRunning())
@@ -89,26 +81,25 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 	wxRect rect = GetRect();
 
 	int refresh = 0;
-	if (!m_data.IsOk() || rect.GetWidth() != m_data.GetWidth() || rect.GetHeight() != m_data.GetHeight())
-	{
-		delete m_mdc;
+	if (!m_data.IsOk() || rect.GetWidth() != m_data.GetWidth() || rect.GetHeight() != m_data.GetHeight()) {
+		m_mdc.reset();
 		m_data = wxBitmap(rect.GetWidth(), rect.GetHeight());
-		m_mdc = new wxMemoryDC(m_data);
+		m_mdc = std::make_unique<wxMemoryDC>(m_data);
 		// Use same layout direction as the DC which bitmap is drawn on.
 		// This avoids problem with mirrored characters on RTL locales.
 		m_mdc->SetLayoutDirection(dc.GetLayoutDirection());
 		refresh = 31;
 	}
 
-	wxTimeSpan elapsed;
+	fz::duration elapsed;
 	int left = -1;
 	wxFileOffset rate;
 	wxString bytes_and_rate;
 	int bar_split = -1;
 	int permill = -1;
 
-	if (!status_valid_) {
-		if (!m_pPreviousStatusText || *m_pPreviousStatusText != m_statusText) {
+	if (status_.empty()) {
+		if (m_previousStatusText != m_statusText) {
 			// Clear background
 			m_mdc->SetFont(GetFont());
 			m_mdc->SetPen(GetBackgroundColour());
@@ -117,37 +108,33 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 			m_mdc->DrawRectangle(0, 0, rect.GetWidth(), rect.GetHeight());
 			wxCoord h = (rect.GetHeight() - m_textHeight) / 2;
 			m_mdc->DrawText(m_statusText, 50, h);
-			delete m_pPreviousStatusText;
-			m_pPreviousStatusText = new wxString(m_statusText);
+			m_previousStatusText = m_statusText;
 			refresh = 0;
 		}
 	}
 	else {
-		if (m_pPreviousStatusText) {
-			delete m_pPreviousStatusText;
-			m_pPreviousStatusText = 0;
+		if (!m_previousStatusText.empty()) {
+			m_previousStatusText.clear();
 			refresh = 31;
 		}
 
-		int elapsed_seconds = 0;
-		if (status_.started.IsValid()) {
-			elapsed = wxDateTime::Now().Subtract(status_.started);
-			elapsed_seconds = elapsed.GetSeconds().GetLo(); // Assume GetHi is always 0
+		int elapsed_milli_seconds = 0;
+		if (status_.started.empty()) {
+			elapsed = fz::datetime::now() - status_.started;
+			elapsed_milli_seconds = static_cast<int>(elapsed.get_milliseconds()); // Assume it doesn't overflow
 		}
 
-		if (elapsed_seconds != m_last_elapsed_seconds)
-		{
+		if (elapsed_milli_seconds / 1000 != m_last_elapsed_seconds) {
 			refresh |= 1;
-			m_last_elapsed_seconds = elapsed_seconds;
+			m_last_elapsed_seconds = elapsed_milli_seconds / 1000;
 		}
 
 		if (COptions::Get()->GetOptionVal(OPTION_SPEED_DISPLAY))
 			rate = GetCurrentSpeed();
 		else
-			rate = GetSpeed(elapsed_seconds);
+			rate = GetSpeed(elapsed_milli_seconds);
 
-		if (status_.totalSize > 0 && elapsed_seconds && rate > 0)
-		{
+		if (status_.totalSize > 0 && elapsed_milli_seconds >= 1000 && rate > 0) {
 			wxFileOffset r = status_.totalSize - status_.currentOffset;
 			left = r / rate + 1;
 			if (r)
@@ -157,15 +144,13 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 				left = 0;
 		}
 
-		if (m_last_left != left)
-		{
+		if (m_last_left != left) {
 			refresh |= 2;
 			m_last_left = left;
 		}
 
 		const wxString bytestr = CSizeFormat::Format(status_.currentOffset, true, CSizeFormat::bytes, COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0, 0);
-		if (elapsed_seconds && rate > -1)
-		{
+		if (elapsed_milli_seconds >= 1000 && rate > -1) {
 			CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
 			if (format == CSizeFormat::bytes)
 				format = CSizeFormat::iec;
@@ -178,34 +163,30 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		else
 			bytes_and_rate.Printf(_("%s (? B/s)"), bytestr);
 
-		if (m_last_bytes_and_rate != bytes_and_rate)
-		{
+		if (m_last_bytes_and_rate != bytes_and_rate) {
 			refresh |= 8;
 			m_last_bytes_and_rate = bytes_and_rate;
 		}
 
-		if (status_.totalSize > 0)
-		{
-			bar_split = wxLongLong(status_.currentOffset * (PROGRESSBAR_WIDTH - 2) / status_.totalSize).GetLo();
+		if (status_.totalSize > 0) {
+			bar_split = static_cast<int>(status_.currentOffset * (PROGRESSBAR_WIDTH - 2) / status_.totalSize);
 			if (bar_split > (PROGRESSBAR_WIDTH - 2))
 				bar_split = PROGRESSBAR_WIDTH - 2;
 
 			if (status_.currentOffset > status_.totalSize)
 				permill = 1001;
 			else
-				permill = wxLongLong(status_.currentOffset * 1000 / status_.totalSize).GetLo();
+				permill = static_cast<int>(status_.currentOffset * 1000 / status_.totalSize);
 		}
 
-		if (m_last_bar_split != bar_split || m_last_permill != permill)
-		{
+		if (m_last_bar_split != bar_split || m_last_permill != permill) {
 			refresh |= 4;
 			m_last_bar_split = bar_split;
 			m_last_permill = permill;
 		}
 	}
 
-	if (refresh)
-	{
+	if (refresh) {
 		m_mdc->SetFont(GetFont());
 		m_mdc->SetPen(GetBackgroundColour());
 		m_mdc->SetBrush(GetBackgroundColour());
@@ -214,105 +195,103 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		// Get character height so that we can center the text vertically.
 		wxCoord h = (rect.GetHeight() - m_textHeight) / 2;
 
-		if (refresh & 1)
-		{
+		if (refresh & 1) {
 			m_mdc->DrawRectangle(0, 0, m_fieldOffsets[0], rect.GetHeight());
-			DrawRightAlignedText(*m_mdc, elapsed.Format(_("%H:%M:%S elapsed")), m_fieldOffsets[0], h);
+			DrawRightAlignedText(*m_mdc, wxTimeSpan::Milliseconds(elapsed.get_milliseconds()).Format(_("%H:%M:%S elapsed")), m_fieldOffsets[0], h);
 		}
-		if (refresh & 2)
-		{
+		if (refresh & 2) {
 			m_mdc->DrawRectangle(m_fieldOffsets[0], 0, m_fieldOffsets[1] - m_fieldOffsets[0], rect.GetHeight());
-			if (left != -1)
-			{
+			if (left != -1) {
 				wxTimeSpan timeLeft(0, 0, left);
 				DrawRightAlignedText(*m_mdc, timeLeft.Format(_("%H:%M:%S left")), m_fieldOffsets[1], h);
 			}
 			else
 				DrawRightAlignedText(*m_mdc, _("--:--:-- left"), m_fieldOffsets[1], h);
 		}
-		if (refresh & 8)
-		{
+		if (refresh & 8) {
 			m_mdc->DrawRectangle(m_fieldOffsets[3], 0, rect.GetWidth() - m_fieldOffsets[3], rect.GetHeight());
 			m_mdc->DrawText(bytes_and_rate, m_fieldOffsets[3], h);
 		}
-		if (refresh & 16)
-		{
+		if (refresh & 16) {
 			m_mdc->DrawRectangle(m_fieldOffsets[1], 0, m_fieldOffsets[2] - m_fieldOffsets[1], rect.GetHeight());
 		}
-		if (refresh & 4)
-		{
+		if (refresh & 4) {
 			m_mdc->DrawRectangle(m_fieldOffsets[2], 0, m_fieldOffsets[3] - m_fieldOffsets[2], rect.GetHeight());
 			if (bar_split != -1)
 				DrawProgressBar(*m_mdc, m_fieldOffsets[2], 1, rect.GetHeight() - 2, bar_split, permill);
 		}
 	}
-	dc.Blit(0, 0, rect.GetWidth(), rect.GetHeight(), m_mdc, 0, 0);
+	dc.Blit(0, 0, rect.GetWidth(), rect.GetHeight(), m_mdc.get(), 0, 0);
 }
 
-void CStatusLineCtrl::SetTransferStatus(const CTransferStatus* pStatus)
+void CStatusLineCtrl::ClearTransferStatus()
 {
-	if (!pStatus) {
-		if (status_valid_ && status_.totalSize >= 0)
-			m_pParent->UpdateItemSize(m_pEngineData->pItem, status_.totalSize);
-		status_valid_ = false;
+	if (!status_.empty() && status_.totalSize >= 0) {
+		m_pParent->UpdateItemSize(m_pEngineData->pItem, status_.totalSize);
+	}
+	status_.clear();
 
-		switch (m_pEngineData->state)
-		{
-		case t_EngineData::disconnect:
-			m_statusText = _("Disconnecting from previous server");
-			break;
-		case t_EngineData::cancel:
-			m_statusText = _("Waiting for transfer to be cancelled");
-			break;
-		case t_EngineData::connect:
-			m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastServer.FormatServer());
-			break;
-		default:
-			m_statusText = _("Transferring");
-			break;
-		}
+	switch (m_pEngineData->state)
+	{
+	case t_EngineData::disconnect:
+		m_statusText = _("Disconnecting from previous server");
+		break;
+	case t_EngineData::cancel:
+		m_statusText = _("Waiting for transfer to be cancelled");
+		break;
+	case t_EngineData::connect:
+		m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastServer.FormatServer());
+		break;
+	default:
+		m_statusText = _("Transferring");
+		break;
+	}
 
-		if (m_transferStatusTimer.IsRunning())
-			m_transferStatusTimer.Stop();
+	if (m_transferStatusTimer.IsRunning())
+		m_transferStatusTimer.Stop();
 
-		m_past_data_index = -1;
-		m_gcLastOffset = -1;
-		m_gcLastSpeed = -1;
+	m_past_data_count = 0;
+	m_gcLastOffset = -1;
+	m_gcLastSpeed = -1;
+	Refresh(false);
+}
+
+void CStatusLineCtrl::SetTransferStatus(CTransferStatus const& status)
+{
+	if (!status) {
+		ClearTransferStatus();
 	}
 	else {
-		status_valid_ = true;
-		status_ = *pStatus;
+		status_ = status;
 
-		m_lastOffset = pStatus->currentOffset;
+		m_lastOffset = status.currentOffset;
 
 		if (!m_transferStatusTimer.IsRunning())
 			m_transferStatusTimer.Start(100);
+		Refresh(false);
 	}
-	Refresh(false);
 }
 
 void CStatusLineCtrl::OnTimer(wxTimerEvent&)
 {
-	bool changed;
-	CTransferStatus status;
-
-	if (!m_pEngineData || !m_pEngineData->pEngine)
-	{
+	if (!m_pEngineData || !m_pEngineData->pEngine) {
 		m_transferStatusTimer.Stop();
 		return;
 	}
 
-	if (!m_pEngineData->pEngine->GetTransferStatus(status, changed))
-		SetTransferStatus(0);
-	else if (changed)
-	{
+	bool changed;
+	CTransferStatus status = m_pEngineData->pEngine->GetTransferStatus(changed);
+
+	if (status.empty())
+		ClearTransferStatus();
+	else if (changed) {
 		if (status.madeProgress && !status.list &&
 			m_pEngineData->pItem->GetType() == QueueItemType::File)
 		{
 			CFileItem* pItem = (CFileItem*)m_pEngineData->pItem;
 			pItem->set_made_progress(true);
 		}
-		SetTransferStatus(&status);
+		SetTransferStatus(status);
 	}
 	else
 		m_transferStatusTimer.Stop();
@@ -342,16 +321,14 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 	dc.SetBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 	dc.DrawRectangle(x + 1 + bar_split, y + 1, PROGRESSBAR_WIDTH - bar_split - 1, height - 2);
 
-	if (bar_split && height > 2)
-	{
+	if (bar_split && height > 2) {
 		// Draw pretty gradient
 
 		int greenmin = 128;
 		int greenmax = 255;
 		int colourCount = ((height + 1) / 2);
 
-		for (int i = 0; i < colourCount; i++)
-		{
+		for (int i = 0; i < colourCount; ++i) {
 			int curGreen = greenmax - ((greenmax - greenmin) * i / (colourCount - 1));
 			dc.SetPen(wxPen(wxColour(0, curGreen, 0)));
 			dc.DrawLine(x + 1, y + colourCount - i, x + 1 + bar_split, y + colourCount - i);
@@ -364,50 +341,52 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 	dc.DrawRectangle(x, y, PROGRESSBAR_WIDTH, height);
 
 	// Draw percentage-done text
-	wxString prefix;
-	if( permill > 1000)
-	{
-		prefix = _T("> ");
-		permill = 1000;
+	wxString text;
+	if (permill > 1000) {
+		text = _T("> 100.0%");
+	}
+	else {
+		text = wxString::Format(_T("%d.%d%%"), permill / 10, permill % 10);
 	}
-
-	wxString text = wxString::Format(_T("%s%d.%d%%"), prefix, permill / 10, permill % 10);
 
 	wxCoord w, h;
 	dc.GetTextExtent(text, &w, &h);
 	dc.DrawText(text, x + PROGRESSBAR_WIDTH / 2 - w / 2, y + height / 2 - h / 2);
 }
 
-wxFileOffset CStatusLineCtrl::GetSpeed(int elapsedSeconds)
+wxFileOffset CStatusLineCtrl::GetSpeed(int elapsed_milli_seconds)
 {
-	if (!status_valid_)
+	if (status_.empty())
 		return -1;
 
-	if (elapsedSeconds <= 0)
+	if (elapsed_milli_seconds <= 0) {
 		return -1;
+	}
 
-	if (m_past_data_index < 9) {
-		if (m_past_data_index == -1 || m_past_data[m_past_data_index].elapsed < elapsedSeconds) {
-			m_past_data_index++;
-			m_past_data[m_past_data_index].elapsed = elapsedSeconds;
-			m_past_data[m_past_data_index].offset = status_.currentOffset - status_.startOffset;
-		}
+	int elapsed_seconds = elapsed_milli_seconds / 1000;
+	while (m_past_data_count < 10 && elapsed_seconds > m_past_data_count) {
+		m_past_data[m_past_data_count].elapsed = elapsed_milli_seconds;
+		m_past_data[m_past_data_count].offset = status_.currentOffset - status_.startOffset;
+		++m_past_data_count;
 	}
 
 	_past_data forget;
-	for (int i = m_past_data_index; i >= 0; i--) {
-		if (m_past_data[i].elapsed < elapsedSeconds) {
-			forget = m_past_data[i];
-			break;
-		}
+
+	int offset = (elapsed_seconds - 1) / 2;
+	if (offset > 0) {
+		forget = m_past_data[std::min(offset, m_past_data_count - 1)];
 	}
 
-	return (status_.currentOffset - status_.startOffset - forget.offset) / (elapsedSeconds - forget.elapsed);
+	if (elapsed_milli_seconds <= forget.elapsed) {
+		return -1;
+	}
+
+	return ((status_.currentOffset - status_.startOffset - forget.offset) * 1000) / (elapsed_milli_seconds - forget.elapsed);
 }
 
 wxFileOffset CStatusLineCtrl::GetCurrentSpeed()
 {
-	if (!status_valid_)
+	if (status_.empty())
 		return -1;
 
 	const wxTimeSpan timeDiff( wxDateTime::UNow().Subtract(m_gcLastTimeStamp) );
@@ -431,8 +410,7 @@ wxFileOffset CStatusLineCtrl::GetCurrentSpeed()
 
 bool CStatusLineCtrl::Show(bool show /*=true*/)
 {
-	if (show)
-	{
+	if (show) {
 		if (!m_transferStatusTimer.IsRunning())
 			m_transferStatusTimer.Start(100);
 	}
