@@ -81,14 +81,12 @@ wxBitmap CThemeProvider::CreateBitmap(const wxArtID& id, const wxArtClient& /*cl
 
 	std::list<wxString> dirs = GetSearchDirs(size);
 
-	wxString name = id.Mid(4);
-
 	// The ART_* IDs are always given in uppercase ASCII,
 	// all filenames used by FileZilla for the resources
 	// are lowercase ASCII. Locale-independent transformation
 	// needed e.g. if using Turkish locale.
-	MakeLowerAscii(name);
-
+	wxString name = fz::str_tolower_ascii(id.Mid(4));
+	
 	wxLogNull logNull;
 
 	for (auto const& dir : dirs) {
@@ -118,13 +116,11 @@ wxAnimation CThemeProvider::CreateAnimation(const wxArtID& id, const wxSize& siz
 
 	std::list<wxString> dirs = GetSearchDirs(size);
 
-	wxString name = id.Mid(4);
-
 	// The ART_* IDs are always given in uppercase ASCII,
 	// all filenames used by FileZilla for the resources
 	// are lowercase ASCII. Locale-independent transformation
 	// needed e.g. if using Turkish locale.
-	MakeLowerAscii(name);
+	wxString name = fz::str_tolower_ascii(id.Mid(4));
 
 	wxLogNull logNull;
 
@@ -161,12 +157,15 @@ std::vector<wxString> CThemeProvider::GetThemes()
 
 std::vector<std::unique_ptr<wxBitmap>> CThemeProvider::GetAllImages(const wxString& theme, const wxSize& size)
 {
-	wxString path = wxGetApp().GetResourceDir().GetPath() + theme;
+	wxString path = wxGetApp().GetResourceDir().GetPath() + theme + _T("/");
 
 	wxLogNull log;
 
 	wxString strSize = wxString::Format(_T("%dx%d/"), size.GetWidth(), size.GetHeight());
-	if (!wxDir::Exists(strSize)) {
+	if (wxDir::Exists(path + strSize)) {
+		path += strSize;
+	}
+	else {
 		if (size.GetWidth() > 32)
 			path += _T("48x48/");
 		else if (size.GetWidth() > 16)
@@ -202,8 +201,7 @@ bool CThemeProvider::GetThemeData(const wxString& themePath, wxString& name, wxS
 {
 	wxString const file(wxGetApp().GetResourceDir().GetPath() + themePath + _T("theme.xml"));
 	CXmlFile xml(file);
-	TiXmlElement* root = xml.Load();
-	TiXmlElement* theme = root ? root->FirstChildElement("Theme") : 0;
+	auto theme = xml.Load().child("Theme");
 	if (!theme)
 		return false;
 
@@ -219,35 +217,28 @@ std::vector<wxString> CThemeProvider::GetThemeSizes(const wxString& themePath)
 
 	wxString const file(wxGetApp().GetResourceDir().GetPath() + themePath + _T("theme.xml"));
 	CXmlFile xml(file);
-	TiXmlElement* root = xml.Load();
-	TiXmlElement* theme = root ? root->FirstChildElement("Theme") : 0;
+	auto theme = xml.Load().child("Theme");
 	if (!theme)
 		return sizes;
 
-	for (TiXmlElement* pSize = theme->FirstChildElement("size"); pSize; pSize = pSize->NextSiblingElement("size")) {
-		const char* txt = pSize->GetText();
-		if (!txt)
-			continue;
-
-		wxString size = ConvLocal(txt);
+	for (auto xSize = theme.child("size"); xSize; xSize = xSize.next_sibling("size")) {
+		wxString size = GetTextElement(xSize);
 		if (size.empty())
 			continue;
-
 		sizes.push_back(size);
 	}
-	
+
 	return sizes;
 }
 
-wxIconBundle CThemeProvider::GetIconBundle(const wxArtID& id, const wxArtClient& client /*=wxART_OTHER*/)
+wxIconBundle CThemeProvider::GetIconBundle(const wxArtID& id, const wxArtClient&)
 {
 	wxIconBundle iconBundle;
 
 	if (id.Left(4) != _T("ART_"))
 		return iconBundle;
 
-	wxString name = id.Mid(4);
-	MakeLowerAscii(name);
+	wxString name = fz::str_tolower_ascii(id.Mid(4));
 
 	const wxChar* dirs[] = { _T("16x16/"), _T("32x32/"), _T("48x48/") };
 
@@ -268,18 +259,14 @@ bool CThemeProvider::ThemeHasSize(const wxString& themePath, const wxString& siz
 {
 	wxString const file(wxGetApp().GetResourceDir().GetPath() + themePath + _T("theme.xml"));
 	CXmlFile xml(file);
-	TiXmlElement* root = xml.Load();
-	TiXmlElement* theme = root ? root->FirstChildElement("Theme") : 0;
+	auto theme = xml.Load().child("Theme");
 	if (!theme) {
 		return false;
 	}
 
-	for (TiXmlElement* pSize = theme->FirstChildElement("size"); pSize; pSize = pSize->NextSiblingElement("size")) {
-		const char* txt = pSize->GetText();
-		if (!txt)
-			continue;
-
-		if (size == ConvLocal(txt)) {
+	for (auto xSize = theme.child("size"); xSize; xSize = xSize.next_sibling("size")) {
+		wxString s = GetTextElement(xSize);
+		if (size == s) {
 			return true;
 		}
 	}
@@ -302,10 +289,8 @@ wxString CThemeProvider::GetThemePath()
 	return resourceDir.GetPath();
 }
 
-void CThemeProvider::OnOptionChanged(int option)
+void CThemeProvider::OnOptionsChanged(changed_options_t const&)
 {
-	wxASSERT(option == OPTION_THEME);
-
 	m_themePath = GetThemePath();
 
 	wxArtProvider::Remove(this);
@@ -320,12 +305,21 @@ wxSize CThemeProvider::GetIconSize(enum iconSize size)
 		if (s <= 0)
 			s = 16;
 	}
-	else if (size == iconSizeLarge) {
+	else if (size == iconSize24) {
 		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X);
+		if (s <= 0) {
+			s = 24;
+		}
+		else {
+			s += s/2;
+		}
+	}
+	else if (size == iconSizeLarge) {
+		s = wxSystemSettings::GetMetric(wxSYS_ICON_X);
 		if (s <= 0)
 			s = 48;
 		else
-			s *= 3;
+			s += s/2;
 	}
 	else {
 		s = wxSystemSettings::GetMetric(wxSYS_ICON_X);
@@ -335,3 +329,18 @@ wxSize CThemeProvider::GetIconSize(enum iconSize size)
 
 	return wxSize(s, s);
 }
+
+wxSize CThemeProvider::GetIconSize(wxString const& str)
+{
+	wxSize iconSize;
+	if (str == _T("24x24"))
+		return CThemeProvider::GetIconSize(iconSize24);
+	else if (str == _T("32x32"))
+		iconSize = CThemeProvider::GetIconSize(iconSizeNormal);
+	else if (str == _T("48x48"))
+		iconSize = CThemeProvider::GetIconSize(iconSizeLarge);
+	else
+		iconSize = CThemeProvider::GetIconSize(iconSizeSmall);
+
+	return iconSize;
+}
