@@ -176,7 +176,7 @@ void CUpdateDialog::UpdaterStateChanged( UpdaterState s, build const& v )
 		XRCCTRL(*this, "ID_VERSION", wxStaticText)->SetLabel(v.version_);
 
 		wxString news = updater_.GetChangelog();
-		
+
 		auto pos = news.find(v.version_ + _T(" (2"));
 		if (pos != wxString::npos) {
 			news = news.Mid(pos);
@@ -243,15 +243,15 @@ void CUpdateDialog::OnInstall(wxCommandEvent&)
 
 void CUpdateDialog::OnTimer(wxTimerEvent&)
 {
-	wxULongLong size = updater_.AvailableBuild().size_;
-	wxULongLong downloaded = updater_.BytesDownloaded();
+	int64_t size = updater_.AvailableBuild().size_;
+	int64_t downloaded = updater_.BytesDownloaded();
 
 	unsigned int percent = 0;
-	if( size > 0 ) {
-		percent = ((downloaded * 100) / size).GetLo();
+	if (size > 0 && downloaded >= 0) {
+		percent = static_cast<unsigned int>((downloaded * 100) / size);
 	}
 
-	XRCCTRL(*this, "ID_DOWNLOAD_PROGRESS", wxStaticText)->SetLabel(wxString::Format(_T("(%u%% downloaded)"), percent));
+	XRCCTRL(*this, "ID_DOWNLOAD_PROGRESS", wxStaticText)->SetLabel(wxString::Format(_("(%u%% downloaded)"), percent));
 }
 
 void CUpdateDialog::ShowDetails(wxHyperlinkEvent&)
@@ -269,7 +269,7 @@ void CUpdateDialog::Retry(wxHyperlinkEvent&)
 
 void CUpdateDialog::OnDebugLog(wxCommandEvent&)
 {
-	wxMessageBox(updater_.GetLog());
+	wxMessageBoxEx(updater_.GetLog());
 }
 
 #endif
