@@ -6,10 +6,12 @@
 #include "updater.h"
 #include "Options.h"
 #include "file_utils.h"
-#include <local_filesys.h>
+#include <wx/base64.h>
 #include <wx/tokenzr.h>
 #include <string>
 
+#include <libfilezilla/local_filesys.hpp>
+
 #include <string>
 
 // This is ugly but does the job
@@ -25,7 +27,8 @@ EVT_FZ_NOTIFICATION(wxID_ANY, CUpdater::OnEngineEvent)
 EVT_TIMER(wxID_ANY, CUpdater::OnTimer)
 END_EVENT_TABLE()
 
-static wxChar s_update_cert[] = _T("-----BEGIN CERTIFICATE-----\n\
+// BASE-64 encoded DER without the BEGIN/END CERTIFICATE
+static char s_update_cert[] = "\
 MIIFsTCCA5ugAwIBAgIESnXLbzALBgkqhkiG9w0BAQ0wSTELMAkGA1UEBhMCREUx\n\
 GjAYBgNVBAoTEUZpbGVaaWxsYSBQcm9qZWN0MR4wHAYDVQQDExVmaWxlemlsbGEt\n\
 cHJvamVjdC5vcmcwHhcNMDkwODAyMTcyMjU2WhcNMzEwNjI4MTcyMjU4WjBJMQsw\n\
@@ -57,8 +60,7 @@ qJU36tgxiO0XLRRSetl7jkSIO6U1okVH0/tvstrXEWp4XwdlmoZf92VVBrkg3San\n\
 fA5hBaI2gpQwtpyOJzwLzsd43n4b1YcPiyzhifJGcqRCBZA1uArNsH5iG6z/qHXp\n\
 KjuMxZu8aM8W2gp8Yg8QZfh5St/nut6hnXb5A8Qr+Ixp97t34t264TBRQD6MuZc3\n\
 PqQuF7sJR6POArUVYkRD/2LIWsB7\n\
------END CERTIFICATE-----\n\
-");
+";
 
 void version_information::update_available()
 {
@@ -76,50 +78,14 @@ void version_information::update_available()
 	}
 }
 
-
-class CUpdaterOptions : public COptionsBase
-{
-public:
-	CUpdaterOptions()
-		: m_use_internal_rootcert(true)
-	{
-	}
-
-	virtual int GetOptionVal(unsigned int nID)
-	{
-		return COptions::Get()->GetOptionVal(nID);
-	}
-
-	virtual wxString GetOption(unsigned int nID)
-	{
-		if (nID == OPTION_INTERNAL_ROOTCERT && m_use_internal_rootcert)
-			return s_update_cert;
-
-		return COptions::Get()->GetOption(nID);
-	}
-
-	virtual bool SetOption(unsigned int nID, int value)
-	{
-		return COptions::Get()->SetOption(nID, value);
-	}
-
-	virtual bool SetOption(unsigned int nID, wxString const& value)
-	{
-		return COptions::Get()->SetOption(nID, value);
-	}
-
-	bool m_use_internal_rootcert;
-};
-
 static CUpdater* instance = 0;
 
-CUpdater::CUpdater(CUpdateHandler& parent)
+CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context)
 	: state_(UpdaterState::idle)
-	, engine_(new CFileZillaEngine)
-	, update_options_(new CUpdaterOptions())
+	, engine_(new CFileZillaEngine(engine_context))
 {
 	AddHandler(parent);
-	engine_->Init(this, update_options_);
+	engine_->Init(this);
 }
 
 void CUpdater::Init()
@@ -150,7 +116,6 @@ CUpdater::~CUpdater()
 		instance  =0;
 	}
 	delete engine_;
-	delete update_options_;
 }
 
 CUpdater* CUpdater::GetInstance()
@@ -186,21 +151,20 @@ bool CUpdater::LongTimeSinceLastCheck() const
 	if (lastCheckStr.empty())
 		return true;
 
-	wxDateTime lastCheck;
-	lastCheck.ParseDateTime(lastCheckStr);
-	if (!lastCheck.IsValid())
+	fz::datetime lastCheck(lastCheckStr.ToStdWstring(), fz::datetime::utc);
+	if (!lastCheck.empty())
 		return true;
 
-	wxTimeSpan const span = wxDateTime::Now() - lastCheck;
+	auto const span = fz::datetime::now() - lastCheck;
 
-	if (span.GetSeconds() < 0)
+	if (span.get_seconds() < 0)
 		// Last check in future
 		return true;
 
 	int days = 1;
 	if (!CBuildInfo::IsUnstable())
 		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
-	return span.GetDays() >= days;
+	return span.get_days() >= days;
 }
 
 wxString CUpdater::GetUrl()
@@ -225,6 +189,12 @@ wxString CUpdater::GetUrl()
 	else
 		url += _T("&osarch=32");
 #endif
+
+	wxString const cpuCaps = CBuildInfo::GetCPUCaps(',');
+	if (!cpuCaps.empty()) {
+		url += _T("&cpuid=") + cpuCaps;
+	}
+
 	return url;
 }
 
@@ -236,11 +206,11 @@ bool CUpdater::Run()
 		return false;
 	}
 
-	wxDateTime const t = wxDateTime::Now();
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.Format(_T("%Y-%m-%d %H:%M:%S")));
+	auto  const t = fz::datetime::now();
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::utc));
 
 	local_file_.clear();
-	log_ = wxString::Format(_("Started update check on %s\n"), t.Format());
+	log_ = wxString::Format(_("Started update check on %s\n"), t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::local));
 
 	wxString build = CBuildInfo::GetBuildType();
 	if( build.empty() ) {
@@ -250,7 +220,7 @@ bool CUpdater::Run()
 
 	SetState(UpdaterState::checking);
 
-	update_options_->m_use_internal_rootcert = true;
+	m_use_internal_rootcert = true;
 	int res = Download(GetUrl(), wxString());
 
 	if (res != FZ_REPLY_WOULDBLOCK) {
@@ -263,28 +233,45 @@ bool CUpdater::Run()
 
 int CUpdater::Download(wxString const& url, wxString const& local_file)
 {
-	engine_->Execute(CDisconnectCommand());
-	int res = SendConnectCommand(url);
-	if( res == FZ_REPLY_OK ) {
-		res = SendTransferCommand(url, local_file);
+	wxASSERT(pending_commands_.empty());
+	pending_commands_.clear();
+	pending_commands_.emplace_back(new CDisconnectCommand);
+	if (!CreateConnectCommand(url) || !CreateTransferCommand(url, local_file)) {
+		return FZ_REPLY_ERROR;
+	}
+
+	return ContinueDownload();
+}
+
+int CUpdater::ContinueDownload()
+{
+	if (pending_commands_.empty()) {
+		return FZ_REPLY_OK;
+	}
+
+	int res = engine_->Execute(*pending_commands_.front());
+	if (res == FZ_REPLY_OK) {
+		pending_commands_.pop_front();
+		return ContinueDownload();
 	}
 
 	return res;
 }
 
-int CUpdater::SendConnectCommand(wxString const& url)
+bool CUpdater::CreateConnectCommand(wxString const& url)
 {
 	CServer s;
 	CServerPath path;
 	wxString error;
 	if( !s.ParseUrl( url, 0, wxString(), wxString(), error, path ) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS) ) {
-		return FZ_REPLY_ERROR;
+		return false;
 	}
 
-	return engine_->Execute(CConnectCommand(s));
+	pending_commands_.emplace_back(new CConnectCommand(s));
+	return true;
 }
 
-int CUpdater::SendTransferCommand(wxString const& url, wxString const& local_file)
+bool CUpdater::CreateTransferCommand(wxString const& url, wxString const& local_file)
 {
 	CFileTransferCommand::t_transferSettings transferSettings;
 
@@ -292,31 +279,27 @@ int CUpdater::SendTransferCommand(wxString const& url, wxString const& local_fil
 	CServerPath path;
 	wxString error;
 	if( !s.ParseUrl( url, 0, wxString(), wxString(), error, path ) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS) ) {
-		return FZ_REPLY_ERROR;
+		return false;
 	}
 	wxString file = path.GetLastSegment();
 	path = path.GetParent();
 
-	CFileTransferCommand cmd(local_file, path, file, true, transferSettings);
-	int res = engine_->Execute(cmd);
-
-	wxASSERT(res != FZ_REPLY_OK);
-	return res;
+	pending_commands_.emplace_back(new CFileTransferCommand(local_file, path, file, true, transferSettings));
+	return true;
 }
 
-void CUpdater::OnEngineEvent(wxEvent&)
+void CUpdater::OnEngineEvent(wxFzEvent& event)
 {
-	if (!engine_)
+	if (!engine_ || engine_ != event.engine_)
 		return;
 
-	CNotification *notification;
+	std::unique_ptr<CNotification> notification;
 	while( (notification = engine_->GetNextNotification()) ) {
-		ProcessNotification(notification);
-		delete notification;
+		ProcessNotification(std::move(notification));
 	}
 }
 
-void CUpdater::ProcessNotification(CNotification* notification)
+void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notification)
 {
 	if (state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading) {
 		return;
@@ -326,27 +309,43 @@ void CUpdater::ProcessNotification(CNotification* notification)
 	{
 	case nId_asyncrequest:
 		{
-			CAsyncRequestNotification* pData = reinterpret_cast<CAsyncRequestNotification *>(notification);
+			auto pData = unique_static_cast<CAsyncRequestNotification>(std::move(notification));
 			if (pData->GetRequestID() == reqId_fileexists) {
-				reinterpret_cast<CFileExistsNotification *>(pData)->overwriteAction = CFileExistsNotification::resume;
+				static_cast<CFileExistsNotification *>(pData.get())->overwriteAction = CFileExistsNotification::resume;
 			}
 			else if (pData->GetRequestID() == reqId_certificate) {
-				CCertificateNotification* pCertNotification = (CCertificateNotification*)pData;
-				pCertNotification->m_trusted = true;
+				auto & certNotification = static_cast<CCertificateNotification &>(*pData.get());
+				if (m_use_internal_rootcert) {
+					auto certs = certNotification.GetCertificates();
+					if( certs.size() > 1 ) {
+						auto ca = certs.back();
+
+						unsigned int ca_data_length{};
+						unsigned char const* ca_data = ca.GetRawData(ca_data_length);
+
+						wxMemoryBuffer updater_root = wxBase64Decode(s_update_cert, wxNO_LEN, wxBase64DecodeMode_SkipWS);
+						if( ca_data_length == updater_root.GetDataLen() && !memcmp(ca_data, updater_root.GetData(), ca_data_length) ) {
+							certNotification.m_trusted = true;
+						}
+					}
+				}
+				else {
+					certNotification.m_trusted = true;
+				}
 			}
-			engine_->SetAsyncRequestReply(pData);
+			engine_->SetAsyncRequestReply(std::move(pData));
 		}
 		break;
 	case nId_data:
-		ProcessData(notification);
+		ProcessData(static_cast<CDataNotification&>(*notification.get()));
 		break;
 	case nId_operation:
-		ProcessOperation(notification);
+		ProcessOperation(static_cast<COperationNotification const&>(*notification.get()));
 		break;
 	case nId_logmsg:
 		{
-			CLogmsgNotification* msg = reinterpret_cast<CLogmsgNotification *>(notification);
-			log_ += msg->msg + _T("\n");
+			auto const& msg = static_cast<CLogmsgNotification const&>(*notification.get());
+			log_ += msg.msg + _T("\n");
 		}
 		break;
 	default:
@@ -360,29 +359,29 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 	ParseData();
 
-	if( version_information_.available_.version_.empty() ) {
+	if (version_information_.available_.version_.empty()) {
 		s = UpdaterState::idle;
 	}
-	else if( !version_information_.available_.url_.empty() ) {
+	else if (!version_information_.available_.url_.empty()) {
 
 		wxString const temp = GetTempFile();
 		wxString const local_file = GetLocalFile(version_information_.available_, true);
-		if( !local_file.empty() && CLocalFileSystem::GetFileType(local_file) != CLocalFileSystem::unknown) {
+		if (!local_file.empty() && fz::local_filesys::get_file_type(fz::to_native(local_file)) != fz::local_filesys::unknown) {
 			local_file_ = local_file;
 			log_ += wxString::Format(_("Local file is %s\n"), local_file);
 			s = UpdaterState::newversion_ready;
 		}
 		else {
 			// We got a checksum over a secure channel already.
-			update_options_->m_use_internal_rootcert = false;
+			m_use_internal_rootcert = false;
 
-			if( temp.empty() || local_file.empty() ) {
+			if (temp.empty() || local_file.empty()) {
 				s = UpdaterState::newversion;
 			}
 			else {
 				s = UpdaterState::newversion_downloading;
-				wxLongLong size = CLocalFileSystem::GetSize(temp);
-				if( size >= 0 && static_cast<unsigned long long>(size.GetValue()) >= version_information_.available_.size_ ) {
+				auto size = fz::local_filesys::get_size(fz::to_native(temp));
+				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
 				else if( !can_download || Download( version_information_.available_.url_, GetTempFile() ) != FZ_REPLY_WOULDBLOCK ) {
@@ -397,17 +396,32 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 	return s;
 }
-void CUpdater::ProcessOperation(CNotification* notification)
+
+void CUpdater::ProcessOperation(COperationNotification const& operation)
 {
 	if( state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading ) {
 		return;
 	}
 
+	if (pending_commands_.empty()) {
+		SetState(UpdaterState::failed);
+		return;
+	}
+
+
 	UpdaterState s = UpdaterState::failed;
 
-	COperationNotification* operation = reinterpret_cast<COperationNotification*>(notification);
-	if (operation->nReplyCode != FZ_REPLY_OK) {
-		if( state_ != UpdaterState::checking ) {
+	int res = operation.nReplyCode;
+	if (res == FZ_REPLY_OK || (operation.commandId == Command::disconnect && res & FZ_REPLY_DISCONNECTED)) {
+		pending_commands_.pop_front();
+		res = ContinueDownload();
+		if (res == FZ_REPLY_WOULDBLOCK) {
+			return;
+		}
+	}
+
+	if (res != FZ_REPLY_OK) {
+		if (state_ != UpdaterState::checking) {
 			s = UpdaterState::newversion;
 		}
 	}
@@ -452,7 +466,7 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 	return s;
 }
 
-wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
+wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 {
 	wxString const fn = GetFilename( b.url_ );
 	wxString const dl = GetDownloadDir().GetPath();
@@ -460,20 +474,20 @@ wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
 	int i = 1;
 	wxString f = dl + fn;
 
-	while( CLocalFileSystem::GetFileType(f) != CLocalFileSystem::unknown && (!allow_existing || !VerifyChecksum(f, b.size_, b.hash_))) {
-		if( ++i > 99 ) {
+	while (fz::local_filesys::get_file_type(fz::to_native(f)) != fz::local_filesys::unknown && (!allow_existing || !VerifyChecksum(f, b.size_, b.hash_))) {
+		if (++i > 99) {
 			return wxString();
 		}
 		wxString ext;
 		int pos;
-		if( !fn.Right(8).CmpNoCase(_T(".tar.bz2")) ) {
+		if (!fn.Right(8).CmpNoCase(_T(".tar.bz2"))) {
 			pos = fn.size() - 8;
 		}
 		else {
 			pos = fn.Find('.', true);
 		}
 
-		if( pos == -1 ) {
+		if (pos == -1) {
 			f = dl + fn + wxString::Format(_T(" (%d)"), i);
 		}
 		else {
@@ -484,16 +498,14 @@ wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
 	return f;
 }
 
-void CUpdater::ProcessData(CNotification* notification)
+void CUpdater::ProcessData(CDataNotification& dataNotification)
 {
 	if( state_ != UpdaterState::checking ) {
 		return;
 	}
 
-	CDataNotification* pData = reinterpret_cast<CDataNotification*>(notification);
-
 	int len;
-	char* data = pData->Detach(len);
+	char* data = dataNotification.Detach(len);
 
 	if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
 		log_ += wxString::Format(_T("ProcessData %d\n"), len);
@@ -501,7 +513,7 @@ void CUpdater::ProcessData(CNotification* notification)
 
 	if( raw_version_information_.size() + len > 131072 ) {
 		log_ += _("Received version information is too large");
-		engine_->Execute(CCancelCommand());
+		engine_->Cancel();
 		SetState(UpdaterState::failed);
 	}
 	else {
@@ -509,7 +521,7 @@ void CUpdater::ProcessData(CNotification* notification)
 			if (data[i] < 10 || (unsigned char)data[i] > 127) {
 				log_ += _("Received invalid character in version information");
 				SetState(UpdaterState::failed);
-				engine_->Execute(CCancelCommand());
+				engine_->Cancel();
 				break;
 			}
 		}
@@ -523,7 +535,7 @@ void CUpdater::ProcessData(CNotification* notification)
 
 void CUpdater::ParseData()
 {
-	const wxLongLong ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str());
+	int64_t const ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str());
 	version_information_ = version_information();
 
 	wxString raw_version_information = raw_version_information_;
@@ -566,16 +578,16 @@ void CUpdater::ParseData()
 		wxString versionOrDate = tokens.GetNextToken();
 
 		if (type == _T("nightly")) {
-			wxDateTime nightlyDate;
-			if( !nightlyDate.ParseFormat(versionOrDate, _T("%Y-%m-%d")) ) {
-				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+			fz::datetime nightlyDate(versionOrDate.ToStdWstring(), fz::datetime::utc);
+			if (!nightlyDate.empty()) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += _T("Could not parse nightly date\n");
 				}
 				continue;
 			}
 
-			wxDateTime buildDate = CBuildInfo::GetBuildDate();
-			if (!buildDate.IsValid() || !nightlyDate.IsValid() || nightlyDate <= buildDate) {
+			fz::datetime buildDate = CBuildInfo::GetBuildDate();
+			if (!buildDate.empty() || !nightlyDate.empty() || nightlyDate <= buildDate) {
 				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
 					log_ += _T("Nightly isn't newer\n");
 				}
@@ -583,7 +595,7 @@ void CUpdater::ParseData()
 			}
 		}
 		else {
-			wxLongLong v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
+			int64_t v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
 			if (v <= ownVersionNumber)
 				continue;
 		}
@@ -647,14 +659,14 @@ void CUpdater::OnTimer(wxTimerEvent&)
 	AutoRunIfNeeded();
 }
 
-bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString const& checksum )
+bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum)
 {
-	if( file.empty() || checksum.empty() ) {
+	if (file.empty() || checksum.empty()) {
 		return false;
 	}
 
-	wxLongLong filesize = CLocalFileSystem::GetSize(file);
-	if( filesize < 0 || static_cast<unsigned long long>(filesize.GetValue()) != size ) {
+	auto filesize = fz::local_filesys::get_size(fz::to_native(file));
+	if (filesize < 0 || filesize != size) {
 		return false;
 	}
 
@@ -681,7 +693,7 @@ bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString
 	SHA512_Final(&state, raw_digest);
 
 	wxString digest;
-	for( unsigned int i = 0; i < sizeof(raw_digest); i++ ) {
+	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
 		unsigned char l = raw_digest[i] >> 4;
 		unsigned char r = raw_digest[i] & 0x0F;
 
@@ -709,8 +721,8 @@ wxString CUpdater::GetTempFile() const
 {
 	wxASSERT( !version_information_.available_.hash_.empty() );
 	wxString ret = wxFileName::GetTempDir();
-	if( !ret.empty() ) {
-		if( ret.Last() != wxFileName::GetPathSeparator() ) {
+	if (!ret.empty()) {
+		if (ret.Last() != wxFileName::GetPathSeparator()) {
 			ret += wxFileName::GetPathSeparator();
 		}
 
@@ -724,7 +736,7 @@ wxString CUpdater::GetFilename( wxString const& url) const
 {
 	wxString ret;
 	int pos = url.Find('/', true);
-	if( pos != -1 ) {
+	if (pos != -1) {
 		ret = url.Mid(pos + 1);
 	}
 	size_t p = ret.find_first_of(_T("?#"));
@@ -742,6 +754,10 @@ void CUpdater::SetState( UpdaterState s )
 {
 	if( s != state_ ) {
 		state_ = s;
+
+		if (s != UpdaterState::checking && s != UpdaterState::newversion_downloading) {
+			pending_commands_.clear();
+		}
 		build b = version_information_.available_;
 		for (auto const& handler : handlers_ ) {
 			if( handler ) {
@@ -787,24 +803,21 @@ void CUpdater::RemoveHandler( CUpdateHandler& handler )
 	}
 }
 
-wxULongLong CUpdater::BytesDownloaded() const
+int64_t CUpdater::BytesDownloaded() const
 {
-	wxLongLong ret;
-	if( state_ == UpdaterState::newversion_ready ) {
-		if( !local_file_.empty() ) {
-			ret = CLocalFileSystem::GetSize(local_file_);
+	int64_t ret{-1};
+	if (state_ == UpdaterState::newversion_ready) {
+		if (!local_file_.empty()) {
+			ret = fz::local_filesys::get_size(fz::to_native(local_file_));
 		}
 	}
 	else if( state_ == UpdaterState::newversion_downloading ) {
 		wxString const temp = GetTempFile();
 		if( !temp.empty() ) {
-			ret = CLocalFileSystem::GetSize(temp);
+			ret = fz::local_filesys::get_size(fz::to_native(temp));
 		}
 	}
-	if( ret < 0 ) {
-		ret = 0;
-	}
-	return static_cast<unsigned long long>(ret.GetValue());
+	return ret;
 }
 
 bool CUpdater::UpdatableBuild() const
