@@ -3,12 +3,14 @@
 
 #if FZ_MANUALUPDATECHECK
 
+#include <wx/timer.h>
+
 struct build
 {
 	wxString url_;
 	wxString version_;
 	wxString hash_;
-	wxULongLong size_;
+	int64_t size_{-1};
 };
 
 struct version_information
@@ -44,11 +46,11 @@ public:
 	virtual void UpdaterStateChanged( UpdaterState s, build const& v ) = 0;
 };
 
-class CUpdaterOptions;
-class CUpdater : wxEvtHandler
+class CFileZillaEngineContext;
+class CUpdater : public wxEvtHandler
 {
 public:
-	CUpdater( CUpdateHandler& parent );
+	CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context);
 	virtual ~CUpdater();
 
 	// 2-Stage initialization
@@ -63,7 +65,7 @@ public:
 
 	wxString DownloadedFile() const;
 
-	wxULongLong BytesDownloaded() const;
+	int64_t BytesDownloaded() const; // Returns -1 on error
 
 	wxString GetLog() const { return log_; }
 
@@ -77,37 +79,38 @@ public:
 
 protected:
 	int Download(wxString const& url, wxString const& local_file = _T(""));
+	int ContinueDownload();
 
 	void AutoRunIfNeeded();
 	bool Run();
 
-	int SendConnectCommand(wxString const& url);
-	int SendTransferCommand(wxString const& url, wxString const& local_file);
+	bool CreateConnectCommand(wxString const& url);
+	bool CreateTransferCommand(wxString const& url, wxString const& local_file);
 
 	wxString GetUrl();
-	void ProcessNotification(CNotification* notification);
-	void ProcessOperation(CNotification* notification);
-	void ProcessData(CNotification* notification);
+	void ProcessNotification(std::unique_ptr<CNotification> && notification);
+	void ProcessOperation(COperationNotification const& operation);
+	void ProcessData(CDataNotification& dataNotification);
 	void ParseData();
 	UpdaterState ProcessFinishedDownload();
 	UpdaterState ProcessFinishedData(bool can_download);
 
-	bool VerifyChecksum( wxString const& file, wxULongLong size, wxString const& checksum );
+	bool VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum);
 
 	wxString GetTempFile() const;
-	wxString GetFilename( wxString const& url) const;
-	wxString GetLocalFile( build const& b, bool allow_existing );
+	wxString GetFilename(wxString const& url) const;
+	wxString GetLocalFile(build const& b, bool allow_existing);
 
 	void SetState( UpdaterState s );
 
 	DECLARE_EVENT_TABLE()
-	void OnEngineEvent(wxEvent& ev);
+	void OnEngineEvent(wxFzEvent& event);
 	void OnTimer(wxTimerEvent& ev);
 
 	UpdaterState state_;
 	wxString local_file_;
 	CFileZillaEngine* engine_;
-	CUpdaterOptions* update_options_;
+	bool m_use_internal_rootcert{};
 
 	wxString raw_version_information_;
 
@@ -118,6 +121,8 @@ protected:
 	wxString log_;
 
 	wxTimer update_timer_;
+
+	std::deque<std::unique_ptr<CCommand>> pending_commands_;
 };
 
 #endif //FZ_MANUALUPDATECHECK
