@@ -1,11 +1,15 @@
 #include <filezilla.h>
 #include "filezillaapp.h"
 #include "verifycertdialog.h"
-#include <wx/statbox.h>
-#include <wx/tokenzr.h>
 #include "dialogex.h"
 #include "ipcmutex.h"
 #include "Options.h"
+#include "timeformatting.h"
+#include "xrc_helper.h"
+
+#include <wx/scrolwin.h>
+#include <wx/statbox.h>
+#include <wx/tokenzr.h>
 
 CVerifyCertDialog::CVerifyCertDialog()
 	: m_xmlFile(wxGetApp().GetSettingsFile(_T("trustedcerts")))
@@ -23,29 +27,30 @@ CVerifyCertDialog::~CVerifyCertDialog()
 bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 {
 	bool warning = false;
-	if (cert.GetActivationTime().IsValid()) {
-		if (cert.GetActivationTime() > wxDateTime::Now()) {
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), cert.GetActivationTime().FormatDate()));
+	if (cert.GetActivationTime().empty()) {
+		if (cert.GetActivationTime() > fz::datetime::now()) {
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), CTimeFormat::Format(cert.GetActivationTime())));
+			xrc_call(*pDlg, "ID_ACTIVATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
 			warning = true;
 		}
 		else
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), cert.GetActivationTime().FormatDate());
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), CTimeFormat::Format(cert.GetActivationTime()));
 	}
 	else {
 		warning = true;
 		pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
 	}
 
-	if (cert.GetExpirationTime().IsValid()) {
-		if (cert.GetExpirationTime() < wxDateTime::Now()) {
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), cert.GetExpirationTime().FormatDate()));
+	if (cert.GetExpirationTime().empty()) {
+		if (cert.GetExpirationTime() < fz::datetime::now()) {
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), CTimeFormat::Format(cert.GetExpirationTime())));
+			xrc_call(*pDlg, "ID_EXPIRATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
 			warning = true;
 		}
 		else
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), cert.GetExpirationTime().FormatDate());
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), CTimeFormat::Format(cert.GetExpirationTime()));
 	}
-	else
-	{
+	else {
 		warning = true;
 		pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
 	}
@@ -58,43 +63,98 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	pDlg->SetChildLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), cert.GetPkAlgoName(), cert.GetPkAlgoBits()));
 	pDlg->SetChildLabel(XRCID("ID_SIGNALGO"), cert.GetSignatureAlgorithm());
 
-	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_MD5"), cert.GetFingerPrintMD5());
+	wxString const& sha256 = cert.GetFingerPrintSHA256();
+	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA256"), sha256.Left(sha256.size() / 2 + 1) + _T("\n") + sha256.Mid(sha256.size() / 2 + 1));
 	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
 
-	ParseDN(XRCCTRL(*pDlg, "ID_SUBJECT_BOX", wxStaticBox), cert.GetSubject(), m_pSubjectSizer);
 	ParseDN(XRCCTRL(*pDlg, "ID_ISSUER_BOX", wxStaticBox), cert.GetIssuer(), m_pIssuerSizer);
 
+	auto subjectPanel = XRCCTRL(*pDlg, "ID_SUBJECT_PANEL", wxScrolledWindow);
+	subjectPanel->Freeze();
+
+	ParseDN(subjectPanel, cert.GetSubject(), m_pSubjectSizer);
+
+	auto const& altNames = cert.GetAltSubjectNames();
+	if (!altNames.empty()) {
+		wxString str;
+		for (auto const& altName : altNames) {
+			str += altName + _T("\n");
+		}
+		str.RemoveLast();
+		m_pSubjectSizer->Add(new wxStaticText(subjectPanel, wxID_ANY, wxPLURAL("Alternative name:", "Alternative names:", altNames.size())));
+		m_pSubjectSizer->Add(new wxStaticText(subjectPanel, wxID_ANY, str));
+	}
+	m_pSubjectSizer->Fit(subjectPanel);
+
+	wxSize min = m_pSubjectSizer->CalcMin();
+	int const maxHeight = (line_height_ + m_pDlg->ConvertDialogToPixels(wxPoint(0, 1)).y) * 15;
+	if (min.y >= maxHeight) {
+		min.y = maxHeight;
+		min.x += wxSystemSettings::GetMetric(wxSYS_VSCROLL_X);
+	}
+
+	// Add extra safety margin to prevent squishing on OS X.
+	min.x += 2;
+
+	subjectPanel->SetMinSize(min);
+	subjectPanel->Thaw();
+
 	return warning;
 }
 
-void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotification, bool displayOnly /*=false*/)
+#include <wx/scrolwin.h>
+
+bool CVerifyCertDialog::DisplayAlgorithm(int controlId, wxString name, bool insecure)
+{
+	if (insecure) {
+		name += _T(" - ");
+		name += _("Insecure algorithm!");
+
+		auto wnd = m_pDlg->FindWindow(controlId);
+		if (wnd) {
+			wnd->SetForegroundColour(wxColour(255, 0, 0));
+		}
+	}
+
+	m_pDlg->SetChildLabel(controlId, name);
+
+	return insecure;
+}
+
+void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly /*=false*/)
 {
 	LoadTrustedCerts();
 
 	m_pDlg = new wxDialogEx;
-	if (displayOnly)
-		m_pDlg->Load(0, _T("ID_DISPLAYCERT"));
-	else
-	{
-		m_pDlg->Load(0, _T("ID_VERIFYCERT"));
+	if (!m_pDlg->Load(0, _T("ID_VERIFYCERT"))) {
+		wxBell();
+		delete m_pDlg;
+		m_pDlg = 0;
+		return;
+	}
 
+	if (displayOnly) {
+		xrc_call(*m_pDlg, "ID_DESC", &wxWindow::Hide);
+		xrc_call(*m_pDlg, "ID_ALWAYS_DESC", &wxWindow::Hide);
+		xrc_call(*m_pDlg, "ID_ALWAYS", &wxWindow::Hide);
+		xrc_call(*m_pDlg, "wxID_CANCEL", &wxWindow::Hide);
+		m_pDlg->SetTitle(_T("Certificate details"));
+	}
+	else {
 		m_pDlg->WrapText(m_pDlg, XRCID("ID_DESC"), 400);
 
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Hide();
 	}
 
-	m_certificates = pNotification->GetCertificates();
-	if (m_certificates.size() == 1)
-	{
+	m_certificates = notification.GetCertificates();
+	if (m_certificates.size() == 1) {
 		XRCCTRL(*m_pDlg, "ID_CHAIN_DESC", wxStaticText)->Hide();
 		XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice)->Hide();
 	}
-	else
-	{
+	else {
 		wxChoice* pChoice = XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice);
-		for (unsigned int i = 0; i < m_certificates.size(); i++)
-		{
+		for (unsigned int i = 0; i < m_certificates.size(); ++i) {
 			pChoice->Append(wxString::Format(_T("%d"), i));
 		}
 		pChoice->SetSelection(0);
@@ -102,7 +162,9 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 		pChoice->Connect(wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(CVerifyCertDialog::OnCertificateChoice), 0, this);
 	}
 
-	m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), pNotification->GetHost(), pNotification->GetPort()));
+	m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), notification.GetHost(), notification.GetPort()));
+
+	line_height_ = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetSize().y;
 
 	m_pSubjectSizer = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
 	m_pSubjectSizer->Clear(true);
@@ -111,8 +173,7 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 	m_pIssuerSizer->Clear(true);
 
 	wxSize minSize(0, 0);
-	for (unsigned int i = 0; i < m_certificates.size(); ++i)
-	{
+	for (unsigned int i = 0; i < m_certificates.size(); ++i) {
 		DisplayCert(m_pDlg, m_certificates[i]);
 		m_pDlg->Layout();
 		m_pDlg->GetSizer()->Fit(m_pDlg);
@@ -122,16 +183,18 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 
 	bool warning = DisplayCert(m_pDlg, m_certificates[0]);
 
-	m_pDlg->SetChildLabel(XRCID("ID_PROTOCOL"), pNotification->GetProtocol());
-	m_pDlg->SetChildLabel(XRCID("ID_KEYEXCHANGE"), pNotification->GetKeyExchange());
-	m_pDlg->SetChildLabel(XRCID("ID_CIPHER"), pNotification->GetSessionCipher());
-	m_pDlg->SetChildLabel(XRCID("ID_MAC"), pNotification->GetSessionMac());
+	DisplayAlgorithm(XRCID("ID_PROTOCOL"), notification.GetProtocol(), (notification.GetAlgorithmWarnings() & CCertificateNotification::tlsver) != 0);
+	DisplayAlgorithm(XRCID("ID_KEYEXCHANGE"), notification.GetKeyExchange(), (notification.GetAlgorithmWarnings() & CCertificateNotification::kex) != 0);
+	DisplayAlgorithm(XRCID("ID_CIPHER"), notification.GetSessionCipher(), (notification.GetAlgorithmWarnings() & CCertificateNotification::cipher) != 0);
+	DisplayAlgorithm(XRCID("ID_MAC"), notification.GetSessionMac(), (notification.GetAlgorithmWarnings() & CCertificateNotification::mac) != 0);
 
-	if (warning)
-	{
+	if (notification.GetAlgorithmWarnings() != 0) {
+		warning = true;
+	}
+
+	if (warning) {
 		XRCCTRL(*m_pDlg, "ID_IMAGE", wxStaticBitmap)->SetBitmap(wxArtProvider::GetBitmap(wxART_WARNING));
-		if (!displayOnly)
-			XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
+		XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
 	}
 
 	m_pDlg->GetSizer()->Fit(m_pDlg);
@@ -139,29 +202,28 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 
 	int res = m_pDlg->ShowModal();
 
-	if (!displayOnly)
-	{
-		if (res == wxID_OK)
-		{
-			wxASSERT(!IsTrusted(pNotification));
+	if (!displayOnly) {
+		if (res == wxID_OK) {
+			wxASSERT(!IsTrusted(notification));
 
-			pNotification->m_trusted = true;
+			notification.m_trusted = true;
 
-			if (!warning && XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->GetValue())
-				SetPermanentlyTrusted(pNotification);
-			else
-			{
-				t_certData cert;
-				cert.host = pNotification->GetHost();
-				cert.port = pNotification->GetPort();
-				const unsigned char* data = m_certificates[0].GetRawData(cert.len);
-				cert.data = new unsigned char[cert.len];
-				memcpy(cert.data, data, cert.len);
-				m_sessionTrustedCerts.push_back(cert);
+			if (!notification.GetAlgorithmWarnings()) {
+				if (!warning && XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->GetValue())
+					SetPermanentlyTrusted(notification);
+				else {
+					t_certData cert;
+					cert.host = notification.GetHost();
+					cert.port = notification.GetPort();
+					const unsigned char* data = m_certificates[0].GetRawData(cert.len);
+					cert.data = new unsigned char[cert.len];
+					memcpy(cert.data, data, cert.len);
+					m_sessionTrustedCerts.push_back(cert);
+				}
 			}
 		}
 		else
-			pNotification->m_trusted = false;
+			notification.m_trusted = false;
 	}
 
 	delete m_pDlg;
@@ -258,17 +320,20 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxWindow* parent, std::list<wxString>&
 	}
 }
 
-bool CVerifyCertDialog::IsTrusted(CCertificateNotification* pNotification)
+bool CVerifyCertDialog::IsTrusted(CCertificateNotification const& notification)
 {
+	if (notification.GetAlgorithmWarnings() != 0) {
+		// These certs are never trusted.
+		return false;
+	}
+
 	LoadTrustedCerts();
 
-	wxASSERT(pNotification);
-
 	unsigned int len;
-	CCertificate cert =  pNotification->GetCertificates()[0];
+	CCertificate cert =  notification.GetCertificates()[0];
 	const unsigned char* data = cert.GetRawData(len);
 
-	return IsTrusted(pNotification->GetHost(), pNotification->GetPort(), data, len, false);
+	return IsTrusted(notification.GetHost(), notification.GetPort(), data, len, false);
 }
 
 bool CVerifyCertDialog::DoIsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, std::list<CVerifyCertDialog::t_certData> const& trustedCerts)
@@ -376,55 +441,54 @@ void CVerifyCertDialog::LoadTrustedCerts()
 		return;
 	}
 
-	TiXmlElement* pElement = m_xmlFile.Load();
-	if (!pElement) {
+	auto element = m_xmlFile.Load();
+	if (!element) {
 		return;
 	}
 
 	m_trustedCerts.clear();
 
-	if (!(pElement = pElement->FirstChildElement("TrustedCerts")))
+	if (!(element = element.child("TrustedCerts")))
 		return;
 
 	bool modified = false;
 
-	TiXmlElement* pCert = pElement->FirstChildElement("Certificate");
-	while (pCert) {
-		wxString value = GetTextElement(pCert, "Data");
+	auto cert = element.child("Certificate");
+	while (cert) {
+		wxString value = GetTextElement(cert, "Data");
 
-		TiXmlElement* pRemove = 0;
+		pugi::xml_node remove;
 
 		t_certData data;
 		if (value.empty() || !(data.data = ConvertStringToHex(value, data.len)))
-			pRemove = pCert;
+			remove = cert;
 
-		data.host = GetTextElement(pCert, "Host");
-		data.port = GetTextElementInt(pCert, "Port");
+		data.host = GetTextElement(cert, "Host");
+		data.port = GetTextElementInt(cert, "Port");
 		if (data.host.empty() || data.port < 1 || data.port > 65535)
-			pRemove = pCert;
+			remove = cert;
 
-		wxLongLong activationTime = GetTextElementLongLong(pCert, "ActivationTime", 0);
+		int64_t activationTime = GetTextElementInt(cert, "ActivationTime", 0);
 		if (activationTime == 0 || activationTime > wxDateTime::GetTimeNow())
-			pRemove = pCert;
+			remove = cert;
 
-		wxLongLong expirationTime = GetTextElementLongLong(pCert, "ExpirationTime", 0);
+		int64_t expirationTime = GetTextElementInt(cert, "ExpirationTime", 0);
 		if (expirationTime == 0 || expirationTime < wxDateTime::GetTimeNow())
-			pRemove = pCert;
+			remove = cert;
 
 		if (IsTrusted(data.host, data.port, data.data, data.len, true)) // Weed out duplicates
-			pRemove = pCert;
+			remove = cert;
 
-		if (!pRemove)
+		if (!remove)
 			m_trustedCerts.push_back(data);
 		else
 			delete [] data.data;
 
-		pCert = pCert->NextSiblingElement("Certificate");
+		cert = cert.next_sibling("Certificate");
 
-		if (pRemove)
-		{
+		if (remove) {
 			modified = true;
-			pElement->RemoveChild(pRemove);
+			element.remove_child(remove);
 		}
 	}
 
@@ -432,22 +496,22 @@ void CVerifyCertDialog::LoadTrustedCerts()
 		m_xmlFile.Save(false);
 }
 
-void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* const pNotification)
+void CVerifyCertDialog::SetPermanentlyTrusted(CCertificateNotification const& notification)
 {
-	const CCertificate certificate = pNotification->GetCertificates()[0];
+	const CCertificate certificate = notification.GetCertificates()[0];
 	unsigned int len;
 	const unsigned char* const data = certificate.GetRawData(len);
 
 	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
 	LoadTrustedCerts();
 
-	if (IsTrusted(pNotification->GetHost(), pNotification->GetPort(), data, len, true))	{
+	if (IsTrusted(notification.GetHost(), notification.GetPort(), data, len, true))	{
 		return;
 	}
 
 	t_certData cert;
-	cert.host = pNotification->GetHost();
-	cert.port = pNotification->GetPort();
+	cert.host = notification.GetHost();
+	cert.port = notification.GetPort();
 	cert.len = len;
 	cert.data = new unsigned char[len];
 	memcpy(cert.data, data, len);
@@ -457,27 +521,21 @@ void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* co
 		return;
 	}
 
-	TiXmlElement* pElement = m_xmlFile.GetElement();
-	if (!pElement) {
+	auto element = m_xmlFile.GetElement();
+	if (!element) {
 		return;
 	}
 
-	TiXmlElement* pCerts = pElement->FirstChildElement("TrustedCerts");
-	if (!pCerts)
-		pCerts = pElement->LinkEndChild(new TiXmlElement("TrustedCerts"))->ToElement();
+	auto certs = element.child("TrustedCerts");
+	if (!certs)
+		certs = element.append_child("TrustedCerts");
 
-	TiXmlElement* pCert = pCerts->LinkEndChild(new TiXmlElement("Certificate"))->ToElement();
-
-	AddTextElement(pCert, "Data", ConvertHexToString(data, len));
-
-	wxLongLong time = certificate.GetActivationTime().GetTicks();
-	AddTextElement(pCert, "ActivationTime", time.ToString());
-
-	time = certificate.GetExpirationTime().GetTicks();
-	AddTextElement(pCert, "ExpirationTime", time.ToString());
-
-	AddTextElement(pCert, "Host", pNotification->GetHost());
-	AddTextElement(pCert, "Port", pNotification->GetPort());
+	auto xCert = certs.append_child("Certificate");
+	AddTextElement(xCert, "Data", ConvertHexToString(data, len));
+	AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.GetActivationTime().get_time_t()));
+	AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.GetExpirationTime().get_time_t()));
+	AddTextElement(xCert, "Host", notification.GetHost());
+	AddTextElement(xCert, "Port", notification.GetPort());
 
 	m_xmlFile.Save(true);
 }
