@@ -4,16 +4,16 @@
 #include "xmlfunctions.h"
 
 class wxDialogEx;
-class CVerifyCertDialog : protected wxEvtHandler
+class CVerifyCertDialog final : protected wxEvtHandler
 {
 public:
 	CVerifyCertDialog();
 	virtual ~CVerifyCertDialog();
 
-	bool IsTrusted(CCertificateNotification* pNotification);
-	void ShowVerificationDialog(CCertificateNotification* pNotification, bool displayOnly = false);
+	bool IsTrusted(CCertificateNotification const& notification);
+	void ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly = false);
 
-protected:
+private:
 	struct t_certData {
 		wxString host;
 		int port{};
@@ -24,6 +24,8 @@ protected:
 	bool IsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, bool permanentOnly);
 	bool DoIsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, std::list<t_certData> const& trustedCerts);
 
+	bool DisplayAlgorithm(int controlId, wxString name, bool insecure);
+
 	bool DisplayCert(wxDialogEx* pDlg, const CCertificate& cert);
 
 	void ParseDN(wxWindow* parent, const wxString& dn, wxSizer* pSizer);
@@ -34,7 +36,7 @@ protected:
 	wxString ConvertHexToString(const unsigned char* data, unsigned int len);
 	unsigned char* ConvertStringToHex(const wxString& str, unsigned int &len);
 
-	void SetPermanentlyTrusted(const CCertificateNotification* const pNotification);
+	void SetPermanentlyTrusted(CCertificateNotification const& notification);
 
 	void LoadTrustedCerts();
 
@@ -47,6 +49,7 @@ protected:
 	wxDialogEx* m_pDlg{};
 	wxSizer* m_pSubjectSizer{};
 	wxSizer* m_pIssuerSizer{};
+	int line_height_{};
 
 	void OnCertificateChoice(wxCommandEvent& event);
 };
