@@ -1,3 +1,5 @@
+#define FZSFTP_PROTOCOL_VERSION 4
+
 typedef enum
 {
     sftpUnknown = -1,
@@ -6,19 +8,19 @@ typedef enum
     sftpError,
     sftpVerbose,
     sftpStatus,
-    sftpRecv,
-    sftpSend,
+    sftpRecv, /* socket */
+    sftpSend, /* socket */
     sftpClose,
     sftpRequest,
     sftpListentry,
-    sftpRead,
-    sftpWrite,
+    sftpTransfer, /* payload: when written to local file (download) or acknowledged by server (upload) */
     sftpRequestPreamble,
     sftpRequestInstruction,
     sftpUsedQuotaRecv,
     sftpUsedQuotaSend,
     sftpKexAlgorithm,
     sftpKexHash,
+	sftpKexCurve,
     sftpCipherClientToServer,
     sftpCipherServerToClient,
     sftpMacClientToServer,
