@@ -9,10 +9,6 @@
 
 #include "winstuff.h"
 
-#elif defined(macintosh)
-
-#include "macstuff.h"
-
 #elif defined(MACOSX)
 
 #include "osx.h"
