@@ -11,6 +11,7 @@
 #include "misc.h"
 #include "int64.h"
 #include "tree234.h"
+#include "ssh.h"
 #include "sftp.h"
 
 #include "fzprintf.h"
@@ -23,11 +24,10 @@ struct sftp_packet {
     int type;
 };
 
-static const char *fxp_error_message;
-static char fxp_error_message_long[200];
+static char *fxp_error_message = NULL;
 static int fxp_errtype;
 
-static void fxp_internal_error(char *msg);
+static void fxp_internal_error(const char *msg);
 
 /* ----------------------------------------------------------------------
  * SFTP packet construction functions.
@@ -39,7 +39,8 @@ static void sftp_pkt_ensure(struct sftp_packet *pkt, int length)
 	pkt->data = sresize(pkt->data, pkt->maxlen, char);
     }
 }
-static void sftp_pkt_adddata(struct sftp_packet *pkt, void *data, int len)
+static void sftp_pkt_adddata(struct sftp_packet *pkt,
+                             const void *data, int len)
 {
     pkt->length += len;
     sftp_pkt_ensure(pkt, pkt->length);
@@ -49,15 +50,22 @@ static void sftp_pkt_addbyte(struct sftp_packet *pkt, unsigned char byte)
 {
     sftp_pkt_adddata(pkt, &byte, 1);
 }
+static void sftp_pkt_adduint32(struct sftp_packet *pkt,
+			       unsigned long value)
+{
+    unsigned char x[4];
+    PUT_32BIT(x, value);
+    sftp_pkt_adddata(pkt, x, 4);
+}
 static struct sftp_packet *sftp_pkt_init(int pkt_type)
 {
     struct sftp_packet *pkt;
     pkt = snew(struct sftp_packet);
     pkt->data = NULL;
     pkt->savedpos = -1;
+    pkt->length = 0;
     pkt->maxlen = 0;
-    pkt->length = 4;
-    sftp_pkt_ensure(pkt, 4);
+    sftp_pkt_adduint32(pkt, 0); /* length field will be filled in later */
     sftp_pkt_addbyte(pkt, (unsigned char) pkt_type);
     return pkt;
 }
@@ -67,13 +75,6 @@ static void sftp_pkt_addbool(struct sftp_packet *pkt, unsigned char value)
     sftp_pkt_adddata(pkt, &value, 1);
 }
 */
-static void sftp_pkt_adduint32(struct sftp_packet *pkt,
-			       unsigned long value)
-{
-    unsigned char x[4];
-    PUT_32BIT(x, value);
-    sftp_pkt_adddata(pkt, x, 4);
-}
 static void sftp_pkt_adduint64(struct sftp_packet *pkt, uint64 value)
 {
     unsigned char x[8];
@@ -86,18 +87,18 @@ static void sftp_pkt_addstring_start(struct sftp_packet *pkt)
     sftp_pkt_adduint32(pkt, 0);
     pkt->savedpos = pkt->length;
 }
-static void sftp_pkt_addstring_str(struct sftp_packet *pkt, char *data)
+static void sftp_pkt_addstring_str(struct sftp_packet *pkt, const char *data)
 {
     sftp_pkt_adddata(pkt, data, strlen(data));
     PUT_32BIT(pkt->data + pkt->savedpos - 4, pkt->length - pkt->savedpos);
 }
 static void sftp_pkt_addstring_data(struct sftp_packet *pkt,
-				    char *data, int len)
+				    const char *data, int len)
 {
     sftp_pkt_adddata(pkt, data, len);
     PUT_32BIT(pkt->data + pkt->savedpos - 4, pkt->length - pkt->savedpos);
 }
-static void sftp_pkt_addstring(struct sftp_packet *pkt, char *data)
+static void sftp_pkt_addstring(struct sftp_packet *pkt, const char *data)
 {
     sftp_pkt_addstring_start(pkt);
     sftp_pkt_addstring_str(pkt, data);
@@ -116,28 +117,9 @@ static void sftp_pkt_addattrs(struct sftp_packet *pkt, struct fxp_attrs attrs)
     if (attrs.flags & SSH_FILEXFER_ATTR_PERMISSIONS) {
 	sftp_pkt_adduint32(pkt, attrs.permissions);
     }
-/*FZ: Does not work
-    if (attrs.flags & (SSH_FILEXFER_ATTR_CREATETIME | SSH_FILEXFER_ATTR_MODIFICATIONTIME)) {
-	if (attrs.flags & SSH_FILEXFER_ATTR_ACCESSTIME) {
-	    sftp_pkt_adduint32(pkt, attrs.atime.hi);
-	    sftp_pkt_adduint32(pkt, attrs.atime.lo);
-	}
-	if (attrs.flags & SSH_FILEXFER_ATTR_CREATETIME) {
-	    sftp_pkt_adduint32(pkt, attrs.ctime.hi);
-	    sftp_pkt_adduint32(pkt, attrs.ctime.lo);
-	}
-	if (attrs.flags & SSH_FILEXFER_ATTR_MODIFICATIONTIME) {
-	    sftp_pkt_adduint32(pkt, attrs.mtime.hi);
-	    sftp_pkt_adduint32(pkt, attrs.mtime.lo);
-	}
-    }
-    else
-*/
-    {
-	if (attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
-	    sftp_pkt_adduint32(pkt, attrs.atime.lo);
-	    sftp_pkt_adduint32(pkt, attrs.mtime.lo);
-	}
+    if (attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
+	sftp_pkt_adduint32(pkt, attrs.atime);
+	sftp_pkt_adduint32(pkt, attrs.mtime);
     }
     if (attrs.flags & SSH_FILEXFER_ATTR_EXTENDED) {
 	/*
@@ -173,7 +155,7 @@ static int sftp_pkt_getstring(struct sftp_packet *pkt,
     *p = NULL;
     if (pkt->length - pkt->savedpos < 4)
 	return 0;
-    *length = GET_32BIT(pkt->data + pkt->savedpos);
+    *length = toint(GET_32BIT(pkt->data + pkt->savedpos));
     pkt->savedpos += 4;
     if ((int)(pkt->length - pkt->savedpos) < *length || *length < 0) {
 	*length = 0;
@@ -203,38 +185,10 @@ static int sftp_pkt_getattrs(struct sftp_packet *pkt, struct fxp_attrs *ret)
 	if (!sftp_pkt_getuint32(pkt, &ret->permissions))
 	    return 0;
     }
-/*FZ: Does not work
-    if (ret->flags & (SSH_FILEXFER_ATTR_CREATETIME | SSH_FILEXFER_ATTR_MODIFICATIONTIME)) {
-	unsigned long hi, lo;
-	if (ret->flags & SSH_FILEXFER_ATTR_ACCESSTIME) {
-    	    if (!sftp_pkt_getuint32(pkt, &hi) ||
-		!sftp_pkt_getuint32(pkt, &lo))
-		return 0;
-	    ret->atime = uint64_make(hi, lo);
-	}
-	if (ret->flags & SSH_FILEXFER_ATTR_CREATETIME) {
-    	    if (!sftp_pkt_getuint32(pkt, &hi) ||
-		!sftp_pkt_getuint32(pkt, &lo))
-		return 0;
-	    ret->ctime = uint64_make(hi, lo);
-	}
-	if (ret->flags & SSH_FILEXFER_ATTR_MODIFICATIONTIME) {
-    	    if (!sftp_pkt_getuint32(pkt, &hi) ||
-		!sftp_pkt_getuint32(pkt, &lo))
-		return 0;
-	    ret->mtime = uint64_make(hi, lo);
-	}
-    }
-    else 
-*/
-    {
-	if (ret->flags & SSH_FILEXFER_ATTR_ACMODTIME) {
-    	    if (!sftp_pkt_getuint32(pkt, &ret->atime.lo) ||
-		!sftp_pkt_getuint32(pkt, &ret->mtime.lo))
-		return 0;
-	}
-	ret->atime.hi = ret->mtime.hi = 0;
-	ret->ctime.lo = ret->ctime.hi = 0;
+    if (ret->flags & SSH_FILEXFER_ATTR_ACMODTIME) {
+	if (!sftp_pkt_getuint32(pkt, &ret->atime) ||
+	    !sftp_pkt_getuint32(pkt, &ret->mtime))
+	    return 0;
     }
     if (ret->flags & SSH_FILEXFER_ATTR_EXTENDED) {
 	unsigned long count;
@@ -267,7 +221,7 @@ static void sftp_pkt_free(struct sftp_packet *pkt)
 int sftp_send(struct sftp_packet *pkt)
 {
     int ret;
-    PUT_32BIT(pkt->data, pkt->length-4);
+    PUT_32BIT(pkt->data, pkt->length - 4);
     ret = sftp_senddata(pkt->data, pkt->length);
     sftp_pkt_free(pkt);
     return ret;
@@ -409,11 +363,7 @@ struct sftp_request *sftp_find_request(struct sftp_packet *pktin)
     struct sftp_request *req;
 
     if (!pktin) {
-	char* tmp = dupcat("did not receive a valid SFTP packet: ", fxp_error_message, NULL);
-	strncpy(fxp_error_message_long, tmp, 200);
-	fxp_error_message_long[199] = 0;
-	sfree(tmp);
-	fxp_internal_error(fxp_error_message_long);
+	fxp_internal_error("did not receive a valid SFTP packet");
 	return NULL;
     }
 
@@ -425,7 +375,6 @@ struct sftp_request *sftp_find_request(struct sftp_packet *pktin)
 
     if (!req || !req->registered) {
 	fxp_internal_error("request ID mismatch");
-        sftp_pkt_free(pktin);
 	return NULL;
     }
 
@@ -472,21 +421,45 @@ static int fxp_got_status(struct sftp_packet *pktin)
 	"operation unsupported",
     };
 
+    sfree(fxp_error_message);
     if (pktin->type != SSH_FXP_STATUS) {
-	fxp_error_message = "expected FXP_STATUS packet";
+	fxp_error_message = dupprintf("expected FXP_STATUS packet, got packet type %d instead", pktin->type);
 	fxp_errtype = -1;
     } else {
 	unsigned long ul;
 	if (!sftp_pkt_getuint32(pktin, &ul)) {
-	    fxp_error_message = "malformed FXP_STATUS packet";
+	    fxp_error_message = dupstr("received malformed FXP_STATUS packet");
 	    fxp_errtype = -1;
 	} else {
+	    char *error_desc;
+	    int error_desc_len;
+
+	    if (sftp_pkt_getstring(pktin, &error_desc, &error_desc_len)) {
+		if (!error_desc || error_desc_len <= 0 || !*error_desc) {
+		    error_desc = 0;
+		}
+		else {
+		    error_desc = mkstr(error_desc, error_desc_len);
+		}
+	    }
+
 	    fxp_errtype = ul;
-	    if (fxp_errtype < 0 ||
-		fxp_errtype >= sizeof(messages) / sizeof(*messages))
-		fxp_error_message = "unknown error code";
-	    else
-		fxp_error_message = messages[fxp_errtype];
+
+	    if (error_desc && fxp_errtype >= SSH_FX_FAILURE) {
+		if (fxp_errtype < 0 ||
+		    fxp_errtype >= sizeof(messages) / sizeof(*messages))
+		    fxp_error_message = dupprintf("received unknown error code %d with description '%s'", fxp_errtype, error_desc);
+		else
+		    fxp_error_message = dupprintf("received %s with description '%s'", messages[fxp_errtype], error_desc);
+	    }
+	    else {
+		if (fxp_errtype < 0 ||
+		    fxp_errtype >= sizeof(messages) / sizeof(*messages))
+		    fxp_error_message = dupprintf("received unknown error code %d", fxp_errtype);
+		else
+		    fxp_error_message = dupstr(messages[fxp_errtype]);
+            }
+	    sfree(error_desc);
 	}
     }
 
@@ -498,9 +471,10 @@ static int fxp_got_status(struct sftp_packet *pktin)
 	return -1;
 }
 
-static void fxp_internal_error(char *msg)
+static void fxp_internal_error(const char *msg)
 {
-    fxp_error_message = msg;
+    sfree(fxp_error_message);
+    fxp_error_message = dupstr(msg);
     fxp_errtype = -1;
 }
 
@@ -561,7 +535,7 @@ int fxp_init(void)
 /*
  * Canonify a pathname.
  */
-struct sftp_request *fxp_realpath_send(char *path)
+struct sftp_request *fxp_realpath_send(const char *path)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -607,7 +581,8 @@ char *fxp_realpath_recv(struct sftp_packet *pktin, struct sftp_request *req)
 /*
  * Open a file.
  */
-struct sftp_request *fxp_open_send(char *path, int type)
+struct sftp_request *fxp_open_send(const char *path, int type,
+                                   struct fxp_attrs *attrs)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -616,7 +591,10 @@ struct sftp_request *fxp_open_send(char *path, int type)
     sftp_pkt_adduint32(pktout, req->id);
     sftp_pkt_addstring(pktout, path);
     sftp_pkt_adduint32(pktout, type);
-    sftp_pkt_adduint32(pktout, 0);     /* (FIXME) empty ATTRS structure */
+    if (attrs)
+        sftp_pkt_addattrs(pktout, *attrs);
+    else
+        sftp_pkt_adduint32(pktout, 0); /* empty ATTRS structure */
     sftp_send(pktout);
 
     return req;
@@ -652,7 +630,7 @@ struct fxp_handle *fxp_open_recv(struct sftp_packet *pktin,
 /*
  * Open a directory.
  */
-struct sftp_request *fxp_opendir_send(char *path)
+struct sftp_request *fxp_opendir_send(const char *path)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -718,7 +696,7 @@ void fxp_close_recv(struct sftp_packet *pktin, struct sftp_request *req)
     sftp_pkt_free(pktin);
 }
 
-struct sftp_request *fxp_mkdir_send(char *path)
+struct sftp_request *fxp_mkdir_send(const char *path)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -744,7 +722,7 @@ int fxp_mkdir_recv(struct sftp_packet *pktin, struct sftp_request *req)
     return 1;
 }
 
-struct sftp_request *fxp_rmdir_send(char *path)
+struct sftp_request *fxp_rmdir_send(const char *path)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -769,7 +747,7 @@ int fxp_rmdir_recv(struct sftp_packet *pktin, struct sftp_request *req)
     return 1;
 }
 
-struct sftp_request *fxp_remove_send(char *fname)
+struct sftp_request *fxp_remove_send(const char *fname)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -794,7 +772,8 @@ int fxp_remove_recv(struct sftp_packet *pktin, struct sftp_request *req)
     return 1;
 }
 
-struct sftp_request *fxp_rename_send(char *srcfname, char *dstfname)
+struct sftp_request *fxp_rename_send(const char *srcfname,
+                                     const char *dstfname)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -824,7 +803,7 @@ int fxp_rename_recv(struct sftp_packet *pktin, struct sftp_request *req)
  * Retrieve the attributes of a file. We have fxp_stat which works
  * on filenames, and fxp_fstat which works on open file handles.
  */
-struct sftp_request *fxp_stat_send(char *fname)
+struct sftp_request *fxp_stat_send(const char *fname)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -892,7 +871,8 @@ int fxp_fstat_recv(struct sftp_packet *pktin, struct sftp_request *req,
 /*
  * Set the attributes of a file.
  */
-struct sftp_request *fxp_setstat_send(char *fname, struct fxp_attrs attrs)
+struct sftp_request *fxp_setstat_send(const char *fname,
+                                      struct fxp_attrs attrs)
 {
     struct sftp_request *req = sftp_alloc_request();
     struct sftp_packet *pktout;
@@ -1190,9 +1170,9 @@ static struct fxp_xfer *xfer_init(struct fxp_handle *fh, uint64 offset)
     xfer->offset = offset;
     xfer->head = xfer->tail = NULL;
     xfer->req_totalsize = 0;
-    xfer->req_maxsize = 1048576;
+    xfer->req_maxsize = 1048576*4;
     xfer->err = 0;
-    xfer->filesize = uint64_make(0xffffffffU, 0xffffffffU);
+    xfer->filesize = uint64_make(ULONG_MAX, ULONG_MAX);
     xfer->furthestdata = uint64_make(0, 0);
     fz_timer_init(&xfer->send_timer);
     xfer->sent_interval = 0;
@@ -1256,15 +1236,23 @@ struct fxp_xfer *xfer_download_init(struct fxp_handle *fh, uint64 offset)
     return xfer;
 }
 
+/*
+ * Returns INT_MIN to indicate that it didn't even get as far as
+ * fxp_read_recv and hence has not freed pktin.
+ */
 int xfer_download_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
 {
     struct sftp_request *rreq;
     struct req *rr;
 
     rreq = sftp_find_request(pktin);
+    if (!rreq)
+        return INT_MIN;            /* this packet doesn't even make sense */
     rr = (struct req *)fxp_get_userdata(rreq);
-    if (!rr)
-	return 0;		       /* this packet isn't ours */
+    if (!rr) {
+        fxp_internal_error("request ID is not part of the current download");
+	return INT_MIN;		       /* this packet isn't ours */
+    }
     rr->retlen = fxp_read_recv(pktin, rreq, rr->buffer, rr->len);
 #ifdef DEBUG_DOWNLOAD
     printf("read request %p has returned [%d]\n", rr, rr->retlen);
@@ -1323,8 +1311,9 @@ int xfer_download_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
     }
 
     if (uint64_compare(xfer->furthestdata, xfer->filesize) > 0) {
-	fxp_error_message = "received a short buffer from FXP_READ, but not"
-	    " at EOF";
+	sfree(fxp_error_message);
+	fxp_error_message = dupstr("received a short buffer from FXP_READ, but not"
+	    " at EOF");
 	fxp_errtype = -1;
 	xfer_set_error(xfer);
 	return -1;
@@ -1435,6 +1424,10 @@ void xfer_upload_data(struct fxp_xfer *xfer, char *buffer, int len)
 #endif
 }
 
+/*
+ * Returns INT_MIN to indicate that it didn't even get as far as
+ * fxp_write_recv and hence has not freed pktin.
+ */
 int xfer_upload_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
 {
     struct sftp_request *rreq;
@@ -1442,9 +1435,13 @@ int xfer_upload_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
     int ret;
 
     rreq = sftp_find_request(pktin);
+    if (!rreq)
+        return INT_MIN;            /* this packet doesn't even make sense */
     rr = (struct req *)fxp_get_userdata(rreq);
-    if (!rr)
-	return 0;		       /* this packet isn't ours */
+    if (!rr) {
+        fxp_internal_error("request ID is not part of the current upload");
+	return INT_MIN;		       /* this packet isn't ours */
+    }
     ret = fxp_write_recv(pktin, rreq);
 #ifdef DEBUG_UPLOAD
     printf("write request %p has returned [%d]\n", rr, ret);
@@ -1467,7 +1464,7 @@ int xfer_upload_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
     xfer->sent_interval += rr->len;
     if (fz_timer_check(&xfer->send_timer)) {
 	/* The data we sent is the data we earlier read from file */
-	fzprintf(sftpRead, "%d", xfer->sent_interval);
+	fzprintf(sftpTransfer, "%d", xfer->sent_interval);
 	xfer->sent_interval = 0;
     }
     sfree(rr);
@@ -1480,6 +1477,9 @@ int xfer_upload_gotpkt(struct fxp_xfer *xfer, struct sftp_packet *pktin)
 
 void xfer_cleanup(struct fxp_xfer *xfer)
 {
+    if (xfer->sent_interval > 0) {
+	fzprintf(sftpTransfer, "%d", xfer->sent_interval);
+    }
     struct req *rr;
     while (xfer->head) {
 	rr = xfer->head;
@@ -1489,3 +1489,12 @@ void xfer_cleanup(struct fxp_xfer *xfer)
     }
     sfree(xfer);
 }
+
+int pending_receive()
+{
+    char tmp[5];
+    if (!ssh_socket) {
+	return -1;
+    }
+    return recv_peek(ssh_socket, tmp, 5);
+}
