@@ -11,7 +11,7 @@
 
 char *get_ttymode(void *frontend, const char *mode) { return NULL; }
 
-int get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
+int get_userpass_input(prompts_t *p, const unsigned char *in, int inlen)
 {
     int ret;
     ret = cmdline_get_passwd_input(p, in, inlen);
@@ -20,7 +20,7 @@ int get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
     return ret;
 }
 
-void platform_get_x11_auth(struct X11Display *display, const Config *cfg)
+void platform_get_x11_auth(struct X11Display *display, Conf *conf)
 {
     /* Do nothing, therefore no auth. */
 }
@@ -30,50 +30,6 @@ const int platform_uses_x11_unix_by_default = TRUE;
  * File access abstraction.
  */
 
-static wchar_t* utf8_to_wide(char* utf8)
-{
-    wchar_t *w;
-
-    int len = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, 0, 0);
-    if (len <= 0)
-	return NULL;
-
-    w = snewn(len, wchar_t);
-
-    if (!w)
-	return NULL;
-
-    if (MultiByteToWideChar(CP_UTF8, 0, utf8, -1, w, len) <= 0)
-    {
-	sfree(w);
-	return NULL;
-    }
-
-    return w;
-}
-
-static char* wide_to_utf8(wchar_t* w)
-{
-    char* utf8;
-
-    int len = WideCharToMultiByte(CP_UTF8, 0, w, -1, 0, 0, 0, 0);
-    if (len <= 0)
-	return NULL;
-
-    utf8 = snewn(len, char);
-
-    if (!utf8)
-	return NULL;
-
-    if (WideCharToMultiByte(CP_UTF8, 0, w, -1, utf8, len, 0, 0) <= 0)
-    {
-	sfree(utf8);
-	return NULL;
-    }
-    
-    return utf8;
-}
-
 /*
  * Set local current directory. Returns NULL on success, or else an
  * error message which must be freed after printing.
@@ -141,8 +97,9 @@ struct RFile {
     HANDLE h;
 };
 
-RFile *open_existing_file(char *name, uint64 *size,
-			  unsigned long *mtime, unsigned long *atime)
+RFile *open_existing_file(const char *name, uint64 *size,
+			  unsigned long *mtime, unsigned long *atime,
+                          long *perms)
 {
     HANDLE h;
     RFile *ret;
@@ -172,6 +129,9 @@ RFile *open_existing_file(char *name, uint64 *size,
 	    TIME_WIN_TO_POSIX(wrtime, *mtime);
     }
 
+    if (perms)
+        *perms = -1;
+
     return ret;
 }
 
@@ -196,7 +156,7 @@ struct WFile {
     HANDLE h;
 };
 
-WFile *open_new_file(char *name)
+WFile *open_new_file(const char *name, long perms)
 {
     HANDLE h;
     WFile *ret;
@@ -217,7 +177,7 @@ WFile *open_new_file(char *name)
     return ret;
 }
 
-WFile *open_existing_wfile(char *name, uint64 *size)
+WFile *open_existing_wfile(const char *name, uint64 *size)
 {
     HANDLE h;
     WFile *ret;
@@ -304,7 +264,7 @@ uint64 get_file_posn(WFile *f)
     return ret;
 }
 
-int file_type(char *name)
+int file_type(const char *name)
 {
     DWORD attr;
 
@@ -328,7 +288,7 @@ struct DirHandle {
     char *name;
 };
 
-DirHandle *open_directory(char *name)
+DirHandle *open_directory(const char *name)
 {
     HANDLE h;
     WIN32_FIND_DATAW fdat;
@@ -394,7 +354,7 @@ void close_directory(DirHandle *dir)
     sfree(dir);
 }
 
-int test_wildcard(char *name, int cmdline)
+int test_wildcard(const char *name, int cmdline)
 {
     HANDLE fh;
     WIN32_FIND_DATAW fdat;
@@ -430,14 +390,14 @@ struct WildcardMatcher {
     char *srcpath;
 };
 
-/*
- * Return a pointer to the portion of str that comes after the last
- * slash (or backslash or colon, if `local' is TRUE).
- */
-static char *stripslashes(char *str, int local)
+char *stripslashes(const char *str, int local)
 {
     char *p;
 
+    /*
+     * On Windows, \ / : are all path component separators.
+     */
+
     if (local) {
         p = strchr(str, ':');
         if (p) str = p+1;
@@ -451,10 +411,10 @@ static char *stripslashes(char *str, int local)
 	if (p) str = p+1;
     }
 
-    return str;
+    return (char *)str;
 }
 
-WildcardMatcher *begin_wildcard_matching(char *name)
+WildcardMatcher *begin_wildcard_matching(const char *name)
 {
     HANDLE h;
     WIN32_FIND_DATA fdat;
@@ -514,7 +474,7 @@ void finish_wildcard_matching(WildcardMatcher *dir)
     sfree(dir);
 }
 
-int vet_filename(char *name)
+int vet_filename(const char *name)
 {
     if (strchr(name, '/') || strchr(name, '\\') || strchr(name, ':'))
 	return FALSE;
@@ -525,7 +485,7 @@ int vet_filename(char *name)
     return TRUE;
 }
 
-int create_directory(char *name)
+int create_directory(const char *name)
 {
     int res;
 
@@ -539,7 +499,7 @@ int create_directory(char *name)
     return res;
 }
 
-char *dir_file_cat(char *dir, char *file)
+char *dir_file_cat(const char *dir, const char *file)
 {
     return dupcat(dir, "\\", file, NULL);
 }
@@ -585,17 +545,27 @@ extern int select_result(WPARAM, LPARAM);
 int do_eventsel_loop(HANDLE other_event)
 {
     int n, nhandles, nallhandles, netindex, otherindex;
-    long next, ticks;
+    unsigned long next, then;
+    long ticks;
     HANDLE *handles;
     SOCKET *sklist;
     int skcount;
-    long now = GETTICKCOUNT();
+    unsigned long now = GETTICKCOUNT();
 
-    if (run_timers(now, &next)) {
-	ticks = next - GETTICKCOUNT();
-	if (ticks < 0) ticks = 0;  /* just in case */
+    if (toplevel_callback_pending()) {
+        ticks = 0;
+        next = now;
+    } else if (run_timers(now, &next)) {
+	then = now;
+	now = GETTICKCOUNT();
+	if (now - then > next - then)
+	    ticks = 0;
+	else
+	    ticks = next - now;
     } else {
 	ticks = INFINITE;
+        /* no need to initialise next here because we can never get
+         * WAIT_TIMEOUT */
     }
 
     handles = handle_get_events(&nhandles);
@@ -679,6 +649,8 @@ int do_eventsel_loop(HANDLE other_event)
 
     sfree(handles);
 
+    run_toplevel_callbacks();
+
     if (n == WAIT_TIMEOUT) {
 	now = next;
     } else {
@@ -705,7 +677,7 @@ int ssh_sftp_loop_iteration(void)
     if (p_WSAEventSelect == NULL) {
 	fd_set readfds;
 	int ret;
-	long now = GETTICKCOUNT();
+	unsigned long now = GETTICKCOUNT(), then;
 
 	if (sftp_ssh_socket == INVALID_SOCKET)
 	    return -1;		       /* doom */
@@ -714,13 +686,17 @@ int ssh_sftp_loop_iteration(void)
 	    select_result((WPARAM) sftp_ssh_socket, (LPARAM) FD_WRITE);
 
 	do {
-	    long next, ticks;
+	    unsigned long next;
+	    long ticks;
 	    struct timeval tv, *ptv;
 
 	    if (run_timers(now, &next)) {
-		ticks = next - GETTICKCOUNT();
-		if (ticks <= 0)
-		    ticks = 1;	       /* just in case */
+		then = now;
+		now = GETTICKCOUNT();
+		if (now - then > next - then)
+		    ticks = 0;
+		else
+		    ticks = next - now;
 		tv.tv_sec = ticks / 1000;
 		tv.tv_usec = ticks % 1000 * 1000;
 		ptv = &tv;
@@ -774,11 +750,12 @@ static DWORD WINAPI command_read_thread(void *param)
     return 0;
 }
 
-char *ssh_sftp_get_cmdline(char *prompt, int no_fds_ok)
+char *ssh_sftp_get_cmdline(const char *prompt, int no_fds_ok)
 {
     int ret;
     struct command_read_ctx actx, *ctx = &actx;
     DWORD threadid;
+    HANDLE hThread;
 
     /* Not used in fzsftp
     fputs(prompt, stdout);
@@ -797,8 +774,9 @@ char *ssh_sftp_get_cmdline(char *prompt, int no_fds_ok)
     ctx->event = CreateEvent(NULL, FALSE, FALSE, NULL);
     ctx->line = NULL;
 
-    if (!CreateThread(NULL, 0, command_read_thread,
-		      ctx, 0, &threadid)) {
+    hThread = CreateThread(NULL, 0, command_read_thread, ctx, 0, &threadid);
+    if (!hThread) {
+	CloseHandle(ctx->event);
 	fzprintf(sftpError, "Unable to create command input thread");
 	cleanup_exit(1);
     }
@@ -810,6 +788,9 @@ char *ssh_sftp_get_cmdline(char *prompt, int no_fds_ok)
 	assert(ret >= 0);
     } while (ret == 0);
 
+    CloseHandle(hThread);
+    CloseHandle(ctx->event);
+
     return ctx->line;
 }
 
