@@ -1,4 +1,4 @@
-#include <libfilezilla.h>
+#include <libfilezilla_engine.h>
 #include <directorylistingparser.h>
 
 #include <cppunit/extensions/HelperMacros.h>
@@ -50,8 +50,8 @@ std::vector<t_entry> CDirectoryListingParserTest::m_entries;
 
 CPPUNIT_TEST_SUITE_REGISTRATION(CDirectoryListingParserTest);
 
-typedef CRefcountObject<wxString> R;
-typedef CSparseOptional<wxString> O;
+typedef CRefcountObject<std::wstring> R;
+typedef CSparseOptional<std::wstring> O;
 
 static int calcYear(int month, int day)
 {
@@ -83,7 +83,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 1994, 4, 8)
+				fz::datetime(fz::datetime::utc, 1994, 4, 8)
 			},
 			DEFAULT
 		});
@@ -98,7 +98,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(3, 29), 3, 29, 3, 26)
+				fz::datetime(fz::datetime::utc, calcYear(3, 29), 3, 29, 3, 26)
 			},
 			DEFAULT
 		});
@@ -113,7 +113,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 1994, 4, 8)
+				fz::datetime(fz::datetime::utc, 1994, 4, 8)
 			},
 			DEFAULT
 		});
@@ -128,7 +128,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				CDirentry::flag_dir | CDirentry::flag_link,
 				O(_T("usr/bin")),
-				CDateTime(CDateTime::utc, calcYear(1, 25), 1, 25, 0, 17)
+				fz::datetime(fz::datetime::utc, calcYear(1, 25), 1, 25, 0, 17)
 			},
 			DEFAULT
 		});
@@ -145,7 +145,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 9, 26)
+				fz::datetime(fz::datetime::utc, 2000, 9, 26)
 			},
 			DEFAULT
 		});
@@ -159,7 +159,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(9, 26), 9, 26, 13, 45)
+				fz::datetime(fz::datetime::utc, calcYear(9, 26), 9, 26, 13, 45)
 			},
 			DEFAULT
 		});
@@ -173,7 +173,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 6, 7, 21, 22)
+				fz::datetime(fz::datetime::utc, 2005, 6, 7, 21, 22)
 			},
 			DEFAULT
 		});
@@ -189,7 +189,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(10, 5), 10, 5, 21, 22)
+				fz::datetime(fz::datetime::utc, calcYear(10, 5), 10, 5, 21, 22)
 			},
 			DEFAULT
 		});
@@ -206,7 +206,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("supervisor")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(1, 16), 1, 16, 18, 53)
+				fz::datetime(fz::datetime::utc, calcYear(1, 16), 1, 16, 18, 53)
 			},
 			DEFAULT
 		});
@@ -220,7 +220,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("rhesus")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(10, 20), 10, 20, 15, 27)
+				fz::datetime(fz::datetime::utc, calcYear(10, 20), 10, 20, 15, 27)
 			},
 			DEFAULT
 		});
@@ -241,7 +241,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("1391972")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1995, 11, 22)
+				fz::datetime(fz::datetime::utc, 1995, 11, 22)
 			},
 			DEFAULT
 		});
@@ -255,7 +255,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("folder")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 1996, 5, 10)
+				fz::datetime(fz::datetime::utc, 1996, 5, 10)
 			},
 			DEFAULT
 		});
@@ -270,7 +270,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("group domain user")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(1, 29), 1, 29, 3, 26)
+				fz::datetime(fz::datetime::utc, calcYear(1, 29), 1, 29, 3, 26)
 			},
 			DEFAULT
 		});
@@ -289,7 +289,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1996, 3, 1, 22, 15, 3)
+				fz::datetime(fz::datetime::utc, 1996, 3, 1, 22, 15, 3)
 			},
 			DEFAULT
 		});
@@ -303,7 +303,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir | 0,
 				O(),
-				CDateTime(CDateTime::utc, 1996, 2, 13, 23, 58, 27)
+				fz::datetime(fz::datetime::utc, 1996, 2, 13, 23, 58, 27)
 			},
 			DEFAULT
 		});
@@ -320,7 +320,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 4, 27, 12, 9)
+				fz::datetime(fz::datetime::utc, 2000, 4, 27, 12, 9)
 			},
 			DEFAULT
 		});
@@ -335,7 +335,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 4, 6, 15, 47)
+				fz::datetime(fz::datetime::utc, 2000, 4, 6, 15, 47)
 			},
 			DEFAULT
 		});
@@ -349,7 +349,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2002, 9, 2, 18, 48)
+				fz::datetime(fz::datetime::utc, 2002, 9, 2, 18, 48)
 			},
 			DEFAULT
 		});
@@ -363,7 +363,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2002, 9, 2, 19, 6)
+				fz::datetime(fz::datetime::utc, 2002, 9, 2, 19, 6)
 			},
 			DEFAULT
 		});
@@ -378,7 +378,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("500 101")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1973, 11, 29, 21, 33, 9)
+				fz::datetime(fz::datetime::utc, 1973, 11, 29, 21, 33, 9)
 			},
 			DEFAULT
 		});
@@ -395,7 +395,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 4, 4, 21, 6)
+				fz::datetime(fz::datetime::utc, 2000, 4, 4, 21, 6)
 			},
 			DEFAULT
 		});
@@ -409,7 +409,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2002, 12, 12, 2, 13)
+				fz::datetime(fz::datetime::utc, 2002, 12, 12, 2, 13)
 			},
 			DEFAULT
 		});
@@ -425,7 +425,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("user group")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2002, 10, 8, 9, 47)
+				fz::datetime(fz::datetime::utc, 2002, 10, 8, 9, 47)
 			},
 			DEFAULT
 		});
@@ -443,7 +443,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 4, 23, 10, 57)
+				fz::datetime(fz::datetime::utc, 2003, 4, 23, 10, 57)
 			},
 			DEFAULT
 		});
@@ -457,7 +457,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1999, 7, 14, 12, 37)
+				fz::datetime(fz::datetime::utc, 1999, 7, 14, 12, 37)
 			},
 			DEFAULT
 		});
@@ -472,7 +472,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 2, 11, 16, 15)
+				fz::datetime(fz::datetime::utc, 2003, 2, 11, 16, 15)
 			},
 			DEFAULT
 		});
@@ -487,7 +487,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 10, 5, 23, 38)
+				fz::datetime(fz::datetime::utc, 2000, 10, 5, 23, 38)
 			},
 			DEFAULT
 		});
@@ -504,7 +504,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(7, 26), 7, 26, 20, 10)
+				fz::datetime(fz::datetime::utc, calcYear(7, 26), 7, 26, 20, 10)
 			},
 			DEFAULT
 		});
@@ -518,7 +518,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(9, 26), 9, 26, 20, 10)
+				fz::datetime(fz::datetime::utc, calcYear(9, 26), 9, 26, 20, 10)
 			},
 			DEFAULT
 		});
@@ -532,7 +532,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 10, 2)
+				fz::datetime(fz::datetime::utc, 2003, 10, 2)
 			},
 			DEFAULT
 		});
@@ -546,7 +546,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1999, 10, 12, 17, 12)
+				fz::datetime(fz::datetime::utc, 1999, 10, 12, 17, 12)
 			},
 			DEFAULT
 		});
@@ -560,7 +560,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root other")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 4, 24, 17, 12)
+				fz::datetime(fz::datetime::utc, 2003, 4, 24, 17, 12)
 			},
 			DEFAULT
 		});
@@ -577,7 +577,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root sys")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 4, 18)
+				fz::datetime(fz::datetime::utc, 2003, 4, 18)
 			},
 			DEFAULT
 		});
@@ -593,7 +593,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root staff")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 3, 20)
+				fz::datetime(fz::datetime::utc, 2003, 3, 20)
 			},
 			DEFAULT
 		});
@@ -607,7 +607,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root root")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(8, 17), 8, 17, 8, 52)
+				fz::datetime(fz::datetime::utc, calcYear(8, 17), 8, 17, 8, 52)
 			},
 			DEFAULT
 		});
@@ -621,7 +621,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root root")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2004, 7, 15)
+				fz::datetime(fz::datetime::utc, 2004, 7, 15)
 			},
 			DEFAULT
 		});
@@ -638,7 +638,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root,root")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2001, 11, 19, 21, 41)
+				fz::datetime(fz::datetime::utc, 2001, 11, 19, 21, 41)
 			},
 			DEFAULT
 		});
@@ -653,7 +653,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 7, 2, 10, 30, 13)
+				fz::datetime(fz::datetime::utc, 2003, 7, 2, 10, 30, 13)
 			},
 			DEFAULT
 		});
@@ -668,7 +668,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("IV2_XXX")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 1, 7)
+				fz::datetime(fz::datetime::utc, 2000, 1, 7)
 			},
 			DEFAULT
 		});
@@ -683,7 +683,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("PRONAS")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2002, 7, 15)
+				fz::datetime(fz::datetime::utc, 2002, 7, 15)
 			},
 			DEFAULT
 		});
@@ -697,7 +697,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("FTP_CLIENT,SCOT")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 4, 24, 8, 16, 15)
+				fz::datetime(fz::datetime::utc, 2003, 4, 24, 8, 16, 15)
 			},
 			DEFAULT
 		});
@@ -711,7 +711,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("FTP_CLIENT,SCOT")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 7, 2, 10, 30, 8)
+				fz::datetime(fz::datetime::utc, 2003, 7, 2, 10, 30, 8)
 			},
 			DEFAULT
 		});
@@ -726,7 +726,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("SUMMARY")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2006, 8, 2, 13, 5)
+				fz::datetime(fz::datetime::utc, 2006, 8, 2, 13, 5)
 			},
 			DEFAULT
 		});
@@ -740,7 +740,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1994, 6, 17, 17, 25, 37)
+				fz::datetime(fz::datetime::utc, 1994, 6, 17, 17, 25, 37)
 			},
 			DEFAULT
 		});
@@ -758,7 +758,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("QSYS")),
 				CDirentry::flag_dir | 0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 2, 23, 15, 9, 55)
+				fz::datetime(fz::datetime::utc, 2000, 2, 23, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -772,7 +772,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("QSYS")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2000, 2, 23, 15, 9, 55)
+				fz::datetime(fz::datetime::utc, 2000, 2, 23, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -787,7 +787,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("longowner longgroup")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(2, 12), 2, 12, 17, 20)
+				fz::datetime(fz::datetime::utc, calcYear(2, 12), 2, 12, 17, 20)
 			},
 			DEFAULT
 		});
@@ -802,7 +802,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("owner group")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 1999, 6, 1)
+				fz::datetime(fz::datetime::utc, 1999, 6, 1)
 			},
 			DEFAULT
 		});
@@ -817,7 +817,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2004, 6, 3, 10, 20, 3)
+				fz::datetime(fz::datetime::utc, 2004, 6, 3, 10, 20, 3)
 			},
 			DEFAULT
 		});
@@ -832,7 +832,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir | 0,
 				O(),
-				CDateTime(CDateTime::utc, 1998, 2, 28, 5, 23, 30)
+				fz::datetime(fz::datetime::utc, 1998, 2, 28, 5, 23, 30)
 			},
 			DEFAULT
 		});
@@ -847,7 +847,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("B BCC3I1 7670")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(1, 13), 1, 13, 7, 42)
+				fz::datetime(fz::datetime::utc, calcYear(1, 13), 1, 13, 7, 42)
 			},
 			DEFAULT
 		});
@@ -863,7 +863,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("20.20")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2007, 3, 29)
+				fz::datetime(fz::datetime::utc, 2007, 3, 29)
 			},
 			DEFAULT
 		});
@@ -878,7 +878,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("user group")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2004, 10, 1)
+				fz::datetime(fz::datetime::utc, 2004, 10, 1)
 			},
 			DEFAULT
 		});
@@ -892,7 +892,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 11, 11)
+				fz::datetime(fz::datetime::utc, 2005, 11, 11)
 			},
 			DEFAULT
 		});
@@ -907,7 +907,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("root sys")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 1, 6)
+				fz::datetime(fz::datetime::utc, 2005, 1, 6)
 			},
 			DEFAULT
 		});
@@ -922,7 +922,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("QPGMR")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2006, 9, 18, 14, 21, 26)
+				fz::datetime(fz::datetime::utc, 2006, 9, 18, 14, 21, 26)
 			},
 			DEFAULT
 		});
@@ -937,7 +937,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 9, 23, 14, 57, 7)
+				fz::datetime(fz::datetime::utc, 2005, 9, 23, 14, 57, 7)
 			},
 			DEFAULT
 		});
@@ -952,7 +952,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("255, 0")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2007, 4, 6, 14, 21, 18)
+				fz::datetime(fz::datetime::utc, 2007, 4, 6, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -967,7 +967,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("255,255")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2007, 4, 6, 14, 21, 18)
+				fz::datetime(fz::datetime::utc, 2007, 4, 6, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -982,7 +982,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("user sys")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, calcYear(1, 30), 1, 30, 12, 40)
+				fz::datetime(fz::datetime::utc, calcYear(1, 30), 1, 30, 12, 40)
 			},
 			DEFAULT
 		});
@@ -1005,7 +1005,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2003, 5, 21)
+				fz::datetime(fz::datetime::utc, 2003, 5, 21)
 		},
 		DEFAULT
 	});
@@ -1019,7 +1019,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2004, 3, 04)
+				fz::datetime(fz::datetime::utc, 2004, 3, 04)
 		},
 		DEFAULT
 	});
@@ -1033,7 +1033,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime()
+				fz::datetime()
 		},
 		DEFAULT
 	});
@@ -1047,7 +1047,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 6, 6)
+				fz::datetime(fz::datetime::utc, 2005, 6, 6)
 		},
 		DEFAULT
 	});
@@ -1061,7 +1061,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 		},
 		DEFAULT
 	});
@@ -1082,7 +1082,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 		},
 		MVS
 	});
@@ -1097,7 +1097,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2004, 6, 22, 16, 32)
+				fz::datetime(fz::datetime::utc, 2004, 6, 22, 16, 32)
 		},
 		MVS
 	});
@@ -1112,7 +1112,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 		},
 		MVS
 	});
@@ -1126,7 +1126,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			MVS
 		});
@@ -1141,7 +1141,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			MVS
 		});
@@ -1173,7 +1173,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("060191")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2005, 10, 4, 15, 28, 42)
+				fz::datetime(fz::datetime::utc, 2005, 10, 4, 15, 28, 42)
 			},
 			ZVM
 		});
@@ -1187,7 +1187,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("slopri devlab")),
 				CDirentry::flag_dir,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1201,7 +1201,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2008, 11, 5, 16, 52, 15)
+				fz::datetime(fz::datetime::utc, 2008, 11, 5, 16, 52, 15)
 			},
 			DEFAULT
 		});
@@ -1219,7 +1219,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			MVS
 		});
@@ -1233,7 +1233,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1247,7 +1247,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1261,7 +1261,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(_T("1179 1179")),
 				0,
 				O(),
-				CDateTime(CDateTime::utc, 2008, 4, 26, 13, 55, 1)
+				fz::datetime(fz::datetime::utc, 2008, 4, 26, 13, 55, 1)
 			},
 			DEFAULT
 		});
@@ -1275,7 +1275,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
 				O(_T("/foo")),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1289,7 +1289,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1304,7 +1304,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				0,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1318,7 +1318,7 @@ void CDirectoryListingParserTest::InitEntries()
 				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
 				O(),
-				CDateTime()
+				fz::datetime()
 			},
 			DEFAULT
 		});
@@ -1332,7 +1332,7 @@ void CDirectoryListingParserTest::InitEntries()
 			R(),
 			0,
 			O(),
-			CDateTime(CDateTime::utc, 2009, 7, 22, 9, 25, 10)
+			fz::datetime(fz::datetime::utc, 2009, 7, 22, 9, 25, 10)
 		},
 		DEFAULT
 	});
@@ -1348,7 +1348,7 @@ void CDirectoryListingParserTest::InitEntries()
 			R(),
 			0,
 			O(),
-			CDateTime(CDateTime::utc, 2003, 5, 21)
+			fz::datetime(fz::datetime::utc, 2003, 5, 21)
 		},
 		DEFAULT
 	});
@@ -1365,7 +1365,7 @@ void CDirectoryListingParserTest::InitEntries()
 			R(),
 			0,
 			O(),
-			CDateTime(CDateTime::utc, 2011, 8, 25)
+			fz::datetime(fz::datetime::utc, 2011, 8, 25)
 		},
 		DEFAULT
 	});
@@ -1382,7 +1382,7 @@ void CDirectoryListingParserTest::InitEntries()
 			R(),
 			CDirentry::flag_dir,
 			O(),
-			CDateTime(CDateTime::utc, 2003, 5, 21)
+			fz::datetime(fz::datetime::utc, 2003, 5, 21)
 		},
 		DEFAULT
 	});
@@ -1396,7 +1396,7 @@ void CDirectoryListingParserTest::InitEntries()
 			R(_T("0 0")),
 			CDirentry::flag_dir,
 			O(),
-			CDateTime(CDateTime::utc, calcYear(7, 29), 7, 29, 2, 27)
+			fz::datetime(fz::datetime::utc, calcYear(7, 29), 7, 29, 2, 27)
 		},
 		DEFAULT
 	});
@@ -1411,18 +1411,18 @@ void CDirectoryListingParserTest::InitEntries()
 			R(),
 			0,
 			O(),
-			CDateTime(CDateTime::utc, 1968, 11, 5, 16, 52, 15)
+			fz::datetime(fz::datetime::utc, 1968, 11, 5, 16, 52, 15)
 		},
 		DEFAULT
 	});
 
 /*
-	wxString name;
-	wxLongLong size;
-	wxString permissions;
-	wxString ownerGroup;
+	std::wstring name;
+	int64_t size;
+	std::wstring permissions;
+	std::wstring ownerGroup;
 	int flags;
-	wxString target; // Set to linktarget it link is true
+	std::wstring target; // Set to linktarget it link is true
 
 	wxDateTime time;
 */
